/*----------------------------------------------------------------------------*\
**                                BandRecord.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)BandRecord.c	5.1 02/26/99, Copyright DLR 1998"

#include "bandcord.h"
#include "log.h"


#define	SccsId	("@(#)BandRecord.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("BandRecord")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                              Band__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		read Band Record Data
**
**  Other interfaces:
**
**  Description:
**	Function to read all Band Record Data from the given 
**	input stream into the Band Record Data structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Band__read(BandRecord_s *Param, FILE* Stream, long DataSize)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    GOME_Word_t IntgTime;
    GOME_Word_t PixelValue;
    int i;
    char LogBuffer[ LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** read Quality Flags
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->Flag, 1, sizeof( Param->Flag), Stream);

    /*--------------------------------------------------------------------*\
    ** read index to KeyDataCalibration
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->KeyDataCalibrationIndx, 1,
			sizeof( Param->KeyDataCalibrationIndx), Stream);

    /*--------------------------------------------------------------------*\
    ** read index to Calibration record
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->CalibrationIndx, 1,
			sizeof( Param->CalibrationIndx), Stream);

    /*--------------------------------------------------------------------*\
    ** read Integration Time
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &IntgTime, 1, sizeof( IntgTime), Stream);
    Param->IntgTime = (float)IntgTime;

    /*--------------------------------------------------------------------*\
    ** read data array
    \*--------------------------------------------------------------------*/
    for (i = 0; i < DataSize; i++)
    {
	NoOfBytes += fread( &PixelValue, 1, sizeof( PixelValue), Stream);
	Param->Data[ i] = (float)PixelValue;
    }

    if (NoOfBytes !=  sizeof( IntgTime) + sizeof( Param->Flag) +
	sizeof( Param->CalibrationIndx) +
	sizeof( Param->KeyDataCalibrationIndx) + 
	DataSize * sizeof( PixelValue))
    {
	Status = FATAL;
	sprintf( LogBuffer, "Band data record, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.2__2495", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                           Band__Show
**  Input parameters:
**	Stream		output stream
**	Param		Band Record Structure
**
**  Output parameters:
**
**  Other interfaces:
**	Band Record Structure written to the output stream
**
**  Description:
**	Function to write the Band Record Structure to the output
**	stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
Band__Show(const BandRecord_s *Param, FILE* Stream, long DataSize)
{
    int	i;

    fprintf( Stream, "\n*** Band Data\n\n");

    /*--------------------------------------------------------------------*\
    ** show Integration Time, Quality flag, Calibration index
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Quality Flag               : %d\n", Param->Flag);
    fprintf( Stream, "Calibration Index          : %d\n", Param->CalibrationIndx);
    fprintf( Stream, "KeyDataCalibration Index : %d\n",
	     Param->KeyDataCalibrationIndx);
    fprintf( Stream, "IntegrationTime            : %f\n", Param->IntgTime);

    /*--------------------------------------------------------------------*\
    ** show data array
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "\nIndex data:\n");
    for ( i = 0; i < DataSize; i++)
    {
	fprintf( Stream, "%4d %6f\n", i, Param->Data[ i]);
    }

}


/*----------------------------------------------------------------------------*\
**                                WriteBandParameter
**  Input parameters:
**    Stream		Output stream
**    Band		enumeration of Band
**    BandRecord	Parameter of Band Data
**    Nsamp		Size of Band Data
**    FirstWaveLength	first wavelength of band data
**    LastWaveLength	last wavelength of band data
**    ReflectivityJump	flag for reflectivity jump
**    Error		Spectral Calibration Error of Band data
**
**  Output parameters:
**    WriteBandParameter	SUCCESS	= output ok
**				FATAL	= any error occured
**
**  Other interfaces:
**    Output file specified by the given stream
**
**  Description:
**    Function to write the Band parameter to the given stream
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
WriteBandParameter(FILE *Stream, int Band, const BandRecord_s *BandRecord,
		   short Nsamp, float FirstWaveLength, float LastWaveLength,
		   int ReflectivityJump, double Error)
{
    FunctionStatus_e	Status		= SUCCESS;
    char			DeadPixel;
    char			HotPixel;
    char			SaturatedPixel;
    char			SpectralCheck;

    /*----------------------------------------------------------------------*\
    ** get the subflags from Quality Flag
    \*----------------------------------------------------------------------*/
    DeadPixel	= (BandRecord->Flag & 0x3);
    HotPixel	= (BandRecord->Flag & 0xC) / (char)0x4;
    SaturatedPixel= (BandRecord->Flag & 0x30) / (char)0x10;
    SpectralCheck	= (BandRecord->Flag & 0xC0) / (char)0x40;

    /*----------------------------------------------------------------------*\
    ** write measurement pixel number, subset counter, Quality Flags and
    ** the number of detector pixel samples
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "%2s %10.5f %08.4f %08.4f %4d %06.4f %1d %1d %1d %1d %1d\n",
	     BandStrTab[ Band], BandRecord->IntgTime,
	     FirstWaveLength, LastWaveLength, Nsamp,
	     Error,
	     ReflectivityJump,
	     SpectralCheck, SaturatedPixel,
	     HotPixel, DeadPixel);
    if ( ferror( Stream) != 0)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.22__5445", Status, LOG_CANNOT_WRITE,
		  "Band parameter", "");
    }

    return Status;
}



/*============================================================================*\
**  private operations
\*============================================================================*/


