/*----------------------------------------------------------------------------*\
**                              brent.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function brent
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)brent.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "numrecip.h"
#include "log.h"


#define	SS_ID	("brent")


#define	ITMAX	(100)

#define	CGOLD	(0.3819660)

#define	ZEPS	(1.0e-10)

#define SIGN(a,b) ((b) > 0.0 ? fabs(a) : -fabs(a))
#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
/*----------------------------------------------------------------------------*\
**                                brent
**  Input parameters:
**    double ax, bx, cx, tol
**    double (*f)(double)
**  Output parameters:
**    xmin
**    Status: tells if there are some problems during the computation
**  Other interfaces: none
**  Description:
**    Parabolic Interpolation and Brent's Method for minimization or
**    maximization of functions.
**    Given a function f, and a bracketing triplet of abscissas ax, bx, cx
**    (such that bx is between ax and cx, and f(bx) is less than both
**    f(ax) and f(cx)), this routine isolates the minimun to a fraction
**    precision of about tol using Brent's method. The abscissa of the
**    minimun is returned as xmin, and the minimun function value is
**    returned as brent, the returned function value.
**    NOTE: The funcion brent can be used to compute the MAXIMUN with
**          -f instead of the function f.
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    26.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
double brent(double ax, double bx, double cx, double (*f)(double), double tol, double *xmin, FunctionStatus_e * Status)
{
	short iter;
	double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
	double e=0.0;

	*Status = SUCCESS;
	a=((ax < cx) ? ax : cx);
	b=((ax > cx) ? ax : cx);
	x=w=v=bx;
	fw=fv=fx=(*f)(x);
	for (iter=1;iter<=ITMAX;iter++) {
		xm=0.5*(a+b);
		tol2=2.0*(tol1=tol*fabs(x)+ZEPS);
		if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
			*xmin=x;
			return fx;
		}
		if (fabs(e) > tol1) {
			r=(x-w)*(fx-fv);
			q=(x-v)*(fx-fw);
			p=(x-v)*q-(x-w)*r;
			q=2.0*(q-r);
			if (q > 0.0) p = -p;
			q=fabs(q);
			etemp=e;
			e=d;
			if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
				d=CGOLD*(e=(x >= xm ? a-x : b-x));
			else {
				d=p/q;
				u=x+d;
				if (u-a < tol2 || b-u < tol2)
					d=SIGN(tol1,xm-x);
			}
		} else {
			d=CGOLD*(e=(x >= xm ? a-x : b-x));
		}
		u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));
		fu=(*f)(u);
		if (fu <= fx) {
			if (u >= x) a=x; else b=x;
			SHFT(v,w,x,u)
			SHFT(fv,fw,fx,fu)
		} else {
			if (u < x) a=u; else b=u;
			if (fu <= fw || w == x) {
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} else if (fu <= fv || v == x || v == w) {
				v=u;
				fv=fu;
			}
		}
	}
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_109", ERROR, LOG_NUMRECIP, "Too many iterations", "");
	*Status = ERROR;
	*xmin=x;
	return fx;
}

#undef SIGN
