/*----------------------------------------------------------------------------*\
**                                BSDF.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/02/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)BSDF.c	5.2 08/02/99, Copyright DLR 1998"

#include <stdlib.h>
#include <math.h>
#include <errno.h>
#include <string.h>
#include "bsdf.h"
#include "dataconv.h"
#include "numrecip.h"
#include "log.h"
#ifdef DEBUG
#include "debug.h"
#endif

extern int errno;


#define	SccsId	("@(#)BSDF.c 5.2 08/02/99, Copyright DLR 1998")


#define	SS_ID	("BSDF")

static BSDF_azimuth_corr_s azimuth_corr;
static Bool file_read = False;

#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                             BSDF__LoadAzimuthCorrectionParameters
**  Input parameters:
**      filename: name of the degradation parameter file
**
**  Return value:
**      Status
**
**  Description:
**      load BSDF azimuth dependancy correction parameters from given file
**
**  Created:	25.8.98
**
**  Author:	Christian Rehn, DLR/DFD-AP
**
\*----------------------------------------------------------------------------*/
FunctionStatus_e
BSDF__LoadAzimuthCorrectionParameters(const char filename[], BSDF_s BSDF, 
  char SoftwareVersion[LEVEL_1_PRODUCT_VERSION_SIZE])
{
  FunctionStatus_e Status = SUCCESS;
  char line[MAX_FILENAME_LENGTH];
  FILE *deg_stream;
  short iline;

  float dtmp;
  float ProductVersion;
  float AzimCorrVersion = -1;

  /*------------------------------------------------------------------------*\
  ** if not already read, read the degradation parameter file
  \*------------------------------------------------------------------------*/
  if (file_read == False)
  {

    deg_stream = fopen(filename, "r");

    if (deg_stream == NULL)
    {
      Status = FATAL;
      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__80", Status,
        LOG_CANNOT_OPEN, "Degradation parameter file - download the latest version from http://auc.dfd.dlr.de/GOME/", "");
      printf("Degradation parameter file missing - download the latest version from\nhttp://auc.dfd.dlr.de/GOME/\n");
      return Status;
    }

    /*------------------------------------------------------------------------*\
    ** convert ProductVersion to float number
    \*------------------------------------------------------------------------*/
    ProductVersion = (float) atof(SoftwareVersion);
    if (errno == ERANGE)
    {
      Status = FATAL;
      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__80", Status,
        LOG_CANNOT_CALCULATE, "Cannot convert ProductVersion from string to float", "");
      return Status;
    }

    /*------------------------------------------------------------------------*\
    ** assign BSDF.Azimuth to azimuth_corr.BSDF_a
    \*------------------------------------------------------------------------*/
    azimuth_corr.BSDF_a = BSDF.Azimuth;

    /*------------------------------------------------------------------------*\
    ** read line by line, search keywords, copy data to parameter structure
    \*------------------------------------------------------------------------*/
    while (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
    {
      if (strncmp(line, "highest_BSDF_version", 20) == 0)
      {
        while (fgets (line, MAX_FILENAME_LENGTH, deg_stream) && strcmp(line, "\n") != 0 && line[0] != ' ')
        {
          sscanf (line, "%f\n", &AzimCorrVersion);
        }
      }
      if (strncmp(line, "BSDF_a0", 7) == 0)
      {
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%f\n", &azimuth_corr.BSDF_a0);
        }
      }
      if (strncmp(line, "num_BSDF_a1", 11) == 0)
      {
        if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
        {
          sscanf (line, "%d\n ", &azimuth_corr.n_BSDF_a1);
        }
      }

      if (strncmp(line, "BSDF_a1", 7) == 0)
      {
        for (iline = 0; iline < azimuth_corr.n_BSDF_a1; iline++)
        {
          if (fgets (line, MAX_FILENAME_LENGTH, deg_stream))
          {
            sscanf (line, "%f %f\n", &azimuth_corr.BSDF_a1_grid[iline], &azimuth_corr.BSDF_a1[iline]);
          }
        }
      }
    }

    fclose(deg_stream);
  
    file_read = True;

    if (AzimCorrVersion == -1)
    {
      Status = ERROR;
      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__80", Status,
        LOG_CANNOT_READ, "parameters for aymmetric BSDF correction - No correction applied", "");
      return Status;
    }

    if (ProductVersion > AzimCorrVersion)
    {
      Status = ERROR;
      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__80", INFO,
        LOG_CANNOT_READ, "parameters for aymmetric BSDF correction. SoftwareVersion too high", 
          "Aymmetric BSDF correction already applied in l1 product");
      return Status;
    }
  }

  return Status;
}

/*----------------------------------------------------------------------------*\
**                                BSDF__CorrectAzimuth
**  Input parameters: 
**    BSDF_azimuth_corr_s* azimuth_corr : structure containing BSDF azimuth dependancy correction 
**                                parameters from given file
**    Azimuth : angle between sun and diffuser entrance
**    Band (1A .. 4)
**    Reticon data (1024)
**    Lambda grid for Reticon data (1024)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (ASCII string)
**    
**  Output parameters: 
**    Reticon data (1024) (now corrected)
**    
**  Other interfaces:
**  Description:
**    BSDF asymetric azimuth dependancy correction, parameters are read from file
**  References:
**  Libraries:
**  Created:	10.3.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e BSDF__CorrectAzimuth (float Azimuth,
  ScienceData_e Band, float Reticon[], const float LambdaGrid[], short Begin, short End, const UTC_s *Date)
{
  FunctionStatus_e Status = SUCCESS;
  int loop;
  float BSDF_a1_fitted[DETECTOR_PIXELS];
  float corr[DETECTOR_PIXELS];

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    char ASCII_Time[ 30];
    float InputReticon [DETECTOR_PIXELS];
    FILE *out;
    char debug_filename[80];

    UTC_2_ASCII(*Date, ASCII_Time);

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_2))
    {	
	for (loop = Begin; loop <= End; loop++)
	{
	    InputReticon [loop] = Reticon[loop];
	}
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
  /* Interpolate BSDF_a1 onto sun grid */
  fit_grid_akima(azimuth_corr.BSDF_a1_grid, azimuth_corr.BSDF_a1, azimuth_corr.n_BSDF_a1,
    LambdaGrid, BSDF_a1_fitted, Begin, End);

  /* Apply correction */
  for (loop = Begin; loop <= End; loop++)
  {
    corr[loop] =  ( 1.0 + BSDF_a1_fitted[loop] * Azimuth - azimuth_corr.BSDF_a0 * Azimuth * Azimuth )
                     /
            ( 1.0 - azimuth_corr.BSDF_a * Azimuth * Azimuth );

    Reticon[loop] /= corr[loop];
  }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG	
    sprintf (debug_filename, "%s", DbgFileName("BSDF__CorrectAzimuth", ASCII_Time) );

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\n");
		fprintf(out, "Azimuth : %f a : %f a0 : %f\n", Azimuth, azimuth_corr.BSDF_a, azimuth_corr.BSDF_a0);
		fprintf(out, 
			"Band       Lambda     InputRetc BSDFa1_fit corr-fac   OutputRet \n");
		fprintf(out, 
			"[]         []         []        []         []         []        \n\n");
	    }
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %*.*g %*.*g %*.*g %*.*g %*.*g \n", 
			Band, 
			DEBUG_WIDTH, DEBUG_PRECISION, LambdaGrid[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, BSDF_a1_fitted[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, corr[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon[loop]);
	    }
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "3.17_167", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif

  return Status;
}

/*----------------------------------------------------------------------------*\
**                                BSDF__CalculateMeanSunAzimuth
**  Input parameters: 
**    BSDF_azimuth_corr_s* azimuth_corr : structure containing BSDF azimuth dependancy correction 
**                                parameters from given file
**    Azimuth : angle between sun and diffuser entrance
**    Band (1A .. 4)
**    Reticon data (1024)
**    Lambda grid for Reticon data (1024)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (ASCII string)
**    
**  Output parameters: 
**    Azimuth (float)
**    
**  Other interfaces:
**  Description:
**    Calculate azimuth for mean sun reference for BSDF asymetric azimuth 
**    dependancy correction.
**    Only valid for solar zenith angle 90 degrees and ERS2 orbit.
**
**  References:
**  Libraries:
**  Created:	10.3.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e BSDF__CalculateMeanSunAzimuth (float* Azimuth, const UTC_s Date)
{
  FunctionStatus_e Status = SUCCESS;
  int loop;
  UTC_s ReferenceDate = {0, 0};
  double coeffs[] = {0.0861850, -0.00797775, -2.50270, -0.0735632, 5.14285, 0.822299, 0.516461};
  float YearFracArc;
  float elapsed_days = 0.0;  /* elapsed_days since 3.7.95 0:00h */

  if (!ASCII_2_UTC("03-JUL-1995 00:00:00.000", &ReferenceDate) )
  {
    Status = FATAL;
    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__323", Status,
      LOG_CANNOT_CALCULATE, "reference time for azimuth", "");
    return Status;
  }

  elapsed_days = ((float) Date.Days + (float) Date.Msecs / (float) MSECS_PER_DAY)
    - ((float) ReferenceDate.Days + (float) ReferenceDate.Msecs / (float) MSECS_PER_DAY);

  YearFracArc = 2. * M_PI * elapsed_days / 365.24;

  *Azimuth = (float) 
           coeffs[0] * sin(3.* YearFracArc)
         + coeffs[1] * cos(3.* YearFracArc)
         + coeffs[2] * sin(2.* YearFracArc)
         + coeffs[3] * cos(2.* YearFracArc)
         + coeffs[4] * cos(YearFracArc)
         + coeffs[5] * sin(YearFracArc)
         + coeffs[6];

  return Status;
}

/*----------------------------------------------------------------------------*\
**                                BSDF__ComputeAngles
**  Input parameters: float PropagatorZenith, float PropagatorAzimuth
**  Output parameters: float *BSDF_Elevation, float *BSDF_Azimuth
**  Other interfaces:
**  Description:
**    Converts sun angles calculated by the Propagator to the angles suitable for
**    TPD's BSDF function
**  References:
**  Libraries:
**  Created:	10.3.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void BSDF__ComputeAngles (float PropagatorZenith, float PropagatorAzimuth, float *BSDF_Zenith, float *BSDF_Azimuth)
{
    *BSDF_Azimuth = 0.0;
    *BSDF_Zenith = 0.0;

    /*----------------------------------------------------------------------*\
    ** calculate BSDF_Azimuth
    \*----------------------------------------------------------------------*/
    *BSDF_Azimuth = fabs( PropagatorAzimuth - ZERO_AZIMUTH );

    /*----------------------------------------------------------------------*\
    ** calculate BSDF_Elevation
    \*----------------------------------------------------------------------*/
    *BSDF_Zenith = PropagatorZenith;

    /*
     *BSDF_Elevation = PropagatorZenith - 90.0;
     */
}

/*----------------------------------------------------------------------------*\
**                                BSDF__Calculate
**  Input parameters:
**  Output parameters:
**  Other interfaces:
**
**  Description:
**	Calculates the BSDF(Lambda, Azimuth, Elevation)
**
**	BSDF(LAM, EL, AZ) = BSDF_0*(1 + cel*EL)(1 - caz*AZ^2)*(1 + c1*LAM + c2*LAM^2 + ... + c7*LAM^7) 
**
**	LAM = (LAM - 500nm)/500nm
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	12.1.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
float BSDF__Calculate (const BSDF_s *Self, float Lambda, float Elevation, float Azimuth)
{
    short loop;
    float Y, Result;

    float Mult = 1.0;

    Y = 1.0;

    for (loop = 1; loop < BSDF_FITT_ORDER; loop++)
    {
	Y += Self->Coefficients[loop] * pow( ((Lambda - 500.0)/500.0), (float) loop);
    }

    Result = Self->BSDF_0 * ( 1 + (Self->Elevation * Elevation)) * ( 1 - (Self->Azimuth * Azimuth * Azimuth)) * Y;
    return Result;
}

/*----------------------------------------------------------------------------*\
**                                BSDF__Apply
**  Input parameters:
**    Self containing BSDF as polynomial function
**    Intensity wavelength grid (1024)
**    PropagatorZenith (sun zenith angle)
**    PropagatorAzimuth (sun azimuth angle)
**    Band (1A .. 4)
**    Reticon data (1024)
**    Lambda grid for Reticon data (1024)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (ASCII string)
**  Output parameters:
**    BSDF as tabulated spectrum
**    Calibrated Reticon data
**  Other interfaces:
**  Description:
**    Divides a spectrum by the BSDF of the sun diffuser
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	16.3.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void BSDF__Apply(const BSDF_s* Self, float PropagatorZenith, float PropagatorAzimuth,
		 ScienceData_e Band, float Reticon[], const float LambdaGrid[],  float BSDF[], short Begin, short End, const char *PixelDate)
{
    int loop;
    float BSDF_Elevation, BSDF_Zenith, BSDF_Azimuth;

#ifdef DEBUG
    float InputReticon [DETECTOR_PIXELS];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_2))
    {	
	for (loop = Begin; loop <= End; loop++)
	{
	    InputReticon [loop] = Reticon[loop];
	}
    }
#endif


    /*-----------------------------------------------------------------------*\
    ** Convert the angles given from the propagator to BSDF suitable angles
    ** BSDF__ComputeAngles must not be called here, it is already called in the
    ** Processor and the angles are passed through
    \*-----------------------------------------------------------------------*/
    /* BSDF__ComputeAngles (PropagatorZenith, PropagatorAzimuth, &BSDF_Zenith, &BSDF_Azimuth); */	
    BSDF_Zenith = PropagatorZenith;
    BSDF_Azimuth = PropagatorAzimuth;

    BSDF_Elevation = BSDF_Zenith - 90.0;

    /*-----------------------------------------------------------------------*\
    ** Calculate the BSDF for each wavelength
    \*-----------------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
    {
	BSDF[loop] = BSDF__Calculate (Self, LambdaGrid[loop], BSDF_Elevation, BSDF_Azimuth);
    }

    /*-----------------------------------------------------------------------*\
    ** Divide Reticon by BSDF
    \*-----------------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
    {
	if (BSDF[loop] > 1e-10)
	    Reticon[loop] /= BSDF[loop];
	else
	    Reticon[loop] = 0.0;
    }


    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG	
    sprintf (debug_filename, "%s", DbgFileName("BSDF__Apply", PixelDate) );

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Band       Lambda     InputRetc BSDF       OutputRet \n");
		fprintf(out, 
			"[]         []         []        []         []        \n\n");
	    }
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %*.*g %*.*g %*.*g %*.*g \n", 
			Band, 
			DEBUG_WIDTH, DEBUG_PRECISION, LambdaGrid[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, BSDF[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon[loop]);
	    }
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "3.17_167", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif

}


/*----------------------------------------------------------------------------*\
**                                BSDF__Read
**  Input parameters:
**	Stream			input stream
**  Output parameters:
**	BSDF			Key Data Errors
**
**  Other interfaces:
**
**  Description:
**	Function to read all BSDF parameter entries from the given 
**	input stream into the BSDF structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
BSDF__read(BSDF_s *BSDF, FILE* Stream, Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read BSDF
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread( &BSDF->BSDF_0, 1,
			sizeof( BSDF->BSDF_0), Stream); 
    NoOfBytes += fread( &BSDF->Elevation, 1,
			sizeof( BSDF->Elevation), Stream); 
    NoOfBytes += fread( &BSDF->Azimuth, 1,
			sizeof( BSDF->Azimuth), Stream); 

    /*----------------------------------------------------------------------*\
    ** read BSDF Response_f2_SMDep
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread( BSDF->Coefficients, 1,
			sizeof( BSDF->Coefficients), Stream); 

    /*----------------------------------------------------------------------*\
    ** check size
    \*----------------------------------------------------------------------*/
    if (NoOfBytes != BSDF_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "BSDF Parameter, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.2__4434", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if  (convert_sun_2_host)
    {
	BSDF__Sun2Host( BSDF);
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                BSDF__Write
**  Input parameters:
**	Stream			output stream
**	BSDF			Key Data Errors
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the BSDF parameter entries to the given 
**	output stream
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
BSDF__write(const BSDF_s *BSDF, FILE* Stream)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read BSDF
    \*----------------------------------------------------------------------*/
    NoOfBytes += fwrite( &BSDF->BSDF_0, 1,
			 sizeof( BSDF->BSDF_0), Stream); 
    NoOfBytes += fwrite( &BSDF->Elevation, 1,
			 sizeof( BSDF->Elevation), Stream); 
    NoOfBytes += fwrite( &BSDF->Azimuth, 1,
			 sizeof( BSDF->Azimuth), Stream); 

    /*----------------------------------------------------------------------*\
    ** read BSDF Response_f2_SMDep
    \*----------------------------------------------------------------------*/
    NoOfBytes += fwrite( BSDF->Coefficients, 1,
			 sizeof( BSDF->Coefficients), Stream); 

    /*----------------------------------------------------------------------*\
    ** check size
    \*----------------------------------------------------------------------*/
    if (NoOfBytes != BSDF_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "BSDF Parameter, #written bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.2__4495", Status,
		  LOG_CANNOT_WRITE, LogBuffer, "");
    }
    return Status;
}

/*----------------------------------------------------------------------------*\
**                                BSDF__Show
**  Input parameters:
**	Stream			output stream
**	BSDF			Key Data Error structure
**
**  Output parameters:
**
**  Other interfaces:
**	Key Data Error structure written to the output stream
**
**  Description:
**	Function to write the Key Data Error structure to the
**	output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
BSDF__show(const BSDF_s *BSDF, FILE* Stream)
{
    int	i;

    fprintf( Stream, "BSDF Function\n");
    fprintf( Stream, "%f %f %f\n", BSDF->BSDF_0, BSDF->Elevation, BSDF->Azimuth);
    for ( i = 0; i < BSDF_FITT_ORDER; i++)
	fprintf( Stream, "\t%f", BSDF->Coefficients[ i]);
    fprintf( Stream, "\n");

}



/*============================================================================*\
**  private operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            BSDF__Sun2Host
**  Input parameters:
**    BSDF		in Sun memory representation
**  Output parameters:
**    BSDF		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the BSDF struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
BSDF__Sun2Host( BSDF_s *Param)
{
    int	i;

    Param->BSDF_0 =
	SunFloat2Float( (unsigned char*)&Param->BSDF_0);
    Param->Elevation =
	SunFloat2Float( (unsigned char*)&Param->Elevation);
    Param->Azimuth =
	SunFloat2Float( (unsigned char*)&Param->Azimuth);

    for ( i = 0; i < BSDF_FITT_ORDER; i++)
    {
	Param->Coefficients[ i] =
	    SunFloat2Float( (unsigned char*)&Param->Coefficients[ i]);

    }
}

