/*----------------------------------------------------------------------------*\
**                                DR.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)DR.c	5.1 02/26/99, Copyright DLR 1998"

#include "dr.h"
#include "log.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)DR.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("DR")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
extern Debug_s* debug_bdr_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                DR__Init
**  Input parameters:
**
**  Output parameters:
**    DR		Spectrum data Description record (Radiance) structure
**    DR__Init		SUCCESS	= Initialization ok
**			else	= any error
**
**  Other interfaces:
**
**  Description:
**    Function to initialize the DR with the default values.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
DR__Init(DR_s *DR)
{
    FunctionStatus_e	Status		= SUCCESS;
    int			i, j;
    /*-----------------------------------------------------------------*\
    ** define wavelength range
    \*-----------------------------------------------------------------*/
    DR->StartWavelength	= 0.0;
    DR->EndWavelength	= 0.0;

    /*-----------------------------------------------------------------*\
    ** set default values
    \*-----------------------------------------------------------------*/
    for ( i = 0; i < PMD_DEFINED; i++) {
	DR->PMD_Flag[ i] = 0;
    }
    for ( i = 0; i < OPTICAL_CHANNELS; i++) {
	for ( j = 0; j < DETECTOR_PIXELS; j++) {
		DR->Radiance[i][j] = 0.0;
		DR->AbsErrorRad[i][j] = 0.0;
		DR->RelErrorRad[i][j] = 0.0;
		DR->Flag[i][j] = 0;
	    }
    }
    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)	{
	    DR->BandRecord[i].IntgTime = 0.0;
	    DR->OrigBandRecord[i].IntgTime = 0;
	    DR->ReflectivityJump[i] = REFLECTIVITY_JUMP_NO_CHECK;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                GetBandDataLevel_1_Product
**  Input parameters:
**    Config		Extract configuration structure
**    FSR1		File structure record of Level 1 Product
**    FCD		Fixed Calibration record of Level 1 Product
**    ProductType	product type
**    Band		enumeration of the Band data to be read
**    PixelPar		pixel parameter description
**    CorFactor		the multiplication factor of the raw data
**			(co-adding mode)
**
**  Output parameters:
**    Spectrum				spectrum description record
**					structure
**    GetBandDataLevel_1_Product	SUCCESS = Band read ok and complete
**					FATAL	= any error occured reading Band
**
**  Other interfaces:
**    Input file of Level 1 Product specified in Config
**
**  Description:
**    Function to read one complete Band from the according BDR list of the 
**    Level 1 Product into the DR structure. The BDR list index is selected
**    by the ScienceIndex of the Current PCD or SCD or MCD.
**    The Band data and Parameter are converted and multiplied by the CorFactor
**    (Co-adding mode).
**    The KeyDataCalibrationIndx of the Band is checked for availability.
** 
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
GetBandDataLevel_1_Product(const Config_s *Config, const FSR1_s *FSR1,
			   const FCD_s *FCD, ProductType_e ProductType, 
			   ScienceData_e Band, const PixelPar_s *PixelPar,
			   DR_s *Spectrum, float CorFactor, 
			   Bool CONVERT_SUN_2_HOST)
{
    FunctionStatus_e Status = SUCCESS;
    long FilePos_SD;
    int i, j;
    GOME_Word_t RadianceOrig[OPTICAL_CHANNELS][DETECTOR_PIXELS];
    char LogBuffer[LOG_FIELD_SIZE];
#ifdef DEBUG  
    char debug_message[255];
#endif

    /*------------------------------------------------------------------------*\
    ** position to the Band within the level 1 product
    \*------------------------------------------------------------------------*/
    FilePos_SD = Config->FilePos_SD[ Band] + FSR1->BDR_Rec_Size[ Band] *
	PixelPar->ScienceIndex[ Band];
    if (fseek (Config->Level_1_ProductStream, FilePos_SD, SEEK_SET) == -1) {
	Status = FATAL;
	sprintf( LogBuffer, "Band %2d from level 1 product file (fseek)", Band);
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.1__154", Status,
		 LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

#ifdef DEBUG  
    sprintf(debug_message, "BDR__Read: %d\n", Band);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message); 
    Debug__print(debug_bdr_ptr, DBG_EXTRACTING, DEBUG_LEVEL_BDR,
		 debug_message);
#endif
    /*--------------------------------------------------------------------*\
    ** set default values
    \*--------------------------------------------------------------------*/
    for ( i = 0; i < OPTICAL_CHANNELS; i++) {
	for ( j = 0; j < DETECTOR_PIXELS; j++) {
	    RadianceOrig[i][j] = 0;
	}
    }

    /*--------------------------------------------------------------------*\
    ** read Band
    \*--------------------------------------------------------------------*/
    if (BDR__read(&Spectrum->OrigBandRecord[Band],
		  Config->Level_1_ProductStream,
		  FSR1->BDR_Rec_Size[ Band],
		  &RadianceOrig[ Band2Channel( Band)]
		  [ FCD->BCR[ Band].StartDetector],
		  FCD->BCR[ Band].EndDetector 
		  - FCD->BCR[ Band].StartDetector + 1,
		  CONVERT_SUN_2_HOST) != SUCCESS) {
	Status = FATAL;
	sprintf( LogBuffer, "Band %2d from level 1 product file", Band);
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__187", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	    return Status;
    }

    /*----------------------------------------------------------------*\
    ** convert the original band data to the necessary format
    \*----------------------------------------------------------------*/
    for (j = FCD->BCR[ Band].StartDetector;
	 j <= FCD->BCR[ Band].EndDetector; j++) {
	Spectrum->Radiance[ Band2Channel( Band)][ j] = (float)
	    RadianceOrig[ Band2Channel( Band)][ j] * CorFactor;
    }
    Spectrum->BandRecord[ Band].Flag =
	Spectrum->OrigBandRecord[ Band].Flag;
    Spectrum->BandRecord[ Band].CalibrationIndx =
	Spectrum->OrigBandRecord[ Band].CalibrationIndx;
    Spectrum->BandRecord[ Band].KeyDataCalibrationIndx =
	Spectrum->OrigBandRecord[ Band].KeyDataCalibrationIndx;
    Spectrum->BandRecord[ Band].IntgTime = (float)
	Spectrum->OrigBandRecord[ Band].IntgTime *
	INTG_TIME_CONV_FACTOR;
    Spectrum->BandRecord[ Band].Data = 0;
    
    /*----------------------------------------------------------------*\
    ** check the band data indices
    \*----------------------------------------------------------------*/
    if ((Spectrum->BandRecord[ Band].KeyDataCalibrationIndx < 0) ||
	(Spectrum->BandRecord[ Band].KeyDataCalibrationIndx >= FCD->Nang))
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__218", Status, LOG_NUM,
		  "BDR, Polarization Sensitivity Parameter index out of range",
		  "");
	    return Status;
    }
    if (Spectrum->BandRecord[ Band].KeyDataCalibrationIndx ==
	INVALID_INDEX) 
    {
	Status = FATAL;
	sprintf( LogBuffer, "BDR (Band %2d), Polarization Sensitivity Parameter == INVALID_INDEX", Band);
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__228", Status, LOG_NUM,
		  LogBuffer, "");
	    return Status;
    }

    return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
