/*----------------------------------------------------------------------------*\
**                                EDR.c
**
**  Description:
** 
**  Version:    5.3
**  Date:       07/28/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)EDR.c	5.3 07/28/99, Copyright DLR 1998"

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "edr.h"
#include "filter.h"
#include "calilter.h"
#include "log.h"
#include "reflvity.h"
#include "qa.h"
#include "qualheck.h"
#include "pcd.h"
#include "smcd.h"
#include "wat2phot.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)EDR.c 5.3 07/28/99, Copyright DLR 1998")


#define	SS_ID	("EDR")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*----------------------------------------------------------------------------*\
**  extern global variables
\*----------------------------------------------------------------------------*/
extern float previous_PMD_16[OPTICAL_CHANNELS][DETECTOR_PIXELS];
extern char previous_PMD_16_time[ASCII_UTC_TIME_LENGTH];

/*----------------------------------------------------------------------------*\
**  global variables
\*----------------------------------------------------------------------------*/
int EDR_WrittenCounter = 0;

/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                EDR__Init
**  Input parameters:
**    Config		Extract Configuration structure
**    Current_PCD	the current processed PCD (Split Mode)
**
**  Output parameters:
**    EDR		Extract Description record (Radiance) structure
**    EDR__Init		SUCCESS	= Initialization ok
**			else	= any error
**
**  Other interfaces:
**
**  Description:
**    Function to initialize the EDR with the default values.
**    In Split mode:
**	- the EDR->First_PCD and EDR->Last_PCD are set to the current
**	  processed PCD.
**	- the EDR->BeginTime and EDR->EndTime are not initialized.
**    In non Split mode the EDR->First_PCD, EDR->Last_PCD, EDR->BeginTime and
**    EDR->EndTime are initialized with the corresponding values of the
**    corresponding extract configuration structure.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__Init(const Config_s *Config, int Current_PCD, EDR_s *EDR)
{
    FunctionStatus_e	Status		= SUCCESS;
    int			i, j;

    /*------------------------------------------------------------------------*\
    ** set the first and the last PCD of the EDR structure according the split
    ** flag
    ** Case Split: the time range is set later during reading
    \*------------------------------------------------------------------------*/
    EDR->ProductType = Config->ProductType;
    if (Config->Filter.splitFlag || Config->Filter.noSunFlag || Config->Filter.albedoFlag)
    {
	EDR->First_PCD	= Current_PCD;
	if (Config->LastMeasurementPixel < Config->FirstMeasurementPixel)
	    EDR->Last_PCD	= Current_PCD - 1;
	else
	    EDR->Last_PCD	= Current_PCD;
    }
    else
    {
	EDR->First_PCD	= Config->FirstMeasurementPixel;
	EDR->Last_PCD	= Config->LastMeasurementPixel;
	EDR->BeginTime	= Config->BeginTime;
	EDR->EndTime	= Config->EndTime;
    }

    /*-----------------------------------------------------------------*\
    ** initialize Spectrum data
    \*-----------------------------------------------------------------*/
    DR__Init( &EDR->Spectrum);

    /*-----------------------------------------------------------------*\
    ** reset the geolocations (PCD) history
    \*-----------------------------------------------------------------*/
    EDR->N_PCD_History	= 0;
    EDR->N_PCD_HistoryRead= 0;
    EDR->PCD_History = 0;

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                EDR__CalculateErrors
**  Input parameters:
**    bandFilter		Band Filter for error calculation
**    calibrationFilter		Calibration filter to seletct the switch on or
**					off the different Calibration steps
**    FCD			Fixed calibrartion record of the Level 1 Product
**    Grid			all grids of the Level 1 Product
**    EDR			Extract description record structure
**    Signal			signal leakage and PPG corrected
**    BSDF			BSDF function
**    P7_BAND_1A_MeanCalculated	flag indicating if the mean value of P7 of
**				Band 1a was calculated
**    PolarizationParP7_Mean	Polarization structure for the mean value of P7
**				of Band 1a
**
**  Output parameters:
**    EDR			Extract description record structure
**    EDR__CalculateErrors	SUCCESS	= error calculation of Spectrum ok
**				else	= any error occured during error
**								calculation
**
**  Other interfaces:
**
**  Description:
**    Function to calculate the absolute and relative error of the 
**    spectrum according the product type and the calibration steps intensity,
**    polarization and BSDF.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__CalculateErrors(const Bool bandFilter[ SCIENCE_DATA_DEFINED],
		     const Bool calibrationFilter[CALIBRATION_DEFINED],
		     const FCD_s *FCD, const GRID_s *GRID, EDR_s *EDR,
		     const float Signal[ OPTICAL_CHANNELS][ DETECTOR_PIXELS],
		     const float BSDF[ OPTICAL_CHANNELS][ DETECTOR_PIXELS],
		     Bool P7_BAND_1A_MeanCalculated,
		     const PolarizationC_s *PolarizationParP7_Mean)
{
    FunctionStatus_e Status = SUCCESS;
    int i, j;
    float AbsErrorRadianceResponse;
    float AbsErrorSig;
    float RelErrorSig;
    ScienceData_e Band;
    OpticalChannel_e Ch;
    float RadResp;
    float PolCor;
    float PolCorFak[ OPTICAL_CHANNELS][ DETECTOR_PIXELS];
    float ErrPolCor[ OPTICAL_CHANNELS][ DETECTOR_PIXELS];
    char ASCII_Time[ 30];
    float BSDF_Val;
    float AbsError_BSDF;
    CalibrationType_e CalType;
    const PolarizationC_s* Polarization_pt;
    Bool PolCorValid;
    char LogBuffer[LOG_FIELD_SIZE];

    /*------------------------------------------------------------------------*\
    ** convert the pixel time
    \*------------------------------------------------------------------------*/
    if (UTC_2_ASCII(EDR->PixelPar.Date, ASCII_Time) != SUCCESS)
    {
	Status = WARN;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__202", Status,
		  LOG_NUM, "converting the UTC time of Geolocation", "");
	strcpy(ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
    }

    /*---------------------------------------------------------------------*\
    ** Get calibration type
    \*---------------------------------------------------------------------*/
    CalType = CalibrationTypeGet( EDR->ProductType, calibrationFilter);

    /*---------------------------------------------------------------------*\
    ** calculate absolute and relative error
    \*---------------------------------------------------------------------*/
    for ( j = 0; j < SCIENCE_DATA_DEFINED; j++)	
    {
	Band	= (ScienceData_e)j;
	Ch		= Band2Channel( Band);
	if (bandFilter[ j] && (EDR->Spectrum.ScienceDataAvail[ j])) 
	{
	    /*----------------------------------------------------------------*\
	    **  calculate the errors of the polarization correction 
	    **  faktors
	    \*----------------------------------------------------------------*/
	    if ((CalType == CAL_RAD_POL) || (CalType == CAL_RAD_POL_INTENS)) 
	    {
		if (((Band == BAND_1A) && P7_BAND_1A_MeanCalculated) 
		    | ((Band == BLIND_1A) && P7_BAND_1A_MeanCalculated) 
		    | ((Band == STRAYLIGHT_1A) && P7_BAND_1A_MeanCalculated)) 
		{
		    Polarization_pt = PolarizationParP7_Mean;
		}
		else 
		{
		    Polarization_pt = &EDR->PixelPar.Specific.Earth.PolarizationPar;
		}

		/*------------------------------------------------------------*\
		**  Check if polarization correction valid
		\*------------------------------------------------------------*/
		PolCorValid = True;
		for (i = 0; i < POLARIZATION_COEFFICIENTS; i++) 
		{
		    /* round INVALID_FRAC_POL because it may be 99.990000 or 99.989998 but p is always 99.989998 */
		    if (Polarization_pt->Coefficient[ i] > (int)INVALID_FRAC_POL) 
		    {
			PolCorValid = False;
		    }
		}
		/*------------------------------------------------------------*\
		**  calculate only polarization errors if polarization 
		**  correction valid
		\*------------------------------------------------------------*/
		if (PolCorValid) 
		{
		    PolarizationC__ComputePolFakError(Polarization_pt,
						     FCD->KeyDataCalibrationL1_Par[EDR->Spectrum.BandRecord[Band].KeyDataCalibrationIndx].Eta_Omega,
						     GRID->Grid[ FCD->IndexSpectralCalPar][ Ch],
						     GRID->Grid[ EDR->PixelPar.IndexSpectralCalPar][ Ch],
						     Band,
						     FCD->BCR[ Band].StartDetector,
						     FCD->BCR[ Band].EndDetector,
						     &FCD->KeyDataError,
						     PolCorFak[ Ch],
						     ErrPolCor[ Ch],
						     ASCII_Time);
		}
		else 
		{
		    /*--------------------------------------------------------*\
		    ** disable PolCorFak and ErrPolCor for error calculation
		    \*-----------------------------------------------------------------*/
		    for (i = FCD->BCR[ Band].StartDetector;
			 i <= FCD->BCR[ Band].EndDetector; i++)	
		    {
			PolCorFak[ Ch][ i]	= 1.0;
			ErrPolCor[ Ch][ i]	= 0.0;
		    }
		}
	    }
	    else 
	    {
		/*------------------------------------------------------------*\
		**  disable PolCorFak and ErrPolCor for error calculation
		\*------------------------------------------------------------*/
		for (i = FCD->BCR[ Band].StartDetector;
		     i <= FCD->BCR[ Band].EndDetector; i++) 
		{
		    PolCorFak[ Ch][ i]	= 1.0;
		    ErrPolCor[ Ch][ i]	= 0.0;
		}
	    }

	    /*----------------------------------------------------------------*\
	    ** calculate errors of one band
	    \*----------------------------------------------------------------*/
	    for (i = FCD->BCR[ Band].StartDetector;
		 i <= FCD->BCR[ Band].EndDetector; i++) 
	    {
		/*------------------------------------------------------------*\
		** calculate absolute error of signal
		\*------------------------------------------------------------*/
		AbsErrorSig = (float)sqrt((double)
					  (Signal[ Ch][ i] * NE 
					   + FCD->LeakagePar[EDR->PixelPar.IndexLeakagePar].ArrayNoise *
					   NE *
					   FCD->LeakagePar[ EDR->PixelPar.IndexLeakagePar].ArrayNoise *
					   NE + 0.5 * NE * 0.5 * NE)) / NE;
        
		if ((float)fabs( (double)Signal[Ch][ i]) < MIN_VAL) 
		{
		    Status = WARN;
#ifdef DEBUG
		    if (Debug__isActive(debug_ptr, DBG_EXTRACTING,
					DEBUG_LEVEL))
		    {
			sprintf(LogBuffer, "Cannot calculate Spectrum Errors"
				"of Channel %2d Pixel %3d, reason Signal: %f",
				Ch + 1, i, Signal[Ch][i]);
			Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__320",
				 Status, LOG_NUM, LogBuffer, "");
		    }
#endif
		    EDR->Spectrum.AbsErrorRad[Ch][i] = UNKNOWN_ERROR_VAL;
		    EDR->Spectrum.RelErrorRad[Ch][i] = UNKNOWN_ERROR_VAL;
		}
		else 
		{
		    /*--------------------------------------------------------*\
		    ** calculate relative error of signal
		    \*--------------------------------------------------------*/
		    RelErrorSig = AbsErrorSig / Signal[ Ch][ i];

		    /*--------------------------------------------------------*\
		    ** calculate absolute error extract format (E1)
		    \*--------------------------------------------------------*/
		    EDR->Spectrum.AbsErrorRad[ Ch][ i] = RelErrorSig *
			EDR->Spectrum.Radiance[Ch][ i];

		    /*--------------------------------------------------------*\
		    ** get the Radiance response and the according error
		    \*--------------------------------------------------------*/
		    if ((CalType == CAL_RAD_INTENS) 
			|| (CalType == CAL_RAD_POL_INTENS) 
			|| (CalType == CAL_IRR_INTENS)
			|| (CalType == CAL_IRR_INTENS_BSDF)) 
		    {
			RadResp = 
			    FCD->KeyDataCalibrationL1_Par[EDR->Spectrum.BandRecord[Band].KeyDataCalibrationIndx].RadianceResponse_Omega[i];
			AbsErrorRadianceResponse = 
			    FCD->KeyDataError.Response_f2_SMDep[ Ch][ i] / 3.0 * RadResp;
		    }
		    else 
		    {
			RadResp	= 1.0;
			AbsErrorRadianceResponse = 0.0;
		    }

		    if ((CalType == CAL_IRR_BSDF)
			|| (CalType == CAL_IRR_INTENS_BSDF)) 
		    {
			/*----------------------------------------------------*\
			** get the BSDF value and error
			\*----------------------------------------------------*/
			BSDF_Val = BSDF[ Ch][ i];
			AbsError_BSDF = KeyDataError__Get( &FCD->KeyDataError, KDE_BSDF,
							   Ch, i) / 3.0 * BSDF_Val;
		    }
		    else 
		    {
			BSDF_Val = 1.0;
			AbsError_BSDF = 0.0;
		    }

		    /*--------------------------------------------------------*\
		    ** check the signal value
		    \*--------------------------------------------------------*/
		    if ((float)fabs( (double)RadResp) < MIN_VAL) 
		    {
			Status = WARN;
#ifdef DEBUG
			if (Debug__isActive(debug_ptr, DBG_EXTRACTING,
					    DEBUG_LEVEL))
			{
			    sprintf(LogBuffer, "Cannot calculate Spectrum "
				    "Errors (E2) of Channel %2d Pixel %3d, "
				    "reason H_Res: %f", Ch + 1, i, RadResp);
			    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__388", 
				     Status, LOG_NUM, LogBuffer, "");
			}
#endif
			EDR->Spectrum.RelErrorRad[Ch][i] = UNKNOWN_ERROR_VAL;
		    }
		    else 
		    {
			/*--------------------------------------------------------------*\
			** calculate absolute error extract format (E2)
			\*--------------------------------------------------------------*/
			PolCor = PolCorFak[ Ch][ i];
			switch (CalType) 
			{
			case CAL_IRR:
			case CAL_RAD: 
			    {
				EDR->Spectrum.RelErrorRad[ Ch][ i] = AbsErrorSig / 
				    EDR->Spectrum.Radiance[Ch][ i];
				break;
			    }
			case CAL_IRR_BSDF:
			case CAL_IRR_INTENS:
			case CAL_IRR_INTENS_BSDF: 
			    {
				EDR->Spectrum.RelErrorRad[ Ch][ i] = 
				    (float)sqrt((double)(1.0/RadResp * 1.0/RadResp * AbsErrorSig * AbsErrorSig *
							 1.0 / BSDF_Val * 1.0 / BSDF_Val + 
							 -Signal[ Ch][ i]/RadResp/RadResp/BSDF_Val *
							 -Signal[ Ch][ i]/RadResp/RadResp/BSDF_Val *
							 AbsErrorRadianceResponse * AbsErrorRadianceResponse +
							 -Signal[ Ch][ i]/RadResp/BSDF_Val/BSDF_Val *
							 -Signal[ Ch][ i]/RadResp/BSDF_Val/BSDF_Val *
							 AbsError_BSDF * AbsError_BSDF));
				break;
			    }
			case CAL_RAD_POL:
			case CAL_RAD_INTENS:
			case CAL_RAD_POL_INTENS: 
			    {
				EDR->Spectrum.RelErrorRad[ Ch][ i] = (float)sqrt((double)(
											  PolCor/RadResp * PolCor/RadResp * AbsErrorSig * AbsErrorSig + 
											  Signal[ Ch][ i]/RadResp * Signal[ Ch][ i]/RadResp *
											  ErrPolCor[ Ch][ i] * ErrPolCor[ Ch][ i] +
											  -Signal[ Ch][ i] / RadResp * PolCor / RadResp *
											  -Signal[ Ch][ i] / RadResp * PolCor / RadResp *
											  AbsErrorRadianceResponse * AbsErrorRadianceResponse));
				break;
			    }
			}
		    }
		}
	    }
	}
    }
    return Status;
}


/*----------------------------------------------------------------------------*\
**                                EDR__CalculateErrorsAndFlags
**  Input parameters:
**    bandFilter	Band Filter for error calculation
**    calibrationFilter	calibration filter
**    FCD		fixed calibration record of Level 1 Product
**    Grid		all Grids of the Level 1 Product
**    EDR		Extract description record structure
**    ErrCorrRadiance	signal of spectrum to calculate the signal error
**    BSDF		BSDF function
**    P7_BAND_1A_MeanCalculated	flag indicating if the mean value of P7 of
**				Band 1a was calculated
**    PolarizationParP7_Mean	Polarization structure for the mean value of P7
**				of Band 1a
**
**  Output parameters:
**    EDR				Extract description record structure
**    EDR__CalculateErrorsAndFlags	SUCCESS	= Error calculation ok
**					else	= error calculation not ok
**
**  Other interfaces:
**
**  Description:
**    Calculate the errors of the Radiance and set the flags of the EDR.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__calculateErrorsAndFlags(EDR_s *EDR, const GRID_s *GRID, 
			     const Bool bandFilter[ SCIENCE_DATA_DEFINED],
			     const Bool calibrationFilter[ CALIBRATION_DEFINED],
			     const FCD_s *FCD,
			     const float ErrCorrRadiance[OPTICAL_CHANNELS][DETECTOR_PIXELS],
			     const float BSDF[ OPTICAL_CHANNELS][DETECTOR_PIXELS],
			     Bool P7_BAND_1A_MeanCalculated,
			     const PolarizationC_s* PolarizationParP7_Mean)
{
    FunctionStatus_e	Status		= SUCCESS;
    int			i, j;
    char			ASCII_Time[ 30];
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** Calculate error of EDR
    \*----------------------------------------------------------------------*/
    if (EDR__CalculateErrors(bandFilter, calibrationFilter, FCD, GRID, EDR,
			     ErrCorrRadiance, BSDF, P7_BAND_1A_MeanCalculated,
			     PolarizationParP7_Mean) != SUCCESS)
    {
	Status = WARN;
	UTC_2_ASCII(EDR->PixelPar.Date, ASCII_Time);
	sprintf( LogBuffer, "Cannot calculate Spectrum Errors in Pixel: %s",
		 ASCII_Time);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__505", Status, LOG_NUM,
		 LogBuffer, "");
    }

    /*----------------------------------------------------------------------*\
    ** set flags for all defined bands
    \*----------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
	if (EDR->Spectrum.ScienceDataAvail[ i])
	{
	    /*---------------------------------------------------------------------*\
	    **  set flags 
	    \*---------------------------------------------------------------------*/
	    for (j = FCD->BCR[ (ScienceData_e)i].StartDetector;
		 j <= FCD->BCR[ (ScienceData_e)i].EndDetector; j++)
	    {
		if (((EDR->Spectrum.Flag[ Band2Channel( (ScienceData_e)i)][ j] ==
		      QF_GOOD) ||
		     (EDR->Spectrum.Flag[ Band2Channel( (ScienceData_e)i)][ j] >
		      QF_OTHER)) &&
		    (EDR->Spectrum.Radiance[ Band2Channel( (ScienceData_e)i)][ j] <
		     0.0))
		    EDR->Spectrum.Flag[ Band2Channel( (ScienceData_e)i)][ j] = QF_OTHER;
	    }
	}
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                EDR__Process
**  Input parameters:
**    Config		Extract Configuration structure
**    FSR1		File structure record of Level 1 Product
**    SPH1		Specific Product Header of Level 1 Product
**    FCD		fixed calibration record of Level 1 Product
**    GRID		all grids of Level 1 Product
**    EDR		Extract description record structure
**    Irradiance	Irradiance of Solar Spectrum
**
**  Output parameters:
**    EDR		Extract description record structure
**    EDR__Process	SUCCESS	= Processing of the EDR is ok
**			else	= any error occured during EDR processing
**
**  Other interfaces:
**    Output file of Extract Level 1 Product specified in Config
**    Input file of Level 1 Product specified in Config
**
**  Description:
**    Function the process one complete spectrum of the current PCD or SCD or
**    MCD (Measurement pixel number ) of the Level 1 Product.
**    First all necessary spectrum data are read from the Level 1 Product.
**    Then the calibrations of the PMDs are done. Then the calibrations of the
**    spectrum data are done. Finally the error and flag calculation are done.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__process(EDR_s* EDR, const GRID_s* GRID, Config_s *Config, 
	     const Level_1_Product_s* level_1_product_ptr,
	     const float Irradiance[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	     Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    FunctionStatus_e Stat = SUCCESS;
    int	i, j;
    float ErrCorrRadiance[OPTICAL_CHANNELS][DETECTOR_PIXELS];
    float BSDF[OPTICAL_CHANNELS][ DETECTOR_PIXELS];
    Bool P7_BAND_1A_MeanCalculated = False;
    PolarizationC_s PolarizationParP7_Mean;

    /*------------------------------------------------------------------------*\
    ** initialize Spectrum data
    \*------------------------------------------------------------------------*/
    DR__Init(&EDR->Spectrum);

    /*------------------------------------------------------------------------*\
    ** get geolocation data, integration time and the spectrum out of the
    ** level 1 product
    \*------------------------------------------------------------------------*/
    if (EDR__getSpectrumDataLevel_1_Product(EDR, Config, level_1_product_ptr,
					    convert_sun_2_host) != SUCCESS) 
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__598", Status, LOG_CANNOT_READ,
		  "spectrum from level 1 product", "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** calibrate PMD, spectrum and calculate the error flags
    \*--------------------------------------------------------------------*/
#ifdef DEBUG
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
		 "calibrate PMD and spectrum\n");
#endif
    Stat = EDR__calibratePMD(EDR, Config->Filter.calibrationFilter, 
			     &(level_1_product_ptr->FCD));
    if ((Stat != SUCCESS) && (Stat != WARN)) 
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__615", Status, LOG_NUM,
		 "Cannot calibrate PMD", "");
	return Status;
    }
    
    if (EDR__calibrate(EDR, GRID, Config, &(level_1_product_ptr->FCD),
		       ErrCorrRadiance, Irradiance, BSDF,
		       &P7_BAND_1A_MeanCalculated,
		       &PolarizationParP7_Mean) != SUCCESS) 
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__626", Status, LOG_NUM,
		  "Cannot calibrate spectrum", "");
	return Status;
    }

#ifdef DEBUG  
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
		 "calculte error flags\n");
#endif
    if (EDR__calculateErrorsAndFlags(EDR, GRID, Config->Filter.bandFilter,
				     Config->Filter.calibrationFilter,
				     &(level_1_product_ptr->FCD),
				     (const float(*)[])ErrCorrRadiance,
				     (const float(*)[])BSDF,
				     P7_BAND_1A_MeanCalculated,
				     &PolarizationParP7_Mean) == FATAL)
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__644", Status, LOG_NUM,
		  "Cannot Calculate Errors and Flags", "");
	return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                EDR__write
**  Input parameters:
**    Config		Extract Configuration structure
**    FCD		fixed calibration record of Level 1 Product
**    GRID		all grids of Level 1 Product
**    EDR		Extract description record structure
**
**  Output parameters:
**    EDR__Write	SUCCESS	= output ok
**			else	= any error occured during output
**
**  Other interfaces:
**    Output file Extract Level 1 Product specified in Config
**
**  Description:
**    Function the write the whole spectrum on the Extract Level 1
**    Product. First the Spectrum Parameter are written followed by the
**    complete Band data (Band parameter and Band Data itself).
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
**  update:		albedoFlag handling, Bernd Aberle, DLR/WT-DA-SE, 27.08.96
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__write(const EDR_s *EDR,  const GRID_s *GRID, 
	   const Config_s *Config, const FCD_s *FCD)
{
    FunctionStatus_e Status = SUCCESS;
    int i;
    int j;
    int Nband;

    /*------------------------------------------------------------------------*\
    ** if QA mode write only selected EDR data into QA (Quality Assurance) file
    \*----------------------------------------------------------------------*/
    if ((Config->QA_ProductStream != NULL) && Config->Filter.qaInfoFlag)
    {
	if (QA__writeEDR(Config, EDR) != SUCCESS)
	{
	    Status = FATAL;
	}
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** don't write the EDR if the pixel SubsetCounter is not active
    \*------------------------------------------------------------------------*/
    if (!Config->Filter.subsetCounterFilter[EDR->PixelPar.AuxiliaryPar.SubsetCounter])
	return Status;

    /*------------------------------------------------------------------------*\
    ** normal writing (no albedo flag)
    \*------------------------------------------------------------------------*/
    /*----------------------------------------------------------------------*\
    ** write measurement pixel number, subset counter, number of detector pixels,
    ** geolocation and the PMD data into file
    \*----------------------------------------------------------------------*/
    if (WriteAllSpectrumParameter(Config, EDR) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__718", Status, LOG_CANNOT_WRITE,
		  "spectrum parameter", "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** write spectrum
    \*----------------------------------------------------------------------*/
    if (WriteSpectrum(Config, GRID, FCD, EDR) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__729", Status, LOG_CANNOT_WRITE,
		  "spectrum", "");
	return Status;
    }
    EDR_WrittenCounter++;

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                GetSpectrumDataLevel_1_Product
**  Input parameters:
**    Config		Extract configuration structure
**    FSR1		File structure record of Level 1 Product
**    FCD		Fixed Calibration record of Level 1 Product
**    SPH1		Specific Product Header of Level 1 Product
**    EDR		Extract description record structure
**
**  Output parameters:
**    EDR				Extract description record structure
**    GetSpectrumDataLevel_1_Product	SUCCESS	= Spectrum read ok
**					FATAL	= any error occured reading
**							one Spectrum
**
**  Other interfaces:
**    Input file of Level 1 Product specified in Config
**
**  Description:
**    Function the read one complete spectrum of the current PCD or SCD or MCD
**    (Measurement pixel number ) of the EDR from the Level 1 Product.
**    First the PCD or SCD or MCD is read and all indices are checked for
**    validity.
**    Then all selected and completed Bands are read. The Calibration 
**    indices of all Bands are checked for validity too.
**    Finaly the PCD or SCD or MCD history is read according the max.
**    Integration time.
**    In Split mode the following additional actions will be executed:
**	- the time range of the EDR is set to the PCD, SCD or MCD history
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__getSpectrumDataLevel_1_Product(EDR_s* EDR, const Config_s *Config, 
				    const Level_1_Product_s* 
				    level_1_product_ptr,
				    Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    int i;
    long FilePosPCD;
    Bool ReticonFilter[ OPTICAL_CHANNELS];
    short Factor;
    char ASCII_Time[ 30];
    char LogBuffer[LOG_FIELD_SIZE];
#ifdef DEBUG
    char debug_message[255];
#endif

    /*------------------------------------------------------------------------*\
    ** read pixel specific spectrum parameter (PCD, SCD or MCD)
    \*------------------------------------------------------------------------*/
    FilePosPCD = Config->FilePosPCD +
	Config->PCD_Size * EDR->Spectrum.Current_PCD;
    if (PixelPar__Get(Config->Level_1_ProductStream, FilePosPCD,
		      Config->PCD_Size, &(level_1_product_ptr->SPH1),
		      EDR->ProductType,
		      EDR->Spectrum.Current_PCD, &EDR->PixelPar,
		      convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__805", Status, LOG_CANNOT_READ,
		  "PCD or SCD or MCD from level 1 product file", "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** Write PCD, SCD or MCD into PCD output stream
    \*--------------------------------------------------------------------*/
    if ((Config->PCD_ProductStream != NULL) && Config->Filter.calFileFlag) {
	if (UTC_2_ASCII( EDR->PixelPar.Date, ASCII_Time) != SUCCESS) {
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__815", WARN, LOG_NUM, 
		      "converting the UTC time of Geolocation", "");
	    strcpy( ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
	}
	PixelPar__show(&EDR->PixelPar, Config->PCD_ProductStream,
		       EDR->ProductType, False);
    }

    /*--------------------------------------------------------------------*\
    ** check the indices of the PixelPar
    \*--------------------------------------------------------------------*/
    if ((EDR->PixelPar.IndexSpectralCalPar < 0) 
	|| (EDR->PixelPar.IndexSpectralCalPar 
	    >= level_1_product_ptr->FCD.Nspec)) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__831", Status, LOG_NUM,
		  "PCD or SCD or MCD, Spectral Calibration Parameter "
		  "Index out of range", "");
	return Status;
    }
    if (EDR->PixelPar.IndexSpectralCalPar == INVALID_INDEX) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__839", Status, LOG_NUM,
		  "PCD or SCD or MCD, Spectral Calibration Parameter "
		  "Index == INVALID_INDEX", "");
	return Status;
    }

    if ((EDR->PixelPar.IndexLeakagePar < 0) ||
	(EDR->PixelPar.IndexLeakagePar >= level_1_product_ptr->FCD.Nleak)) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__849", Status, LOG_NUM,
		  "PCD or SCD or MCD, Leakage Calibration Parameter "
		  "Index out of range", "");
	return Status;
    }
    if (EDR->PixelPar.IndexLeakagePar == INVALID_INDEX) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__857", Status, LOG_NUM,
		  "PCD or SCD or MCD, Leakage Calibration Parameter "
		  "Index == INVALID_INDEX", "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** check from which reticon data has to be copied
    \*--------------------------------------------------------------------*/
    for ( i = 0; i < OPTICAL_CHANNELS; i++) 
    {
	ReticonFilter[i] = False;
    }

    for ( i = Config->StartBand; i <= Config->EndBand; i++)
    {
	if ((Config->Filter.bandFilter[ i]) &&
	    (EDR->PixelPar.ScienceIndex[ i] != INVALID_INDEX)) 
	{
	    ReticonFilter[ Band2Channel( (ScienceData_e)i)] = True;
	}
    }
	
    /*--------------------------------------------------------------------*\
    ** read the necessary science data from Level 1 Product. (e.g if only 
    ** the Band_1A is needed, read also Blind_1A, Straylight_1A/1B, Band_1b)
    \*--------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	EDR->Spectrum.ScienceDataAvail[i] = False;
	if (ReticonFilter[Band2Channel((ScienceData_e)i)] 
		&& (EDR->PixelPar.ScienceIndex[i] != INVALID_INDEX)) 
	{
	    if ( (EDR->PixelPar.ScienceIndex[i] < 0) 
		 || (EDR->PixelPar.ScienceIndex[i] 
		     >= level_1_product_ptr->FSR1.NoOfBDR_Rec[i])) 
	    {
		Status = FATAL;
		Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__895", Status,
			  LOG_INVALID, "Band index detected", "");
		return Status;
	    }
	    
	    /*--------------------------------------------------------*\
	    ** get the Multiplication factor of the raw data
	    ** (co-adding mode)
	    \*--------------------------------------------------------*/
	    Factor = GetAverageFactor((ScienceData_e)i,
				      EDR->PixelPar.AuxiliaryPar.AverageMode,
				      EDR->PixelPar.AuxiliaryPar.IntegrStatus);

#ifdef DEBUG  
	    UTC_2_ASCII( EDR->PixelPar.Date, ASCII_Time);
	    if (EDR->PixelPar.AuxiliaryPar.AverageMode) 
	    {
		sprintf(debug_message, "Pixel time: %s, Average Mode "
		       "detected on Band: %1d, Factor: %hd\n",
		       ASCII_Time, i, Factor);
		Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
			     debug_message);
	    }
#endif
	    /*--------------------------------------------------------*\
	    ** read the Band Data
	    \*--------------------------------------------------------*/
	    if (GetBandDataLevel_1_Product(Config, &(level_1_product_ptr->FSR1),
					   &(level_1_product_ptr->FCD), 
					   EDR->ProductType,
					   (ScienceData_e)i, 
					   &EDR->PixelPar,
					   &EDR->Spectrum,
					   (float)Factor,
					   convert_sun_2_host) != SUCCESS) 
	    {
		Status = FATAL;
		sprintf( LogBuffer, "Band (%2d) Data from Level 1 "
			 "Product file", i);
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__934", Status,
			  LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }
		 
	    EDR->Spectrum.ScienceDataAvail[i] = True;
	}
    }
	
    /*----------------------------------------------------------------*\
    ** read the PCD or SCD or MCD History
    \*----------------------------------------------------------------*/
    if (Get_PCD_History(Config, &(level_1_product_ptr->SPH1), EDR,
			convert_sun_2_host) > SUCCESS) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__950", Status,
		  LOG_CANNOT_READ, "Geolocations of Band", "");
	return Status;
    }

    /*------------------------------------------------------------*\
    ** if split mode set the time range to the current measurement
    ** pixel time and the begin time of the history
    \*------------------------------------------------------------*/
    if (Config->Filter.splitFlag || Config->Filter.noSunFlag 
	|| Config->Filter.albedoFlag) 
    {
	/*--------------------------------------------------------*\
	** set the time range
	\*--------------------------------------------------------*/
	if ( EDR->N_PCD_HistoryRead > 0) 
	{
	    EDR->BeginTime = EDR->PCD_History[EDR->N_PCD_History
					     - EDR->N_PCD_HistoryRead].Date;
	}
	else {
	    EDR->BeginTime	= EDR->PixelPar.Date;
	}
	EDR->EndTime	= EDR->PixelPar.Date;
    }

#ifdef DEBUG  
    sprintf(debug_message, "N_PCD_History    : %d\n", EDR->N_PCD_History);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
    sprintf(debug_message, "N_PCD_HistoryRead: %d\n", EDR->N_PCD_HistoryRead);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
		
    for (i = EDR->N_PCD_History - EDR->N_PCD_HistoryRead;
	 i < EDR->N_PCD_History; i++) 
    {
	UTC_2_ASCII( EDR->PCD_History[ i].Date, ASCII_Time);
	sprintf(debug_message, "PCD or SCD or MCD[ %d] %s Days: %u, Msecs: %u\n",
	       i, ASCII_Time, EDR->PCD_History[i].Date.Days,
	       EDR->PCD_History[i].Date.Msecs);
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL,
		     debug_message);
    }

    UTC_2_ASCII(EDR->BeginTime, ASCII_Time);
    sprintf(debug_message, "set EDR BeginTime to %s Days: %u, "
	   "Msecs: %u\n", ASCII_Time, EDR->BeginTime.Days, 
	   EDR->BeginTime.Msecs);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
    
    UTC_2_ASCII( EDR->EndTime, ASCII_Time);
    sprintf(debug_message, "set EDR EndTime to %s Days: %u, "
	   "Msecs: %u\n", ASCII_Time, EDR->EndTime.Days, EDR->EndTime.Msecs);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
#endif

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                WriteAllSpectrumParameter
**  Input parameters:
**    Config		Extract configuration structure
**    EDR		Extract description record structure
**
**  Output parameters:
**    WriteAllSpectrumParameter		SUCCESS	= ok
**					FATAL	= any error occured during write
**
**  Other interfaces:
**    Output file of Extract Level 1 Product specified in Config
**
**  Description:
**    Function to write all Parameter of the Spectrum into the Extract Level
**    1 Product. In Split mode the spectral parameter of the PCD or SCD or MCD
**    history are additionally preceded.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
WriteAllSpectrumParameter(const Config_s *Config, const EDR_s *EDR)
{
    FunctionStatus_e	Status		= SUCCESS;
    int			i;
    int			Nband;

    /*--------------------------------------------------------------------*\
    ** if split flag write
    \*--------------------------------------------------------------------*/
    Nband = 0;
    if (Config->Filter.splitFlag || Config->Filter.noSunFlag)
    {
	for (i = EDR->N_PCD_History - EDR->N_PCD_HistoryRead;
	     i < EDR->N_PCD_History; i++)
	{
	    if (WriteSpectrumParameterSplit(Config->Extract_1_ProductStream,
					    EDR->ProductType, &EDR->PCD_History[ i], Nband) != SUCCESS)
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__1054", Status, LOG_CANNOT_WRITE,
			  "spectrum parameter into exract level 1 file", "");
		return Status;
	    }
	    EDR_WrittenCounter++;
	}
    }

    /*----------------------------------------------------------------------*\
    ** write measurement pixel number, subset counter, number of detector pixel
    ** samples
    \*----------------------------------------------------------------------*/
    Nband = 0;
    for ( i = Config->StartBand; i <= Config->EndBand; i++)
    {
	if ((Config->Filter.bandFilter[ i]) &&
	    (EDR->PixelPar.ScienceIndex[ i] != INVALID_INDEX))
	    Nband += 1;
    }
    if (WriteSpectrumParameter(Config->Extract_1_ProductStream,
			       EDR->ProductType, &EDR->PixelPar, Nband,
			       EDR->Spectrum.PMD_Flag) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__1078", Status, LOG_CANNOT_WRITE,
		  "spectrum parameter into exract level 1 file", "");
	return Status;
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                WriteSpectrum
**  Input parameters:
**    Config		Extract configuration structure
**    GRID		all wavelength grids of the level 1 product
**    FCD		Fixed calibration record of level 1 product
**    EDR		Extract description record structure
**
**  Output parameters:
**    WriteSpectrum	SUCCESS	= output ok
**			else	= any error occured during output
**
**  Other interfaces:
**    Output file of Extract Level 1 Product specified in Config
**
**  Description:
**    Function to write the whole spectrum into the Extract Level 1 Product.
**    The Spectrum includes all selected and completted Bands. The Band 
**    includes the Band Parameter followed by the Band Data itself.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
WriteSpectrum(const Config_s *Config, const GRID_s *GRID, const FCD_s *FCD,
	      const EDR_s *EDR)
{
    FunctionStatus_e Status = SUCCESS;
    int	i, j;

    /*------------------------------------------------------------------------*\
    ** spectrum parameter sampling and calculations
    \*------------------------------------------------------------------------*/
    for (i = Config->StartBand; i <= Config->EndBand; i++)
    {
	OpticalChannel_e channel = Band2Channel( (ScienceData_e)i);

	if ( (Config->Filter.bandFilter[i]) 
	     && (EDR->PixelPar.ScienceIndex[i] != INVALID_INDEX))
	{
	    /*----------------------------------------------------------------*\
	    ** print Band parameter
	    \*----------------------------------------------------------------*/
	    if (WriteBandParameter(Config->Extract_1_ProductStream, i,
				   &EDR->Spectrum.BandRecord[i],
				   FCD->BCR[i].EndDetector - FCD->BCR[i].StartDetector + 1,
				   GRID->Grid[EDR->PixelPar.IndexSpectralCalPar][channel][FCD->BCR[i].StartDetector],
				   GRID->Grid[EDR->PixelPar.IndexSpectralCalPar][channel][FCD->BCR[i].EndDetector],
				   EDR->Spectrum.ReflectivityJump[i],
				   FCD->SpectralPar[EDR->PixelPar.IndexSpectralCalPar].Error[channel])
		!= SUCCESS)
	    {
		Status = FATAL;
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__1144", Status,
			 LOG_CANNOT_WRITE, "Band Parameter", "");
		return Status;
	    }

	    /*----------------------------------------------------------------*\
	    ** print spectrum: wavelength radiance abserror relerror flag
	    \*----------------------------------------------------------------*/
	    for (j = FCD->BCR[i].StartDetector; j <= FCD->BCR[i].EndDetector;
		 j++)
	    {
		fprintf(Config->Extract_1_ProductStream,
			"%08.4f %011.5e %011.5e %011.5e %01d\n",
			GRID->Grid[EDR->PixelPar.IndexSpectralCalPar][channel][j],
			EDR->Spectrum.Radiance[channel][j],
			EDR->Spectrum.AbsErrorRad[channel][j],
			EDR->Spectrum.RelErrorRad[channel][j],
			EDR->Spectrum.Flag[channel][j]);
	    }
	}

	if (ferror(Config->Extract_1_ProductStream) != 0)
	{
	    Status = FATAL;
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__1168", Status, LOG_CANNOT_WRITE,
		      "spectrum into exract level 1 file", "");
	    return Status;
	}
    }

#ifdef DEBUG
    fflush(Config->Extract_1_ProductStream);
#endif
    return Status;
}


/*----------------------------------------------------------------------------*\
**                                CalibratePMD
**  Input parameters:
**    calibrationFilter		Calibration filter to select the switch on or
**					off the different Calibration steps
**    FCD			Fixed calibrartion record of the Level 1 Product
**    EDR			Extract description record structure
**
**  Output parameters:
**    EDR			Extract description record structure
**    CalibratePMD		SUCCESS	= Calibration of PMD ok
**				else	= any error occured during calibration
**
**  Other interfaces:
**
**  Description:
**    Function to calibrate the PMD values of the EDR according the given
**    product type. The calibration steps are:
**	- apply Leakage calibration parameter for PMD
**	- apply Polarization parameter for PMD
**	- apply Intensity calibration parameter for PMD
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__calibratePMD(EDR_s* EDR,
		  const Bool calibrationFilter[CALIBRATION_DEFINED],
		  const FCD_s *FCD)
{
    FunctionStatus_e Status = SUCCESS;
    int i;
    float PMD_WaveLength[ PMD_DEFINED];
    char ASCII_Time[30];

    /*------------------------------------------------------------------------*\
    ** convert the pixel time
    \*------------------------------------------------------------------------*/
    if (UTC_2_ASCII( EDR->PixelPar.Date, ASCII_Time) != SUCCESS)
    {
	Status = WARN;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__1226", Status,
		  LOG_NUM, "converting the UTC time of Geolocation", "");
	strcpy( ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
    }

    /*------------------------------------------------------------------------*\
    ** set the PMD flags to 0
    \*------------------------------------------------------------------------*/
    for ( i = 0; i < PMD_DEFINED; i++)
    {
		    EDR->Spectrum.PMD_Flag[ i] = 0;
	}

    /*------------------------------------------------------------------------*\
    ** apply the PMD's if the Calibration parameter are available
    \*------------------------------------------------------------------------*/
    for ( i = 0; i < PMD_DEFINED; i++)
    {
	/*---------------------------------------------------------------------*\
	**  apply Leakage calibration parameter for PMD
	\*---------------------------------------------------------------------*/
	if (EDR->PixelPar.IndexLeakagePar != INVALID_INDEX)
	{
	    if (calibrationFilter[ CAL_LEAKAGE])
		LeakageC__ApplyPMD( &FCD->LeakagePar[ EDR->PixelPar.IndexLeakagePar],
				   (PMD_Number_e)i, EDR->PixelPar.AuxiliaryPar.PMD[ i],
				   ASCII_Time);
	}
	else
	{
	    Status = FATAL;
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__1249", Status, LOG_NUM,
		      "apply Leakage PMD because of invalid index of Leakage parameter", "");
	    return Status;
	}

	if (EDR->ProductType == PR_EARTHSHINE)
	{
	    /*---------------------------------------------------------------------*\
	    **  apply Polarization parameter for PMD only for EARTHSHINE
	    \*---------------------------------------------------------------------*/
	    if (calibrationFilter[ CAL_POLARIZATION])
	    {
		if (PolarizationC__ApplyPMD(&EDR->PixelPar.Specific.Earth.PolarizationPar,
					   (PMD_Number_e)i, EDR->PixelPar.AuxiliaryPar.PMD[ i],
					   ASCII_Time) != SUCCESS)
		{
		    EDR->Spectrum.PMD_Flag[ i] = 1;
		}
	    }
	}

	if (calibrationFilter[ CAL_INTENSITY])
	{
	    switch (EDR->ProductType)
	    {
	    case PR_EARTHSHINE:
		{
		    /*-----------------------------------------------------------------*\
		    ** apply Intensity calibration parameter for PMD with the
		    ** polarization grid
		    \*-----------------------------------------------------------------*/
			if (EDR->PixelPar.Specific.Earth.PolarizationPar.Wavelength[ PC_PMD_1] <= 0.0 ||
				EDR->PixelPar.Specific.Earth.PolarizationPar.Wavelength[ PC_PMD_2] <= 0.0 ||
				EDR->PixelPar.Specific.Earth.PolarizationPar.Wavelength[ PC_PMD_3] <= 0.0 )
			{
				PMD_WaveLength[ PMD_1] = FCD->IntensityPar.MeanPMD_Wavelength[ PC_PMD_1];
				PMD_WaveLength[ PMD_2] = FCD->IntensityPar.MeanPMD_Wavelength[ PC_PMD_2];
				PMD_WaveLength[ PMD_3] = FCD->IntensityPar.MeanPMD_Wavelength[ PC_PMD_3];
				EDR->Spectrum.PMD_Flag[ i] = 1;
			}
			else
			{
				PMD_WaveLength[ PMD_1] = EDR->PixelPar.Specific.Earth.PolarizationPar.Wavelength[ PC_PMD_1];
				PMD_WaveLength[ PMD_2] = EDR->PixelPar.Specific.Earth.PolarizationPar.Wavelength[ PC_PMD_2];
				PMD_WaveLength[ PMD_3] = EDR->PixelPar.Specific.Earth.PolarizationPar.Wavelength[ PC_PMD_3];
			}
		    if (IntensityC__ApplyPMD( &FCD->IntensityPar, PMD_WaveLength,
					     (PMD_Number_e)i, EDR->PixelPar.AuxiliaryPar.PMD[ i],
					     ASCII_Time) != SUCCESS)
			EDR->Spectrum.PMD_Flag[ i] = 1;
		    break;
		}
	    case PR_SUNSHINE:
	    case PR_MOONSHINE:
		{
		    /*-----------------------------------------------------------------*\
		    ** apply Intensity calibration parameter for PMD with the intensity
		    ** grid
		    \*-----------------------------------------------------------------*/
		    if (IntensityC__ApplyPMD( &FCD->IntensityPar,
					     FCD->IntensityPar.MeanPMD_Wavelength, (PMD_Number_e)i,
					     EDR->PixelPar.AuxiliaryPar.PMD[ i], ASCII_Time) != SUCCESS)
			EDR->Spectrum.PMD_Flag[ i] = 1;
		    break;
		}
	    }
	}
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                CalibrateSpectrum
**  Input parameters:
**    Config (Band Filter)	for Normalize, Apply Polarization,
**				Internsity and Units Conversion
**    Config (calibrationFilter) Calibration filter to seletct the switch on or
**				off the different Calibration steps
**    Config (Flags)
**    FCD			Fixed calibration record of the Level 1 Product
**    Grid			all grids of the Level 1 Product
**    EDR			Extract description record structure
**    Irradiance		irradiance
**
**  Output parameters:
**    EDR			Extract description record structure
**    Signal			spectrum before normalization for further
**				error calculation
**    BSDF			BSDF of measurement pixel
**    P7_BAND_1A_MeanCalculated	flag indicating if the mean value of P7 of
**				Band 1a was calculated
**    PolarizationParP7_Mean	Polarization structure for the mean value of P7
**				of Band 1a
**    CalibrateSpectrum		SUCCESS	= Calibration of Spectrum ok
**				else	= any error occured during calibration
**
**  Other interfaces:
**
**  Description:
**    Function to calibrate the Spectrum values of the EDR according the given
**    product type. The calibration steps are:
**	- apply Leakage calibration parameter
**	- apply FPA correction
**	- apply Fixed calibration parameter
**	- apply Straylight calibration parameter
**	- normalize Spectrum
**	- apply Polarization parameter
**	- apply Intensity calibration parameter
**      - correct for radiance jumps
**      - correct degradation
**	- apply BSDF calibration parameter on sun and moon
**	- take the reflectivity checking
**	- convert units from watts to photons
**	Normalize, Apply Polarization, Apply Internsity and the Units
**	Conversion are only done for the given Band range (StartBand..EndBand).
**	The Signal and the BSDF is returned for later error calculation.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
EDR__calibrate(EDR_s* self, const GRID_s* grid_ptr,
	       Config_s *config_ptr, const FCD_s *fcd_ptr, 
	       float Signal[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	       const float Irradiance[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	       float BSDF[OPTICAL_CHANNELS][DETECTOR_PIXELS],
	       Bool *P7_BAND_1A_MeanCalculated,
	       PolarizationC_s *PolarizationParP7_Mean)
{
    FunctionStatus_e Status = SUCCESS;
    float SunCalibrated[OPTICAL_CHANNELS][ DETECTOR_PIXELS];
    PixelIndex_s PixelIndex[SCIENCE_DATA_DEFINED];
    ScienceData_e Band;
    int i, j, k;
    char ASCII_Time[ASCII_UTC_TIME_LENGTH];
    int ResultRefl;
    float Reflectivity3[DETECTOR_PIXELS];
    float Reflectivity4[DETECTOR_PIXELS];
    float S_PMD_basic[PMD_DEFINED]; /* for radianceJumpFlag */
    short do_correction;	/* for radianceJumpFlag */

    /*------------------------------------------------------------------------*\
    **  convert the pixel time
    \*------------------------------------------------------------------------*/
    if (UTC_2_ASCII( self->PixelPar.Date, ASCII_Time) != SUCCESS) {
	Status = WARN;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.3__1400", Status,
		 LOG_NUM, "converting the UTC time of Geolocation", "");
	strcpy( ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
    }

    /*------------------------------------------------------------------------*\
    ** set the PixelIndex
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	PixelIndex[i].First = fcd_ptr->BCR[i].StartDetector;
	PixelIndex[i].Last = fcd_ptr->BCR[i].EndDetector;
    }

    /*------------------------------------------------------------------------*\
    **  Check for Saturation
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)	
    {
	Band = (ScienceData_e)i;
	if (self->Spectrum.ScienceDataAvail[i])
	{
	    /*----------------------------------------------------------------*\
	    **  flag saturated pixels (only for non-co-added pixels, because 
	    **  preflight saturation limits are not valid for co-adding-mode
	    \*----------------------------------------------------------------*/
	    /*----------------------------------------------------------------*\
	    ** get the Multiplication factor of the raw data (co-adding mode)
	    \*----------------------------------------------------------------*/
	    if (GetAverageFactor((ScienceData_e)i,
				 self->PixelPar.AuxiliaryPar.AverageMode,
				 self->PixelPar.AuxiliaryPar.IntegrStatus) == 1){
		QualityCheck__CheckSaturation(Band,
					      self->Spectrum.Radiance[Band2Channel(Band)],
					      self->Spectrum.Flag[Band2Channel(Band)],
					      fcd_ptr->BCR[Band].StartDetector,
					      fcd_ptr->BCR[Band].EndDetector,
					      ASCII_Time);
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  apply Leakage
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	Band = (ScienceData_e)i;
	if (self->Spectrum.ScienceDataAvail[ i])	
	{
	    /*----------------------------------------------------------------*\
	    **  apply Leakage calibration parameter 
	    \*----------------------------------------------------------------*/
	    if (config_ptr->Filter.calibrationFilter[ CAL_LEAKAGE]) 
	    {
		LeakageC__Apply(&fcd_ptr->LeakagePar[self->PixelPar.IndexLeakagePar],
			       Band,
			       self->Spectrum.Radiance[Band2Channel(Band)],
			       fcd_ptr->BCR[ Band].StartDetector,
			       fcd_ptr->BCR[ Band].EndDetector,
			       ASCII_Time);
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  apply Peltier only for Band_1a and Blind_1a and PR_EARTHSHINE
    \*------------------------------------------------------------------------*/
    if (config_ptr->Filter.calibrationFilter[CAL_FPA] 
	&& (self->ProductType == PR_EARTHSHINE) 
	&& self->Spectrum.ScienceDataAvail[ BAND_1A]) 
    {
	/*--------------------------------------------------------------------*\
	**  apply peltier noise correction for BAND_1A
	\*--------------------------------------------------------------------*/
	FPA__Apply(&fcd_ptr->FPA, self->PixelPar.MeasurementPixel, 
		   self->Spectrum.BandRecord[ BAND_1A].IntgTime,
		   self->Spectrum.Radiance[ Band2Channel( BAND_1A)],
		   fcd_ptr->BCR[ BAND_1A].StartDetector,
		   fcd_ptr->BCR[ BAND_1A].EndDetector,
		   ASCII_Time);
    }
    if (config_ptr->Filter.calibrationFilter[CAL_FPA]
	&& (self->ProductType == PR_EARTHSHINE) 
	&& self->Spectrum.ScienceDataAvail[ BLIND_1A]) 
    {
	/*--------------------------------------------------------------------*\
	**  apply peltier noise correction for BLIND_1A
	\*--------------------------------------------------------------------*/
	FPA__Apply(&fcd_ptr->FPA, self->PixelPar.MeasurementPixel, 
		   self->Spectrum.BandRecord[ BLIND_1A].IntgTime,
		   self->Spectrum.Radiance[ Band2Channel( BLIND_1A)],
		   fcd_ptr->BCR[ BLIND_1A].StartDetector,
		   fcd_ptr->BCR[ BLIND_1A].EndDetector,
		   ASCII_Time);
    }
    if (config_ptr->Filter.calibrationFilter[ CAL_FPA] 
	&& (self->ProductType == PR_EARTHSHINE) 
	&& self->Spectrum.ScienceDataAvail[ STRAYLIGHT_1A]) 
    {
	/*--------------------------------------------------------------------*\
	**  apply peltier noise correction for STRAYLIGHT_1A
	\*--------------------------------------------------------------------*/
	FPA__Apply(&fcd_ptr->FPA, self->PixelPar.MeasurementPixel, 
		   self->Spectrum.BandRecord[ STRAYLIGHT_1A].IntgTime,
		   self->Spectrum.Radiance[ Band2Channel( STRAYLIGHT_1A)],
		   fcd_ptr->BCR[ STRAYLIGHT_1A].StartDetector,
		   fcd_ptr->BCR[ STRAYLIGHT_1A].EndDetector,
		   ASCII_Time);
    }

    /*------------------------------------------------------------------------*\
    **  apply fixed
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	Band = (ScienceData_e)i;
	if (self->Spectrum.ScienceDataAvail[i])	
	{
	    /*----------------------------------------------------------------*\
	    **  apply LED fixed pattern calibration parameter
	    \*----------------------------------------------------------------*/
	    if (config_ptr->Filter.calibrationFilter[CAL_FIXED]) 
	    {
		FixedC__Apply( &fcd_ptr->FixedPar, Band,
			      self->Spectrum.Radiance[ Band2Channel( Band)],
			      fcd_ptr->BCR[ Band].StartDetector,
			      fcd_ptr->BCR[ Band].EndDetector, ASCII_Time);
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  save signal for later error correction
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
	Band = (ScienceData_e)i;
	if (self->Spectrum.ScienceDataAvail[i])
	{
	    for (j = fcd_ptr->BCR[Band].StartDetector;
		 j <= fcd_ptr->BCR[Band].EndDetector; j++)
		Signal[Band2Channel(Band)][j] =
		    self->Spectrum.Radiance[Band2Channel(Band)][j];
	}
	else 
	{
	    for (j = fcd_ptr->BCR[Band].StartDetector;
		 j <= fcd_ptr->BCR[Band].EndDetector; j++) 
	    {
		Signal[Band2Channel(Band)][j] = 0.0;
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  apply Straylight calibration parameter
    \*------------------------------------------------------------------------*/
    if (config_ptr->Filter.calibrationFilter[CAL_STRAYLIGHT]) 
    {
	if (self->Spectrum.ScienceDataAvail[BAND_1A] 
	    || self->Spectrum.ScienceDataAvail[BAND_1B]) 
	{
	    StrayLight__Correction(&fcd_ptr->StraylightCharacteristic,
				   Band2Channel(BAND_1A),
				   self->Spectrum.Radiance[Band2Channel(BAND_1A)],
				   self->Spectrum.BandRecord[BAND_1A].IntgTime,
				   self->Spectrum.BandRecord[BAND_1B].IntgTime,
				   PixelIndex, ASCII_Time);
	}
	if (self->Spectrum.ScienceDataAvail[BAND_2A] 
	    || self->Spectrum.ScienceDataAvail[BAND_2B]) 
	{
	    StrayLight__Correction(&fcd_ptr->StraylightCharacteristic,
				   Band2Channel(BAND_2A),
				   self->Spectrum.Radiance[Band2Channel( BAND_2A)],
				   self->Spectrum.BandRecord[ BAND_2A].IntgTime,
				   self->Spectrum.BandRecord[ BAND_2B].IntgTime,
				   PixelIndex, ASCII_Time);
	}
	if (self->Spectrum.ScienceDataAvail[BAND_3]) 
	{
	    StrayLight__Correction(&fcd_ptr->StraylightCharacteristic,
				   Band2Channel(BAND_3),
				   self->Spectrum.Radiance[Band2Channel( BAND_3)],
				   self->Spectrum.BandRecord[BAND_3].IntgTime,
				   self->Spectrum.BandRecord[BAND_3].IntgTime,
				   PixelIndex, ASCII_Time);
	}
	if (self->Spectrum.ScienceDataAvail[BAND_4]) 
	{
	    StrayLight__Correction(&fcd_ptr->StraylightCharacteristic,
				   Band2Channel( BAND_4),
				   self->Spectrum.Radiance[Band2Channel(BAND_4)],
				   self->Spectrum.BandRecord[BAND_4].IntgTime,
				   self->Spectrum.BandRecord[BAND_4].IntgTime,
				   PixelIndex, ASCII_Time);
	}
    }

    /*------------------------------------------------------------------------*\
    **  normalize spectrum
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)	
    {
	Band = (ScienceData_e)i;
	if (config_ptr->Filter.bandFilter[i] 
	    && (self->Spectrum.ScienceDataAvail[i])) 
	{
	    /*----------------------------------------------------------------*\
	    **  Normalize the Band data 
	    \*----------------------------------------------------------------*/
	    if (config_ptr->Filter.calibrationFilter[CAL_NORMALIZE]) 
	    {
		Normalize(self->Spectrum.BandRecord[Band].IntgTime,
			  self->Spectrum.Radiance[Band2Channel(Band)],
			  fcd_ptr->BCR[ Band].StartDetector,
			  fcd_ptr->BCR[ Band].EndDetector);
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  write QA (Quality Assurance) output
    \*------------------------------------------------------------------------*/
    if (config_ptr->Filter.qaInfoFlag) 
    {
	if (QA__writeLeakage(config_ptr, self, fcd_ptr) != SUCCESS)
	{
	    Status = FATAL;
	    return Status;
	}
    }

    /*------------------------------------------------------------------------*\
    **  reset the flag P7_BAND_1A_MeanCalculated
    \*------------------------------------------------------------------------*/
    *P7_BAND_1A_MeanCalculated = False;

    if (self->ProductType == PR_EARTHSHINE) 
    {
	/*--------------------------------------------------------------------*\
	**  apply Polarization
	\*--------------------------------------------------------------------*/
	for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
	{
	    Band = (ScienceData_e)i;
	    if (config_ptr->Filter.bandFilter[i] 
		&& (self->Spectrum.ScienceDataAvail[i])) 
	    {
		/*------------------------------------------------------------*\
		**  apply Polarization calibration parameter
		\*------------------------------------------------------------*/
		if (config_ptr->Filter.calibrationFilter[CAL_POLARIZATION]) 
		{
		    /*--------------------------------------------------------*\
		    **  PolarizationC__apply with the 7th point mean value 
		    **  according all subpixels if BAND_1A and integration 
		    **  time > than GROUND_PIXEL_TIME
		    \*--------------------------------------------------------*/
		    if (((Band == BAND_1A) &&
			 (self->Spectrum.BandRecord[ BAND_1A].IntgTime >
			  GROUND_PIXEL_TIME)) |
			((Band == BLIND_1A) &&
			 (self->Spectrum.BandRecord[ BLIND_1A].IntgTime >
			  GROUND_PIXEL_TIME)) |
			((Band == STRAYLIGHT_1A) &&
			 (self->Spectrum.BandRecord[ STRAYLIGHT_1A].IntgTime >
			  GROUND_PIXEL_TIME))) {
			*P7_BAND_1A_MeanCalculated = True;
			PolarizationC__CalculatePolParBand1a(PolarizationParP7_Mean, self, 
							    self->Spectrum.BandRecord[BAND_1A].IntgTime);

			PolarizationC__Apply(PolarizationParP7_Mean, Band, 
					    fcd_ptr->KeyDataCalibrationL1_Par[self->Spectrum.BandRecord[Band].KeyDataCalibrationIndx].Eta_Omega,
					    grid_ptr->Grid[fcd_ptr->IndexSpectralCalPar][Band2Channel(Band)],
					    self->Spectrum.Radiance[Band2Channel( Band)],
					    grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(Band)],
					    fcd_ptr->BCR[ Band].StartDetector,
					    fcd_ptr->BCR[ Band].EndDetector, ASCII_Time);          
		    }
		    else 
		    {
			PolarizationC__Apply(&self->PixelPar.Specific.Earth.PolarizationPar,
					    Band,
					    fcd_ptr->KeyDataCalibrationL1_Par[
									  self->Spectrum.BandRecord[ 
												   Band].KeyDataCalibrationIndx].Eta_Omega,
					    grid_ptr->Grid[fcd_ptr->IndexSpectralCalPar][Band2Channel(Band)],
					    self->Spectrum.Radiance[Band2Channel(Band)],
					    grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(Band)],
					    fcd_ptr->BCR[Band].StartDetector,
					    fcd_ptr->BCR[Band].EndDetector, ASCII_Time);
		    }
		}
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  apply Intensity
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	Band = (ScienceData_e)i;
	if (config_ptr->Filter.bandFilter[i] 
	    && (self->Spectrum.ScienceDataAvail[i])) 
	{
	    /*----------------------------------------------------------------*\
	    **  apply Intensity calibration parameter 
	    \*----------------------------------------------------------------*/
	    if (config_ptr->Filter.calibrationFilter[ CAL_INTENSITY]) 
	    {
		IntensityC__Apply(&fcd_ptr->IntensityPar,
				     grid_ptr->Grid[fcd_ptr->IndexSpectralCalPar][Band2Channel(Band)],
				     fcd_ptr->KeyDataCalibrationL1_Par[self->Spectrum.BandRecord[Band].KeyDataCalibrationIndx].RadianceResponse_Omega,
				     Band,
				     self->Spectrum.Radiance[Band2Channel(Band)],
				     grid_ptr->Grid[ self->PixelPar.IndexSpectralCalPar][Band2Channel(Band)],
				     fcd_ptr->BCR[Band].StartDetector,
				     fcd_ptr->BCR[Band].EndDetector, ASCII_Time);

		if (config_ptr->Filter.albedoFlag) 
		{
                    /* divide calibrated radiance by calibrated sun spectrum */
		    IntensityC__CalculateAlbedo(
					       Irradiance[ Band2Channel(Band)],
					       grid_ptr->Grid[fcd_ptr->IndexSpectralCalPar][Band2Channel(Band)],
					       Band,
					       self->Spectrum.Radiance[ Band2Channel( Band)],
					       grid_ptr->Grid[ self->PixelPar.IndexSpectralCalPar][Band2Channel( Band)],
					       fcd_ptr->BCR[ Band].StartDetector,
					       fcd_ptr->BCR[ Band].EndDetector, ASCII_Time);
		}
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  correct for radiance jumps
    \*------------------------------------------------------------------------*/
    if (self->ProductType == PR_EARTHSHINE) 
    {
	/*--------------------------------------------------------------------*\
	**  check for adjacent previous pixel
	**  check also if integration time is less or equal 1.5 (jumps prefer 
	**  that)
	\*--------------------------------------------------------------------*/
	UTC_2_ASCII(self->PixelPar.Date, ASCII_Time);

	if (CheckGap((const char*)previous_PMD_16_time, 
		     (const char*)ASCII_Time) == SUCCESS
	    && self->Spectrum.BandRecord[BAND_4].IntgTime <= GROUND_PIXEL_TIME)
	{
	    do_correction = 1;
	}
	else
	{
	    do_correction = 0;
	}
	if (config_ptr->Filter.radianceJumpFlag) 
	{
	    /*----------------------------------------------------------------*\
	    **  prepare reflectivity jump correction 
	    \*----------------------------------------------------------------*/
	    IntensityC__PrepareJumpCorrection(self->PixelPar.AuxiliaryPar.PMD, 
				     self->Spectrum.BandRecord[BAND_4].IntgTime, 
				     S_PMD_basic, ASCII_Time);
	    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)	
	    {
		Band = (ScienceData_e)i;

		if (config_ptr->Filter.bandFilter[i] 
		    && (self->Spectrum.ScienceDataAvail[i])
		    &&  (self->Spectrum.ScienceDataAvail[BAND_4])
		    && Band != BAND_1A && Band != BLIND_1A 
		    && Band != STRAYLIGHT_1A) 
		{
		    /*--------------------------------------------------------*\
		    **  correct reflectivity jumps 
		    \*--------------------------------------------------------*/
		    IntensityC__CorrectJumps(grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(Band)], 
					    self->Spectrum.Radiance[Band2Channel(Band)], 
					    self->Spectrum.BandRecord[BAND_4].IntgTime, 
					    do_correction,
					    self->PixelPar.AuxiliaryPar.PMD, S_PMD_basic, 
					    previous_PMD_16[Band2Channel( Band)], Band, 
					    fcd_ptr->BCR[Band].StartDetector, 
					    fcd_ptr->BCR[Band].EndDetector, ASCII_Time);
		}
	    }
	}
	strncpy(previous_PMD_16_time, ASCII_Time, ASCII_UTC_TIME_LENGTH);
    }

    /*------------------------------------------------------------------------*\
    **  correct degradation (aging)
    \*------------------------------------------------------------------------*/
    if (config_ptr->Filter.correctDegradationFlag == True)
    {
	for (i = 0; i < SCIENCE_DATA_DEFINED; i++)	
    	{
		Band = (ScienceData_e)i;
		if (config_ptr->Filter.bandFilter[i] 
			&& (self->Spectrum.ScienceDataAvail[i])) 
		{
			/*----------------------------------------------------------------*\
		    	**  if selected (Filter.correctDegradationFlag == True)
		    	\*----------------------------------------------------------------*/
			if (DegradationParameter__Apply(grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(Band)], 
				self->Spectrum.Radiance[Band2Channel(Band)],
				&(self->PixelPar.Date),
				Band,
				fcd_ptr->BCR[ Band].StartDetector,
				fcd_ptr->BCR[ Band].EndDetector) != SUCCESS)
			{
			    Status = FATAL;
			    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "1.5__1806", Status,
				     LOG_NUM, "Error while correcting degradation", "");
			    return Status;
			}
	    	}
	}
    }

    /*------------------------------------------------------------------------*\
    **  apply BSDF
    \*------------------------------------------------------------------------*/
    if (self->ProductType == PR_SUNSHINE) 
    {
	for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
	{
	    Band = (ScienceData_e)i;

	    if (config_ptr->Filter.bandFilter[i] 
		&& (self->Spectrum.ScienceDataAvail[ i])) 
	    {
		/*------------------------------------------------------------*\
		**  apply BSDF calibration parameter 
		\*------------------------------------------------------------*/
		if (config_ptr->Filter.calibrationFilter[ CAL_BSDF]) 
		{
		    BSDF__Apply(&fcd_ptr->BSDF, 
				self->PixelPar.Specific.Sun.BSDF_Angle.Zenith,
				self->PixelPar.Specific.Sun.BSDF_Angle.Azimuth,
				Band,
				self->Spectrum.Radiance[ Band2Channel( Band)],
				grid_ptr->Grid[ self->PixelPar.IndexSpectralCalPar][Band2Channel( Band)],
				BSDF[ Band2Channel( Band)],
				fcd_ptr->BCR[ Band].StartDetector,
				fcd_ptr->BCR[ Band].EndDetector, ASCII_Time);
		}
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  correct sun for BSDF azimuth dependancy
    \*------------------------------------------------------------------------*/
    if (self->ProductType == PR_SUNSHINE && config_ptr->Filter.correct_BSDF_Flag == True) 
    {
	for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
	{
	    Band = (ScienceData_e)i;

	    if (config_ptr->Filter.bandFilter[i] 
		&& (self->Spectrum.ScienceDataAvail[ i])) 
	    {
		/*------------------------------------------------------------*\
		** correct sun for BSDF azimuth dependancy
		** use SunAngle.Azimuth - ZERO_AZIMUTH because it must be signed
		** BSDF_Angle.Azimuth comes as unsigned (abs) from the l1 product
		\*------------------------------------------------------------*/
		if (BSDF__CorrectAzimuth(
                    self->PixelPar.Specific.Sun.SunAngle.Azimuth - ZERO_AZIMUTH,
                    Band,
                    self->Spectrum.Radiance[Band2Channel(Band)],
                    grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(Band)], 
                    fcd_ptr->BCR[ Band].StartDetector,
                    fcd_ptr->BCR[ Band].EndDetector,
                    &(self->PixelPar.Date) ) != SUCCESS)
		{
                  Status = FATAL;
                    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "1.5__1806", Status,
                    LOG_NUM, "Error while correcting BSDF azimuth dependancy", "");
                  return Status;
		}
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  set Reflectivity jump flag if Band 3 and 4 completed
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	self->Spectrum.ReflectivityJump[ i] = REFLECTIVITY_JUMP_NO_CHECK;
    }
    if ((self->ProductType == PR_EARTHSHINE) 
	&& config_ptr->Filter.calibrationFilter[ CAL_INTENSITY]) {
	if (self->Spectrum.ScienceDataAvail[ BAND_3] &&
	    self->Spectrum.ScienceDataAvail[ BAND_4]) {
	    Reflectivity__Compute(
				  self->PixelPar.Specific.Earth.Geolocation.NorthSunAngle[ 1].Zenith,
				  Irradiance[ Band2Channel( BAND_3)],
				  self->Spectrum.Radiance[ Band2Channel( BAND_3)],
				  Reflectivity3,
				  fcd_ptr->BCR[ BAND_3].StartDetector, fcd_ptr->BCR[ BAND_3].EndDetector);

	    Reflectivity__Compute( 
				  self->PixelPar.Specific.Earth.Geolocation.NorthSunAngle[ 1].Zenith,
				  Irradiance[ Band2Channel( BAND_4)],
				  self->Spectrum.Radiance[ Band2Channel( BAND_4)],
				  Reflectivity4,
				  fcd_ptr->BCR[ BAND_4].StartDetector, fcd_ptr->BCR[ BAND_4].EndDetector);

	    if (Reflectivity__Check(
				    grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(BAND_3)],
				    Reflectivity3,
				    fcd_ptr->BCR[ BAND_3].StartDetector, 
				    fcd_ptr->BCR[ BAND_3].EndDetector,
				    grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(BAND_4)],
				    Reflectivity4,
				    fcd_ptr->BCR[ BAND_4].StartDetector, 
				    fcd_ptr->BCR[ BAND_4].EndDetector,
				    self->Spectrum.BandRecord[ BAND_4].IntgTime,
				    &ResultRefl, WAVELENGTH_OVERL_3_4, ASCII_Time) 
		== SUCCESS) {
		self->Spectrum.ReflectivityJump[ BAND_3] = ResultRefl;
		self->Spectrum.ReflectivityJump[ BAND_4] = ResultRefl;
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    **  convert units
    \*------------------------------------------------------------------------*/
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	Band = (ScienceData_e)i;
	if (config_ptr->Filter.bandFilter[i] 
	    && (self->Spectrum.ScienceDataAvail[i])) 
	{
	    /*----------------------------------------------------------------*\
	    **  convert units 
	    \*----------------------------------------------------------------*/
	    if ((config_ptr->Filter.calibrationFilter[ CAL_LEAKAGE]) &&
		(config_ptr->Filter.calibrationFilter[ CAL_FIXED]) &&
		(config_ptr->Filter.calibrationFilter[ CAL_STRAYLIGHT]) &&
		(config_ptr->Filter.calibrationFilter[ CAL_NORMALIZE]) &&
		(config_ptr->Filter.calibrationFilter[ CAL_INTENSITY]) &&
		(config_ptr->Filter.calibrationFilter[ CAL_UNITS])) {
		for (j = fcd_ptr->BCR[ Band].StartDetector;
		     j <= fcd_ptr->BCR[ Band].EndDetector; j++) {
		    self->Spectrum.Radiance[Band2Channel(Band)][j] = 
			Watt2Photon(
				    self->Spectrum.Radiance[Band2Channel(Band)][j],
				    grid_ptr->Grid[self->PixelPar.IndexSpectralCalPar][Band2Channel(Band)][j]);
		}
	    }
	}
    }

    return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                Get_PCD_History
**  Input parameters:
**    Config		Extract configuration structure
**    SPH1		Specific Product Header of Level 1 Product
**
**  Output parameters:
**    EDR		Eartshine description record
**    Get_PCD_History	SUCCESS	= History complete and ok
**			WARN	= History not complete
**			FATAL	= error occured reading the PCD history
**
**  Other interfaces:
**    Input file of Level 1 Product specified in Config
**
**  Description:
**    Function to get the complete PCD or SCD or MCD history according the
**    max. Integration time of the Band Data of the EDR. The History starts
**    with the previous PCD or SCD or MCD from the current PCD or SCD or MCD
**    of the EDR.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Get_PCD_History(const Config_s *Config, const SPH1_s *SPH1, EDR_s *EDR,
		Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    FunctionStatus_e Stat;
    float MaxIntgTime;
    float IntgTime;
    int	NoOfMeasurementPixels;
    int	First_PCD;
    int	i;
    long FilePosPCD;
    PCD_s PCD;
    SMCD_s SMCD;
    PCD_s *xCD;
    unsigned long TimeTol = 2;	/* Time Tolerance in msec */
    UTC_s IntgTime_UTC;
    UTC_s TimeFirst_PCD;
    UTC_s PixelTime;
    char ASCII_Time[ 30];
    char LogBuffer[LOG_FIELD_SIZE];
#ifdef DEBUG
    char debug_message[255];
#endif

    /*------------------------------------------------------------------------*\
    ** calculate max( OrigIntgTime[ i])
    \*------------------------------------------------------------------------*/
    MaxIntgTime = 0;
    for (i = Config->StartBand; i <= Config->EndBand; i++) {
	if ((Config->Filter.bandFilter[ i]) 
	    && (EDR->PixelPar.ScienceIndex[ i] != INVALID_INDEX) 
	    && MaxIntgTime < EDR->Spectrum.BandRecord[ i].IntgTime) {
	    MaxIntgTime = EDR->Spectrum.BandRecord[ i].IntgTime;
	}
    }

    /*------------------------------------------------------------------------*\
    ** calculate the number of Geolocations corresponding to this Integration
    ** time and the time of the first PCD or SCD or MCD
    \*------------------------------------------------------------------------*/
    IntgTime_UTC.Days = 0;
    IntgTime_UTC.Msecs = (unsigned long)(MaxIntgTime * 1000.0) - TimeTol;
    UTC_Sub( EDR->PixelPar.Date, IntgTime_UTC, &TimeFirst_PCD);

    IntgTime = MaxIntgTime - INTG_TIME_CONV_FACTOR;
    if (IntgTime < 0.0)
	IntgTime = 0.0;
    NoOfMeasurementPixels = (short)(IntgTime / GROUND_PIXEL_TIME);
    First_PCD = EDR->Spectrum.Current_PCD - NoOfMeasurementPixels;
    if (First_PCD < 0)
    {
	Status = WARN;
	UTC_2_ASCII( EDR->PixelPar.Date, ASCII_Time);
	sprintf( LogBuffer,
		 "all Measurement Pixel Parameter (Geolocations) corresponding"
		 " to the written Band in Pixel: %s", ASCII_Time);
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__2022", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	First_PCD = 0;
    }

#ifdef DEBUG  
    sprintf(debug_message, "get PCD or SCD or MCD History from PCD or SCD "
	   "or MCD[ %d] to PCD or SCD or MCD[ %d]\n", First_PCD,
	   EDR->Spectrum.Current_PCD);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
    sprintf(debug_message, "MaxIntgTime : %f\n", MaxIntgTime);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
#endif

    /*------------------------------------------------------------------------*\
    ** allocate memory for Geolocations and read the Geolocations
    \*------------------------------------------------------------------------*/
    if (First_PCD < EDR->Spectrum.Current_PCD) {    
	EDR->N_PCD_History = EDR->Spectrum.Current_PCD - First_PCD;
#ifdef __cplusplus
	EDR->PCD_History = new PixelPar_s[ EDR->N_PCD_History];
#else
	EDR->PCD_History = (PixelPar_s*)malloc( sizeof( PixelPar_s) *
						EDR->N_PCD_History);
#endif
	if (EDR->PCD_History == 0) {
	    Status = FATAL;
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__2049", Status,
		      LOG_CANNOT_ALLOCATE,
		      "PixelPar_s in Get_PCD_History", "");
	    return Status;
	}

	/*----------------------------------------------------------------*\
	**  read the PCDs or SCDs or MCDs of all measurement pixels
	**  (current PCD or SCD or MCD excluded)
	\*----------------------------------------------------------------*/
	EDR->N_PCD_HistoryRead = 0;
	for (i = First_PCD; i < EDR->Spectrum.Current_PCD; i++) {
	    FilePosPCD = Config->FilePosPCD + Config->PCD_Size * i;
	    switch (EDR->ProductType) {
	    case PR_EARTHSHINE: {
		Stat = PCD__ReadTime( Config->Level_1_ProductStream, 
				      FilePosPCD, Config->PCD_Size, 
				      &PixelTime, &PCD, convert_sun_2_host);
		xCD	= &PCD;
		break;
	    }
	    case PR_SUNSHINE:
	    case PR_MOONSHINE: {
		Stat = SMCD__ReadTime( Config->Level_1_ProductStream, 
				       FilePosPCD,
				       Config->PCD_Size, &PixelTime,
				       (PCD_s*)&SMCD, convert_sun_2_host);
		xCD = ( PCD_s*)&SMCD;
		break;
	    }
	    default: {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__2081", Status,
			  LOG_NUM, "Illegal product type", "");
		return Status;
	    }
	    }

	    if (Stat != SUCCESS) 
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__2090", Status,
			  LOG_CANNOT_READ,
			  "PCD or SCD or MCD of history from Level 1 Product file", "");
		return Status;
	    }

	    /*--------------------------------------------------------*\
	    **  put the read PCD or SCD or MCD into the PCD or SCD or
	    **  MCD history
	    \*--------------------------------------------------------*/
	    if (UTC_Comp( '<', TimeFirst_PCD, PixelTime)) 
	    {
		EDR->N_PCD_HistoryRead += 1;
		EDR->PCD_History[ i - First_PCD].MeasurementPixel = i;
		    
		/*----------------------------------------------------*\
		**  get the pixel spezific spectrum parameter
		\*----------------------------------------------------*/
		if (PixelPar__Init(SPH1, i, EDR->ProductType, xCD,
				   Config->PCD_Size, &EDR->PCD_History
				   [i - First_PCD]) != SUCCESS)
		{
		    Status = FATAL;
		    UTC_2_ASCII( EDR->PixelPar.Date, ASCII_Time);
		    sprintf( LogBuffer, "Cannot get PCD or SCD or MCD "
			     "Parameter of history corresponding to "
			     "the Bands in Pixel: %s", ASCII_Time);
		    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__2117",
			     Status, LOG_NUM, LogBuffer, "");
		    return Status;
		}
	    }
	    else 
	    {
		Status = WARN;
		UTC_2_ASCII( EDR->PixelPar.Date, ASCII_Time);
		sprintf(LogBuffer, "all Measurement Pixel parameter "
			"(Geolocations) corresponding to the written "
			"spectrum (GAP) in Pixel: %s", ASCII_Time);
		Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.3__2129", Status,
			 LOG_CANNOT_READ, LogBuffer, "");
		break;
	    }
	}
    }

    return Status;
}
 

/*----------------------------------------------------------------------------*\
**                                Normalize
**  Input parameters:
**    Integration time
**    Reticon data
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**  Output parameters:
**    Normalized reticon data
**  Other interfaces:
**  Description:
**    Normalize reticon data to 1 second
**  References:
**  Libraries:
**  Created:	26.4.94
**  Author:		Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void
Normalize(float IntegrationTime, float *Reticon, short Begin, short End)
{
    short i;

    for (i=Begin; i<=End; i++) 
    {
	Reticon[i] /= IntegrationTime;
    }
}
