/*----------------------------------------------------------------------------*\
**                                ERS_PropagatorState.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)ERS_PropagatorState.c	5.1 02/26/99, Copyright DLR 1998"

#include "ers_tate.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)ERS_PropagatorState.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("ERS_PropagatorState")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                           ERS_PropagatorState__Sun2Host
**  Input parameters:
**    ERS_PropagatorState		in Sun memory representation
**  Output parameters:
**    ERS_PropagatorState		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the ERS_PropagatorState struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
ERS_PropagatorState__Sun2Host( ERS_PropagatorState_s *Param)
{
    int	i;

    for ( i = 0; i < ATTITUDE_SIZE; i++)
    {
	/* double */
	Param->ATT[ i] = SunDouble2Double( (unsigned char*)&Param->ATT[ i]);
	Param->DATT[ i] = SunDouble2Double( (unsigned char*)&Param->DATT[ i]);
    }
    /* long */
    Param->IATT = SunLong2Long( (unsigned char*)&Param->IATT);
    /* long */
    Param->StatusInit = SunLong2Long( (unsigned char*)&Param->StatusInit);
    /* double */
    Param->RTIME = SunDouble2Double( (unsigned char*)&Param->RTIME);

    for ( i = 0; i < KEPLER_STATE_SIZE; i++)
    {
	/* double */
	Param->XM[ i] = SunDouble2Double( (unsigned char*)&Param->XM[ i]);
    }
}

/*----------------------------------------------------------------------------*\
**                              ERS_PropagatorState__Read
**  Input parameters:
**	Stream		input stream to read ERS_PropagatorState struct
**  Output parameters:
**	Param		read ERS_PropagatorState parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all ERS_PropagatorState parameter entries from the
**      given input stream into the ERS_PropagatorState structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
ERS_PropagatorState__read(ERS_PropagatorState_s* self, FILE* file_ptr,
			  Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;  
 
    /*------------------------------------------------------------------------*\
    **  read StateVector
    \*------------------------------------------------------------------------*/
    if ( (Status = StateVectorC__read(&(self->StateVector), file_ptr, 
				     convert_sun_2_host)) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__106", Status,
		 LOG_CANNOT_READ, "ERS_PropagatorState, StateVector", "");
	return Status;
    }
    NoOfBytes += STATE_VECTOR_SIZE;

    /*--------------------------------------------------------------------*\
    **  read Combined Effect of AOCS Mispointing, Mispointing and 
    **  Misalignment of GOME Scan Mirror Axis
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread(self->ATT, 1, sizeof(self->ATT), file_ptr);

    /*--------------------------------------------------------------------*\
    **  read AOCS Mispointing Rate
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread(self->DATT, 1, sizeof(self->DATT), file_ptr);

    /*--------------------------------------------------------------------*\
    **  read Attitude Flag
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread(&self->IATT, 1, LONG_SIZE, file_ptr);

    /*--------------------------------------------------------------------*\
    **  read Status Returned from GOME_INIT
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread(&self->StatusInit, 1, LONG_SIZE, file_ptr);

    /*--------------------------------------------------------------------*\
    **  read Modified Julian Day (MJD50) at True Ascending Node Crossing
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread(&self->RTIME, 1, sizeof(self->RTIME), file_ptr);

    /*--------------------------------------------------------------------*\
    **  read Reference Mean Kepler State at True Ascending Node Crossing
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread(self->XM, 1, sizeof(self->XM), file_ptr);

    if (NoOfBytes != ERS_PROPAGATOR_STATE_SIZE) 
    {
	char LogBuffer[LOG_FIELD_SIZE];
	Status = FATAL;
	sprintf(LogBuffer, "ERS_PropagatorState Parameter, #read bytes: %d",
		NoOfBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__149", Status,
		 LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*--------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	ERS_PropagatorState__Sun2Host(self);
    }
    
    return Status;
}


/*----------------------------------------------------------------------------*\
**                              ERS_PropagatorState__Write
**  Input parameters:
**	Stream		input stream to write ERS_PropagatorState struct
**  Output parameters:
**	Param		write ERS_PropagatorState parameter
**
**  Other interfaces:
**
**  Description:
**	Function to write all ERS_PropagatorState parameter entries to the
**      given output stream
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
ERS_PropagatorState__Write( FILE* Stream, const ERS_PropagatorState_s *Param)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;  
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    **  write StateVector
    \*--------------------------------------------------------------------*/
    if (StateVectorC__write(&Param->StateVector, Stream) != SUCCESS)
    {
	Status = FATAL;
	sprintf( LogBuffer, "ERS_PropagatorState, StateVector, #written bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__201", Status,
		  LOG_CANNOT_WRITE, LogBuffer, "");
	return Status;
    }

    NoOfBytes += STATE_VECTOR_SIZE;

    /*--------------------------------------------------------------------*\
    ** write Combined Effect of AOCS Mispointing, Mispointing and Misalignment
    ** of GOME Scan Mirror Axis
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( Param->ATT, 1, sizeof( Param->ATT), Stream);

    /*--------------------------------------------------------------------*\
    **  write AOCS Mispointing Rate
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( Param->DATT, 1, sizeof( Param->DATT), Stream);

    /*--------------------------------------------------------------------*\
    **  write Attitude Flag
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( &Param->IATT, 1, sizeof( Param->IATT), Stream);

    /*--------------------------------------------------------------------*\
    **  write Status Returned from GOME_INIT
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( &Param->StatusInit, 1, sizeof( Param->StatusInit),
			 Stream);

    /*--------------------------------------------------------------------*\
    **  write Modified Julian Day (MJD50) at True Ascending Node Crossing
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( &Param->RTIME, 1, sizeof( Param->RTIME), Stream);

    /*--------------------------------------------------------------------*\
    **  write Reference Mean Kepler State at True Ascending Node Crossing
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( Param->XM, 1, sizeof( Param->XM), Stream);

    /*--------------------------------------------------------------------*\
    **  check 
    \*--------------------------------------------------------------------*/
    if (NoOfBytes != ERS_PROPAGATOR_STATE_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "ERS_PropagatorState Parameter, #write bytes: %d", NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__248", Status, LOG_CANNOT_WRITE,
		  LogBuffer, "");
	return Status;
    }
	
    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                           ERS_PropagatorState__Show
**  Input parameters:
**	Stream		output stream
**	Param		ERS_PropagatorState structure
**
**  Output parameters:
**
**  Other interfaces:
**	ERS_PropagatorState structure written to the output stream
**
**  Description:
**	Function to write the ERS_PropagatorState structure to the output
**      stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void ERS_PropagatorState__Show( FILE* Stream,
				const ERS_PropagatorState_s *Param)
{
    int	i;

    fprintf( Stream, "ERS_PropagatorState\n");

    /*--------------------------------------------------------------------*\
    ** show state vector
    \*--------------------------------------------------------------------*/
    StateVectorC__show(&Param->StateVector, Stream);

    /*--------------------------------------------------------------------*\
    ** write ERS_PropagatorState parameter
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Index\tATT\tDATT\n");
    for ( i = 0; i < ATTITUDE_SIZE; i++)
	fprintf( Stream, "%5d\t%lf\t%lf\n", i, Param->ATT[ i], Param->DATT[ i]);

    fprintf( Stream, "IATT\t%d\n", Param->IATT);
    fprintf( Stream, "StatusInit\t%d\n", Param->StatusInit);
    fprintf( Stream, "RTIME\t%lf\n", Param->RTIME);
    fprintf( Stream, "XM");
    for ( i = 0; i < KEPLER_STATE_SIZE; i++)
	fprintf( Stream, "\t%lf", Param->XM[ i]);
    fprintf( Stream, "\n");

}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
