/*----------------------------------------------------------------------------*\
**                                FCD.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)FCD.c	5.2 08/19/02, Copyright DLR 1998"

#include <stdlib.h>
#include "fcd.h"
#include "dataconv.h"
#include "log.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)FCD.c 5.2 08/19/02, Copyright DLR 1998")


#define	SS_ID	("gdp01_ex")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/
/*----------------------------------------------------------------------------*\
**                                CalculateGrids
**  Input parameters:
**    FCD			Fixed calibration record
**
**  Output parameters:
**    GRID			all calculated grids
**    CalculateGrids		SUCCESS	=	Read ok
**				else	=	any error occured during read
**
**  Other interfaces:
**
**  Description:
**    Function to calculate all grid of the FCD. First the function allocates
**    enough memory for all grids. Then the grids are calculated.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	28.9.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FCD__calculateGrids(FCD_s *FCD, GRID_s* grid_ptr)
{
    FunctionStatus_e Status = SUCCESS;
    int	i, j;
    char LogBuffer[LOG_FIELD_SIZE];
#ifdef DEBUG
    char debug_message[255];
#endif

    /*------------------------------------------------------------------------*\
    **  allocate memory for grids
    \*------------------------------------------------------------------------*/
    grid_ptr->NumberOfGrids = FCD->Nspec;
#ifdef __cplusplus
    grid_ptr->Grid = new float[grid_ptr->NumberOfGrids][OPTICAL_CHANNELS][DETECTOR_PIXELS];
#else
    grid_ptr->Grid = (float (*)[OPTICAL_CHANNELS][DETECTOR_PIXELS])
	malloc(sizeof(float) * grid_ptr->NumberOfGrids * OPTICAL_CHANNELS 
	       * DETECTOR_PIXELS);
#endif
    if (grid_ptr->Grid == 0)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__85", Status,
		 LOG_CANNOT_ALLOCATE, "for Grids", "");
	return Status;
    }

    /*------------------------------------------------------------------*\
    ** calculate grids of all Science Data
    \*------------------------------------------------------------------*/
#ifdef DEBUG  
    sprintf(debug_message, "Calculate %d Grids\n", grid_ptr->NumberOfGrids);
    Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
#endif
    for (j = 0; j < grid_ptr->NumberOfGrids; j++)
    {
	for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
	{
	    SpectralC__apply(&FCD->SpectralPar[j], (ScienceData_e)i,
			    grid_ptr->Grid[j][Band2Channel((ScienceData_e)i)],
			    FCD->BCR[i].StartDetector,
			    FCD->BCR[i].EndDetector, j);
	}
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                            FCD__Sun2Host
**  Input parameters:
**    FCD			in Sun memory representation
**  Output parameters:
**    FCD			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the FCD struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FCD__Sun2Host( FCD_s *Param)
{
    int	i;

    /* short */
    Param->DetectorConfFlags = SunShort2Short( (unsigned char*)
					       &Param->DetectorConfFlags);
    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
	/* short */
	Param->BCR[ i].DetectorArrayNo = SunShort2Short( (unsigned char*)
							 &Param->BCR[ i].DetectorArrayNo);
	/* short */
	Param->BCR[ i].StartDetector = SunShort2Short( (unsigned char*)
						       &Param->BCR[ i].StartDetector);
	/* short */
	Param->BCR[ i].EndDetector = SunShort2Short( (unsigned char*)
						     &Param->BCR[ i].EndDetector);
    }

    Param->IndexSpectralCalPar = SunShort2Short( (unsigned char*)
						 &Param->IndexSpectralCalPar);
}

/*----------------------------------------------------------------------------*\
**                                FCD__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		FCD
**
**  Other interfaces:
**
**  Description:
**	Function to read all FCD entries from the given 
**	input stream into the FCD structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FCD__read(FCD_s* self, FILE* Stream, Bool convert_sun_2_host)
{
    int i;
    FunctionStatus_e Status = SUCCESS;
    long NoOfReadBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read Detector Confidence Fags
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = fread( &self->DetectorConfFlags, 1,
			   FCD_DETECTOR_CONFIDENCE_SIZE, Stream);
    if (NoOfReadBytes != FCD_DETECTOR_CONFIDENCE_SIZE) {
	Status = FATAL;
	sprintf( LogBuffer, "FCD, DetectorConfFlags, #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__193", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }
	
    /*----------------------------------------------------------------------*\
    ** read Band Configuration records
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = 0;
    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	NoOfReadBytes += fread( &self->BCR[ i].DetectorArrayNo, 1,
				SHORT_SIZE, Stream);
	NoOfReadBytes += fread( &self->BCR[ i].StartDetector, 1,
				SHORT_SIZE, Stream);
	NoOfReadBytes += fread( &self->BCR[ i].EndDetector, 1,
				SHORT_SIZE, Stream);
    }
    if (NoOfReadBytes != FCD_BCR_SIZE) 
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "FCD, Band configuration record[ %4d], #read bytes: %d",
		 i, NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__217", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** read preflight errors
    \*----------------------------------------------------------------------*/
    if ( (Status = KeyDataError__read(&(self->KeyDataError), Stream,
				      convert_sun_2_host)) != SUCCESS)
    {
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__228", Status,
		 LOG_CANNOT_READ, "FCD, KeyData Errors", "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** read BSDF
    \*----------------------------------------------------------------------*/
    if (BSDF__read(&self->BSDF, Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__239", Status,
		 LOG_CANNOT_READ, "FCD, BSDF", "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** read Straylight calibration parameter
    \*------------------------------------------------------------------------*/
    if (StraylightCharacteristic__read(&self->StraylightCharacteristic,
				       Stream, convert_sun_2_host)
	!= SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__252", Status,
		 LOG_CANNOT_READ, "FCD, StrayLight Characteristic Parameter",
		  "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** read FPA parameter
    \*------------------------------------------------------------------------*/
    if (FPA__read(&(self->FPA), Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__264", Status,
		  LOG_CANNOT_READ, "FCD, FPA Parameter", "");
	return Status;
    }
    

    /*----------------------------------------------------------------------*\
    ** read Leakage calibration parameter
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = fread( &self->Nleak, 1, FCD_NLEAK_SIZE, Stream);
    if (NoOfReadBytes != FCD_NLEAK_SIZE) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "FCD, Number of Leakage Parameter sets, #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__279", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }
	
    /*---------------------------------------------------------------------*\
    ** Convert Nleak
    \*---------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	self->Nleak = SunShort2Short( (unsigned char*)&self->Nleak);
    }

    if (self->Nleak == 0) 
    {
	self->LeakagePar = 0;
    }
    else
    {
	/*--------------------------------------------------------------------*\
	** create Leakage
	\*--------------------------------------------------------------------*/
#ifdef __cplusplus
	self->LeakagePar = new LeakageC_s [ self->Nleak];
#else
	self->LeakagePar = (LeakageC_s*)malloc(sizeof( LeakageC_s) * self->Nleak);
#endif
	if (self->LeakagePar == 0) 
	{
	    Status = FATAL;
	    sprintf(LogBuffer, "FCD, memory for Leakage Parameter, #bytes to "
		    "allocate: %d", sizeof( LeakageC_s) * self->Nleak);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__311", Status,
		     LOG_CANNOT_ALLOCATE, LogBuffer, "");
	    return Status;
	}

	for (i = 0; i < self->Nleak; i++) {
	    if (LeakageC__read(&self->LeakagePar[i], Stream,
			       convert_sun_2_host) != SUCCESS)
	    {
		Status = FATAL;
		sprintf( LogBuffer, "FCD, Leakage Parameter[%4d]", i);
		Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__322", Status,
			  LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    ** read fixed calibration parameter
    \*------------------------------------------------------------------------*/
    if (FixedC__read(&self->FixedPar, Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__335", Status,
		 LOG_CANNOT_READ, "FCD, Fixed Calalibration Parameter", "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** read number of hot pixel
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = fread( &self->Nhot, 1, FCD_NHOT_SIZE, Stream);
    if (NoOfReadBytes != FCD_NHOT_SIZE) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "FCD, Number of Hot Pixels (Nhot), #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__349", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*---------------------------------------------------------------------*\
    ** Convert Nhot
    \*---------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	self->Nhot = SunShort2Short( (unsigned char*)&self->Nhot);
    }

    /*----------------------------------------------------------------------*\
    ** read hot pixel
    \*----------------------------------------------------------------------*/
    if (self->Nhot == 0) 
    {
	self->HotPixelPar = 0;
    }
    else {
	/*--------------------------------------------------------------------*\
	** create Hot pixels
	\*--------------------------------------------------------------------*/
#ifdef __cplusplus
	self->HotPixelPar = new HotPixelC_s [ self->Nhot];
#else
	self->HotPixelPar = (HotPixelC_s*)malloc( sizeof( HotPixelC_s) *
						 self->Nhot);
#endif
	if (self->HotPixelPar == 0) 
	{
	    Status = FATAL;
	    sprintf( LogBuffer,
		     "FCD, memory for Hot Pixels, #bytes to allocate: %d",
		     sizeof( HotPixelC_s) * self->Nhot);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__385", Status,
		      LOG_CANNOT_ALLOCATE, LogBuffer, "");
	    return Status;
	}

	for ( i = 0; i < self->Nhot; i++) 
	{
	    if (HotPixelC__read(&self->HotPixelPar[i], Stream,
			       convert_sun_2_host) != SUCCESS)
	    {
		Status = FATAL;
		sprintf(LogBuffer,"FCD, Hot Pixel[ %4d]", i);
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__397", Status,
			 LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }
	}
    }

    /*----------------------------------------------------------------------*\
    ** read number of Spectral Parameter sets
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = fread( &self->Nspec, 1, FCD_NSPEC_SIZE, Stream);
    if (NoOfReadBytes != FCD_NSPEC_SIZE) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "FCD, Number of Spectral parameter sets (Nspec), #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__413", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*---------------------------------------------------------------------*\
    ** Convert Nspec
    \*---------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	self->Nspec = SunShort2Short( (unsigned char*)&self->Nspec);
    }

    /*----------------------------------------------------------------------*\
    ** read Spectral Calibration parameter including preflight parameter
    \*----------------------------------------------------------------------*/
    if (self->Nspec == 0) 
    {
	self->SpectralPar = 0;
    }
    else {
	/*--------------------------------------------------------------------*\
	** create Spectral parameter sets
	\*--------------------------------------------------------------------*/
#ifdef __cplusplus
	self->SpectralPar = new SpectralC_s [ self->Nspec];
#else
	self->SpectralPar = (SpectralC_s*)malloc( sizeof( SpectralC_s) *
						 self->Nspec);
#endif
	if (self->SpectralPar == 0) {
	    Status = FATAL;
	    sprintf(LogBuffer,
		    "FCD, memory for spectral parameter, #bytes to allocate: %d",
		    sizeof( SpectralC_s) * self->Nspec);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__448", Status,
		      LOG_CANNOT_ALLOCATE, LogBuffer, "");
	    return Status;
	}

	for (i = 0; i < self->Nspec; i++) {
	    if (SpectralC__read(&self->SpectralPar[i], Stream,
			       convert_sun_2_host) != SUCCESS)
	    {
		Status = FATAL;
		sprintf(LogBuffer, "FCD, Spectral Parameter set[%4d]", i);
		Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__459", Status,
			  LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }
	}
    }

    /*------------------------------------------------------------------------*\
    ** read Intensity calibration parameter
    \*------------------------------------------------------------------------*/
    NoOfReadBytes = fread( &self->IndexSpectralCalPar, 1,
			   FCD_INDEX_SPECTRAL_CAL_PAR_SIZE, Stream);
    if ( NoOfReadBytes != FCD_INDEX_SPECTRAL_CAL_PAR_SIZE) {
	Status = FATAL;
	sprintf(LogBuffer,
		"FCD, Intensity Calibration Parameter, #read bytes: %d",
		NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__476", Status, 
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    if (IntensityC__read(&self->IntensityPar, Stream, convert_sun_2_host)
	!= SUCCESS)
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__485", Status, 
		  LOG_CANNOT_READ, "FCD, Intensity Calibration Parameter", "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** read number of Scan Mirror Angle
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = fread( &self->Nang, 1, FCD_NANG_SIZE, Stream);
    if (NoOfReadBytes != FCD_NANG_SIZE) 
    {
	Status = FATAL;
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__497", Status, LOG_CANNOT_READ,
		 "FCD, Number of Scan Mirror Angle (Nang)", "");
	return Status;
    }

    /*---------------------------------------------------------------------*\
    ** Convert Nang
    \*---------------------------------------------------------------------*/
    if (convert_sun_2_host) 
    {
	self->Nang = SunShort2Short( (unsigned char*)&self->Nang);
    }

    /*----------------------------------------------------------------------*\
    ** read number of Interpolated Polarization Sensitivity Parameter,
    ** and the Interpolated Polarisation Sensitivity parameter itself
    \*----------------------------------------------------------------------*/
    if (self->Nang == 0) {
	self->KeyDataCalibrationL1_Par = 0;
    }
    else {
	/*--------------------------------------------------------------------*\
	** create Polsensitivity
	\*--------------------------------------------------------------------*/
#ifdef __cplusplus
	self->KeyDataCalibrationL1_Par = new
	    KeyDataCalibrationL1_s [ self->Nang];
#else
	self->KeyDataCalibrationL1_Par = 
	    (KeyDataCalibrationL1_s*)malloc(sizeof( KeyDataCalibrationL1_s)
					    * self->Nang);
#endif
	if (self->KeyDataCalibrationL1_Par == 0) {
	    Status = FATAL;
	    sprintf( LogBuffer,
		     "FCD, memory for KeyDataCalibrationL1 Par., #bytes to allocate: %d",
		     sizeof( KeyDataCalibrationL1_s) * self->Nang);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__534", Status,
		      LOG_CANNOT_ALLOCATE, LogBuffer, "");
	    return Status;
	}

	for ( i = 0; i < self->Nang; i++) 
	{
	    if (KeyDataCalibrationL1__read(&self->KeyDataCalibrationL1_Par[i],
					   Stream, convert_sun_2_host)
		 != SUCCESS)
	    {
		Status = FATAL;
		sprintf(LogBuffer, "FCD, KeyDataCalibrationL1[%4d]", i);
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__547", Status,
			 LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }
	}
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	FCD__Sun2Host( self);
    }

    return SUCCESS;  
}


/*----------------------------------------------------------------------------*\
**                                FCD__Show
**  Input parameters:
**	Stream		output stream
**	self		FCD structure
**
**  Output parameters:
**
**  Other interfaces:
**	FCD structure written to the output stream
**
**  Description:
**	Function to write the FCD structure to the
**	output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FCD__show(const FCD_s *self, FILE* Stream, Bool FullInfo)
{
    static const char	*DetConfStr[ ] = {
	"        ok",
	"      dead",
	"dead pixel",
	" undefined"};
    int		i;
    short		Channel_1;
    short		Channel_2;
    short		Channel_3;
    short		Channel_4;
    short		PMD_1;
    short		PMD_2;
    short		PMD_3;

    fprintf( Stream, "\nFCD\n\n");
    /*----------------------------------------------------------------------*\
    ** show Detector Confidence Fags
    \*----------------------------------------------------------------------*/
    Channel_1	= (self->DetectorConfFlags & 0x3);
    Channel_2	= (self->DetectorConfFlags & 0xC) / 0x4;
    Channel_3	= (self->DetectorConfFlags & 0x30) / 0x10;
    Channel_4	= (self->DetectorConfFlags & 0xC0) / 0x40;
    PMD_1		= (self->DetectorConfFlags & 0x100) / 0x100;
    PMD_2		= (self->DetectorConfFlags & 0x200) / 0x200;
    PMD_3		= (self->DetectorConfFlags & 0x400) / 0x400;
    fprintf( Stream, "Detector Confidence Flags\n");
    fprintf( Stream, "Channel 1 %s\n", DetConfStr[ Channel_1]);
    fprintf( Stream, "Channel 2 %s\n", DetConfStr[ Channel_2]);
    fprintf( Stream, "Channel 3 %s\n", DetConfStr[ Channel_3]);
    fprintf( Stream, "Channel 4 %s\n", DetConfStr[ Channel_4]);
    fprintf( Stream, "PMD 1     %s\n", DetConfStr[ PMD_1]);
    fprintf( Stream, "PMD 2     %s\n", DetConfStr[ PMD_2]);
    fprintf( Stream, "PMD 3     %s\n", DetConfStr[ PMD_3]);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show Band Configuration records
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "BCR\n");
    fprintf( Stream, "Band  Channel  Start  End\n");
    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
	fprintf( Stream,
		 "%s\t\t%3d\t%4d\t%4d\n",
		 ScienceData2Str((ScienceData_e)i), self->BCR[ i].DetectorArrayNo,
		 self->BCR[ i].StartDetector, self->BCR[ i].EndDetector);
    }
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show KeyData errors
    \*----------------------------------------------------------------------*/
    KeyDataError__show(&self->KeyDataError, Stream);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show BSDF
    \*----------------------------------------------------------------------*/
    BSDF__show(&self->BSDF, Stream);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show StraylightCharacteristic
    \*----------------------------------------------------------------------*/
    StraylightCharacteristic__show(&self->StraylightCharacteristic, Stream);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show FPA
    \*----------------------------------------------------------------------*/
    FPA__show(&self->FPA, Stream);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show Leakage calibration parameter
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Number of Leakage (Nleak) %d\n\n", self->Nleak);
    for ( i = 0; i < self->Nleak; i++)
    {
	fprintf( Stream, "Leakage Parameter %d\n", i);
	LeakageC__show(&self->LeakagePar[i], Stream);
    }
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show fixed calibration parameter
    \*----------------------------------------------------------------------*/
    FixedC__show(&self->FixedPar, Stream);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show hot pixel
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Number of Hot Pixel (Nhot) %d\n\n", self->Nhot);
    if (self->Nhot > 0)
	fprintf( Stream, "\tRecord\tArray\tPixel\n");
    for ( i = 0; i < self->Nhot; i++)
    {
	HotPixelC__show(&self->HotPixelPar[i], Stream);
    }
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show Spectral Calibration parameter including preflight parameter
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Number of Spectral Paramater (Nspec) %d\n\n", self->Nspec);
    for ( i = 0; i < self->Nspec; i++)
    {
	fprintf( Stream, "Spectral Parameter %d\n", i);
	SpectralC__show(&self->SpectralPar[i], Stream);
	fprintf( Stream, "\n");
    }

    /*----------------------------------------------------------------------*\
    ** show Intensity calibration parameter
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Spectral Calibration Index %d\n\n",
	     self->IndexSpectralCalPar);

    IntensityC__show(&self->IntensityPar, Stream);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show number of Interpolated Polarization Sensitivity Parameter,
    ** and the Interpolated Polarisation Sensitivity parameter itself
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Number of Angles (Nang) %d\n\n", self->Nang);
    for ( i = 0; i < self->Nang; i++)
    {
	fprintf( Stream, "Pre-flight Calibration %d\n", i);
	KeyDataCalibrationL1__show(&self->KeyDataCalibrationL1_Par[i], Stream);
	fprintf( Stream, "\n");
    }

}

/*============================================================================*\
**  private operations
\*============================================================================*/

 
