/*----------------------------------------------------------------------------*\
**                                Filter.c
**
**  Class Filter (user filter parameters) definition of methods
** 
**  Version:    5.17
**  Date:       11/29/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)Filter.c	5.17 11/29/02, Copyright DLR 1996 - 2002"

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <ctype.h>
#include "filter.h"
#include "qa.h"
#include "log.h"
#include "wat2phot.h"



#define	SccsId	("@(#)Filter.c 5.17 11/29/02, Copyright DLR 1996 - 2002")


#define	SS_ID	("Filter")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                Filter__parseUserParameters
**  Input parameters:
**    argc: Number of arguments
**    argv: Array of strings containing the arguments.
**  Output parameters:
**    Filter: a structure containing the filter parameters
**  Other interfaces:
**    <none>
**  Description:
**    This function parse and check the parameters looking for the options
**    -i, -g, -b, -p, -t, -c, -x, -d, -w, -s, -m, input- and output_file.
**    If some error is found, False is returned.
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    15.11.94
**  Author:     Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Filter__parseUserParameters(Filter_s* self, ProductType_e* ProductType,
			    int argc, char *const argv[])
{
    FunctionStatus_e status = SUCCESS;
    static short BLANK_POSITION = 11;
    extern char *optarg;
    extern int optind;
    int i, idx, Opt;
    int QA_option;
    float coord_flt;
    char *Ptr;
    Bool BandFlag = False, CalibrationFlag = False, SubsetCounterFlag = False;
    char timeStart[ASCII_UTC_TIME_LENGTH], timeStop[ASCII_UTC_TIME_LENGTH];

    /*------------------------------------------------------------------------*\
    **  Set the values for ProductType
    \*------------------------------------------------------------------------*/
    *ProductType = PR_EARTHSHINE;
    for (i=1; i<argc; i++) 
    {
	if (strcmp(argv[i], "-s") == 0) 
	{
	    *ProductType = PR_SUNSHINE;
	}
	else if (strcmp(argv[i], "-m") == 0) 
	{
	    *ProductType = PR_MOONSHINE;
	}
    }

    /*------------------------------------------------------------------------*\
    **  Set the defalt values for Filter depending on the ProductType
    \*------------------------------------------------------------------------*/
    Filter__setDefaultValues(self, *ProductType);

    /*------------------------------------------------------------------------*\
    **  Get option letter from argument vector
    \*------------------------------------------------------------------------*/
    while ( ((Opt = getopt(argc, argv, "igq:b:p:t:r:c:x:e:f:ajkdnwsmh")) != EOF) 
	    && (status == SUCCESS))
    {
	switch (Opt) 
	{
	case 'g': /* Geolocation info */
	    self->geolocationInfoFlag = True;

	    if (BandFlag 
		|| CalibrationFlag || self->albedoFlag 
		|| self->radianceJumpFlag
		|| self->correctDegradationFlag
		|| self->correct_BSDF_Flag
		|| self->splitFlag 
		|| self->noSunFlag || self->calFileFlag || self->sunFlag 
		|| self->moonFlag || self->qaInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option -g is incompatible with the "
			"other ones\n", argv[0]);
		status = FATAL;
	    }
	    break;

	case 'i': /* Information */
	    if (BandFlag || self->pixelFlag || self->timeFlag 
		|| CalibrationFlag || SubsetCounterFlag || self->albedoFlag 
		|| self->radianceJumpFlag
		|| self->correctDegradationFlag
		|| self->correct_BSDF_Flag
		|| self->splitFlag 
		|| self->noSunFlag || self->calFileFlag || self->sunFlag 
		|| self->moonFlag || self->qaInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option -i is incompatible with the "
			"other ones\n", argv[0]);
		status = FATAL;
	    }
	    self->infoFlag = True;
	    break;

	case 'h': /* output in HDF format */
	    self->hdfOutputFlag = True;
	    break;

	case 'q': /* QA info */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -q are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    self->qaInfoFlag = True;

            /* get up to three parameters */

	    QA_option = (int)strtol(optarg, &Ptr, 10);
	    if (Ptr == optarg || optind >= argc) 
	    {
		optind--;
		break;
	    }
	    else 
	    {
              /* check if optarg is a pure number */
              idx = 0;
              while (optarg[idx] != 0 && isdigit(optarg[idx]) != 0 ) idx++;
              if (idx < strlen(optarg))
              {
		optind--;
		break;
              }
              self->qa_increment = QA_option;
	    }

	    QA_option = (int)strtol(argv[optind], &Ptr, 10);
	    if (Ptr == argv[optind] || optind >= argc) 
	    {
		break;
	    }
	    else 
	    {
		self->qa_pixelnumber = QA_option;
		optind++;
	    }

            if (optind >= argc) break;
	    QA_option = (int)strtol(argv[optind], &Ptr, 10);
	    if (Ptr == argv[optind] || optind >= argc)
	    {
		break;
	    }
	    else 
	    {
		self->qa_meannumber = QA_option;
		optind++;
	    }

	    break;

	case 'r': /* Region specification, corner coordinates */
	    if (optind >= argc)	
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires four arguments (lat "
			"long lat long) -- r\n", argv[0]);
		status = FATAL;
	    }

	    if (self->infoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -i and -r are incompatible "
			"-- r\n", argv[0]);
		status = FATAL;
	    }
	    self->regionFlag = True;
	    coord_flt = (float)strtod(optarg, &Ptr);
	    if (Ptr == optarg || optind >= argc) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires four arguments "
			"(lat long lat long) -- r\n", argv[0]);
		status = FATAL;
	    }
	    else 
	    {
		self->cornerCoordinate[0].Latitude = coord_flt;
	    }

	    coord_flt = (float)strtod(argv[optind], &Ptr);
	    if (Ptr == argv[optind]) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires four arguments (lat long"
			" lat long) -- r\n", argv[0]);
		status = FATAL;
	    }
	    else 
	    {
		self->cornerCoordinate[0].Longitude = coord_flt;
		optind++;
	    }

	    coord_flt = (float)strtod(argv[optind], &Ptr);
	    if (Ptr == argv[optind]) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires four arguments (lat long"
			"lat long) -- r\n", argv[0]);
		status = FATAL;
	    }
	    else 
	    {
		self->cornerCoordinate[1].Latitude = coord_flt;
		optind++;
	    }

	    coord_flt = (float)strtod(argv[optind], &Ptr);
	    if (Ptr == argv[optind]) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires four arguments (lat long"
			" lat long) -- r\n", argv[0]);
		status = FATAL;
	    }
	    else 
	    {
		self->cornerCoordinate[1].Longitude = coord_flt;
		optind++;
	    }

	    /* assure that latitudes are between -90 and +90 */
	    if (self->cornerCoordinate[0].Latitude < -90.0 
		|| self->cornerCoordinate[0].Latitude > 90.0 
		|| self->cornerCoordinate[1].Latitude < -90.0 
		|| self->cornerCoordinate[1].Latitude > 90.0)	
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: please specify latitudes between -90 and"
			" 90 degrees -- r\n", argv[0]);
		status = FATAL;
	    }

	    break;

	case 'b': /* Band filter */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -b are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (Filter__setYesNoPreferences(self->bandFilter, 
					  SCIENCE_DATA_DEFINED, optarg)) 
	    {
		BandFlag = True;
	    }
	    else 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires a valid argument -- b\n", 
			argv[0]);
		status = FATAL;
	    }
	    break;

	case 'p': /* Pixel p1 to p2 */
	    if (self->infoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -i and -p are incompatible"
			" -- p\n", argv[0]);
		status = FATAL;
	    }
	    if (self->timeFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -t and -p are incompatible -- "
			"p\n", argv[0]);
		status = FATAL;
	    }

	    self->pixelFlag = True;
	    self->pixelStart = (int)strtol(optarg, &Ptr, 10) - 1;
	    if (Ptr == optarg) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires two integers -- p\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (optind >= argc)	
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires two arguments -- p\n", 
			argv[0]);
		status = FATAL;
	    }
	    self->pixelStop = (int)strtol(argv[optind], &Ptr, 10) - 1;
	    if (Ptr == argv[optind]) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires two integers -- p\n", 
			argv[0]);
		status = FATAL;
	    }
	    else 
	    {
		optind++;
	    }

	    if (self->pixelStart > self->pixelStop)	
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option with pixel start greater than "
			"pixel stop -- p\n", argv[0]);
		status = FATAL;
	    }

	    if ((self->pixelStart >= self->pixelStop) 
		&& self->radianceJumpFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "Please process at least two pixels for jump "
			"correction (previous PMD readouts needed)\n");
		status = FATAL;
	    }
	    break;

	case 't': /* Time t1 to t2 */
	    if (self->infoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -i and -t are incompatible -- "
			"t\n", argv[0]);
		status = FATAL;
	    }
	    if (self->pixelFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -t and -p are incompatible -- "
			"p\n", argv[0]);
		status = FATAL;
	    }
	    self->timeFlag = True;
	    strncpy(timeStart, optarg, ASCII_UTC_TIME_LENGTH);
	    timeStart[BLANK_POSITION] = ' ';
	    if (optind >= argc)	
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires two arguments -- t\n", 
			argv[0]);
		status = FATAL;
	    }
	    strncpy(timeStop, argv[optind++], ASCII_UTC_TIME_LENGTH);
	    timeStop[BLANK_POSITION] = ' ';
	    /*---------------------------------------------------------------*\
	    **  Convert to UTC time
	    \*---------------------------------------------------------------*/
	    strncat(timeStart, "00", ASCII_UTC_TIME_LENGTH);
	    strncat(timeStop, "00", ASCII_UTC_TIME_LENGTH);
	    if ((ASCII_2_UTC(timeStart, &self->timeStart) != SUCCESS) 
		|| (ASCII_2_UTC(timeStop, &self->timeStop) != SUCCESS)) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires two valid time "
			"arguments -- t\n", argv[0]);
		status = FATAL;
	    }
	    if (UTC_Comp('>', self->timeStart, self->timeStop)) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option with time start greater than "
			"time stop -- t\n", argv[0]);
		status = FATAL;
	    }
	    break;

	case 'c': /* Calibration apply */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -c are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (Filter__setCalibration(self, optarg, *ProductType)) 
	    {
		CalibrationFlag = True;
		if (self->albedoFlag 
		    && self->calibrationFilter[CAL_UNITS]) 
		{
			Filter__writeDLR_Header(stderr);
		    fprintf(stderr, "%s: options -c \"U\" and -a are "
			    "incompatible\n", argv[0]);
		    status = FATAL;
		}
	    }
	    else 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires a valid argument -- c\n",
			argv[0]);
		status = FATAL;
	    }
	    break;

	case 'x': /* Subset counter filter */
	    if (self->infoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -i and -x are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (Filter__setYesNoPreferences(self->subsetCounterFilter, 
					    SUBSET_COUNTER_DEFINED, optarg)) 
	    {
		SubsetCounterFlag = True;
	    }
	    else 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires a valid argument\n", 
			argv[0]);
		status = FATAL;
	    }
	    break;

	case 'a': /* Albedo */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -a are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (self->noSunFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -n and -a are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (CalibrationFlag && self->calibrationFilter[CAL_UNITS]) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -c \"U\" and -a are "
			"incompatible\n", argv[0]);
		status = FATAL;
	    }
	    self->calibrationFilter[CAL_UNITS] = False;
	    self->albedoFlag = True;
	    break;

	case 'j': /* Radiance jump correction */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -j are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (self->pixelStart >= self->pixelStop) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "Please process at least two pixels for "
			"jump correction (previous PMD readouts needed)\n");
		status = FATAL;
	    }
	    self->radianceJumpFlag = True;
	    break;

	case 'k': /* switch off sun wavelength correction by Cross-Correlation */
			self->xCorrelationFlag = False;
	    break;

	case 'e': /* Degradation correction */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -e are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }

	    strncpy(self->DegradationParameterFile, optarg, MAX_FILENAME_LENGTH);
	    status = DegradationParameter__ReadVersion(self->DegradationParameterFile);
            if (status != SUCCESS)
	    {
	      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__1746", status,
	        LOG_CANNOT_READ, "Degradation parameters", "");
	    }

	    if (optind >= argc)	
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires a parameter file name -- e\n", 
			argv[0]);
		status = FATAL;
	    }

	    self->correctDegradationFlag = True;
	    self->correct_BSDF_Flag = True;
	    break;

	case 'f': /* BSDF correction */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -f are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }

	    strncpy(self->DegradationParameterFile, optarg, MAX_FILENAME_LENGTH);
	    DegradationParameter__ReadVersion(self->DegradationParameterFile);
	    status = DegradationParameter__ReadVersion(self->DegradationParameterFile);
            if (status != SUCCESS)
	    {
	      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.5__1746", status,
	        LOG_CANNOT_READ, "Degradation parameters", "");
	    }

	    if (optind >= argc)	
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: option requires a parameter file name -- f\n", 
			argv[0]);
		status = FATAL;
	    }

	    self->correct_BSDF_Flag = True;
	    break;

	case 'd': /* Divide (split) */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -d are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (self->albedoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -a and -d are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    self->splitFlag = True;
	    break;

	case 'n': /* No mean sun spectrum will be written to divided */
	    /* product files                                   */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -n are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (self->albedoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -a and -n are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    self->splitFlag = True;
	    self->noSunFlag = True;
	    break;

	case 'w': /* Write calibration  */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -w are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    self->calFileFlag = True;
	    break;

	case 's': /* Sun */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -s are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    if (self->moonFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -m and -s are incompatible\n", 
			argv[0]);
		status = FATAL;
	    }
	    self->sunFlag = True;
	    break;

	case 'm': /* Moon  */
	    if (self->infoFlag || self->geolocationInfoFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -ig and -m are incompatible\n", argv[0]);
		status = FATAL;
	    }
	    if (self->sunFlag) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: options -s and -m are incompatible\n", argv[0]);
		status = FATAL;
	    }
	    self->moonFlag = True;
	    break;

	case '?':
	default:
	    status = FATAL;
	}
    }

    if (status == SUCCESS) 
    {
	if (optind >= argc)
	{
		Filter__writeDLR_Header(stderr);
	    fprintf(stderr, "%s: requires an input_file\n", argv[0]);
	    status = FATAL;
	}
	else 
	{
	    self->inputFilename = argv[optind++];
	}
    }

    if (status == SUCCESS) 
    {
	if (! (self->infoFlag || self->geolocationInfoFlag)) 
	{
	    if (optind >= argc) 
	    {
		Filter__writeDLR_Header(stderr);
		fprintf(stderr, "%s: requires an output_file\n", argv[0]);
		status = FATAL;
	    }
	    else 
	    {
		self->outputFilename = argv[optind];
	    }
	}
    }

    if ( (status == SUCCESS) && (!self->pixelFlag && !self->timeFlag))
    {
	self->pixelFlag = True;
    }

    if (status != SUCCESS)
    {
	Filter__printUsage(argv[0]);
    }

    return status;
}


/*----------------------------------------------------------------------------*\
**                                Filter__show
**  Input parameters:
**    Stream		output stream
**    Filter		data structure of structure Filter_s
**
**  Output parameters:
**    <none>
**    
**  Other interfaces:
**
**  Description:
**    Function to write the data structure Filter_s to the output stream
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
Filter__show(const Filter_s* self, FILE* Stream)
{
    int	i;

    /*------------------------------------------------------------------------*\
    **  Show Filter
    \*------------------------------------------------------------------------*/
    fprintf(Stream, "pixelFlag              : %d\n", self->pixelFlag);
    fprintf(Stream, "timeFlag               : %d\n", self->timeFlag);
    fprintf(Stream, "regionFlag             : %d\n", self->regionFlag);
    fprintf(Stream, "calFileFlag            : %d\n", self->calFileFlag);
    fprintf(Stream, "albedoFlag             : %d\n", self->albedoFlag);
    fprintf(Stream, "radianceJumpFlag       : %d\n", self->radianceJumpFlag);
    fprintf(Stream, "xCorrelationFlag       : %d\n", self->xCorrelationFlag);
    fprintf(Stream, "hdfOutputFlag          : %d\n", self->hdfOutputFlag);
    fprintf(Stream, "correctDegradationFlag : %d\n", 
	    self->correctDegradationFlag);
    fprintf(Stream, "correct_BSDF_Flag : %d\n", 
	    self->correct_BSDF_Flag);
    fprintf(Stream, "splitFlag              : %d\n", self->splitFlag);
    fprintf(Stream, "noSunFlag              : %d\n", self->noSunFlag);
    fprintf(Stream, "sunFlag                : %d\n", self->sunFlag);
    fprintf(Stream, "moonFlag               : %d\n", self->moonFlag);
    fprintf(Stream, "infoFlag               : %d\n", self->infoFlag);
    fprintf(Stream, "Geol. infoFlag         : %d\n", self->geolocationInfoFlag);
    fprintf(Stream, "QA infoFlag            : %d\n", self->qaInfoFlag);
    fprintf(Stream, "Top left coord         : %f %f\n",
	    self->cornerCoordinate[0].Latitude,
	    self->cornerCoordinate[0].Longitude);
    fprintf(Stream, "Bottom right coord     : %f %f\n",
	    self->cornerCoordinate[1].Latitude,
	    self->cornerCoordinate[1].Longitude);
    fprintf(Stream, "pixelStop              : %d\n", self->pixelStop);
    fprintf(Stream, "pixelStart             : %d\n", self->pixelStart);
    fprintf(Stream, "pixelStop              : %d\n", self->pixelStop);
    fprintf(Stream, "timeStart              : %d, %d\n",
	    self->timeStart.Days, self->timeStart.Msecs);
    fprintf(Stream, "timeStop               : %d, %d\n",
	    self->timeStop.Days, self->timeStop.Msecs);
    if (self->inputFilename != NULL) 
    {
	fprintf( Stream, "inputFilename      : %s\n", self->inputFilename);
    }
    if (self->outputFilename != NULL) 
    {
	fprintf( Stream, "outputFilename     : %s\n", self->outputFilename);
    }
    fprintf(Stream, "bandself         :");
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) 
    {
	fprintf( Stream, "%d, ", self->bandFilter[ i]);
    }
    fprintf(Stream, "\n");
    fprintf(Stream, "calibrationFilter         :");
    for (i = 0; i < CALIBRATION_DEFINED; i++) 
    {
	fprintf(Stream, "%d, ", self->calibrationFilter[ i]);
    }
    for (i=0; i<SUBSET_COUNTER_DEFINED; i++) 
    {
	fprintf(Stream, "%d, ", self->subsetCounterFilter[i]);
    }
    fprintf(Stream, "\n");
}


/*----------------------------------------------------------------------------*\
**                                Filter__writeDLR_Header
**  Input parameters:
**    stream: output stream
**  Output parameters:
**    Filter__writeDLR_Header	SUCCESS	= output ok
**				else	= any error occured
**  Other interfaces:
**    Extract Level 1 Product file specified by stream
**  Description:
**    This function prints out the DLR Header
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    15.11.94
**  Author:     Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Filter__writeDLR_Header(FILE* stream)
{
    char SoftwareVersion[VERSION_LENGTH];
    sprintf(SoftwareVersion, VERSION_STRING_FORMAT, SOFTWARE_VERSION);
    fprintf(stream, "/*---------------------------------------------------------------------------*\\\n");
    fprintf(stream, "**  GDP Level 0-to-1 Extracting * Version %s", SoftwareVersion);
    fprintf(stream, " * Copyright  DLR 1996-2002  **\n");
    fprintf(stream, "\\*---------------------------------------------------------------------------*/\n");
    if (ferror(stream) != 0) 
    {
	return FATAL;
    }
    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                ExtractHeader__Write
**  Input parameters:
**    self:
**    stream: output stream
**
**  Output parameters:
**    ExtractHeader__Write	SUCCESS	= output ok
**				else	= any error occured
**  Other interfaces:
**    Extract Level 1 Product file specified by stream
**
**  Description:
**    This function prints out a product header according the given ProductType
**    and the given CalibrationFilter.
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    15.11.94
**  Author:     Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Filter__writeExtractHeader(Filter_s* self, FILE* stream, 
			   ProductType_e product_type)
{
    float degradation_version;

    Filter__writeDLR_Header(stream);
    fprintf(stream, "Calibrations Applied\n");
    if (self->calibrationFilter[CAL_LEAKAGE])
    {
	fprintf(stream, "Leakage ");
    }
    if (self->calibrationFilter[CAL_FPA]) 
    {
	fprintf(stream, "FPA ");
    }
    if (self->calibrationFilter[CAL_FIXED]) 
    {
	fprintf(stream, "Fixed ");
    }
    if (self->calibrationFilter[CAL_STRAYLIGHT]) 
    {
	fprintf(stream, "Straylight ");
    }
    if (self->calibrationFilter[CAL_NORMALIZE]) 
    {
	fprintf(stream, "Normalize ");
    }
    if (self->calibrationFilter[CAL_POLARIZATION]) 
    {
	fprintf(stream, "Polarization ");
    }
    if (self->calibrationFilter[CAL_BSDF]) 
    {
	fprintf(stream, "BSDF ");
    }
    if (self->calibrationFilter[CAL_INTENSITY]) 
    {
	fprintf(stream, "Intensity ");
    }
    if (self->calibrationFilter[CAL_UNITS]) 
    {
	fprintf(stream, "Unit_Conversion ");
    }
    if (self->radianceJumpFlag) 
    {
	fprintf(stream, "Radiance_Jump_Correction ");
    }
    if (self->xCorrelationFlag) 
    {
	fprintf(stream, "Cross-Correlation ");
    }
    if (self->correctDegradationFlag == True)
    {
	DegradationParameter__GetVersion(&degradation_version);
	fprintf(stream, "Degradation_%-5.2f ", degradation_version);
    }
    if (self->correct_BSDF_Flag == True)
    {
	DegradationParameter__GetVersion(&degradation_version);
	fprintf(stream, "Asym_BSDF_%-5.2f ", degradation_version);
    }

    fprintf(stream, "\nUnits\n");
    if (self->calibrationFilter[CAL_LEAKAGE] &&
	self->calibrationFilter[CAL_FIXED] &&
	self->calibrationFilter[CAL_STRAYLIGHT] &&
	self->calibrationFilter[CAL_NORMALIZE] &&
	(((product_type == PR_EARTHSHINE) 
	  && self->calibrationFilter[CAL_POLARIZATION]) ||
	 ((product_type == PR_SUNSHINE)
	  && self->calibrationFilter[CAL_BSDF]) ||
	 ((product_type != PR_EARTHSHINE) 
	  && (product_type != PR_SUNSHINE))) 
	&& self->calibrationFilter[CAL_INTENSITY]) 
    {
	if (self->albedoFlag) 
	{
	    fprintf(stream, "Wavelength [nm], Irradiance [mW/m^2.nm], "
		    "Albedo [-]\n");
	}
	else if (self->calibrationFilter[CAL_UNITS])	
	{
	    fprintf(stream, "%s\n", PhotonUnits__Get());
	}
	else 
	{
	    fprintf(stream, "%s\n", WattUnits__Get());
	}
    }
    else 
    {
	fprintf(stream, "Wavelength [-], Irradiance [-], Radiance [-]\n");
    }

/*     if (self->hdfOutputFlag)  */
/*     { */
/* 	fprintf(stream, "Output format is HDF\n"); */
/*     } */
/*     else */
/*     { */
/* 	fprintf(stream, "Output format is ASCII\n"); */
/*     } */

    if (ferror(stream) != 0) 
    {
	return FATAL;
    }
    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                            Filter__getProductType
**  Input parameters:
**    Filter		filter of input parameter
**
**  Output parameters:
**    ProductType	extract product type 
**    GetProductType	SUCCESS if valid product type
**			else invalid product type
**
**  Other interfaces:
**
**  Description:
**    Function to get the extracted product type
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
Filter__getProductType(const Filter_s *self, ProductType_e *ProductType)
{
    FunctionStatus_e Status = SUCCESS;

    /*--------------------------------------------------------------------*\
    ** define the product Type
    \*--------------------------------------------------------------------*/
    if (!(self->infoFlag || self->geolocationInfoFlag) && !self->sunFlag && !self->moonFlag) {
	*ProductType = PR_EARTHSHINE;
    }
    else if (self->sunFlag) {
	*ProductType = PR_SUNSHINE;
    }
    else if (self->moonFlag) {
	*ProductType = PR_MOONSHINE;
    }
    else if (self->qaInfoFlag) {
	*ProductType = PR_INFO_QA;
    }
    else if (self->geolocationInfoFlag) {
	*ProductType = PR_INFO_GEOLOCATION;
    }
    else if ((self->infoFlag || self->geolocationInfoFlag)) {
	*ProductType = PR_INFO;
    }
    else {
	*ProductType = PR_OTHER;
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.17__1015", Status,
		  LOG_NUM, "program error illegal product type", "");
	return Status;
    }

    return Status;
}



/*============================================================================*\
**  private operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                          Filter__setYesNoPreferences
**  Input parameters:
**    N_Pref: number of elements to set
**    Arg: a string with the filter to set
**  Output parameters:
**    Preference: array containing the set parameters
**  Other interfaces:
**    <none>
**  Description:
**    This function checks the band filter argument of commandline option -b.
**    It must be a string with SCIENCE_DATA_DEFINED number of characters,
**    each of them containing a 'y' or 'n'
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    15.11.94
**  Author:     Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
Bool 
Filter__setYesNoPreferences(Bool Preference[], short N_Pref, const char *Arg)
{
    int i, ArgLength = strlen(Arg);

    if (ArgLength > N_Pref) {
	return False;
    }

    for (i=0; i<N_Pref; i++) {
	Preference[i] = False;
    }
    for (i=0; i<ArgLength; i++)	{
	switch (toupper(Arg[i])) {
	case 'Y':
	    Preference[i] = True;
	    break;
	case 'N':
	    break;
	default:
	    return False;
	}
    }
    return True;
}


/*----------------------------------------------------------------------------*\
**                                Filter__setCalibration
**  Input parameters:
**    Arg: a string with the filter to check
**  Output parameters:
**    Filter: a structure containing the filter parameters
**  Other interfaces:
**    <none>
**  Description:
**    This function checks the band filter argument. It must be a string with
**    CALIBRATION_DEFINED characters, each of them containing a 'y' or 'n'
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    24.5.95
**  Author:     Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
Bool
Filter__setCalibration(Filter_s *Filter, const char *Arg, 
		       ProductType_e ProductType)
{
    int i, ArgLength = strlen(Arg);

    for (i=0; i < CALIBRATION_DEFINED; i++)
    {
	Filter->calibrationFilter[ i] = False;
    }

    for (i=0; i<ArgLength; i++)
    {
	switch (Arg[i])
	{
	case 'l':
	case 'L':
	    Filter->calibrationFilter[CAL_LEAKAGE] = True;
	    break;
	case 'a':
	case 'A':
	    Filter->calibrationFilter[CAL_FPA] = True;
	    break;
	case 'f':
	case 'F':
	    Filter->calibrationFilter[CAL_FIXED] = True;
	    break;
	case 's':
	case 'S':
	    Filter->calibrationFilter[CAL_STRAYLIGHT] = True;
	    break;
	case 'n':
	case 'N':
	    Filter->calibrationFilter[CAL_NORMALIZE] = True;
	    break;
	case 'p':
	case 'P':
	    if (ProductType == PR_EARTHSHINE)
	    {
		Filter->calibrationFilter[CAL_POLARIZATION] = True;
	    }
	    else
	    {
		return False;
	    }
	    break;
	case 'i':
	case 'I':
	    Filter->calibrationFilter[CAL_INTENSITY] = True;
	    break;
	case 'b':
	case 'B':
	    if (ProductType == PR_SUNSHINE)
	    {
		Filter->calibrationFilter[CAL_BSDF] = True;
	    }
	    else
	    {
		return False;
	    }
	    break;
	case 'u':
	case 'U':
	    Filter->calibrationFilter[CAL_UNITS] = True;
	    break;
	default:
	    return False;
	}
    }
    return True;
}


/*----------------------------------------------------------------------------*\
**                                Filter__setDefaultValues
**  Input parameters:
**    <none>
**  Output parameters:
**    Filter: a structure containing the default filter parameters
**  Other interfaces:
**    <none>
**  Description:
**    Set default values in Filter
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    14.2.96
**  Author:     Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void 
Filter__setDefaultValues(Filter_s* self, ProductType_e ProductType)
{
    short i;

    self->pixelFlag = False;
    self->timeFlag = False;
    self->regionFlag = False;
    self->calFileFlag = False;
    self->albedoFlag = False;
    self->radianceJumpFlag = False;
    self->xCorrelationFlag = True;
    self->hdfOutputFlag = False;
    self->correctDegradationFlag = False;
    self->correct_BSDF_Flag = False;
    self->splitFlag = False;
    self->noSunFlag = False;
    self->infoFlag = False;
    self->geolocationInfoFlag = False;
    self->qaInfoFlag = False;
    self->sunFlag = False;
    self->moonFlag = False;
    self->pixelStart = 0;
    self->pixelStop = INT_MAX;
    for (i=0; i < CALIBRATION_DEFINED; i++)
    {
	self->calibrationFilter[i] = True;
    }
    switch (ProductType)
    {
    case PR_EARTHSHINE:
	self->calibrationFilter[CAL_BSDF] = False;
	break;
    case PR_SUNSHINE:
	self->calibrationFilter[CAL_POLARIZATION] = False;
	self->calibrationFilter[CAL_FPA] = False;
	self->calibrationFilter[CAL_UNITS] = False;
	break;
    case PR_MOONSHINE:
	self->calibrationFilter[CAL_POLARIZATION] = False;
	self->calibrationFilter[CAL_FPA] = False;
	self->calibrationFilter[CAL_BSDF] = False;
	break;
    }

    for (i=0; i<BAND_DEFINED; i++) 
    {
	self->bandFilter[i] = True;
    }
    for (; i<SCIENCE_DATA_DEFINED; i++)
    {
	self->bandFilter[i] = False;
    }
    for (i=0; i<SUBSET_COUNTER_DEFINED; i++) {
	self->subsetCounterFilter[i] = True;
    }
    self->inputFilename = NULL;
    self->outputFilename = NULL;

    self->qa_increment = QA_increment;
    self->qa_pixelnumber = QA_PixelNumber;
    self->qa_meannumber = QA_MeanNumber;
}
 

/*----------------------------------------------------------------------------*\
**                                printUsage
**  Input parameters:
**    ProgramName: the name of the program being called
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    This function prints out a program usage message
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    15.11.94
**  Author:     Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void 
Filter__printUsage(const char *program_name)
{
    fprintf(stderr, "\nUsage:\n");
    fprintf(stderr, "%s\n\t-i [-g] [-q] input_file\nor\n", program_name);

    fprintf(stderr, 
	    "%s\n\t[-b b_filter] [-p p1 p2 | -t t1 t2] [-r lat long lat long]\n"
	    "\t[-x  x_filter] [-c c_filter]\n"
	    "\t[-a] [-j] [-d] [-w] [-n] [-k]\n"
	    "\t[-e degradation_par_filename]\n"
	    "\t[-f degradation_par_filename]\n"
	    "\tinput_file output_file\n"
	    "or\n", program_name);

    fprintf(stderr, 
	    "%s\n\t-s [-b b_filter] [-p p1 p2 | -t t1 t2] [-r lat long lat long]\n"
	    "\t[-x  x_filter] [-c c_filter]\n"
	    "\t[-w] [-n] [-k]\n"
	    "\t[-e degradation_par_filename]\n"
	    "\t[-f degradation_par_filename]\n"
	    "\tinput_file output_file\n"
	    "or\n", program_name);

    fprintf(stderr, 
	    "%s\n\t-m [-b b_filter] [-p p1 p2 | -t t1 t2] [-r lat long lat long]\n"
	    "\t[-x  x_filter] [-c c_filter]\n"
	    "\t[-w] [-n] [-k]\n"
	    "\t[-e degradation_par_filename]\n"
	    "\t[-f degradation_par_filename]\n"
	    "\tinput_file output_file\n"
	    "\n", program_name);

    fprintf(stderr, "\nwhere:\n");
    fprintf(stderr, "i display general product information\n");
    fprintf(stderr, "g display geolocation information\n");
    fprintf(stderr, "q display selected information\n");
    fprintf(stderr, "b extract bands selected (b_filter = \"BBBBBBBBBB\" "
	    "one \"B\" for band 1A, 1B,\n"
	    "  2A, 2B, 3, 4, blind 1a, straylight 1a, 1b and 2a, "
	    "where B='y' or B='n')\n");
    fprintf(stderr, "p extract ground pixels between p1 and p2 "
	    "(from 1 to number of pixels)\n");
    fprintf(stderr, "r extract ground pixels between corners upper left\n"
	    "  and lower right (lat long in degrees)\n");
    fprintf(stderr, "t extract ground pixels between t1 and t2 "
	    "(format DD-MMM-YYYY_HH:MM:SS.m)\n");
    fprintf(stderr, "c apply the calibration parameters selected "
	    "(c_filter = \"LAFSNPIU\" for\n"
	    "  earthshine, \"LAFSNBIU\" for sun packages or \"LAFSNIU\" "
	    "for moon packages;\n"
	    "  where: L=Leakage, A=FPA, F=Fixed, S=Straylight, N=Normalize, "
	    "B=BSDF,\n"
	    "  P=Polarization, I=Intensity, U=Unit_Conversion)\n");
    fprintf(stderr, "x extract specific ground pixels (x_filter = \"XXXX\" "
	    "one \"X\" for east,\n"
	    "  center, west and back scan pixel, where X='y' or X='n')\n");
    fprintf(stderr, "a compute albedos instead of radiances\n");
    fprintf(stderr, "j correct radiance jumps\n");
    fprintf(stderr, "d divide (split) output creating one file for each ground pixel\n"
            "  containing the sun- and earthshine-spectrum\n");
    fprintf(stderr, "n divide (split) output creating one file without sun\n"
	    "  spectrum for each ground pixel plus only one file with sun spectrum\n");
    fprintf(stderr, "w write the calibration data records (FCD and PCDs) "
	    "in a file\n");
    fprintf(stderr, "s extract sun packages\n");
    fprintf(stderr, "m extract moon packages\n");
    fprintf(stderr, "e instrument degradation plus BSDF asymetric azimuth dependancy correction\n");
    fprintf(stderr, "f BSDF asymetric azimuth dependancy correction\n");
    fprintf(stderr, "k switch off X-correlation (default is on)\n");
    /* fprintf(stderr, "h write output in HDF format (default is ASCII)\n\n"); */
}
