/*----------------------------------------------------------------------------*\
**                              fit_spli.c
**
**  GDP Level 0 to 1 C subroutine of ExtractLevel_1_Data
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the following methods:
**    fit_grid_spline
**    spline1
**    splint1
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)fit_spli.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"
#include "log.h"


#define	SS_ID	("fit_grid_spline")


/*----------------------------------------------------------------------------*\
**                                fit_grid_spline
**  Input parameters:
**    x_axis of the given spectrum, x_axis_in: float array []
**    values of the given spectrum, y_axis_in: float array []
**    number of values of the given spectrum, n_in: short, mostly DETECTOR_PIXELS
**  Output parameters:
**    x_axis of the desired spectrum: float array [TOTAL_DETECTOR_PIXELS]
**    values of the spectrum fitted to the desired spectrum: float array [DETECTOR_PIXELS]
**    number of values of the desired spectrum, from Begin to End, normally 0 to DETECTOR_PIXELS-1
**  Other interfaces:
**  Description:
**    interpolates the values of a given spectrum to another wavelength grid, using "spline"
**  References:
**    This function is described in the Module ...
**  Libraries:
**    numrecip
**  Created:	07.4.94
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void fit_grid_spline (const float xin[], const float yin[], int nin, const float xres[], float yres[], int Begin, int End)
{
	int loop, count;

	FunctionStatus_e Status;	
	char ErrorString [80];

	double result;
	double Akima_Y[AKIMA_EXTRA_POINTS+DETECTOR_PIXELS];
	double Akima_X[AKIMA_EXTRA_POINTS+DETECTOR_PIXELS];
	double Akima_A[DETECTOR_PIXELS];

	for (loop = 0; loop < nin; loop++)
	{
		Akima_X [loop] = xin [loop];
		Akima_Y [loop] = yin [loop];
	}


	/*----------------------------------------------------------------------------*\
	** Spline
	\*----------------------------------------------------------------------------*/
	spline1 (Akima_X, Akima_Y, nin, 0.0, 0.0, Akima_A);

	/*----------------------------------------------------------------------------*\
	** Splint
	\*----------------------------------------------------------------------------*/
	count = 0;

	for (loop = Begin; loop <= End; loop++)
	{
		splint1 (Akima_X, Akima_Y, Akima_A, nin, (double)xres[loop], &result, &Status);

		if (Status == ERROR)
			count++;
		else
			yres[loop] = (float) result;
	}

	if (count > 0)
	{
		sprintf(ErrorString, "%d times X_high and X_low were the same in splint", count);
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "3.1_55", ERROR, LOG_NUMRECIP, ErrorString, "");
	}
}

/*----------------------------------------------------------------------------*\
**                                spline
**  Input parameters:
**    Vector x[1..n], y[1..n]
**    double yp1, ypn
**    int n
**  Output parameters:
**    Vector y2[1..n]
**  Other interfaces: none
**  Description:
**    Given arrays x[1..n] and y[1..n] containing a tabulated function,
**    y=f(x), with x1<x2<..<xn, and given values yp1 and ypn for the
**    first derivative of the interpolating function at points 1 and n,
**    respectively, this routine returns and array y2[1..n] that
**    contains the second derivatives of the interpolating function at
**    the tabulated points xi. If yp1 and/or ypn are larger than
**    NATURAL_SPLINE (0.99e30), the routine is signalled to set the
**    corresponding boundary condition for a natural spline, with zero
**    second derivative on that boundary
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    28.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
**  Changed by: Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void spline1( double x[], double y[], short n, double yp1, double ypn, double y2[])
{
	short i,k;
	double p,qn,sig,un;
	double u[DETECTOR_PIXELS];

	if (yp1 > 0.99e30)
		y2[0]=u[0]=0.0;
	else {
		y2[0] = -0.5;
		u[0]=(3.0/(x[1]-x[0]))*((y[1]-y[0])/(x[1]-x[0])-yp1);
	}
	for (i=1;i<=n-2;i++) {
		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
		p=sig*y2[i-1]+2.0;
		y2[i]=(sig-1.0)/p;
		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}
	if (ypn > 0.99e30)
		qn=un=0.0;
	else {
		qn=0.5;
		un=(3.0/(x[n-1]-x[n-2]))*(ypn-(y[n-1]-y[n-2])/(x[n-1]-x[n-2]));
	}
	y2[n-1]=(un-qn*u[n-2])/(qn*y2[n-2]+1.0);

	for (k=n-2;k>=0;k--)
		y2[k]=y2[k]*y2[k+1]+u[k];
}


/*----------------------------------------------------------------------------*\
**                                splint
**  Input parameters:
**    vector xa[1..n], ya[1..n], y2a[1..n]
**    short n
**    double x
**  Output parameters:
**    double *y
**    Status: tells if there are some problems during the computation
**  Other interfaces: none
**  Description:
**    Given arrays xa[1..n] and ya[1..n] wich tabulate a function and
**    given the array y2a[1..n], wich is the output from spline above,
**    and given a value x, this routine returns a cubic-spline
**    interpolated value y.
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    11.2.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
**  Changed by: Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void splint1(double xa[],double ya[],double y2a[], short n,double x,double *y, FunctionStatus_e * Status)
{
	short klo,khi,k;
	double h,b,a;
	void nrerror();

	*Status = SUCCESS;

	klo=0;
	khi=n-1;
	while (khi-klo > 1) 
	{
		k=(khi+klo) >> 1;
		if (xa[k] > x) khi=k;
		else klo=k;
	}

	h=xa[khi]-xa[klo];
	if (h == 0.0) {
		*Status = ERROR;
		return;
	}

	a = (xa[khi] - x) / h;
	b = (x - xa[klo]) / h;

	*y = a * ya[klo] + b * ya[khi] + ( (a*a*a - a) * y2a[klo] + (b*b*b - b) * y2a[khi]) * (h*h) / 6.0;
}
