/*----------------------------------------------------------------------------*\
**                                FixedC.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)FixedC.c	5.1 02/26/99, Copyright DLR 1998"

#include "fixedc.h"
#include "log.h"
#include "dataconv.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)FixedC.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("Fixed")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                FixedC__Apply
**  Input parameters:
**    Fixed Pattern (Self) (1024)
**    Integration time
**    Band (1A .. 4)
**    Reticon data (1024)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (ASCII string)
**  Output parameters:
**    Corrected Reticon data
**  Other interfaces:
**  Description:
**    Corrects Reticon by multiplying the Fixed Pattern
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	21.7.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void 
FixedC__Apply(const FixedC_s *Self, ScienceData_e Band, float *Reticon,
	      short Begin, short End, const char *PixelDate)
{

    int loop;
    OpticalChannel_e Ch=Band2Channel(Band);

#ifdef DEBUG
    FILE *out;
    float InputReticon [DETECTOR_PIXELS];
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_FIXED, DBG_LEVEL_1))
    {	
	for (loop = Begin; loop <= End; loop++)
	    InputReticon [loop] = Reticon [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** Correct each detector pixel for Pixel-to-pixel-gain
    \*-----------------------------------------------------------------------*/

    for (loop = Begin; loop <= End; loop++)
	Reticon[loop] *= ( Self->Pattern[Ch][loop] );	/* this is the basic line in this module ! */

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("FixedC__Apply", PixelDate) );
    if (Debug__isActive(debug_ptr, DBG_FIXED, DBG_LEVEL_1))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\n"); 
		fprintf(out, 
			"Pixel      Band       InputRetic Pattern    OutputRet  \n");
		fprintf(out, 
			"[]         []         []         []         []\n\n");
	    }
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g\n", 
			loop + (Ch * DETECTOR_PIXELS), Band, 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Self->Pattern[Ch][loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon [loop]);
	    }		
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_104", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif
}



/*----------------------------------------------------------------------------*\
**                           FixedC__Sun2Host
**  Input parameters:
**    Fixed		in Sun memory representation
**  Output parameters:
**    Fixed		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the Fixed struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FixedC__Sun2Host( FixedC_s *Param)
{
    int	i, j;

    for ( j = 0; j < OPTICAL_CHANNELS; j++)
	for ( i = 0; i < DETECTOR_PIXELS; i++)
	{
	    Param->Pattern[ j][ i] = SunFloat2Float( (unsigned char*)
						     &Param->Pattern[ j][ i]);

	}
}

/*----------------------------------------------------------------------------*\
**                              FixedC__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		read Fixed calibration parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all Fixed calibration parameter from the given 
**	input stream into the Fixed structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FixedC__read(FixedC_s *Param, FILE* Stream, Bool CONVERT_SUN_2_HOST)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];
    /*--------------------------------------------------------------------*\
    ** read Pixel to Pixel Gain
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->Pattern, 1, sizeof( Param->Pattern),
			Stream);
    /*--------------------------------------------------------------------*\
    **  check 
    \*--------------------------------------------------------------------*/
    if (NoOfBytes != FIXED_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "Fixed Calibration Parameter, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__197", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (CONVERT_SUN_2_HOST)
    {
	FixedC__Sun2Host( Param);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           FixedC__Show
**  Input parameters:
**	Stream		output stream
**	Param		Fixed parameter structure
**
**  Output parameters:
**
**  Other interfaces:
**	Fixed parameter structure written to the output stream
**
**  Description:
**	Function to write the fixed parameter structure to the output stream in
**	ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FixedC__show(const FixedC_s *Param, FILE* Stream)
{
    int	i, j;

    /*--------------------------------------------------------------------*\
    **  show fixed pattern parameter
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Pixel-to-Pixel Gain\n");

    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%4d", i);
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%f", Param->Pattern[ j][ i]);
	fprintf( Stream, "\n");
    }
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
