/*----------------------------------------------------------------------------*\
**                                FPA.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)FPA.c	5.2 08/19/02, Copyright DLR 1998"

#include <stdlib.h>
#include "fpa.h"
#include "log.h"
#include "dataconv.h"
#include "numrecip.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)FPA.c 5.2 08/19/02, Copyright DLR 1998")


#define	SS_ID	("FPA")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                FPA__Compute
**  Input parameters:
**	PeltierData	The state of the peltier cooler, indicating the cooling
**			activity
**
**	PeltierDataSize	the number of PeltierData values
**
**	Self->FilterCoefficient		The filter, which will be applied to the
**					PeltierData array (FPA_s is defined in defs.h)
**
**	Self->CoefficientsUsed		The number of filter coefficients
**
**  Output parameters:
**
**	Self->Noise	The peltier noise, derived from PeltierData
**
**  Other interfaces:
**  Description:
**
**	Calculate FPA Noise, which is an offset to the detector signals
**	generated by switching the peltier cooler.
**
**  References:
**  Libraries:
**  Created:	13.02.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void FPA__Compute(FPA_s *Self, float PeltierData[], short PeltierDataSize)
{
    short loop;
    short border_tolerance;

#ifdef DEBUG
    FILE *out;
    char debug_filename[80];
#endif

    /*-----------------------------------------------------------------------*\
    ** check if enough values to filter are there
    \*-----------------------------------------------------------------------*/
    if (PeltierDataSize <= Self->CoefficientsUsed)
    {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.2_67", WARN, 
		 LOG_CANNOT_CALCULATE, 
		 "- Not enough FPA noise values: less than filter coeffs.", "");
	Self->NoiseSize = 0;
	Self->Noise = NULL;
	return;
    }

    /*-----------------------------------------------------------------------*\
    ** Set NoiseSize=PeltierDataSize, allocate and compute FPA Noise
    \*-----------------------------------------------------------------------*/
    Self->NoiseSize = PeltierDataSize;

#ifdef __cplusplus
    Self->Noise = new float [PeltierDataSize];
#else
    Self->Noise = (float *) malloc( sizeof( float) * PeltierDataSize);
#endif

    if (Self->Noise == NULL)
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.2_85", FATAL, LOG_CANNOT_ALLOCATE, "- Couldn't allocate memory for Peltier noise", "");

    /*-----------------------------------------------------------------------*\
    ** convolute data and FilterCoefficient
    \*-----------------------------------------------------------------------*/
    convol(PeltierData, Self->Noise, 0, PeltierDataSize-1, Self->FilterCoefficient, Self->CoefficientsUsed);

    /*-----------------------------------------------------------------------*\
    ** extrapolate border regions because filter kernel (60 values) is too big
    \*-----------------------------------------------------------------------*/
    border_tolerance = Self->CoefficientsUsed/2;

    for (loop = 0; loop < border_tolerance; loop++)
	Self->Noise[loop] = LinearInterpolation(loop, border_tolerance, border_tolerance + 2, 
						Self->Noise[border_tolerance], Self->Noise[border_tolerance + 2]);

    for (loop = PeltierDataSize - border_tolerance; loop < PeltierDataSize; loop++)
	Self->Noise[loop] = LinearInterpolation(loop, (PeltierDataSize - border_tolerance) - 2, PeltierDataSize - border_tolerance, 
						Self->Noise[(PeltierDataSize - border_tolerance) - 2], Self->Noise[PeltierDataSize - border_tolerance]);

    /*-----------------------------------------------------------------------*\
    ** in Self->Noise is now the low-pass-filtered PeltierData
    ** So subtract Self->Noise from PeltierData to get the noise
    ** (high-pass-filtered PeltierData)
    \*-----------------------------------------------------------------------*/
    for (loop = 0; loop < PeltierDataSize; loop++)
	Self->Noise[loop] = PeltierData[loop] - Self->Noise[loop];

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", "FPA__Compute.dbg");

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "w")) != NULL)
	{
	    fprintf(out, "\n"); 
	    fprintf(out, "Number     FilterCoefficient \n");
	    fprintf(out, "[]         []                \n\n");

	    for (loop = 0; loop < Self->CoefficientsUsed; loop++)
	    {
		fprintf(out, "%10d %*.*g \n", loop, DEBUG_WIDTH, 
			DEBUG_PRECISION, Self->FilterCoefficient[loop]);
	    }

	    fprintf(out, "\n"); 
	    fprintf(out, "Pixel      Pelt_Data  Pelt_Noise Pelt_filtered \n");
	    fprintf(out, "[]         []         []         []            \n\n");

	    for (loop = 0; loop < PeltierDataSize; loop++)
	    {
		fprintf(out, "%10d %*.*g %*.*g %*.*g\n", loop, 
			DEBUG_WIDTH, DEBUG_PRECISION, PeltierData [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Self->Noise[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, 
			PeltierData [loop] - Self->Noise[loop]);
	    }
		
		
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.2_153", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif
}


/*----------------------------------------------------------------------------*\
**                                FPA__Apply
**  Input parameters:
**	Self	A structure (defined in defs.h) containing 
**			peltier noise, calculated in FPA__Compute
**			ScaleFactor, applied to the noise according integration time
**
**	GroundPixel	The number of the current ground pixel
**
**	IntgTime
**
**	Reticon		signals which are to be corrected
**
**	Begin, End	of the band in the Reticon array
**
**	PixelDate	in ASCII
**
**  Output parameters:
**
**	Reticon		but now corrected
**
**  Other interfaces:
**  Description:
**
**	Corrects reticon signals for the noise, created by the peltier cooler switching
**
**	Steps :
**		- Calculate the number of ground pixels during integration
**		- build a mean value of PeltierNoise over this integration period
**		- subtract this mean noise from the reticon signal
**
**  References:
**  Libraries:
**  Created:	13.02.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FPA__Apply(const FPA_s *Self, short GroundPixel, float IntgTime, 
	   float Reticon[], short Begin, short End, const char *PixelDate)
{
    short loop;
    short number;
    short count;
    float ScaleFactor;
    float MeanNoise;

#ifdef DEBUG
    FILE *out;
    float InputReticon [DETECTOR_PIXELS];
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_3))
    {	
	for (loop = Begin; loop <= End; loop++)
	    InputReticon [loop] = Reticon [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** check if Peltier noise was calculated
    \*-----------------------------------------------------------------------*/
    if (Self->NoiseSize <= GroundPixel || Self->Noise == NULL)
    {
	Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.2_221", WARN, LOG_CANNOT_CALCULATE,
		 "- No FPA noise value for this ground pixel !", "");
	return;
    }

    /*-----------------------------------------------------------------------*\
    ** find the ScaleFactor according IntgTime
    \*-----------------------------------------------------------------------*/
    switch ((int) IntgTime)
    {
    case 12: ScaleFactor = Self->ScaleFactor[FPA_SCALE_12]; break;
    case 24: ScaleFactor = Self->ScaleFactor[FPA_SCALE_24]; break;
    case 30: ScaleFactor = Self->ScaleFactor[FPA_SCALE_30]; break;
    case 48: ScaleFactor = Self->ScaleFactor[FPA_SCALE_48]; break;
    case 60: ScaleFactor = Self->ScaleFactor[FPA_SCALE_60]; break;
    default: return;	/* no valid IntgTime given */
    }


    /*-----------------------------------------------------------------------*\
    ** calculate MeanNoise from all Noise values during IntgTime
    \*-----------------------------------------------------------------------*/
    number = (short)(IntgTime / GROUND_PIXEL_TIME);
    MeanNoise = 0.0;
    count = 0;

    for (loop = 0; loop < number && (GroundPixel - loop) >= 0; loop++)
    {
	MeanNoise += Self->Noise[GroundPixel - loop];
	count++;
    }
	
    if (count > 0) 
    {
	/* calculate MeanNoise */
	MeanNoise = MeanNoise * ScaleFactor / count;

	/* subtract Self->Noise from Reticon */
	for (loop = Begin; loop <= End; loop++) Reticon[loop] -= MeanNoise;
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", "FPA__Apply_summary.dbg" );

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_2))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    fprintf(out, "%10.5f %10.5f %s\n", ScaleFactor, MeanNoise, 
		    PixelDate); 
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "3.13_120", WARN,
		     LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
    sprintf(debug_filename, "%s", DbgFileName("FPA__Apply", PixelDate) );

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    fprintf(out, "\n"); 
	    fprintf(out, "Pixel      InputRetic PeltNoise  OutputRet  \n");
	    fprintf(out, "[]         []         []         []\n\n");

	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %*.*g %*.*g %*.*g\n", loop, 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, MeanNoise, 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon [loop]);
	    }		
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "3.13_103", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif
}


/*----------------------------------------------------------------------------*\
**                           FPA__Sun2Host
**  Input parameters:
**    FPA		in Sun memory representation
**  Output parameters:
**    FPA		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the FPA struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FPA__Sun2Host( FPA_s *Param)
{
    int	i;

    for ( i = 0; i < FPA_SCALE_FACTORS; i++)
	/* float */
	Param->ScaleFactor[ i] = SunFloat2Float( (unsigned char*)
						 &Param->ScaleFactor[ i]);
    /* short */
    Param->CoefficientsUsed = SunShort2Short( (unsigned char*)
					      &Param->CoefficientsUsed);
    for ( i = 0; i < FPA_FILTER_COEFFICIENTS; i++)
	/* float */
	Param->FilterCoefficient[ i] = SunFloat2Float( (unsigned char*)
						       &Param->FilterCoefficient[ i]);
}

/*----------------------------------------------------------------------------*\
**                              FPA__Read
**  Input parameters:
**	Stream		input stream to read polarization
**  Output parameters:
**	Param		read FPA parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all FPA parameter entries from the given 
**	input stream into the FPA structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FPA__read(FPA_s *Param, FILE* Stream, Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;  
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** read FPA ScaleFactor
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->ScaleFactor, 1, sizeof( Param->ScaleFactor),
			Stream);

    /*--------------------------------------------------------------------*\
    **  read FPA CoefficientsUsed
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->CoefficientsUsed, 1,
			sizeof( Param->CoefficientsUsed), Stream);

    /*--------------------------------------------------------------------*\
    ** read FPA ScaleFactor
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->FilterCoefficient, 1,
			sizeof( Param->FilterCoefficient), Stream);

    /*--------------------------------------------------------------------*\
    **  check 
    \*--------------------------------------------------------------------*/
    if (NoOfBytes != FPA_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "FPA Parameter, #read bytes: %d", NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__424", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	FPA__Sun2Host( Param);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                              FPA__Write
**  Input parameters:
**	Stream		input stream 
**	Param		FPA parameter
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write all FPA parameter entries from the given 
**	output stream from the FPA structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
FPA__write(const FPA_s *Param, FILE* Stream)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;  
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** write FPA ScaleFactor
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( Param->ScaleFactor, 1, sizeof( Param->ScaleFactor),
			 Stream);

    /*--------------------------------------------------------------------*\
    **  write FPA CoefficientsUsed
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( &Param->CoefficientsUsed, 1,
			 sizeof( Param->CoefficientsUsed), Stream);

    /*--------------------------------------------------------------------*\
    ** write FPA ScaleFactor
    \*--------------------------------------------------------------------*/
    NoOfBytes += fwrite( Param->FilterCoefficient, 1,
			 sizeof( Param->FilterCoefficient), Stream);

    /*--------------------------------------------------------------------*\
    **  check 
    \*--------------------------------------------------------------------*/
    if (NoOfBytes != FPA_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "FPA Parameter, #write bytes: %d", NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__492", Status, LOG_CANNOT_WRITE,
		  LogBuffer, "");

    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           FPA__Show
**  Input parameters:
**	Stream		output stream
**	Param		polarization structure
**
**  Output parameters:
**
**  Other interfaces:
**	polarization structure written to the output stream
**
**  Description:
**	Function to write the polarization structure to the output stream in
**	ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
FPA__show(const FPA_s *Param, FILE* Stream)
{
    int	i;

    fprintf( Stream, "FPA Parameter\n");
    /*--------------------------------------------------------------------*\
    ** write FPA ScaleFactor
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Index\tScaleFactor\n");
    for ( i = 0; i < FPA_SCALE_FACTORS; i++)
    {
	if ((i % 5) == 0)
	    fprintf( Stream, "%5d", i);
	fprintf( Stream, "\t%f", Param->ScaleFactor[ i]);
	if ((i % 5) == 4)
	    fprintf( Stream, "\n");
    }
    fprintf( Stream, "\n");

    /*--------------------------------------------------------------------*\
    **  write Plane of polarization angle
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "CoefficientsUsed %hd\n", Param->CoefficientsUsed);

    /*--------------------------------------------------------------------*\
    ** write FPA FilterCoefficient
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Index\tFilterCoefficient\n");
    for ( i = 0; i < FPA_FILTER_COEFFICIENTS; i++)
    {
	if ((i % 5) == 0)
	    fprintf( Stream, "%5d", i);
	fprintf( Stream, "\t%f", Param->FilterCoefficient[ i]);
	if ((i % 5) == 4)
	    fprintf( Stream, "\n");
    }
    fprintf( Stream, "\n");

}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
