/*----------------------------------------------------------------------------*\
**                                IntensityC.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       07/28/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)IntensityC.c	5.2 07/28/99, Copyright DLR 1998"

#include <string.h>
#include <math.h>
#include "inteityc.h"
#include "log.h"
#include "dataconv.h"
#include "numrecip.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)IntensityC.c 5.2 07/28/99, Copyright DLR 1998")


#define	SS_ID	("Intensity")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                IntensityC__Apply
**  Input parameters:
**    Self containing Mean Sun Reference Spectrum
**    Intensity wavelength grid (1024)
**    RadianceResponse (1024)
**    Band (1A .. 4)
**    Reticon data (1024)
**    Lambda grid for Reticon data (1024)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (ASCII string)
**  Output parameters:
**    Calibrated Reticon data
**  Other interfaces:
**  Description:
**    Makes an intensity calibration by dividing Reticon data by RadianceResponse
**    function (key data)
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	16.3.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
IntensityC__Apply(const IntensityC_s* Self, const float IntensityGrid[],
		  const float RadianceResponse[], ScienceData_e Band,
		  float Reticon[], const float LambdaGrid[],
		  short Begin, short End, const char *PixelDate)
{
    int loop;
    int ErrorCount;
    OpticalChannel_e Ch = Band2Channel(Band);
    char ErrorString[LOG_FIELD_SIZE];

    float MeanSunReference_Fitted [DETECTOR_PIXELS];
    float RadianceResponse_Fitted [DETECTOR_PIXELS];


#ifdef DEBUG
    float InputReticon [DETECTOR_PIXELS];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_2))
    {	
	for (loop = Begin; loop <= End; loop++)
	    InputReticon [loop] = Reticon [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** RadianceResponse is on IntensityGrid, Reticon is on LambdaGrid.
    ** So fit RadianceResponse onto LambdaGrid :
    \*-----------------------------------------------------------------------*/
    fit_grid_akima(IntensityGrid, RadianceResponse, DETECTOR_PIXELS, LambdaGrid, 		
		   RadianceResponse_Fitted, Begin, End);

    for (loop = Begin; loop <= End; loop++)
    {
	/* Do not interpolate between valid signal and zeros, this will create artificial peaks.
	   To avoid division by very small instrument response values (H), set H to zero,
	   where input H is very small and interpolated output H is below 0.6 or so */ 

	if (fabs(RadianceResponse[loop]) < MIN_VALUE && fabs(RadianceResponse_Fitted[loop]) < MIN_VALUE_INTERPOL_H) 
	    RadianceResponse_Fitted[loop] =  0.0;
    }

    /*-----------------------------------------------------------------------*\
    ** Calculate the Earthshine Radiance Spectrum
    \*-----------------------------------------------------------------------*/
    ErrorCount = 0;

    for (loop = Begin; loop <= End; loop++)
    {
	/* Calculate the Earthshine Radiance Spectrum */

	if (fabs(RadianceResponse_Fitted[loop]) > MIN_VALUE)
	    Reticon[loop] /=  RadianceResponse_Fitted[loop];
	else
	{
	    ErrorCount++; 
	    Reticon[loop] = 0.0;
	}
    }

    /*
      if (ErrorCount > 0)
      {
      sprintf(ErrorString, 
      "%d values of RadianceResponse are zero in %s (causing division by zero, Earthshine Radiance Spectrum) - PixelDate %s", 
      ErrorCount, ScienceData2Str(Band), PixelDate );

      Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.8_126", WARN, LOG_PROBLEMS, ErrorString, "");
      }
      */

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG	
    sprintf (debug_filename, "%s", DbgFileName("IntensityC__Apply", PixelDate) );
    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_2))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Band       IntGrid    RadResp    Lambda     InputRetc RadRespFit OutputRet \n");
		fprintf(out, 
			"[]         []         []         []         []        []         []        \n\n");
	    }
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %*.*g %*.*g %*.*g %*.*g %*.*g %*.*g\n", Band,
			DEBUG_WIDTH, DEBUG_PRECISION, IntensityGrid[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, RadianceResponse[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, LambdaGrid[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, 
			RadianceResponse_Fitted[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon[loop]);
	    }	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.8_164", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif
}


/*----------------------------------------------------------------------------*\
**                                IntensityC__CalculateAlbedo
**  Input parameters:
**    Irradiance : calibrated sun spectrum
**    Irradiance wavelength grid (1024)
**    Band (1A .. 4)
**    Reticon data : calibrated earthshine spectrum (1024)
**    Lambda grid for Reticon data (1024)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (ASCII string)
**  Output parameters:
**    Albedo (BSDF_Earth)
**  Other interfaces:
**  Description:
**    Calculates Albedos (BSDF_Earth) by dividing Reticon data by Irradiance
**    (= leakage-, straylight-, fixed-, bsdf- and instrument-response-corrected mean sun spectrum)
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	14.8.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void 
IntensityC__CalculateAlbedo(const float Irradiance[],
			    const float IntensityGrid[],
			    ScienceData_e Band, float Reticon[],
			    const float LambdaGrid[], 
			    short Begin, short End, const char *PixelDate)
{
    int loop;
    int ErrorCount;
    OpticalChannel_e Ch = Band2Channel(Band);
    char ErrorString[LOG_FIELD_SIZE];

    float Irradiance_Fitted [DETECTOR_PIXELS];


#ifdef DEBUG
    float InputReticon [DETECTOR_PIXELS];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_2))
    {	
	for (loop = Begin; loop <= End; loop++)
	    InputReticon [loop] = Reticon [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** MeanSunReference is on IntensityGrid, Reticon is on LambdaGrid.
    ** So fit MeanSunReference onto LambdaGrid :
    \*-----------------------------------------------------------------------*/
    fit_grid_akima(IntensityGrid, Irradiance, DETECTOR_PIXELS, LambdaGrid, 		
		   Irradiance_Fitted, Begin, End);

    /* Do not divide by small values, this will create peaks */
    for (loop = Begin; loop <= End; loop++)
    {
	if (fabs(Irradiance_Fitted[loop]) < 1e-20) 
	    Irradiance_Fitted[loop] =  0.0;
    }

    /*-----------------------------------------------------------------------*\
    ** Calculate the Earthshine Radiance Spectrum
    \*-----------------------------------------------------------------------*/
    ErrorCount = 0;

    for (loop = Begin; loop <= End; loop++)
    {
	/* Calculate the Earthshine Radiance Spectrum */

	if (fabs(Irradiance_Fitted[loop]) > MIN_VALUE)
	    Reticon[loop] /=  Irradiance_Fitted[loop];
	else
	{
	    ErrorCount++; 
	    Reticon[loop] = 0.0;
	}
    }

    /*
      if (ErrorCount > 0)
      {
      sprintf(ErrorString, 
      "%d values of MeanSunReference are zero in %s (causing division by zero, Earthshine Radiance Spectrum) - PixelDate %s", 
      ErrorCount, ScienceData2Str(Band), PixelDate );

      Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.8_258", WARN, LOG_PROBLEMS, ErrorString, "");
      }
      */

    /*------------------------------------------------------------------------*\
    ** test output
    \*------------------------------------------------------------------------*/
#ifdef DEBUG	
    sprintf (debug_filename, "%s", DbgFileName("IntensityC__CalculateAlbedo",
					       PixelDate) );
    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_2))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Band       IntGrid    MeanSun    Lambda     InputRetc MeanSunFit OutputRet \n");
		fprintf(out, 
			"[]         []         []         []         []        []         []        \n\n");
	    }
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %*.*g %*.*g %*.*g %*.*g %*.*g %*.*g\n", Band,
			DEBUG_WIDTH, DEBUG_PRECISION, IntensityGrid [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, 
			Irradiance[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, LambdaGrid[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, 
			Irradiance_Fitted[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon[loop]);
	    }
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.8_296", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif
}


/*----------------------------------------------------------------------------*\
**                                IntensityC__CalculateSunCalibrated
**  Input parameters:
**    Mean Sun Reference Spectrum
**    CalibrationParameters (H_Response)
**    Band (1A .. 4)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**  Output parameters:
**    Calibrated Sun Spectrum (SunCalibrated)
**  Other interfaces:
**  Description:
**    Calibrates (divides) Mean Sun Reference spectrum by Radiance Response function (both
**    embedded in Self structure) 
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	1.3.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
IntensityC__CalculateSunCalibrated(const IntensityC_s* Self, ScienceData_e Band,
				   float* SunCalibrated, short Begin, short End)
{
    int loop;
    int ErrorCount;
    OpticalChannel_e Ch = Band2Channel(Band);
    char ErrorString[LOG_FIELD_SIZE];

#ifdef DEBUG
    FILE *out;
    char debug_filename[80];
#endif

    /*-----------------------------------------------------------------------*\
    ** Calculate the Sun Irradiance Spectrum
    \*-----------------------------------------------------------------------*/
    ErrorCount = 0;

    for (loop = Begin; loop <= End; loop++)
    {
	if (fabs(Self->Calibration[Ch][loop]) > MIN_VALUE)
	    SunCalibrated[loop] = Self->MeanSunReference[Ch][loop] / Self->Calibration[Ch][loop];
	else
	{
	    ErrorCount++; 
	    SunCalibrated[loop] = 0.0;
	}
    }

    /*
      if (ErrorCount > 0)
      {
      sprintf(ErrorString, 
      "%d values of RadianceResponse are zero in %s (causing division by zero, Sun Irradiance Spectrum)", 
      ErrorCount, ScienceData2Str(Band) );

      Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.8_360", WARN, LOG_PROBLEMS, ErrorString, "");
      }
      */

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", "IntensityC__CalculateSunCalibrated.dbg");

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_1))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Pixel      Band       MeanSunRef Calibrat   SunCalib   \n");
		fprintf(out, 
			"[]         []         []         []         []\n\n");
	    }
	
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g\n", 
			loop + (Ch * DETECTOR_PIXELS), 
			Band, 
			DEBUG_WIDTH, DEBUG_PRECISION, Self->MeanSunReference[Ch][loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Self->Calibration[Ch][loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, SunCalibrated [loop]);
	    }
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.8_397", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif

}


/*----------------------------------------------------------------------------*\
**                                IntensityC__ApplyPMD
**  Input parameters:
**    Mean Sun PMD
**    Polarization p values for the PMDs
**    PMD_Number_e Number (which PMD, 1,2 or 3)
**    PMD values (16)
**    date (ASCII string)
**  Output parameters:
**    Calibrated PMD data
**  Other interfaces:
**  Description:
**    Converts PMD data to reflectances by dividing the PMD readouts by Mean
**    Sun PMD readouts (the latter are divided by BSDF). 
**    Polarisation__ApplyPMD later divides PMD by two times p value of the 
**    corresponding PMD.
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	18.4.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
IntensityC__ApplyPMD(const IntensityC_s* Self, const float LambdaPMD[],
		     PMD_Number_e Number, float PMD[], const char *PixelDate)
{
    int loop;
    char ErrorString[LOG_FIELD_SIZE];

    float MeanPMD_in [2*PMD_DEFINED-1];
    float MeanPMD_in_Lambda [2*PMD_DEFINED-1];

    float MeanPMD_Fitted [PMD_DEFINED];
    float MeanPMD_Fitted_Lambda [PMD_DEFINED];

    short order = 3;
	
    FunctionStatus_e Status = SUCCESS;

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    float InputPMD [PMD_SIZE];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_3))
    {	
	for (loop = 0; loop < PMD_SIZE; loop++)
	    InputPMD [loop] = PMD [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** Init
    \*-----------------------------------------------------------------------*/
    for (loop = 0; loop < PMD_DEFINED; loop++)
    {
	MeanPMD_Fitted [loop] = 1.0;
	MeanPMD_Fitted_Lambda [loop] = -1.0;
    }


    /*-----------------------------------------------------------------------*\
    ** extend input grid by two points to improve akima interpolation
    \*-----------------------------------------------------------------------*/
    MeanPMD_in_Lambda [0] = Self->MeanPMD_Wavelength [PMD_1];
    MeanPMD_in_Lambda [1] = Self->MeanPMD_Wavelength [PMD_1] + 10.0;
    MeanPMD_in_Lambda [2] = Self->MeanPMD_Wavelength [PMD_2];
    MeanPMD_in_Lambda [3] = Self->MeanPMD_Wavelength [PMD_3] - 10.0;
    MeanPMD_in_Lambda [4] = Self->MeanPMD_Wavelength [PMD_3];

    /*-----------------------------------------------------------------------*\
    ** extend Self->MeanPMD by two points to improve akima interpolation
    \*-----------------------------------------------------------------------*/
    fit_grid_linear(Self->MeanPMD_Wavelength, Self->MeanPMD, PMD_DEFINED,
		    MeanPMD_in_Lambda, MeanPMD_in, 0, PMD_DEFINED+1);

    /*-----------------------------------------------------------------------*\
    ** Copy SelfPolarization Wavelengths
    \*-----------------------------------------------------------------------*/
    for (loop = PMD_1; loop <= PMD_3; loop++)
	MeanPMD_Fitted_Lambda [loop] = LambdaPMD [loop];

    /*-----------------------------------------------------------------------*\
    ** Interpolate Self->MeanPMD to the Wavelengths of SelfPolarization
    \*-----------------------------------------------------------------------*/
    fit_grid_akima(MeanPMD_in_Lambda, MeanPMD_in, PMD_DEFINED+2,
		   MeanPMD_Fitted_Lambda, MeanPMD_Fitted, 0, PMD_DEFINED-1);

    /*-----------------------------------------------------------------------*\
    ** Correct each PMD pixel for BSDF
    \*-----------------------------------------------------------------------*/
    if (Self->MeanPMD[Number] < 10e-10 || MeanPMD_Fitted[Number] < 10e-10)
    {
	sprintf(ErrorString, 
		"Invalid MeanPMD (Sun) value for PMD %d - PixelDate %s", Number+1, PixelDate);

	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "3.5_221", WARN, LOG_PROBLEMS, ErrorString, "");
		
	Status = ERROR;
    }
    else
    {
	for (loop = 0; loop < PMD_SIZE; loop++)
	    PMD[loop] /= MeanPMD_Fitted[Number];
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("IntensityC__ApplyPMD", PixelDate) );

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Number == PMD_1)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Pixel      PMD_Number InputPMD   MeanPMDLam MeanPMD    MeanFitLam MeanPMDFit OutputPMD        \n");
		fprintf(out, 
			"[]         []         []         []         []         []         []         []\n\n");
	    }
	
	    for (loop = 0; loop < PMD_SIZE; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g %*.*g %*.*g %*.*g\n", 
			loop,
			Number, 
			DEBUG_WIDTH, DEBUG_PRECISION, InputPMD[loop],
			DEBUG_WIDTH, DEBUG_PRECISION, Self->MeanPMD_Wavelength[Number],
			DEBUG_WIDTH, DEBUG_PRECISION, Self->MeanPMD[Number], 
			DEBUG_WIDTH, DEBUG_PRECISION, MeanPMD_Fitted_Lambda[Number],
			DEBUG_WIDTH, DEBUG_PRECISION, MeanPMD_Fitted[Number],
			DEBUG_WIDTH, DEBUG_PRECISION, PMD[loop]);
	    }
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "3.14_385", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif

    return Status;
}



/*----------------------------------------------------------------------------*\
**                                IntensityC__PrepareJumpCorrection
**  Input parameters:
**    Pmd[PMD_DEFINED][PMD_SIZE]	PMD readouts of the current ground pixel
**    IntgTime				Integration time
**    PixelDate				date (ASCII string)
**
**  Output parameters:
**    S_PMD_basic[PMD_DEFINED]		Sums of the PMD readouts per PMD	
**
**  Other interfaces:
**
**  Description:
**	Summing up the 3 series of PMD readouts (for the 3 PMDs), depending on integration time :
**	If 1.5 sec, sum up 16 readouts, if 0.375 sec, sum up last four readouts...
**  References:
**  Libraries:
**    ....
**  Created:	10.01.97
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void 
IntensityC__PrepareJumpCorrection(float Pmd[PMD_DEFINED][PMD_SIZE], 
				  const float IntgTime, float S_PMD_basic[],
				  const char *PixelDate)
{
    short pmd;
    short loop;

    short PMD_n = PMD_SIZE;

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    FILE *out;
    char debug_filename[80];
#endif

    /*---------------------------------------------------------------*\
    ** check how many PMD readouts to use
    \*---------------------------------------------------------------*/
    if (IntgTime <= GROUND_PIXEL_TIME)
    {
	PMD_n = (short) (PMD_SIZE * IntgTime / GROUND_PIXEL_TIME );
    }
    else return;	/* IT should be less or equal 1.5 */

    /*---------------------------------------------------------------*\
    ** sum up PMD readouts 1 to 16 for each PMD
    \*---------------------------------------------------------------*/
    for (pmd = 0; pmd < PMD_DEFINED; pmd++)
    {
	S_PMD_basic[pmd] = 0.0;

	for (loop = PMD_SIZE-1; loop >= PMD_SIZE - PMD_n; loop--) /* loop over subpixels */
	    S_PMD_basic[pmd] +=  Pmd[pmd][loop];
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf(debug_filename, "%s", 
	    DbgFileName("IntensityC__PrepareJumpCorrection", PixelDate) );
    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    fprintf(out, "\n");
	    fprintf(out, 
		    "index      PMD_Number PMD_value  S_PMD_basic\n");
	    fprintf(out, 
		    "[]         []         []         []\n\n");
	
	    for (pmd = 0; pmd < PMD_DEFINED; pmd++)
	    {
		for (loop = PMD_SIZE-1; loop >= PMD_SIZE - PMD_n; loop--) /* loop over subpixels */
		    fprintf(out, "%10d %10d %*.*g %*.*g\n",
			    loop + pmd * PMD_SIZE, 
			    pmd,
			    DEBUG_WIDTH, DEBUG_PRECISION, Pmd[pmd][loop],
			    DEBUG_WIDTH, DEBUG_PRECISION, S_PMD_basic[pmd]
			    );
	    }
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "3.14_385", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif

}


/*----------------------------------------------------------------------------*\
**                                IntensityC__CorrectJumps
**  Input parameters:
**    Reticon data (1024)
**    Lambda grid for Reticon data (1024)
**    Integration time of band 4
**    do_correction :	1 : do the correction
**			0 : don't do the correction, only calculate previous_PMD_16
**				for the next call
**    PMD readouts of the current ground pixel
**    S_PMD_basic 	The sums of the 16 readouts of the three PMDs
**    previous_PMD_16   An interpolated spectrum of the last PMD readouts of the
**			previous ground pixel
**    Band (1A .. 4)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    PixelDate (ASCII string)
**
**  Output parameters:
**    Corrected Reticon data
**
**  Other interfaces:
**
**  Description:
**    Uses the PMD readouts for correcting radiance jumps in GOME spectra
**
**  References:
**  Libraries:
**    ....
**  Created:	10.01.97
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void 
IntensityC__CorrectJumps(const float LambdaGrid[], float Reticon[], 
			 const float IntgTime, const short do_correction,
			 float Pmd[PMD_DEFINED][PMD_SIZE], float S_PMD_basic[],
			 float previous_PMD_16[], ScienceData_e Band,
			 short Begin, short End, const char *PixelDate)
{
    float PMD_x[PMD_CORNER_POINTS] = {310.0, 332.0, 354.12, 481.54, 699.5, 750.0, 800.0};
    float S_PMD[PMD_CORNER_POINTS];
    float PMD_0[PMD_CORNER_POINTS];
    float PMD_16[PMD_CORNER_POINTS];

    float S_PMD_spec[DETECTOR_PIXELS];
    float PMD_16_spec[DETECTOR_PIXELS];
    float PMD_IT[DETECTOR_PIXELS];
    float f_corr[DETECTOR_PIXELS];

    short pmd, ch;
    short loop;

    short PMD_n = PMD_SIZE;

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    float InputReticon [DETECTOR_PIXELS];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_2))
    {	
	for (loop = Begin; loop <= End; loop++)
	    InputReticon [loop] = Reticon [loop];
    }
#endif

    /*---------------------------------------------------------------*\
    ** preparation
    \*---------------------------------------------------------------*/
    ch = Band2Channel(Band);

    for (loop = 0; loop < DETECTOR_PIXELS; loop++)
    {
	S_PMD_spec[loop] = 0;
	PMD_16_spec[loop] = 0;
	PMD_IT[loop] = 0;
	f_corr[loop] = 0;
    }

    /*---------------------------------------------------------------*\
    ** Interpolate between S_PMD
    \*---------------------------------------------------------------*/
    S_PMD[0] = S_PMD_basic[PMD_1];
    S_PMD[1] = S_PMD_basic[PMD_1];
    S_PMD[2] = S_PMD_basic[PMD_1];
    S_PMD[3] = S_PMD_basic[PMD_2];
    S_PMD[4] = S_PMD_basic[PMD_3];
    S_PMD[5] = S_PMD_basic[PMD_3];
    S_PMD[6] = S_PMD_basic[PMD_3];

    fit_grid_akima(PMD_x, S_PMD, PMD_CORNER_POINTS, LambdaGrid,
		   S_PMD_spec, Begin, End);

    /*---------------------------------------------------------------*\
    ** Interpolate between all 16th PMD readouts
    \*---------------------------------------------------------------*/
    PMD_16[0] = Pmd[PMD_1][PMD_SIZE - 1];
    PMD_16[1] = Pmd[PMD_1][PMD_SIZE - 1];
    PMD_16[2] = Pmd[PMD_1][PMD_SIZE - 1];
    PMD_16[3] = Pmd[PMD_2][PMD_SIZE - 1];
    PMD_16[4] = Pmd[PMD_3][PMD_SIZE - 1];
    PMD_16[5] = Pmd[PMD_3][PMD_SIZE - 1];
    PMD_16[6] = Pmd[PMD_3][PMD_SIZE - 1];

    fit_grid_akima(PMD_x, PMD_16, PMD_CORNER_POINTS, LambdaGrid,
		   PMD_16_spec, Begin, End);

    /*---------------------------------------------------------------*\
    ** check how many PMD readouts to use
    \*---------------------------------------------------------------*/
    if (IntgTime < GROUND_PIXEL_TIME)
    {
	/*---------------------------------------------------------------*\
	** if IT less than 1.5, replace previous_PMD_16 by Pmd(n)
	\*---------------------------------------------------------------*/
	PMD_n = (short) (PMD_SIZE * IntgTime / GROUND_PIXEL_TIME );

	PMD_0[0] = Pmd[PMD_1][(PMD_SIZE - PMD_n) - 1];
	PMD_0[1] = Pmd[PMD_1][(PMD_SIZE - PMD_n) - 1];
	PMD_0[2] = Pmd[PMD_1][(PMD_SIZE - PMD_n) - 1];
	PMD_0[3] = Pmd[PMD_2][(PMD_SIZE - PMD_n) - 1];
	PMD_0[4] = Pmd[PMD_3][(PMD_SIZE - PMD_n) - 1];
	PMD_0[5] = Pmd[PMD_3][(PMD_SIZE - PMD_n) - 1];
	PMD_0[6] = Pmd[PMD_3][(PMD_SIZE - PMD_n) - 1];

	fit_grid_akima(PMD_x, PMD_0, PMD_CORNER_POINTS, LambdaGrid,
		       previous_PMD_16, Begin, End);
    }

    /*---------------------------------------------------------------*\
    ** Calculate PMD_IT for each detector pixel
    \*---------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
	PMD_IT[loop] = ((float)loop / (float)DETECTOR_PIXELS) * (previous_PMD_16[loop] - PMD_16_spec[loop]);

    /*---------------------------------------------------------------*\
    ** Calculate the correction factor for each detector pixel
    \*---------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
	f_corr[loop] = S_PMD_spec[loop] / (S_PMD_spec[loop] + PMD_IT[loop]);

    /*---------------------------------------------------------------*\
    ** Apply the correction factor to each detector pixel.
    **
    ** IT should be less or equal 1.5, if not, don't correct but 
    ** calculate previous_PMD_16 for the next ground pixel
    \*---------------------------------------------------------------*/
    if (IntgTime <= GROUND_PIXEL_TIME && do_correction == 1)
	for (loop = Begin; loop <= End; loop++)
	    Reticon[loop] *= f_corr[loop];

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("IntensityC__CorrectJumps",
					       PixelDate) );
    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1B)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Pixel      Band       LambdaGrid InputRetic S_PMD_spec prevPMD_16 PMD_16_spe PMD_IT     do_corr     f_corr     Reticon \n");
		fprintf(out, 
			"[]         []         []         []         []         []         []         []         []          []         [] \n\n");
	    }
	
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g %*.*g %*.*g %*.*g %10d %*.*g %*.*g\n", 
			loop + (ch * DETECTOR_PIXELS), 
			Band,
			DEBUG_WIDTH, DEBUG_PRECISION, LambdaGrid [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, S_PMD_spec[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, previous_PMD_16[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, PMD_16_spec[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, PMD_IT[loop], 
			do_correction, 
			DEBUG_WIDTH, DEBUG_PRECISION, f_corr[loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon[loop]
			);
	    }

	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.8_845", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif

    /*---------------------------------------------------------------*\
    ** Save 16th PMD readouts for next ground pixel
    \*---------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
	previous_PMD_16[loop] = PMD_16_spec[loop];
}


/*----------------------------------------------------------------------------*\
**                           IntensityC__Sun2Host
**  Input parameters:
**    Intensity		in Sun memory representation
**  Output parameters:
**    Intensity		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the Intensity struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
IntensityC__Sun2Host(IntensityC_s *Param)
{
    int	i, j;

    for ( j = 0; j < OPTICAL_CHANNELS; j++)
	for ( i = 0; i < DETECTOR_PIXELS; i++)
	{
	    /* float */
	    Param->Calibration[ j][ i] = SunFloat2Float( (unsigned char*)
							 &Param->Calibration[ j][ i]);
	    /* float */
	    Param->MeanSunReference[ j][ i] = SunFloat2Float( (unsigned char*)
							      &Param->MeanSunReference[ j][ i]);
	    /* float */
	    Param->RelativeError[ j][ i] = SunFloat2Float( (unsigned char*)
							   &Param->RelativeError[ j][ i]);
	}

    for ( i = 0; i < PMD_DEFINED; i++)
    {
	/* float */
	Param->MeanPMD[ i] = SunFloat2Float( (unsigned char*)
					     &Param->MeanPMD[ i]);
	/* float */
	Param->MeanPMD_Wavelength[ i] = SunFloat2Float( (unsigned char*)
							&Param->MeanPMD_Wavelength[ i]);
    }
}

/*----------------------------------------------------------------------------*\
**                              IntensityC__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		read Intensity Calibration parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all Intensity Calibration parameter from the given 
**	input stream into the Intensity Calibration parameter structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
IntensityC__read(IntensityC_s *Param, FILE* Stream, Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    long NoOfReadBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];
 
    /*------------------------------------------------------------------------*\
    ** read Intensity Parameter
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread(Param->Calibration, 1, sizeof( Param->Calibration),
		       Stream);

    /*------------------------------------------------------------------------*\
    ** read Sun Reference Spectrum (Mean Value)
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( Param->MeanSunReference, 1,
			sizeof( Param->MeanSunReference), Stream);

    /*------------------------------------------------------------------------*\
    ** read relative errors of Sun Mean Reference Spectrum
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( Param->RelativeError, 1,
			sizeof( Param->RelativeError), Stream);

    /*------------------------------------------------------------------------*\
    ** read PMD Mean Values of the Sun Reference Spectra
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( Param->MeanPMD, 1,
			sizeof( Param->MeanPMD), Stream);

    /*------------------------------------------------------------------------*\
    ** read PMD Wavelength
    \*------------------------------------------------------------------------*/
    NoOfBytes += fread( Param->MeanPMD_Wavelength, 1,
			sizeof( Param->MeanPMD_Wavelength), Stream);

    /*------------------------------------------------------------------------*\
    ** read Date
    \*------------------------------------------------------------------------*/
    if (UTC__read(&Param->Date,  Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	sprintf(LogBuffer, "Intensity UTC time, #read bytes: %d",
		NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__994", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }
    NoOfBytes += UTC_SIZE;

    /*------------------------------------------------------------------------*\
    ** error check
    \*------------------------------------------------------------------------*/
    if (NoOfBytes != INTENSITY_SIZE) 
    {
	Status = FATAL;
	sprintf( LogBuffer, "Intensity Calibration Parameter, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__1008", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*------------------------------------------------------------------------*/
    if (convert_sun_2_host) 
    {
	IntensityC__Sun2Host( Param);
    }

    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                           IntensityC__Show
**  Input parameters:
**	Stream		output stream
**	Param		Intensity parameter structure
**
**  Output parameters:
**
**  Other interfaces:
**	Intensity parameter structure written to the output stream
**
**  Description:
**	Function to write the Intensity parameter structure to the output
**	stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
IntensityC__show(const IntensityC_s *Param, FILE* Stream)
{
    int	i, j;
    char ASCII_Time[ 30];

    /*----------------------------------------------------------------------*\
    ** show Intensity Parameter
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Instrument Response\n");
    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%4d", i);
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%f",Param->Calibration[ j][ i]);
	fprintf( Stream, "\n");
    }
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show Sun Reference Spectrum (Mean Value)
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Sun Mean Reference Spectrum\n");
    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%4d", i);
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%f", Param->MeanSunReference[ j][ i]);
	fprintf( Stream, "\n");
    }
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show relative error of Sun Reference Spectrum (Mean Value)
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Sun Mean Reference Spectrum Relative Error\n");
    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%4d", i);
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%f", Param->RelativeError[ j][ i]);
	fprintf( Stream, "\n");
    }
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show PMD Mean Values and Wavelength of the Sun Reference Spectra
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Sun Mean PMD (Wavelength  Value)\n");
    for ( j = 0; j < PMD_DEFINED; j++)
	fprintf( Stream, "\t%f\t%f\n", Param->MeanPMD_Wavelength[ j],
		 Param->MeanPMD[ j]);
    fprintf( Stream, "\n");

    /*----------------------------------------------------------------------*\
    ** show the data of the sun spectrum
    \*----------------------------------------------------------------------*/
    fprintf( Stream, "Sun Mean Reference Spectrum Date\n");
    if (UTC_2_ASCII( Param->Date, ASCII_Time) != SUCCESS)
    {
/*
  Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__1108", ERROR,
  LOG_NUM, "converting the UTC time of Sun Mean Reference Spectrum", "");
  */
	strcpy( ASCII_Time, "dd.mmm.yyyy hh:mm:ss.sss");
    }
    fprintf( Stream, "%s\n", ASCII_Time);

}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
