/*----------------------------------------------------------------------------*\
**                                KeyDataError.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)KeyDataError.c	5.1 02/26/99, Copyright DLR 1998"

#include "keydrror.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)KeyDataError.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("KeyDataError")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                KeyDataError__Get
**  Input parameters:
**    Self: KeyDataError_s structure
**    Function: the KeyDataError_e to return
**    Ch: Optical Channel
**    Pixel: Detector Pixel
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    Returns the Relative Error Budget on KeyData Functions
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	06.12.95
**  Author:		Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
float KeyDataError__Get(const KeyDataError_s *Self, KeyDataError_e Function, OpticalChannel_e Ch, short Pixel)
{
    if (Function == KDE_RESPONSE_F2_SMDEP)
    {
	return Self->Response_f2_SMDep[Ch][Pixel];
    }
    else
    {
	static const short PIXEL_LIMIT = DETECTOR_PIXELS / 2;
	short Indx;

	Indx = Ch*KEYDATA_ERRORS_PER_CHANNEL;
	if (Pixel >= PIXEL_LIMIT) Indx++;
	return Self->Budget[Function][Indx];
    }
}


/*----------------------------------------------------------------------------*\
**                            KeyDataError__Sun2Host
**  Input parameters:
**    KeyDataError		in Sun memory representation
**  Output parameters:
**    KeyDataError		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the KeyDataError struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
KeyDataError__Sun2Host( KeyDataError_s *Param)
{
    int	i, j;

    for ( i = 0; i < KEYDATA_ERRORS_FUNCTIONS; i++)
	for ( j = 0; j < OPTICAL_CHANNELS * KEYDATA_ERRORS_PER_CHANNEL; j++)
	{
	    Param->Budget[ i][ j] =
		SunFloat2Float( (unsigned char*)&Param->Budget[ i][ j]);
	}

    for ( i = 0; i < OPTICAL_CHANNELS; i++)
	for ( j = 0; j < DETECTOR_PIXELS; j++)
	{
	    Param->Response_f2_SMDep[ i][ j] =
		SunFloat2Float( (unsigned char*)&Param->Response_f2_SMDep[ i][ j]);

	}
}

/*----------------------------------------------------------------------------*\
**                                KeyDataError__Read
**  Input parameters:
**	Stream			input stream
**  Output parameters:
**	KeyDataError		Key Data Errors
**
**  Other interfaces:
**
**  Description:
**	Function to read all Key Data Error entries from the given 
**	input stream into the Key Data Error structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
KeyDataError__read(KeyDataError_s *KeyDataError, FILE* Stream,
		   Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** read KeyDataError Budget
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread( KeyDataError->Budget, 1,
			sizeof( KeyDataError->Budget), Stream); 

    /*----------------------------------------------------------------------*\
    ** read KeyDataError Response_f2_SMDep
    \*----------------------------------------------------------------------*/
    NoOfBytes += fread( KeyDataError->Response_f2_SMDep, 1,
			sizeof( KeyDataError->Response_f2_SMDep), Stream); 

    /*----------------------------------------------------------------------*\
    ** check size
    \*----------------------------------------------------------------------*/
    if (NoOfBytes != KEYDATA_ERRORS_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "KEY_DATA_ERRORS Parameter, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__152", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	KeyDataError__Sun2Host( KeyDataError);
    }

    return SUCCESS;
}

/*----------------------------------------------------------------------------*\
**                                KeyDataError__Write
**  Input parameters:
**	Stream			output stream
**	KeyDataError		Key Data Errors
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the KeyData Errors to the given 
**	output stream
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
KeyDataError__Write( FILE * Stream, const KeyDataError_s *KeyDataError)
{
    FunctionStatus_e	Status		= SUCCESS;
    long			NoOfBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** write KeyDataError Budget
    \*----------------------------------------------------------------------*/
    NoOfBytes += fwrite( KeyDataError->Budget, 1,
			 sizeof( KeyDataError->Budget), Stream); 

    /*----------------------------------------------------------------------*\
    ** write KeyDataError Response_f2_SMDep
    \*----------------------------------------------------------------------*/
    NoOfBytes += fwrite( KeyDataError->Response_f2_SMDep, 1,
			 sizeof( KeyDataError->Response_f2_SMDep), Stream); 

    /*----------------------------------------------------------------------*\
    ** check size
    \*----------------------------------------------------------------------*/
    if (NoOfBytes != KEYDATA_ERRORS_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "KEY_DATA_ERRORS Parameter, #written bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__213", Status,
		  LOG_CANNOT_WRITE, LogBuffer, "");
    }
    return Status;
}

/*----------------------------------------------------------------------------*\
**                                KeyDataError__Show
**  Input parameters:
**	Stream			output stream
**	KeyDataError		Key Data Error structure
**
**  Output parameters:
**
**  Other interfaces:
**	Key Data Error structure written to the output stream
**
**  Description:
**	Function to write the Key Data Error structure to the
**	output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
KeyDataError__show(const KeyDataError_s *KeyDataError, FILE * Stream)
{
    int	i, j, k;

    fprintf( Stream, "Key Data Errors Budget\n");
    for ( i = 0; i < KEYDATA_ERRORS_FUNCTIONS; i++)
    {
	for ( k = 0; k < KEYDATA_ERRORS_PER_CHANNEL; k++)
	{
	    fprintf( Stream, "%s %1d\t", KeyDataError2Str( (KeyDataError_e)i), k + 1);
	    for ( j = 0; j < OPTICAL_CHANNELS; j++)
		fprintf( Stream, "\t%f",
			 KeyDataError->Budget[ i][ j * KEYDATA_ERRORS_PER_CHANNEL + k]);
	    fprintf( Stream, "\n");
	}
    }

    /*--------------------------------------------------------------------*\
    **  show fixed pattern parameter
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Key Data Errors Response_f2_SMDep\n");

    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%4d", i);
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%f", KeyDataError->Response_f2_SMDep[ j][ i]);
	fprintf( Stream, "\n");
    }
}


/*----------------------------------------------------------------------------*\
**                                KeyDataError2Str
**  Input parameters:
**    Enum KeyDataError enumerator
**  Output parameters:
**    <none>
**  Other interfaces:
**    <none>
**  Description:
**    A string representing the KeyDataError_e Enum is returned
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:    24.1.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
const char*
KeyDataError2Str(KeyDataError_e Enum)
{
    static const char* KeyDataErrorStr[] = {"KDE_BSDF", "KDE_RESPONSE",
					    "KDE_F2", "KDE_SMDEP", "KDE_CHI", 
					    "KDE_ETA", "KDE_KSI",
					    "KDE_RESPONSE_F2_SMDEP"};
    return KeyDataErrorStr[Enum];
}



/*============================================================================*\
**  private operations
\*============================================================================*/


