/*----------------------------------------------------------------------------*\
**                                LeakageC.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)LeakageC.c	5.1 02/26/99, Copyright DLR 1998"

#include "leakagec.h"
#include "log.h"
#include "dataconv.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)LeakageC.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("Leakage")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                LeakageC__Apply
**  Input parameters:
**    Leakage Signal (Self)
**    Band (1A .. 4)
**    Reticon data
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (ASCII string)
**  Output parameters:
**    Corrected Reticon data
**  Other interfaces:
**  Description:
**    Subtracts Leakage signal from Reticon data
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	16.3.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void LeakageC__Apply (const LeakageC_s *Self, ScienceData_e Band, float * Reticon, short Begin, short End, const char *PixelDate)
{
    int loop;
    int ErrorCount;
    char ErrorString[LOG_FIELD_SIZE];
    OpticalChannel_e Ch=Band2Channel(Band);


#ifdef DEBUG
    FILE *out;
    float InputReticon [DETECTOR_PIXELS];
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_2))
    {	
	for (loop = Begin; loop <= End; loop++)
	    InputReticon [loop] = Reticon [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** Correct each detector pixel for leakage current
    \*-----------------------------------------------------------------------*/
    ErrorCount = 0;

    for (loop = Begin; loop <= End; loop++)
    {
	Reticon[loop] -= ( Self->DarkCurrent[Ch][loop] );

	if ( Reticon[loop] < 0.0 ) ErrorCount++;
    }

    /*
      if (ErrorCount > 0)
      {
      sprintf(ErrorString, 
      "%d values negativ after Dark subtraction in Band %s PixelDate %s", ErrorCount, ScienceData2Str(Band), PixelDate);

      Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_84", WARN, LOG_PROBLEMS, ErrorString, "");
      }
      */

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("LeakageC__Apply", PixelDate) );

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_2))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\n"); 
		fprintf(out, 
			"Pixel      Band       InputRetic DarkCur    OutputRet  \n");
		fprintf(out, 
			"[]         []         []         []         []\n\n");
	    }
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g\n", 
			loop + (Ch * DETECTOR_PIXELS), 
			Band, 
			DEBUG_WIDTH, DEBUG_PRECISION, InputReticon [loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Self->DarkCurrent[Ch][loop], 
			DEBUG_WIDTH, DEBUG_PRECISION, Reticon [loop]);
	    }
		
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_120", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif
}

/*----------------------------------------------------------------------------*\
**                                LeakageC__ApplyPMD
**  Input parameters:
**    PMD Offset (Self)
**    PMD Number (which PMD 1, 2 or 3)
**    PMD signal (16)
**    date (string)
**  Output parameters:
**    Corrected PMD signal
**  Other interfaces:
**  Description:
**    Subtracts PMD Offset from PMD signal
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	16.3.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void LeakageC__ApplyPMD(const LeakageC_s *Self, PMD_Number_e Number, float PMD[], const char *PixelDate)
{
    int loop;
    char ErrorString[LOG_FIELD_SIZE];
    int error_count;

#ifdef DEBUG
    float InputPMD [PMD_SIZE];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_3))
    {	
	for (loop = 0; loop < PMD_SIZE; loop++)
	    InputPMD [loop] = PMD [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** Correct each PMD pixel for Dark Signal
    \*-----------------------------------------------------------------------*/
    error_count = 0;

    for (loop = 0; loop < PMD_SIZE; loop++)
    {
	PMD[loop] -= ( Self->PMD_Offset[Number]);

	if ( PMD[loop] < 0.0 ) error_count++;
    }
	
    /*		
		if (error_count > 0)
		{
		sprintf(ErrorString, 
		"%d values negativ after Dark correction in PMD %d - PixelDate : %s", error_count, Number+1, PixelDate);

		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_181", WARN, LOG_PROBLEMS, ErrorString, "");
		}
		*/

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("LeakageC__ApplyPMD", PixelDate) );

    if (Debug__isActive(debug_ptr, DBG_LEAKAGE, DBG_LEVEL_3))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Number == PMD_1)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Pixel      PMD_Number PMD_Offset InputPMD   PMD\n");
		fprintf(out, 
			"[]         []         []         []         []\n\n");
	    }
	
	    for (loop = 0; loop < PMD_SIZE; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g\n", 
			loop,
			Number, 
			DEBUG_WIDTH, DEBUG_PRECISION, Self->PMD_Offset[Number],
			DEBUG_WIDTH, DEBUG_PRECISION, InputPMD[loop],
			DEBUG_WIDTH, DEBUG_PRECISION, PMD[loop]);
	    }
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_218", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif
}


/*----------------------------------------------------------------------------*\
**                           LeakageC__Sun2Host
**  Input parameters:
**    Leakage		in Sun memory representation
**  Output parameters:
**    Leakage		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the Leakage struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
LeakageC__Sun2Host(LeakageC_s *Param)
{
    int	i;
    int j;
    
    Param->ArrayNoise = SunFloat2Float( (unsigned char*)&Param->ArrayNoise);

    for ( i = 0; i < PMD_DEFINED; i++)
    {
	/* float */
	Param->PMD_Offset[ i] = SunFloat2Float( (unsigned char*)
						&Param->PMD_Offset[ i]);
    }
    /* float */
    Param->PMD_Noise = SunFloat2Float( (unsigned char*)&Param->PMD_Noise);

    for ( j = 0; j < OPTICAL_CHANNELS; j++)
	for ( i = 0; i < DETECTOR_PIXELS; i++)
	{
	    /* float */
	    Param->DarkCurrent[ j][ i] = SunFloat2Float( (unsigned char*)
							 &Param->DarkCurrent[ j][ i]);
	}
}

/*----------------------------------------------------------------------------*\
**                              LeakageC__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		read Leakage parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all Leakage parameter from the given 
**	input stream into the Leakage structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
LeakageC__read(LeakageC_s *Param, FILE* Stream, Bool CONVERT_SUN_2_HOST)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** read Detector Array Noise
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->ArrayNoise, 1, sizeof( Param->ArrayNoise),
			Stream);

    /*--------------------------------------------------------------------*\
    **  read PMD Offset
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->PMD_Offset, 1, sizeof( Param->PMD_Offset),
			Stream);

    /*--------------------------------------------------------------------*\
    **  read PMD noise during this measurement
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->PMD_Noise, 1, sizeof( Param->PMD_Noise),
			Stream);

    /*--------------------------------------------------------------------*\
    **  read Dark Current
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->DarkCurrent, 1, sizeof( Param->DarkCurrent),
			Stream);

    /*--------------------------------------------------------------------*\
    **  check 
    \*--------------------------------------------------------------------*/
    if (NoOfBytes != LEAKAGE_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "Leakage Calibration Parameter, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__343", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (CONVERT_SUN_2_HOST)
    {
	LeakageC__Sun2Host( Param);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           LeakageC__Show
**  Input parameters:
**	Stream		output stream
**	Param		Leakage structure
**
**  Output parameters:
**
**  Other interfaces:
**	Leakage structure written to the output stream
**
**  Description:
**	Function to write the leakage structure to the output stream in
**	ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
LeakageC__show(const LeakageC_s *Param, FILE* Stream)
{
    int	i, j;

    /*--------------------------------------------------------------------*\
    ** write Detector Array Noise
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Array Noise %f\n", Param->ArrayNoise);

    /*--------------------------------------------------------------------*\
    **  write PMD Offset
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "PMD Offset ");
    for ( i = 0; i < PMD_DEFINED; i++)
    {
	fprintf( Stream, "%f ", Param->PMD_Offset[ i]);
    }
    fprintf( Stream, "\n");

    /*--------------------------------------------------------------------*\
    **  write PMD noise during this measurement
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "PMD Noise: %f\n", Param->PMD_Noise);

    /*--------------------------------------------------------------------*\
    **  write Dark Current
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Dark Current\n");

    for ( i = 0; i < DETECTOR_PIXELS; i++)
    {
	fprintf( Stream, "%4d", i);
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%f", Param->DarkCurrent[ j][ i]);
	fprintf( Stream, "\n");
    }
}


/*============================================================================*\
**  private operations
\*============================================================================*/


