/*----------------------------------------------------------------------------*\
**                              log.c
**
**  GDP Level 0 to 1: Log functions implementation file
**
**  Version:	1.1
**  Date:		08/12/98
**  Contents:
**    This file contains the following functions:
**        Bool Log__Open
**        void Log__Msg
**        Bool Log__Read
**        void Log__Close
\*----------------------------------------------------------------------------*/
#include <string.h>
#include <errno.h>
#include "log.h"

#pragma ident	"@(#)log.c	1.1 08/12/98, Copyright DLR 1994"


/*----------------------------------------------------------------------------*\
**  Declaration of static variables
\*----------------------------------------------------------------------------*/
static char Hostname[25] = "UNKNOW HOST MACHINE NAME";	/* Host Name */
static long Pid = -1;				/* Process ID */
static FILE *Stream;			  /* Log file */
static Bool IsOpen=False;			/* Checks if the file is already opened */
static  const char *PidFormat = "%li ~ ";	/* Different input/output formats */
static  const char Separator = '~';
static  const char *MsgFormat = "%s ~ %s ~ %li ~ %s ~ %s ~ %s ~ %s ~ %s ~ %s ~\n";



/*----------------------------------------------------------------------------*\
**                                Log__Open
**  Input parameters:
**    LogFile: the name of the file to put the log messages. If an empty string
**             is given, stderr will be used for write.
**    Write: a boolean indicating the mode.
**  Output parameters:
**  Other interfaces:
**    Output file ...
**  Description:
**    Set the Hostname and Pid variables and try to open the file LogFile
**    in append or read mode. If the file will be opened for writing and the
**    name of the LogFile is an empty string (default value) the standard
**    error will be used to write the log messages.
**    If there are some problems, False is returned
**  References:
**  Libraries:
**  Created:	2.3.94
**  Author:	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
Bool Log__Open(const char* LogFile, Bool Write)
{
    char Type[2] = " ";

    if (IsOpen)
    {
	fprintf(stderr, "LOG ERROR - Log__Open: First close the file with Log__Close\n");
	return False;
    }
    else
    {
	IsOpen = True;
    }

    if (Write)
    {
	*Type = 'a';

	/*--------------------------------------------------------------------*\
	** Set the Hostname and process id
	\*--------------------------------------------------------------------*/
	if (!Sys__GetHostName(Hostname, strlen(Hostname)))
	{
	    fprintf(stderr, "LOG ERROR - Log__Open: Can't read the hostname (sysinfo). Error %d\n", errno);
	    return False;
	}
	if (!Sys__GetPid(&Pid))
	{
	    fprintf(stderr, "LOG ERROR - Log__Open: Can't read the process id (getpid). Error %d\n", errno);
	    return False;
	}
    }
    else
    {
	*Type = 'r';
    }

    if (Write && (*LogFile == '\0')) {
	Stream = stderr;
    }
    else
    {
	if ((Stream = fopen(LogFile, Type)) == NULL)
	{
	    Stream = stderr;
	    fprintf(stderr, "LOG ERROR - Log__Open: Can't open log file '%s'. Error %d\n", LogFile, errno);
	    return False;
	}
    }
    return True;
}



/*----------------------------------------------------------------------------*\
**                                Log__Msg
**  Input parameters:
**    System a Log_System_e that identifies the calling system
**    Subsystem a string that identifies the calling subsystem
**    ErrorNumber a system dependent number unique for the subsystem.
**    Severity a FunctionStatus_e for the message.
**    Number a Log_e that identifies the message.
**    MoreDescription a string containing more description text
**    MoreAction a string containing more action text.
**  Output parameters:
**  Other interfaces:
**    Output file: Log file
**  Description:
**    Writes a log message in the log file using the format:
**        Date - Hostname - Pid - System - Subsystem - ErrorNumber - Severity - Description - Action -
**  References:
**  Libraries:
**  Created:	2.3.94
**  Author:	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void 
Log__Msg(Log_System_e System, const char *Subsystem, char *ErrorNumber,
	 FunctionStatus_e Severity, Log_e Number, const char *MoreDescription,
	 const char *MoreAction)
{
    typedef struct {
	char *Description;
	char *Action;
    } LogData_s;
    /*------------------------------------------------------------------------*\
    **  static LogData containing Description and Action parts
    \*------------------------------------------------------------------------*/
    static const char Substitute = '=';
    static const char *Log_SystemStr[] = { "LEVEL_0_TO_1", "TAPE_GENERATOR", "EXTRACT_DATA", "BDDN_INTERFACE" };
    static const LogData_s LogData[] = {
	{"", "No further action needed "},
	{"Program error: ", "Contact the GDP level 0 to 1 development team "},
	{"Problems ", ""},
	{"Can't open ", "Check the file "},
	{"Can't find ", ""},
	{"Can't read ", ""},
	{"Can't write ", ""},
	{"Can't rename ", ""},
	{"Can't remove ", ""},
	{"Can't reopen ", ""},
	{"Can't close ", ""},
	{"Can't calculate ", ""},
	{"Can't allocate memory ", "Increment the swap space and start the system again"},
	{"Can't generate ", ""},
	{"Numerical Recipes error: ", "Contact the GDP level 0 to 1 development team "},
	{"Invalid ", ""},
	{"Inconstistent ", ""},
	{"Error during range check ", ""},
	{"Sequence error ", ""},
	{"Unknown ", ""},
	/* NEXT */
    };
    char Date[LOG_SUBFIELD_SIZE];
    static char AuxSubsystem[LOG_SUBFIELD_SIZE], AuxDescription[LOG_FIELD_SIZE], AuxAction[LOG_FIELD_SIZE];
    char *Ptr;


#ifdef DEBUG
    /*------------------------------------------------------------------------*\
    ** Check if Number is defined in LogData
    \*------------------------------------------------------------------------*/
    if ((Number < 0) || (Number >= (sizeof(LogData) / sizeof(LogData_s))))
    {
	fprintf(stderr, "LOG ERROR - Log__Msg: Program error. Ilegal enumeration value 'LongNumber_e %d'\n", Number);
	return;
    }

    /*------------------------------------------------------------------------*\
    ** Check if Severity is defined
    \*------------------------------------------------------------------------*/
    if ((Severity < INFO) || (Severity > FATAL))
    {
	fprintf(stderr, "LOG ERROR - Log__Msg: Program error. Ilegal enumeration value 'FunctionStatus_e %d'\n", Number);
	return;
    }
#else
    /*------------------------------------------------------------------------*\
    ** If Severity is other than ERROR or FATAL don't write the msg
    \*------------------------------------------------------------------------*/
    if (Severity < ERROR)
    {
	return;
    }

#endif /* DEBUG */

    /*------------------------------------------------------------------------*\
    ** Set the current Date using the format MM DD HH:MM:SS
    \*------------------------------------------------------------------------*/
    if (!Sys__GetDate(Date, LOG_SUBFIELD_SIZE))
    {
	fprintf(stderr, "LOG ERROR - Log__Msg: Can't set the system date\n");
    }

    strncpy(AuxSubsystem, Subsystem, sizeof(AuxSubsystem)-1);
    for (Ptr=AuxSubsystem; *Ptr != '\0'; Ptr++)
    {
	if ((*Ptr == Separator) || (*Ptr == '\n')) *Ptr = Substitute;
    }
    strncpy(AuxDescription, LogData[Number].Description, sizeof(AuxDescription)-1);
    strncat(AuxDescription, MoreDescription, sizeof(AuxDescription)-strlen(AuxDescription)-1);
    for (Ptr=AuxDescription; *Ptr != '\0'; Ptr++)
    {
	if ((*Ptr == Separator) || (*Ptr == '\n')) *Ptr = Substitute;
    }
    strncpy(AuxAction, LogData[Number].Action, sizeof(AuxAction)-1);
    strncat(AuxAction, MoreAction, sizeof(AuxAction)-strlen(AuxAction)-1);
    for (Ptr=AuxAction; *Ptr != '\0'; Ptr++)
    {
	if ((*Ptr == Separator) || (*Ptr == '\n')) *Ptr = Substitute;
    }
    if ( (fprintf(Stream, MsgFormat,
		  Date,
		  Hostname,
		  Pid,
		  Log_SystemStr[System],
		  AuxSubsystem,
		  ErrorNumber,
		  FunctionStatus2Str(Severity),
		  AuxDescription,
		  AuxAction) < 0) ||
	 (fflush(Stream) == EOF) )
    {
	fprintf(stderr, "LOG ERROR - Log__Msg: Problems while writting a log message\n");
    }
}


/*----------------------------------------------------------------------------*\
**                                Log__Info
**  Input parameters:
**    System a Log_System_e that identifies the calling system
**    Subsystem a two character string that identifies the calling subsystem
**    ErrorNumber a system dependent number unique for the subsystem.
**    MoreDescription a string containing more description text
**  Output parameters:
**  Other interfaces:
**    Output file: Log file
**  Description:
**    Writes a log message in the log using the Log__Msg function:
**  References:
**  Libraries:
**  Created:	22.3.94
**  Author:	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void 
Log__Info(Log_System_e System, const char *Subsystem, char *ErrorNumber,
	  const char *MoreDescription)
{
    Log__Msg(System, Subsystem, ErrorNumber, INFO, LOG_NUM, MoreDescription, "");
}


#ifdef __cplusplus
extern "C"      /* External C functions */
{
#endif

    static Bool 
    Log__ReadField(FILE *, char *, short , char );

#ifdef __cplusplus
}               /* end C linkage */
#endif
/*----------------------------------------------------------------------------*\
**                                Log__ReadField
**  Input parameters:
**    Stream where to read
**    Size maximum of the field
**    Separator of the fields
**  Output parameters:
**    Field string to be readed
**  Other interfaces:
**    Output file: Log file
**  Description:
**    Reads a log message writen by Log__Msg or Log__Info
**    In the case of failure of EOF, False is returned, True otherwise.
**  References:
**  Libraries:
**  Created:	15.9.94
**  Author:	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
static Bool
Log__ReadField(FILE *Stream, char *Field, short Size, char Separator)
{
    char Ch;
    short i=0;

    while ((i< Size) && ((Ch = fgetc(Stream)) != EOF) && (Ch != '\n') && (Ch != Separator))
    {
	Field[i++] = Ch;
    }
    if (Ch != EOF)
    {
	fgetc(Stream);
	if (i > 0) i--;
	Field[i] = '\0';
    }
    return (Ch == Separator)? True: False;
}


/*----------------------------------------------------------------------------*\
**                                Log__Read
**  Input parameters:
**  Output parameters:
**    Date a string for the log date in the format: MM DD HH:MM:SS
**    Hostname a string for the hostname
**    Pid a pit_d for the process id
**    System a string for the system
**    Subsystem a string for the subsystem
**    ErrorNumber a short for the subsystem error identification
**    Description a string for the log description
**    Action a string for the action to be taked
**  Other interfaces:
**    Output file: Log file
**  Description:
**    Reads a log message writen by Log__Msg or Log__Info
**    In the case of failure of EOF, False is returned, True otherwise.
**  References:
**  Libraries:
**  Created:	2.3.94
**  Author:	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
Bool
Log__Read(char *Date, char *Hostname, long *Pid, char *System,
	  char *Subsystem, char *ErrorNumber, char *Severity, char *Description,
	  char *Action)
{
    if (Log__ReadField(Stream, Date, LOG_SUBFIELD_SIZE, Separator) &&
	Log__ReadField(Stream, Hostname, LOG_SUBFIELD_SIZE, Separator) &&
	(fscanf(Stream, PidFormat, Pid) == 1) &&
	Log__ReadField(Stream, System, LOG_SUBFIELD_SIZE, Separator) &&
	Log__ReadField(Stream, Subsystem, LOG_SUBFIELD_SIZE, Separator) &&
	Log__ReadField(Stream, ErrorNumber, LOG_SUBFIELD_SIZE, Separator) &&
	Log__ReadField(Stream, Severity, LOG_SUBFIELD_SIZE, Separator) &&
	Log__ReadField(Stream, Description, LOG_FIELD_SIZE, Separator) &&
	Log__ReadField(Stream, Action, LOG_FIELD_SIZE, Separator) ) 
    {
	return True;
    }
    else
    {
	if (!feof(Stream))
	{
	    fprintf(stderr, "LOG ERROR - Log__Read: Can`t read a log file line. Invalid format\n");
	}
	return False;
    }
}



/*----------------------------------------------------------------------------*\
**                                Log__Close
**  Input parameters:
**  Output parameters:
**  Other interfaces:
**  Description:
**    Close the log file
**  References:
**  Libraries:
**  Created:	2.3.94
**  Author:	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
Bool
Log__Close()
{
    if (IsOpen)
    {
	IsOpen = False;
	return ((Stream != stderr) && (fclose(Stream) != 0))? False: True;
    }
    else
    {
	fprintf(stderr, "LOG ERROR - Log__Close: Trying to close a log file that is not open\n");
	return False;
    }
}

