/*----------------------------------------------------------------------------*\
**                                PCD.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)PCD.c	5.2 08/19/02, Copyright DLR 1998"

#include "pcd.h"
#include "smcd.h"
#include "log.h"
#include "dataconv.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)PCD.c 5.2 08/19/02, Copyright DLR 1998")


#define	SS_ID	("PCD")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                PCD__ReadTime
**  Input parameters:
**	Stream			input file
**      FilePos			file position of the read
**      PCD_Size		size of the PCD
**	PCD			NULL_OBJECT = the PCD is read local only the
**						time is returned
**				else = the PCD is read into the variable PCD
**					and the time is returned
**  Output parameters:
**      PCD_Time		the time of the PCD
**	PCD			NULL_OBJECT = the PCD is read local only the
**						time is returned
**				else = the PCD is read into the variable PCD
**					and the time is returned
**	PCD__ReadTime		SUCCESS == time and PCD read
**				FATAL	== time and PCD not read
**
**  Other interfaces:
**    Input file Level 1 Product defined via Stream
**
**  Description:
**	Function to read the next PCD and return the Time. If the input
**	variable PCD is NULL_OBJECT, only a local read of the PCD is done. If
**      the input variable PCD is not NULL_OBJECT, the read PCD is returned
**      via the variable PCD.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__ReadTime(FILE* Stream, long FilePos, long PCD_Size, UTC_s* PCD_Time,
	      PCD_s* PCD, Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    PCD_s PCD_Local;
    PCD_s *PCD_pt;
    char LogBuffer[LOG_FIELD_SIZE];
 
    /*--------------------------------------------------------------------*\
    ** check PCD is defined
    \*--------------------------------------------------------------------*/
    if (PCD == 0)
    {
	PCD_pt = &PCD_Local;
    }
    else
    {
	PCD_pt = PCD;
    }

    /*--------------------------------------------------------------------*\
    ** read the first PCD
    \*--------------------------------------------------------------------*/
    if (fseek ( Stream, FilePos, SEEK_SET) == -1) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__101", Status,
		  LOG_CANNOT_READ,
		  "PCD of history from Level 1 Product file (fseek)", "");
	return Status;
    }
    
    if (PCD__read(PCD_pt, Stream, convert_sun_2_host) != SUCCESS) 
    {
	    Status = FATAL;
	    sprintf(LogBuffer,
		    "the PCD of the given FilePos %d from the Level 1 Product",
		    FilePos);
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "5.2__113", Status,
		     LOG_CANNOT_READ, LogBuffer, "");
	    return Status;
    }

    /*--------------------------------------------------------------------*\
    ** define the search condition
    \*--------------------------------------------------------------------*/
    *PCD_Time = PCD_pt->Geolocation.DateUTC;

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PCD__SearchTime
**  Input parameters:
**    Stream		Input stream
**    xCD__ReadTime	function to read either the PCD or SMCD
**    FirstFilePosPCD	the file position of the first PCD or SMCD
**    NoOf_PCD_Rec	number of PCD or SMCD in the file
**    PCD_Size		the size of the PCD or SMCD
**    SearchTime	Time to be searched within the PCD list of the Level 1
**			Product
**    MeasurementPixel	the number of measurement pixel
**
**  Output parameters:
**    FoundMeasPixel	PCD or SMCD index (Measurement pixel number) of the
**				given SearchTime
**    FoundTime		the time of the PCD or SMCD which corresponds to the
**				returned PCD or SMCD index (Measurement pixel
				number)
**    PCD__SearchTime		SUCCESS == time found in Level 1 Product
**				FATAL	== time not found in Level 1 Product
**
**  Other interfaces:
**    Input file Level 1 Product defined in Config
**
**  Description:
**	Function to search a given Time within the PCD or SMCD list of the Level
**	1 Product. If the search was successful 'FoundMeasurementPixel' contains
**	the corresonding PCD or SMCD index, 'FoundTime' the corresponding PCD
**      or SMCD time.
**	The search run is started by the given Pixel index ('MeasurementPixel').
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__SearchTime(FILE* Stream,
		FunctionStatus_e (*xCD__ReadTime)
		(FILE *Stream, long FilePos, long PCD_Size, UTC_s *PCD_Time, 
		 PCD_s *PCD, Bool convert_sun_2_host),
		long FirstFilePosPCD, short NoOf_PCD_Rec,
		long PCD_Size, UTC_s SearchTime, int MeasurementPixel,
		int* FoundMeasurementPixel, UTC_s* FoundTime,
		Bool convert_sun_2_host)
{
    FunctionStatus_e	Status		= SUCCESS;
    UTC_s			PrevTime;
    UTC_s			TimeDiff;
    UTC_s			PrevTimeDiff;
    int			PrevMeasurementPixel;
    int			Limit;
    Search_e		Search;
    char			CompMode;
    int			SearchDelta;
    char			ASCII_Time[ 30];
    UTC_s			PixelTime;
    char LogBuffer[LOG_FIELD_SIZE];
#ifdef DEBUG
    char debug_message[255];
#endif

    /*--------------------------------------------------------------------*\
    ** read the first PCD or SMCD
    \*--------------------------------------------------------------------*/
    if ((*xCD__ReadTime)(Stream, FirstFilePosPCD + PCD_Size * MeasurementPixel,
			 PCD_Size, &PixelTime, 0, convert_sun_2_host) 
	!= SUCCESS)
    {
	Status = FATAL;
	UTC_2_ASCII( SearchTime, ASCII_Time);
	sprintf( LogBuffer,
		 "the PCD or SMCD of the given time %s from the Level 1 Product",
		 ASCII_Time);
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__203", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** define the search condition
    \*--------------------------------------------------------------------*/
    if ( UTC_Comp( '<', PixelTime, SearchTime))
    {
	SearchDelta = 1;
	Search = SEARCH_UP;
	CompMode = '>';
	Limit = NoOf_PCD_Rec;
    }
    else if ( UTC_Comp( '>', PixelTime, SearchTime))
    {
	SearchDelta = -1;
	Search = SEARCH_DOWN;
	CompMode = '<';
	Limit = -1;
    }
    else if ( UTC_Comp( '=', PixelTime, SearchTime))
    {
	SearchDelta = 0;
	Search = SEARCH_FOUND;
	CompMode = '=';
	Limit = 0;
    }
    else
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__235", Status,
		  LOG_NUM, "Program error UTC_Comp", "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** Search loop
    \*--------------------------------------------------------------------*/
    PrevTime		= PixelTime;
    PrevMeasurementPixel	= MeasurementPixel;
    while (((Search == SEARCH_DOWN) || (Search == SEARCH_UP)) &&
	   ((MeasurementPixel + SearchDelta) != Limit))
    {
	/*--------------------------------------------------------------------*\
	** Read PCD or SMCD
	\*--------------------------------------------------------------------*/
	PrevTime		= PixelTime;
	PrevMeasurementPixel= MeasurementPixel;
	MeasurementPixel	= MeasurementPixel + SearchDelta;

	if ((*xCD__ReadTime)(Stream, FirstFilePosPCD + PCD_Size
			     * MeasurementPixel, PCD_Size, &PixelTime, 0,
			     convert_sun_2_host) != SUCCESS) {
	    Status = FATAL;
	    UTC_2_ASCII( SearchTime, ASCII_Time);
	    sprintf( LogBuffer,
		     "the PCD or SMCD of the given time %s from the Level 1 Product",
		     ASCII_Time);
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__263", Status,
		      LOG_CANNOT_READ, LogBuffer, "");
	    return Status;
	}

	/*--------------------------------------------------------------------*\
	** check if time found
	\*--------------------------------------------------------------------*/
	if ( UTC_Comp( CompMode, PixelTime, SearchTime) ||
	     UTC_Comp( '=', PixelTime, SearchTime))
	    Search = SEARCH_FOUND;

#ifdef DEBUG  
	sprintf(debug_message, "Search time PCD or SMCD[ %d] %s Days: %u, "
	       "Msecs: %u\n", MeasurementPixel, ASCII_Time, PixelTime.Days,
	       PixelTime.Msecs);
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
#endif

    }

    if (Search == SEARCH_FOUND)
    {
	/*--------------------------------------------------------------------*\
	** calculate time differences in case search down and equal yo
	** select the nearest measurement pixel
	\*--------------------------------------------------------------------*/
	if ((CompMode == '<') || (CompMode == '='))
	{
	    if ((UTC_Sub( SearchTime, PixelTime, &TimeDiff) != SUCCESS)
		|| (UTC_Sub( PrevTime, SearchTime, &PrevTimeDiff) != SUCCESS))
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__296", Status,
			  LOG_NUM, "Program error UTC_Sub (PCD__SearchTime)", "");
		return Status;
	    }      
	}
	else
	{
	    /*--------------------------------------------------------------------*\
	    ** calculate time differences in case search up to select the nearest
	    ** measurement pixel
	    \*--------------------------------------------------------------------*/
	    if (CompMode == '>')
	    {
		if ((UTC_Sub( PixelTime, SearchTime, &TimeDiff) !=
		     SUCCESS) ||
		    (UTC_Sub( SearchTime, PrevTime, &PrevTimeDiff) != SUCCESS))
		{
		    Status = FATAL;
		    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__314", Status,
			      LOG_NUM, "Program error UTC_Sub (PCD__SearchTime)", "");
		    return Status;
		}          
	    }
	    /*--------------------------------------------------------------------*\
	    ** program error
	    \*--------------------------------------------------------------------*/
	    else
	    {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__325", Status,
			  LOG_NUM, "Program error PCD__SearchTime", "");
		return Status;
	    }
	}

	/*-----------------------------------------------------------------------*\
	** select the nearest measurement pixel
	\*-----------------------------------------------------------------------*/
	if (UTC_Comp( '>', TimeDiff, PrevTimeDiff))
	{
	    *FoundMeasurementPixel	= PrevMeasurementPixel;
	    *FoundTime		= PrevTime;
	}
	else
	{
	    *FoundMeasurementPixel	= MeasurementPixel;
	    *FoundTime		= PixelTime;
	}
#ifdef DEBUG  
	UTC_2_ASCII( *FoundTime, ASCII_Time);
	sprintf(debug_message, "Time found at PCD or SMCD[ %d], FoundTime "
	       "Days: %s %u, Msecs: %u\n", *FoundMeasurementPixel,
	       ASCII_Time, FoundTime->Days, FoundTime->Msecs);
	Debug__print(debug_ptr, DBG_EXTRACTING, DEBUG_LEVEL, debug_message);
#endif
    }
    else
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__355", Status,
		  LOG_NUM, "Time not found in PCDs or SMCDs", "");
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PCD__GetTimeRange
**  Input parameters:
**    Stream			input stream
**    xCD__ReadTime		function to read the PCD or SCD or MCD
**    FilePosPCD		file position of the PCD or SCD or MCD
**    NoOf_PCD_Rec		number of PCD or SCD or MCD records
**    PCD_Size			size of PCD or SCD or MCD
**
**  Output parameters:
**	PCD__GetTimeRange	SUCCESS == time found in Level 1 Product
**				FATAL	== time not found in Level 1 Product
**
**  Other interfaces:
**    Input file Level 1 Product defined in Config
**
**  Description:
**	Function to return the time range of the PCD, SCD or MCD.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__getTimeRange(FILE* Stream, 
  		  FunctionStatus_e (*xCD__ReadTime)
  		  (FILE* Stream, long FilePos, long PCD_Size, 
 		   UTC_s* PCD_Time, PCD_s* PCD, Bool convert_sun_2_host),
  		  long FilePosPCD, short NoOf_PCD_Rec, long PCD_Size,
 		  UTC_s* TimeFirst_PCD, UTC_s* TimeLast_PCD,
		  Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    UTC_s PixelTime;

    /*----------------------------------------------------------------------*\
    ** read the first PCD or SMCD and the last PCD or SMCD and get the time of
    ** the first PCD or SMCD and the last PCD or SMCD
    \*----------------------------------------------------------------------*/
    if (xCD__ReadTime(Stream, FilePosPCD, PCD_Size, &PixelTime, 0,
		      convert_sun_2_host) != SUCCESS) 
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__409", Status, LOG_CANNOT_READ,
		  "first PCD, SCD or MCD from Level 1 Product file", "");
	return Status;
    }
    else
    {     
	*TimeFirst_PCD = PixelTime;
	/*--------------------------------------------------------------------*\
	** read the last PCD or SMCD to get the end of the time of the product
	\*--------------------------------------------------------------------*/
	if (NoOf_PCD_Rec > 1) {
	    if ((*xCD__ReadTime)(Stream, FilePosPCD + PCD_Size 
				 * (NoOf_PCD_Rec - 1), PCD_Size, &PixelTime, 0,
				 convert_sun_2_host) != SUCCESS) {
		Status = FATAL;
		Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__424", Status,
			  LOG_CANNOT_READ,
			  "last PCD, SCD or MCD from Level 1 Product file", "");
		return Status;
	    }
	}
	*TimeLast_PCD = PixelTime;	
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PCD__GetSpectrumParameter
**  Input parameters:
**    SPH1: SPH1 of Level 1 Product
**    PCD: PCD of the of Level 1 Product 
**
**  Output parameters:
**    PixelPar			pixel specific product parameter
**    GetSpectrumParameter	SUCCESS	= Parameter extracted
**				else	= error during parameter extraction
**
**  Other interfaces:
**
**  Description:
**    Function to get the pixel specific product parameter from the PCD.
**    The Parameter are listed under Output Parameter.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__GetSpectrumParameter(const SPH1_s *SPH1, const PCD_s *PCD,
			  PixelPar_s *PixelPar)
{
    FunctionStatus_e	Status		= SUCCESS;
    int			i;

    /*----------------------------------------------------------------------*\
    ** get the common pixel specific product parameter
    \*----------------------------------------------------------------------*/
    PixelPar->Date			= PCD->Geolocation.DateUTC;
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++) {
	PixelPar->ScienceIndex[i] = PCD->ScienceIndex[i];
    }
    PixelPar->IndexSpectralCalPar = PCD->IndexSpectralCalPar;
    PixelPar->IndexLeakagePar = PCD->IndexLeakagePar;
    PixelPar->DarkCurrentCorFactor = PCD->DarkCurrentCorFactor;
    PixelPar->DarkCurrentNoiseCorFactor	= PCD->DarkCurrentNoiseCorFactor;

    /*----------------------------------------------------------------------*\
    ** get the pixel specific auxiliary parameter ( subset counter, PMD, 
    ** AverageMode, RawDataCorFactor... )
    \*----------------------------------------------------------------------*/
    SPH1__getAuxiliaryParameter(SPH1, PCD->AuxiliaryData,
				&PixelPar->AuxiliaryPar);

    /*----------------------------------------------------------------------*\
    ** get the earthshine specific pixel parameter
    \*----------------------------------------------------------------------*/
    PixelPar->Specific.Earth.Geolocation = PCD->Geolocation;
    PixelPar->Specific.Earth.PolarizationPar = PCD->PolarizationPar;

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PixelPar__Init
**  Input parameters:
**    SPH1		SPH1 of Level 1 Product
**    MeasurementPixel	number of measurement pixel
**    ProductType	product type to be produced (Earthshine, moonshine,
**			 sunshine)
**    xCD		PCD, SCD or MCD of the of Level 1 Product
**    xCD_Size		size of xCD
**
**  Output parameters:
**    PixelPar			pixel spectific pectrum parameter of product
**    PixelPar__Init		SUCCESS	= Parameter extracted
**				else	= error during parameter extraction
**
**  Other interfaces:
**
**  Description:
**    Function to get the pixel specific parameter from the PCD, SCD or MCD.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PixelPar__Init(const SPH1_s *SPH1, int MeasurementPixel,
	       ProductType_e ProductType, const PCD_s *xCD, long xCD_Size,
	       PixelPar_s *PixelPar)
{
    FunctionStatus_e Status = SUCCESS;

    PixelPar->MeasurementPixel = MeasurementPixel;
    switch (ProductType) {
    case PR_EARTHSHINE:	{
	PCD__GetSpectrumParameter(SPH1, xCD, PixelPar);
	break;
    }
    case PR_SUNSHINE:
    case PR_MOONSHINE: {
	SMCD__GetSpectrumParameter(ProductType, SPH1, ( SMCD_s*)xCD, PixelPar);
	break;
    }
    default: {
	Status = FATAL;
	break;
    }
    }
    return Status;
}


/*----------------------------------------------------------------------------*\
**                           PCD__Sun2Host
**  Input parameters:
**    PCD			in Sun memory representation
**  Output parameters:
**    PCD			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the PCD struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PCD__Sun2Host( PCD_s *Param)
{
    int	i;

    /* float */
    Param->DarkCurrentCorFactor = SunFloat2Float( (unsigned char*)
						  &Param->DarkCurrentCorFactor);
    /* float */
    Param->DarkCurrentNoiseCorFactor = SunFloat2Float( (unsigned char*)
						       &Param->DarkCurrentNoiseCorFactor);
    /* short */
    Param->IndexSpectralCalPar = SunShort2Short( (unsigned char*)
						 &Param->IndexSpectralCalPar);
    /* short */
    Param->IndexLeakagePar = SunShort2Short( (unsigned char*)
					     &Param->IndexLeakagePar);
    for ( i = 0; i < AUXILIARY_DATA_SIZE; i++)
    {
	/* short */
	Param->AuxiliaryData[ i] = SunShort2Short( (unsigned char*)
						   &Param->AuxiliaryData[ i]);
    }
    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
	/* short */
	Param->ScienceIndex[ i] = SunShort2Short( (unsigned char*)
						  &Param->ScienceIndex[ i]);
    }
}


/*----------------------------------------------------------------------------*\
**                                PCD__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		PCD
**
**  Other interfaces:
**
**  Description:
**	Function to read all PCD entries from the given 
**	input stream into the PCD structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PCD__read(PCD_s *Param, FILE * Stream, Bool convert_sun_2_host)
{
    int	i;
    FunctionStatus_e Status = SUCCESS;
    long NoOfReadBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];
			 
    /*--------------------------------------------------------------------*\
    ** read Geolocation
    \*--------------------------------------------------------------------*/
    if (GeolocationC__read(&Param->Geolocation, Stream, convert_sun_2_host)
	!= SUCCESS)
    {
	Status = FATAL;
	sprintf(LogBuffer, "PCD, Geolocation entry, #read bytes: %d",
		NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__638", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** read Dark Current & Noise Correction factor
    \*--------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->DarkCurrentCorFactor, 1,
			   sizeof( Param->DarkCurrentCorFactor), Stream);
    NoOfReadBytes += fread( &Param->DarkCurrentNoiseCorFactor, 1,
			    sizeof( Param->DarkCurrentNoiseCorFactor), Stream);
    if (NoOfReadBytes != PCD_DARK_CURRENT_AND_NOISE_COR_SIZE) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "PCD, Dark Current & Noice Correction Factor, #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__655", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** read index to Spectral Calibration Parameter
    \*--------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->IndexSpectralCalPar, 1,
			   PCD_INDEX_SPECTRAL_CAL_PAR, Stream);
    if ( NoOfReadBytes != PCD_INDEX_SPECTRAL_CAL_PAR) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "PCD, Index to Spectral Calibration Parameter, #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__670", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** read index to Leakage calibration parameter
    \*--------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->IndexLeakagePar, 1,
			   PCD_INDEX_LEAKAGE_COR_PAR, Stream);
    if ( NoOfReadBytes != PCD_INDEX_LEAKAGE_COR_PAR) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "PCD, Index to Pol. Sens. Calibration Parameter, #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__685", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
	}

    /*--------------------------------------------------------------------*\
    ** read Polarization Parameter
    \*--------------------------------------------------------------------*/
    if (PolarizationC__read(&Param->PolarizationPar, Stream, convert_sun_2_host)
	!= SUCCESS)
    {
	Status = FATAL;
	sprintf(LogBuffer, "PCD, Polarization Parameter, #read bytes: %d",
		NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__699", Status, LOG_CANNOT_READ,
		 LogBuffer, "");
	return Status;
    }
    
    /*----------------------------------------------------------------------*\
    ** read MPH_L1, SPH_L1, AuxiliaryData, ScienceIndex
    \*----------------------------------------------------------------------*/
    NoOfReadBytes = xCD__Read(Stream, Param->MPH_L1, Param->SPH_L1,
			      Param->AuxiliaryData, Param->ScienceIndex);
    if (NoOfReadBytes != (PCD_MPH_EXTRACT_SIZE + PCD_SPH_EXTRACT_SIZE +
			  PCD_INSTRUMENT_HEADER_SIZE +
			  SCIENCE_DATA_DEFINED * PCD_SCIENCE_INDICE_SIZE)) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "PCD, xCD, #read bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__715", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	PCD__Sun2Host( Param);
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PCD__Show
**  Input parameters:
**	Stream		output stream
**	Param		PCD Structure
**
**  Output parameters:
**
**  Other interfaces:
**	PCD Structure written to the output stream
**
**  Description:
**	Function to write the PCD Structure to the output
**	stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PCD__show(const PCD_s *Param, FILE* Stream, Bool FullInfo)
{
    int	i;

    if (FullInfo)
    {
	GeolocationC__show(&Param->Geolocation, Stream);
	fprintf( Stream, "\n");
    }

    fprintf( Stream, "Dark Current & Noise Correction Factor\n");
    fprintf( Stream, "%f %f\n\n", Param->DarkCurrentCorFactor,
	     Param->DarkCurrentNoiseCorFactor);

    PolarizationC__show(&Param->PolarizationPar, Stream);
    fprintf( Stream, "\n");

    if (FullInfo)
    {
	fprintf( Stream, "MPH_L1_SIZE %d", MPH_L1_SIZE);
	for (i = 0; i < MPH_L1_SIZE; i++)
	{
	    if (i % 10 == 0)
		fprintf( Stream, "\ni: %3d", i);
	    fprintf( Stream, "%3d ", Param->MPH_L1[ i]);
	}
	fprintf( Stream, "\n\n");

	fprintf( Stream, "SPH_L1_SIZE %d", SPH_L1_SIZE);
	for (i = 0; i < SPH_L1_SIZE; i++)
	{
	    if (i % 10 == 0)
		fprintf( Stream, "\ni: %3d", i);
	    fprintf( Stream, "%3d ", Param->SPH_L1[ i]);
	}
	fprintf( Stream, "\n\n");

	fprintf( Stream, "AUXILIARY_DATA_SIZE %d", AUXILIARY_DATA_SIZE);
	for (i = 0; i < AUXILIARY_DATA_SIZE; i++)
	{
	    if (i % 10 == 0)
		fprintf( Stream, "\ni: %3d", i);
	    fprintf( Stream, "%5d ", Param->AuxiliaryData[ i]);
	}
	fprintf( Stream, "\n\n");
    }

    fprintf( Stream, "Spectral Calibration Index %d\n",
	     Param->IndexSpectralCalPar);
    fprintf( Stream, "Leakage Parameter Index    %d\n",
	     Param->IndexLeakagePar);

    fprintf( Stream, "Science Data Index\n");
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
	fprintf( Stream, "%s\t%d\n", ScienceData2Str((ScienceData_e)i),
		 Param->ScienceIndex[ i]);
    }
    fprintf( Stream, "\n");
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
