/*----------------------------------------------------------------------------*\
**                                PIR.c
**
**  Class PIR definition of methods
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)PIR.c	5.1 02/26/99, Copyright DLR 1998"

#include "pir.h"
#include "log.h"


#define	SccsId	("@(#)PIR.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("PIR")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                            PIR__generate
**  Input parameters:
**    OrbitNumber               Orbit number
**    ProductType               type of the product
**    ProcessingInfo            Processing Information of the product
**  Output parameters:
**    PIR                       the product identifier record (PIR)
**    Generate_PIR              SUCCESS = PIR generation successful
**                              else    = not successful
**  Other interfaces:
**  Description:
**    Function to generate the Product Identifier Record( PIR). The PIR
**    is generated with some fix values and some input values( see Inputs).
**    The Header is generated entry by entry. This ensures that there are no
**    unnessessary wholes(chars) within the generated PIR.
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:    10.8.94
**  Author:     Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PIR__generate(char* PIR, unsigned long OrbitNumber, const char* ProductType,
              const char* ProcessingInfo)
{
   const unsigned long NUMBER_OF_ORBITS        = 1;

    int                 i;
    char                        I_O_String[ 20];
    FunctionStatus_e    Stat;

    Stat = SUCCESS;

    for ( i = 0; i < PIR_SATELLITE_ID_SIZE; i++)
        PIR[ PIR_SATELLITE_ID_ENTRY + i] = PIR_SATELLITE_ID_STRING[ i];

    for ( i = 0; i < PIR_SENSOR_ID_SIZE; i++)
        PIR[ PIR_SENSOR_ID_ENTRY + i] = PIR_SENSOR_ID_STRING[ i];

    if (sprintf( I_O_String, "%-.*u", PIR_ORBIT_NUMBER_SIZE, OrbitNumber) !=
        PIR_ORBIT_NUMBER_SIZE)
    {
        Stat = FATAL;
    }
    for ( i = 0; i < PIR_ORBIT_NUMBER_SIZE; i++)
        PIR[ PIR_ORBIT_NUMBER_ENTRY + i] = I_O_String[ i];

    if (sprintf( I_O_String, "%-.*u", PIR_NUMBER_OF_ORBITS_SIZE,
                 NUMBER_OF_ORBITS) != PIR_NUMBER_OF_ORBITS_SIZE)
    {
        Stat = FATAL;
    }
    for ( i = 0; i < PIR_NUMBER_OF_ORBITS_SIZE; i++)
        PIR[ PIR_NUMBER_OF_ORBITS_ENTRY + i] = I_O_String[ i];

    for ( i = 0; i < PIR_ACQ_FAC_ID_SIZE; i++)
        PIR[ PIR_ACQ_FAC_ID_ENTRY + i] = ProcessingInfo[ i];

    for ( i = 0; i < PIR_PRODUCT_TYPE_SIZE; i++)
        PIR[ PIR_PRODUCT_TYPE_ENTRY + i] = ProductType[ i];

    PIR[ PIR_SPACE_ENTRY] = ' ';


    for ( i = 0; i < PIR_PROC_FAC_ID_SIZE; i++)
        PIR[ PIR_PROC_FAC_ID_ENTRY + i] = PIR_PROC_FAC_ID_STRING[ i];

    for ( i = 0; i < PIR_PROC_DATE_SIZE; i++)
        PIR[ PIR_PROC_DATE_ENTRY + i] = ProcessingInfo[ PIR_ACQ_FAC_ID_SIZE + i];

    for ( i = 0; i < PIR_PROC_TIME_SIZE; i++)
        PIR[ PIR_PROC_TIME_ENTRY + i] = ProcessingInfo[ PIR_ACQ_FAC_ID_SIZE +
                                                      PIR_PROC_DATE_SIZE + i];

    return Stat;
}

/*----------------------------------------------------------------------------*\
**                        PIR__writeExtractLevel_1_Product
**  Input parameters:
**    Stream		File stream of the output file
**    PIR               Product Identifier Record of the Product
**
**  Output parameters:
**    PIR__WriteExtr		SUCCESS	= output ok
**				else	= any error during output
**
**  Other interfaces:
**    output file specified by Stream
**
**  Description:
**    This Function writes the PIR into the output file. The output is formated 
**    according the output format description of the Extract Level 1 Product.
**
**  References:
**    <none>
**  Libraries:
**    <none>
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PIR__writeExtractLevel_1_Product(const char PIR[PIR_SIZE], FILE* Stream)
{
    FunctionStatus_e Status = SUCCESS;
    int i;

    /*------------------------------------------------------------------------*\
    ** write PIR
    \*------------------------------------------------------------------------*/
    for ( i = 0; i < PIR_SIZE; i++)
    {
	fprintf( Stream, "%c", PIR[ i]);
    }
    fprintf( Stream, "\n");

    if ( ferror( Stream) != 0)
    {
	Status = FATAL;
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.1__146", Status, LOG_CANNOT_WRITE,
		  "PIR into exract level 1 file", "");
    }
    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PIR__read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	PIR		PIR
**
**  Other interfaces:
**
**  Description:
**	Function to read all PIR entries from the given 
**	input stream into the PIR data structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PIR__read(char PIR[PIR_SIZE], FILE* file_ptr)
{
  FunctionStatus_e Status = SUCCESS;
  char LogBuffer[LOG_FIELD_SIZE];
  long number_of_bytes = fread(PIR, 1, PIR_SIZE, file_ptr);

  if (number_of_bytes != PIR_SIZE)
  {
      Status = FATAL;
      sprintf(LogBuffer, "PIR, #read bytes: %d", number_of_bytes);
      Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1__182", Status,
	       LOG_CANNOT_READ, LogBuffer, "");
      return FATAL;
  }
  return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                PIR__Write
**  Input parameters:
**	Stream		output stream
**	PIR		PIR
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the PIR to the given 
**	output stream
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PIR__write(const char PIR[ PIR_SIZE], FILE* Stream)
{
  FunctionStatus_e	Status		= SUCCESS;
  long			NoOfBytes	= 0;
  char LogBuffer[LOG_FIELD_SIZE];

  /*----------------------------------------------------------------------*\
  ** write the PIR into the product
  \*----------------------------------------------------------------------*/
  NoOfBytes += fwrite( PIR, 1, PIR_SIZE, Stream);

  if (NoOfBytes != PIR_SIZE)
  {
    Status = FATAL;
    sprintf( LogBuffer, "PIR, #written bytes: %d", NoOfBytes);
    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__224", Status,
		 LOG_CANNOT_WRITE, LogBuffer, "");
  }

  return Status;
}

/*----------------------------------------------------------------------------*\
**                                PIR__Show
**  Input parameters:
**	Stream		output stream
**	PIR
**
**  Output parameters:
**
**  Other interfaces:
**	PIR written to the output stream
**
**  Description:
**	Function to write the PIR to the output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PIR__show(const char PIR[ PIR_SIZE], FILE * Stream)
{
  int	i;

  /*----------------------------------------------------------------------*\
  ** write the PIR into the product
  \*----------------------------------------------------------------------*/
  for (i = 0; i < PIR_SIZE; i++)
  {
    if (i % 10 == 0)
      fprintf( Stream, "\ni: %3d = ", i);
    fprintf( Stream, "%c ", PIR[ i]);
  }
  fprintf( Stream, "\n");
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
