/*----------------------------------------------------------------------------*\
**                                PolarizationC.c
**
**  Description:
** 
**  Version:    5.3
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)PolarizationC.c	5.3 08/19/02, Copyright DLR 1998"

#include <stdlib.h>
#include "polaionc.h"
#include "edr.h"
#include "log.h"
#include "dataconv.h"
#include "numrecip.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)PolarizationC.c 5.3 08/19/02, Copyright DLR 1998")


#define	SS_ID	("Polarization")


static double k1;
static double xd1;


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                PolarizationC__Calculate_p7_mean
**  Input parameters:
**    Polarization		array of all Polarizations according the given
**				integration time
**    NoOfPol			number of Polarizations
**    ASCII_Time		pixel time
**
**  Output parameters:
**    PolarizationParP7_Mean	polarization struct with mean value of 7th
**				point
**    PolarizationC__Calculate_p7_mean	SUCCESS	= Calibration of mean value of
**							7th point is ok
**					else	= 7th point calculation not ok
**
**  Other interfaces:
**
**  Description:
**    Function to calculate the mean value of all Polarizations belonging to
**    the given integration time. The calculation is ok when it was possible
**    to calculate all 7th points of each given Polarization.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	12.10.1995
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
**  Updated:    19.12.1995 Bernd Aberle, DLR/WT-DA-SE, Parameter ParamChi added in 
**                      PolarizationC__Calculate_p7
**  Updated:    17.04.2000 Bernd Aberle, DLR/IMF, case alpha == 0 considered in
**			PolarizationC__GDF
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PolarizationC__Calculate_p7_mean(PolarizationC_s *PolarizationParP7_Mean,
				PolarizationC_s Polarization[], 
				short NoOfPol, char *ASCII_Time)
{
    FunctionStatus_e	Status		= SUCCESS;
    short			i;
    Bool			p7_mean_ok;
    float 		ChiMean;
    float 		p7_ErrorMean;
    float			p7_CoefficientMean;
    float			p7_WavelengthMean;
    char LogBuffer[LOG_FIELD_SIZE];

    /*-------------------------------------------------------------------------*\
    ** calculate the 7th point of all given Polarizations and sum up their value
    \*-------------------------------------------------------------------------*/
    p7_mean_ok		= True;
    p7_CoefficientMean	= 0.0;
    p7_WavelengthMean	= 0.0;
    p7_ErrorMean		= 0.0;
    ChiMean		= 0.0;
    for ( i = 0; i < NoOfPol; i++)
    {
	/*----------------------------------------------------------------------*\
	** convert the pixel time
	\*----------------------------------------------------------------------*/
	if (Polarization[ PC_P7_BAND_1A].Coefficient[ PC_P7_BAND_1A] !=
	    INVALID_FRAC_POL)
	{
	    p7_CoefficientMean	+= Polarization[ i].Coefficient[ PC_P7_BAND_1A];
	    p7_WavelengthMean		+= Polarization[ i].Wavelength[ PC_P7_BAND_1A];
	    p7_ErrorMean		+= Polarization[ i].Error[ PC_P7_BAND_1A];
	    ChiMean			+= Polarization[ i].Chi;
	}
	else
	{
	    p7_mean_ok = False;
	}
    }

    /*-------------------------------------------------------------------------*\
    ** calculate the 7th point mean value if 7th point calculation is ok else
    ** return invalid value
    \*-------------------------------------------------------------------------*/
    for ( i = 0; i < POLARIZATION_COEFFICIENTS; i++)
    {
	PolarizationParP7_Mean->Coefficient[ i] 	= INVALID_FRAC_POL;
	PolarizationParP7_Mean->Error[ i]		= 0.0;
    }
    if (p7_mean_ok)
    {
	PolarizationParP7_Mean->Coefficient[ PC_P7_BAND_1A]	=
	    p7_CoefficientMean / NoOfPol;
	PolarizationParP7_Mean->Wavelength[ PC_P7_BAND_1A]	=
	    p7_WavelengthMean / NoOfPol;
	PolarizationParP7_Mean->Error[ PC_P7_BAND_1A]	=
	    p7_ErrorMean / NoOfPol;
	PolarizationParP7_Mean->Chi	= ChiMean / NoOfPol;
	Status = SUCCESS;
    }
    else
    {
	PolarizationParP7_Mean->Coefficient[ PC_P7_BAND_1A]	= INVALID_FRAC_POL;
	PolarizationParP7_Mean->Error[ PC_P7_BAND_1A]	= 0.0;
	Status = WARN;
	sprintf( LogBuffer,
		 "Not possible to calculate mean value of 7th point in Pixel %s",
		 ASCII_Time);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__147", Status, LOG_NUM, LogBuffer,
		  "");
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PolarizationC__CalculatePolParBand1a
**  Input parameters:
**    EDR			spectrum structure
**    OrigIntgTime		integration time of the band
**
**  Output parameters:
**    PolarizationParP7_Mean	polarization struct with P7_Mean
**    PolarizationC__CalculatePolParBand1a
**				SUCCESS	= Calibration of mean value of
**							7th point is ok
**				else	= 7th point calculation not ok
**
**  Other interfaces:
**
**  Description:
**    Function to calculate the mean value of all Polarizations belonging to
**    the given integration time. The calculation is ok when it was possible
**    to calculate all 7th points of each given Polarization.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	12.10.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PolarizationC__CalculatePolParBand1a(PolarizationC_s *PolarizationParP7_Mean,
				    const EDR_s *EDR, float OrigIntgTime)
{
    FunctionStatus_e	Status		= SUCCESS;
    float			IntgTime;
    short			NoOfPol;
    PolarizationC_s	*Polarization;
    int			i, j;
    char			ASCII_Time[ 30];
    char LogBuffer[LOG_FIELD_SIZE];

    /*---------------------------------------------------------------*\
    ** get ascii time of pixel
    \*---------------------------------------------------------------*/
    UTC_2_ASCII( EDR->PixelPar.Date, ASCII_Time);

    /*---------------------------------------------------------------*\
    ** get all Polarizations corresponding to that integration time
    \*---------------------------------------------------------------*/
    IntgTime = OrigIntgTime - INTG_TIME_CONV_FACTOR;
    if (IntgTime < 0.0)
	IntgTime = 0.0;
    NoOfPol = (short)(IntgTime / GROUND_PIXEL_TIME) + 1;
    if (EDR->N_PCD_HistoryRead < (NoOfPol - 1))
    {
	Status = WARN;
	sprintf( LogBuffer,
		 "Not possible to calculate mean value of 7th point (Band_1a) because NoOfPol(%d) > N_PCD_History(%d) in Pixel %s",
		 NoOfPol, EDR->N_PCD_HistoryRead, ASCII_Time);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__212", Status, LOG_NUM, LogBuffer,
		  "");
	/*---------------------------------------------------------------*\
	** set 7th point to INVALID_FRAC_POL
	\*---------------------------------------------------------------*/
	for ( i = 0; i < POLARIZATION_COEFFICIENTS; i++)
	{
	    PolarizationParP7_Mean->Coefficient[ i] 	= INVALID_FRAC_POL;
	    PolarizationParP7_Mean->Error[ i]		= 0.0;
	}
    }
    else
    {
#ifdef __cplusplus
	Polarization = new PolarizationC_s[ NoOfPol];
#else
	Polarization = (PolarizationC_s*)malloc( sizeof( PolarizationC_s) *
						NoOfPol);
#endif
	if (Polarization == 0)
	{
	    Status = FATAL;
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.3__234", Status,
		      LOG_CANNOT_ALLOCATE, "PolarizationC_s", "");
	    return Status;
	}
          
	Polarization[ NoOfPol - 1] = EDR->PixelPar.Specific.Earth.PolarizationPar;
	j = 1;
	for ( i = NoOfPol - 2 ; i >= 0; i--)
	{
	    Polarization[ i] =
		EDR->PCD_History[ EDR->N_PCD_History -
				j].Specific.Earth.PolarizationPar;
	    j += 1;
	}

	/*---------------------------------------------------------------*\
	** calculate the mean value of the 7th point and apply polarization
	\*---------------------------------------------------------------*/
	*PolarizationParP7_Mean = EDR->PixelPar.Specific.Earth.PolarizationPar;
	PolarizationC__Calculate_p7_mean( PolarizationParP7_Mean, 
					 Polarization, NoOfPol, ASCII_Time);

	/*---------------------------------------------------------------------*\
	** delete the Polarization
	\*---------------------------------------------------------------------*/
#ifdef __cplusplus
	delete [] Polarization;
#else
	free((void *)Polarization);
#endif

    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PolarizationC__Compute
**  Input parameters:
**    Fractional polarization values p (7)(in Self)
**    Wavelengths (7)(in Self)
**    Wavelengths of the given band (BAND_1A .. BAND_4)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    PixelDate (Ascii string)
**
**  Output parameters:
**    Computed polarisations for given band (BandCoefficients)
**
**  Other interfaces:
**
**  Description:
**    Finds a p value for each wavelength of the spectrum by interpolating the 7 p values (3 PMD, 3
**    overlap regions, Raleigh Single Scattering point (7th point))
**
**  References:
**    This function is described in the Module ...
**
**  Libraries:  numrecip
**  Version     1.1
**  Created:	28.7.94
**  Author:		Rob Spurr, DLR/WT-DA-SE
**  Revision:           Rob Spurr, 18 July 1995
**  Updated:		Bernd Aberle, DLR/WT-DA-SE, 26 September 1995, GDF leaves hole if failing, hole filled up with Akima Interpolation
**  Updated:		Bernd Aberle, 16 October 1995, Constant value for whole spectrum, also if only 7th point is there 
\*----------------------------------------------------------------------------*/
void PolarizationC__Compute (const PolarizationC_s *Self, const float *Lambda, float *BandCoefficients, short Begin, short End, const char *PixelDate)
{
#ifdef DEBUG
    FILE *out;
    char debug_filename[80];
#endif

    /*----------------------------------------------------------------------------*\
    **  Important note (updated 18 July 1995 by rjds)
    **
    **  It is assumed that the coefficients in PolarizationC_s are FRACTIONAL POLARISATIONS (all of them!)
    **  There is now NO CONVERSION to DEGREES OF POLARISATION before the interpolation takes place.
    **  All parties now agree to carry out interpolation with fractional polarisation.  Range [0,1].
    **
    \*----------------------------------------------------------------------------*/

    /*----------------------------------------------------------------------------*\
    ** Currently only PMD values and the 7th point are used, but no overlap point !
    **
    ** Fractional polarization value p is valid, if the value is between 0 and 1.
    **
    ** There are now 8 p values in the PolarizationC_s *Self structure :
    ** The first seven for the common integration time of bands 1b to 4 plus one 
    ** Raleigh single scattering point for the integration time of band 1a. If
    ** the integration time of band 1a is greater than 1.5 sec, this p value is a
    ** mean value over several 1.5sec p values, calculated by the calling routine.
    ** The indizes of the p values are defined in defs.h :
    ** 
    **	PC_PMD_1 = 0,
    **	PC_PMD_2,
    **	PC_PMD_3,
    **	PC_OVERLAP_1,
    **	PC_OVERLAP_2,
    **	PC_OVERLAP_3,
    **	PC_P7,
    **	PC_P7_BAND_1A
    **
    ** Description of cases :
    **
    **	If no valid p, all p's are set to 0.5 -> no polarization
    **
    **	If only one valid p, this p value is used for the whole range.
    **	If only p(PC_P7_BAND_1A) valid, the calling routine applys that only for band 1a.
    **
    **	If two or more, find all p values by Akima interpolation. As input, use
    **	valid p's, copy the first p and the last p three times to some lower or higher 
    **	Lambdas, respectively. Then apply Akima. Thus, the gradient of the curve at 
    **	both ends tends to zero and it is impossible, that the curve leaves the valid
    **	range between 0 and 1.
    **	
    **	If p7 and pPMD1 are valid, compute the p's between p7 and pPMD1 by using
    **	the Generalized Distribution Function. Thus if the GDF fails for any reason
    **	(which happened already) the Akima interpolated values still are left alone
    **	in the array.
    **	
    \*----------------------------------------------------------------------------*/

    /*----------------------------------------------------------------------------*\
    **  local declarations
    \*----------------------------------------------------------------------------*/

    /*----------------------------------------------------------------------------*\
    **  Akima
    \*----------------------------------------------------------------------------*/
    double x_arr[POLARIZATION_COEFFICIENTS+AKIMA_EXTRA_POINTS];
    double y_arr[POLARIZATION_COEFFICIENTS+AKIMA_EXTRA_POINTS];
    double a_arr[POLARIZATION_COEFFICIENTS+AKIMA_EXTRA_POINTS];
    double b_arr[POLARIZATION_COEFFICIENTS+AKIMA_EXTRA_POINTS];
    double c_arr[POLARIZATION_COEFFICIENTS+AKIMA_EXTRA_POINTS];
    double d_arr[POLARIZATION_COEFFICIENTS+AKIMA_EXTRA_POINTS];

    int akima_points = 0;

    float delta_w = 0.01;
    float big_delta_w = 20.0;

    double x, y;

    /*----------------------------------------------------------------------------*\
    **  Generalised Distribution Function
    \*----------------------------------------------------------------------------*/
    float grad_first;
    float p1, p0, x0, x1;

    /*----------------------------------------------------------------------------*\
    **  Others
    \*----------------------------------------------------------------------------*/
    int loop;
    float p_interp[POLARIZATION_COEFFICIENTS];
    int n_interp_points = 0;
    int badloop;
    char ErrorString[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------------*\
    **  Initialise
    \*----------------------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++) BandCoefficients[loop] = 0.5;

    /*-----------------------------------------------------------------------*\
    ** Count valid w- (wavelengths) and p-values (p7 and PMDs, not PC_P7_BAND_1A)
    \*-----------------------------------------------------------------------*/
    n_interp_points = 0;
    for (loop = 0; loop < POLARIZATION_COEFFICIENTS - 1; loop++)
    {
	if (Self->Coefficient[loop] >= 0.0 && Self->Coefficient[loop] <= 1.0)
	{
	    p_interp[n_interp_points] = Self->Coefficient[loop];
	    n_interp_points++;
	}
    }

    /*-----------------------------------------------------------------------*\
    ** Sort out the various cases 
    ** Messages if number of interpolation points less than 3 
    \*-----------------------------------------------------------------------*/
    if (n_interp_points == 0)
    {
	if (Self->Coefficient[PC_P7_BAND_1A] >= 0.0 && Self->Coefficient[PC_P7_BAND_1A] <= 1.0)
	{
	    /*-----------------------------------------------------------------------*\
	    ** all p's constant to p7 band 1A - this is the case if IT 1a != IT 1b..4
	    \*-----------------------------------------------------------------------*/
	    sprintf(ErrorString, 
		    ". Constant interpolated fractional p-values (only point for BAND 1A) PixelDate : %s", PixelDate);
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_212", WARN, LOG_CANNOT_CALCULATE, ErrorString, "");

	    for ( loop = Begin; loop <= End; loop++ ) BandCoefficients[loop] = Self->Coefficient[PC_P7_BAND_1A];
	}
	else
	{
	    /*-----------------------------------------------------------------------*\
	    ** all p's 0.5 -> no polarization
	    \*-----------------------------------------------------------------------*/
	    sprintf(ErrorString, "interpolated p-values. No p-values from polarisation determination. No correction ! PixelDate : %s",
		    PixelDate);
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_223", WARN, LOG_CANNOT_CALCULATE, ErrorString, "");
	}
    }

    else if (n_interp_points == 1)
    {
	/*-----------------------------------------------------------------------*\
	** all p's constant
	\*-----------------------------------------------------------------------*/
	sprintf(ErrorString, 
		". Constant interpolated fractional p-values (only ONE point from polarisation determination) PixelDate : %s", PixelDate);
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_234", WARN, LOG_CANNOT_CALCULATE, ErrorString, "");

	for ( loop = Begin; loop <= End; loop++ ) BandCoefficients[loop] = p_interp[0];
    }

    else
    {
	/*-----------------------------------------------------------------------*\
	** 
	** Akima Interpolation and Generalized Distribution Function
	** 
	\*-----------------------------------------------------------------------*/
	if (n_interp_points == 2)
	{
	    sprintf(ErrorString, 
		    ". Only TWO p-values from polarisation determination. PixelDate : %s", PixelDate);
	    Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_250", WARN, LOG_PROBLEMS, ErrorString, "");
	}

	/*-----------------------------------------------------------------------*\
	** collect points for Akima interpolation
	** add p7 and pPMD1,2,3 to x_arr and y_arr 
	\*-----------------------------------------------------------------------*/
	akima_points = 0;
	if (Self->Coefficient[PC_P7] >= 0.0 && Self->Coefficient[PC_P7] <= 1.0)	
	{
	    x_arr[akima_points] = (double) Self->Wavelength[PC_P7];
	    y_arr[akima_points] = (double) Self->Coefficient[PC_P7];
	    akima_points++;
	}
	for (loop = PC_PMD_1; loop <= PC_PMD_3 ; loop++)
	{
	    if (Self->Coefficient[loop] >= 0.0 && Self->Coefficient[loop] <= 1.0)	
	    {
		x_arr[akima_points] = (double) Self->Wavelength[loop];
		y_arr[akima_points] = (double) Self->Coefficient[loop];
		akima_points++;
	    }
	}

	/*-----------------------------------------------------------------------*\
	** shift x_arr and y_arr two points up
	\*-----------------------------------------------------------------------*/
	for (loop = akima_points + 1; loop > 1; loop--)
	{
	    x_arr[loop] = x_arr[loop - 2];
	    y_arr[loop] = y_arr[loop - 2];
	}

	/*-----------------------------------------------------------------------*\
	** add two points left and right
	\*-----------------------------------------------------------------------*/
	x_arr[1] = x_arr[2] - (2 * big_delta_w);
	y_arr[1] = y_arr[2];
	x_arr[0] = x_arr[1] - big_delta_w;
	y_arr[0] = y_arr[1];
	x_arr[akima_points + 2] = x_arr[akima_points + 1] + big_delta_w;
	y_arr[akima_points + 2] = y_arr[akima_points + 1];
	x_arr[akima_points + 3] = x_arr[akima_points + 2] + (2 * big_delta_w);
	y_arr[akima_points + 3] = y_arr[akima_points + 2];

	akima_points += 4;

	/*-----------------------------------------------------------------------*\
	**  Carry out interpolation
	\*-----------------------------------------------------------------------*/
	akima_su (x_arr, y_arr, akima_points, a_arr, b_arr, c_arr, d_arr);
	for ( loop = Begin; loop <= End; loop++ )
	{
	    x = (double) Lambda[loop];
	    akima_po (akima_points,x_arr,a_arr,b_arr,c_arr,d_arr,x,&y);
	    BandCoefficients[loop] = (float) y;
	}

	/*-----------------------------------------------------------------------*\
	**
	**  Compute parameters for the Analytic curves
	**  (over wavelengths less than that of first Akima Interpolation point)
	**  
	\*-----------------------------------------------------------------------*/
	if ( (Self->Coefficient[PC_P7] >= 0.0 && Self->Coefficient[PC_P7] <= 1.0) && 	
	     (Self->Coefficient[PC_PMD_1] >= 0.0 && Self->Coefficient[PC_PMD_1] <= 1.0) )	
	{
	    /*-----------------------------------------------------------------------*\
	    ** gradient of curve at first PMD point 
	    \*-----------------------------------------------------------------------*/
	    x = (double) Self->Wavelength[PC_PMD_1] + delta_w;
	    akima_po (akima_points, x_arr, a_arr, b_arr, c_arr, d_arr, x, &y);
	    grad_first = ( (float) y - Self->Coefficient[PC_PMD_1] ) / delta_w;
	
	    if (grad_first > SMALL || grad_first < -SMALL)
		/* continue only if significant gradiant and avoid zero division */
	    {
				/*-----------------------------------------------------------------------*\
				** border points for analytic curve 
				\*-----------------------------------------------------------------------*/
		x0 = Self->Wavelength[PC_P7];
		p0 = Self->Coefficient[PC_P7];
	
		x1 = Self->Wavelength[PC_PMD_1];
		p1 = Self->Coefficient[PC_PMD_1];
	
				/*-----------------------------------------------------------------------*\
				** calculate GDF 
				\*-----------------------------------------------------------------------*/
		PolarizationC__GDF(grad_first, x0, p0, x1, p1, Lambda, BandCoefficients, Begin, End, PixelDate);
	    }
	}

    }
		

    /*-----------------------------------------------------------------------*\
    ** ..check for out of band interpolated polarisation values, 
    ** re-set to zero or one if they occur 
    \*-----------------------------------------------------------------------*/
    badloop = 0;
    for ( loop = Begin; loop <= End; loop++ )
    {
	if ( BandCoefficients[loop] < 0.0 ) 
	{
	    badloop++;
	    BandCoefficients[loop] = 0.0;
	}
	else if ( BandCoefficients[loop] > 1.0 )
	{
	    badloop++;
	    BandCoefficients[loop] = 1.0;
	}
    }

    /*-----------------------------------------------------------------------*\
    ** output to log file if any negative values obtained 
    \*-----------------------------------------------------------------------*/
    if (badloop > 0 )
    {
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_370", WARN, LOG_INCONSISTENT,
		  "Some out-of-range interpolated p-values occurred", "re-set to 0.0 or 1.0 - some loss of accuracy");
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", "PolarizationC__Compute.dbg" );
    if (Debug__isActive(debug_ptr, DBG_POLARIZATION_APPLY, DBG_LEVEL_1))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    fprintf(out, "\nfraction Polarisation Coefficients (p values)\n");
	    for (loop = 0; loop < POLARIZATION_COEFFICIENTS; loop++)
	    {
		fprintf(out, "%f  %f\n", Self->Wavelength[loop], Self->Coefficient[loop]);
	    }
	    fprintf(out, "\n");
		
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_395", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif
}



/*----------------------------------------------------------------------------*\
**                                PolarizationC__ComputeCorFak
**  Input parameters:
**    PolarizationPar	Coefficients and their Wavelengths (p values) (7) (Self)
**    Sensitivity	Polarization Sensitivity Eta (1024)
**    SensitivityGrid	Wavelength grid for Eta
**    Lambda		Wavelengths for Reticon data
**    Begin		(first pixel in 1024 array)
**    End		(last pixel in 1024 array)
**
**  Output parameters:
**    Correction	polarization correction factor for one band referring
**			to the reticon grid
**    BandCoefficients	polarization values for one band referring to the
**			reticon grid
**    Eta_Omega		Polarization Sensitivity for one band referring to the
**			reticon grid
**
**  Other interfaces:
**
**  Description:
**    Function to calculate the polarization correction factors (Correction),
**    the polarization values (BandCoefficients) and the polarization
**    sensitivity (Eta_Omega) for one band referring to the given reticon
**    grid (Lambda).
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
**			adopted from PolarizationC__Apply
\*----------------------------------------------------------------------------*/
void
PolarizationC__ComputeCorFak( const PolarizationC_s *PolarizationPar,
			     const float Sensitivity[ DETECTOR_PIXELS],
			     const float SensitivityGrid[ DETECTOR_PIXELS],
			     const float Lambda[ DETECTOR_PIXELS], short Begin, short End,
			     float Correction[ DETECTOR_PIXELS],
			     float BandCoefficients[ DETECTOR_PIXELS],
			     float Eta_Omega[ DETECTOR_PIXELS], const char *PixelDate)
{
    int			loop;

    /*-----------------------------------------------------------------------*\
    ** Call to compute polarization values (BandCoefficients) for the given
    ** band referring to the reticon grid (Lambda)
    \*-----------------------------------------------------------------------*/
    PolarizationC__Compute ( PolarizationPar, Lambda, BandCoefficients,
			    Begin, End, PixelDate);

    /*-----------------------------------------------------------------------*\
    ** Calculate the polarization sensitivity (Eta_Omega) for each pixel of the
    ** given band referring to the reticon grid (Lambda)
    \*-----------------------------------------------------------------------*/
    for (loop = 0; loop < DETECTOR_PIXELS; loop++)
	Eta_Omega[ loop] = 0.0;

    fit_grid_akima( SensitivityGrid, Sensitivity, DETECTOR_PIXELS,
		    Lambda, Eta_Omega, Begin, End);

    /*-----------------------------------------------------------------------*\
    ** Calculate the correction factor (Correction) for each pixel of the given
    ** band referring to the reticon grid (Lambda)
    \*-----------------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
    {
	Correction[ loop] = (1.0 + Eta_Omega[ loop]) / 
	    (2.0 * (( BandCoefficients[ loop] *
		      (1.0 - Eta_Omega[ loop])) + Eta_Omega[ loop]));
    }
}


/*----------------------------------------------------------------------------*\
**                                PolarizationC__ComputePolFakError
**  Input parameters:
**    PolarizationPar	Coefficients and their Wavelengths (p values) (7) (Self)
**    Sensitivity	Polarization Sensitivity Eta (1024)
**    SensitivityGrid	Wavelength grid for Eta
**    Lambda		Wavelengths for Reticon data
**    Band		the Band for which the polarization correction factors
**                      and the according errors are to be calculated
**    Begin		(first pixel in 1024 array)
**    End		(last pixel in 1024 array)
**    KeyDataError	Structure containing the Key Data Errors
**    PixelDate		pixel time of the given band (only used for debug)
**
**  Output parameters:
**    PolCorFak		Polarization Correction factors referring to
**			the reticon grid
**    ErrPolCor		errors of Polarization Correction factors
**			referring to the reticon grid
**
**  Other interfaces:
**
**  Description:
**    Function to calculate the polarization correction factors (PolCorFak)
**    and the according errors (ErrPolCor) for the given band (Band) referring
**    to the given reticon grid (Lambda). The error calculation is done via
**    the calculation of a maximum and minimum function of the polarization
**    values (BandCoefficients) in combination with a further application
**    of the Gaussian Error Propagation Law to get the final errors of
**    polarization correction factors.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.12.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PolarizationC__ComputePolFakError( const PolarizationC_s *PolarizationPar,
				  const float Sensitivity[ DETECTOR_PIXELS],
				  const float SensitivityGrid[ DETECTOR_PIXELS],
				  const float Lambda[ DETECTOR_PIXELS], 
				  ScienceData_e Band, short Begin, short End,
				  const KeyDataError_s *KeyDataError, 
				  float PolCorFak[ DETECTOR_PIXELS],
				  float ErrPolCor[ DETECTOR_PIXELS], const char *PixelDate)
{
    int			i;
    float			Err_P_Val;
    float			AbsPreflightErrEta;
    float			N1, N2, Z1, Z2;
    PolarizationC_s	MaxPolPar;
    PolarizationC_s	MinPolPar;
    float 		Eta_Omega[ DETECTOR_PIXELS];
    float 		BandCoefficients[ DETECTOR_PIXELS];
    float 		MaxBandCoefficients[ DETECTOR_PIXELS];
    float 		MinBandCoefficients[ DETECTOR_PIXELS];
/*
  float			PolCorFakMax[ DETECTOR_PIXELS];
  float			PolCorFakMin[ DETECTOR_PIXELS];
  */

    /*-----------------------------------------------------------------------*\
    ** Compute the polarization values( BandCoefficients) and the according
    ** polarization sensitivity (Eta_Omega) for the given band and the given
    ** reticon grid (Lambda)
    \*-----------------------------------------------------------------------*/
    PolarizationC__ComputeCorFak( PolarizationPar, Sensitivity,
				 SensitivityGrid, Lambda, Begin, End, PolCorFak,
				 BandCoefficients, Eta_Omega, PixelDate);

    /*-----------------------------------------------------------------------*\
    ** Calculate the maximum and the minimum values of the polarization
    ** coefficients to calculate later the error of the polarization values
    ** (BandCoefficients)
    \*-----------------------------------------------------------------------*/
    MaxPolPar	= *PolarizationPar;
    for (i = 0; i < POLARIZATION_COEFFICIENTS; i++)
    {
	MaxPolPar.Coefficient[ i] = PolarizationPar->Coefficient[ i] +
	    PolarizationPar->Error[ i];
	if (MaxPolPar.Coefficient[ i] > 1.0)
	    MaxPolPar.Coefficient[ i] = 1.0;
    }
    MinPolPar	= *PolarizationPar;
    for (i = 0; i < POLARIZATION_COEFFICIENTS; i++)
    {
	MinPolPar.Coefficient[ i] = PolarizationPar->Coefficient[ i] -
	    PolarizationPar->Error[ i];
	if (MinPolPar.Coefficient[ i] < 0.0)
	    MinPolPar.Coefficient[ i] = 0.0;
    }

    /*-----------------------------------------------------------------------*\
    ** Calculate the maximum polarization value (MaxBandCoefficients) per pixel
    ** and the minimum polarization value (MinBandCoefficients) per pixel of the
    ** given band referring to the reticon grid (Lambda).
    \*-----------------------------------------------------------------------*/
    PolarizationC__Compute( &MaxPolPar, Lambda, MaxBandCoefficients, Begin, End,
			   PixelDate);
    PolarizationC__Compute ( &MinPolPar, Lambda, MinBandCoefficients, Begin, End, PixelDate);

    /*-----------------------------------------------------------------------*\
    ** Calculate the error of the polarization correction factor (ErrPolCor)
    ** per pixel of the given band referring to the reticon grid (Lambda)
    \*-----------------------------------------------------------------------*/
    for (i = Begin; i <= End; i++)
    {
	/*---------------------------------------------------------------------*\
	** Calculate the absolute error of the polarization value (BandCoefficients)
	\*---------------------------------------------------------------------*/
	Err_P_Val = (float)fabs( (double)((MaxBandCoefficients[ i] -
					   MinBandCoefficients[ i]) / 2.0));

	/*---------------------------------------------------------------------*\
	** Calculate the absolute error of polarization sensitivity (Eta_Omega)
	\*---------------------------------------------------------------------*/
	AbsPreflightErrEta = KeyDataError__Get(KeyDataError, KDE_ETA,
					       Band2Channel(Band), i) / 3.0 * Eta_Omega[ i];

	/*---------------------------------------------------------------------*\
	** Calculate the error of the polarization correction factor (ErrPolCor)
	** per pixel of the given band referring to the reticon grid (Lambda).
	** The error calculation is done via the Gaussian Error Propagation Law.
	\*---------------------------------------------------------------------*/
	N1 = BandCoefficients[ i] * Eta_Omega[ i] - BandCoefficients[ i] -
	    Eta_Omega[ i];
	Z1 = 2.0 * BandCoefficients[ i] - 1.0;
	N2 = N1;
	Z2 = 1.0 + Eta_Omega[ i] * (Eta_Omega[ i] - 1.0);
	ErrPolCor[ i] = (float)sqrt((double)
				    (0.5 * Z1 / N1 / N1 * 0.5 * Z1 / N1 / N1 *
				     AbsPreflightErrEta * AbsPreflightErrEta +
				     0.5 * Z2 / N2 / N2 * 0.5 * Z2 / N2 / N2 *
				     Err_P_Val * Err_P_Val));
    }

/*
  PolarizationC__ComputeCorFak( MaxPolPar, Sensitivity,
  SensitivityGrid, Lambda, Begin, End, PolCorFakMax, PixelDate);
  PolarizationC__ComputeCorFak( MinPolPar, Sensitivity,
  SensitivityGrid, Lambda, Begin, End, PolCorFakMin, PixelDate);
  for (i = Begin; i <= End; i++)
  ErrPolCor[ i] = (float)fabs( (double)((PolCorFakMax[ i] -
  PolCorFakMin[ i]) / 2.0));
  */
}


/*----------------------------------------------------------------------------*\
**                                PolarizationC__Apply
**  Input parameters:
**    Coefficients and their Wavelengths (p values) (7) (Self)
**    Polarization Sensitivity Eta (1024)
**    Wavelength grid for Eta
**    Reticon data
**    Wavelengths for Reticon data
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**    date (string)
**  Output parameters:
**    Corrected Reticon data
**  Other interfaces: Call to PolarizationC__Compute
**  Description:
**    Calculates and applys the Polarization Correction factor to the Reticon
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	16.3.94
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void PolarizationC__Apply(const PolarizationC_s *Self, ScienceData_e Band, const float Sensitivity[], const float SensitivityGrid[], float Reticon[], const float Lambda[], short Begin, short End, const char *PixelDate)
{
    int loop;
    float BandCoefficients [DETECTOR_PIXELS];
    float Eta_Omega [DETECTOR_PIXELS];
    float Correction [DETECTOR_PIXELS];

#ifdef DEBUG
    float InputReticon [DETECTOR_PIXELS];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_POLARIZATION_APPLY, DBG_LEVEL_0))
    {	
	for (loop = Begin; loop <= End; loop++)
	    InputReticon [loop] = Reticon [loop];
    }
#endif

/*-----------------------------------------------------------------------*\
** Find a fractional polarisation value p for each detector pixel by inter-
** and extrapolation of the seven input values (Coefficients)
\*-----------------------------------------------------------------------*/
    /*-----------------------------------------------------------------------*\
    ** Clear parameter
    \*-----------------------------------------------------------------------*/
    for (loop = 0; loop < DETECTOR_PIXELS; loop++)
    {
	BandCoefficients [loop] = 0.0;
	Correction [loop] = 0.0;
	Eta_Omega [loop] = 0.0;
    }

    /*-----------------------------------------------------------------------*\
    ** Calculate BandCoefficients, Eta_Omega and correction factor for each pixel
    \*-----------------------------------------------------------------------*/
    PolarizationC__ComputeCorFak( Self, Sensitivity, SensitivityGrid, Lambda, Begin, End, Correction, BandCoefficients, Eta_Omega, PixelDate);

    /*-----------------------------------------------------------------------*\
    ** Apply the correction factor
    \*-----------------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
    {
	Reticon[loop] = Reticon[loop] *  Correction [loop];
    }


    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("PolarizationC__Apply", PixelDate) );

    if (Debug__isActive(debug_ptr, DBG_POLARIZATION_APPLY, DBG_LEVEL_0))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Band == BAND_1A)
	    {
		fprintf(out, "\nValid Fractional Polarization Coefficients (p values)\n");
		fprintf(out, "Wavelength   Coefficient\n");
		fprintf(out, "[]           []\n");
		for (loop = 0; loop < POLARIZATION_COEFFICIENTS; loop++)
		{
		    if ( Self->Coefficient [loop] >=0.0 && Self->Coefficient [loop] <= 1.0 )
			fprintf(out, "%12.5g %12.5g\n", Self->Wavelength [loop], Self->Coefficient [loop]);
		}

		fprintf(out, "\nChi : %f cos_2chi : %f\n", Self->Chi, (float) cos( Self->Chi * 2.0 ));
	
		fprintf(out, 
			"\nApply Polarization Correction\n");
		fprintf(out, 
			"Lambda       Band         InputReticon BandCoeff    SensitivGrid Sensitivity  Eta          CorrFactor   OutputReticon\n");
		fprintf(out, 
			"[]           []           []           []           []           []           []           []           []\n\n");
	    }
	
	    for (loop = Begin; loop <= End; loop++)
	    {
		fprintf(out, 
			"%12.5g %12d %12.5g %12.7g %12.5g %12.5g %12.5g %12.5g %12.5g\n", 
			Lambda [loop], Band, InputReticon [loop], BandCoefficients [loop], 
			SensitivityGrid[loop], Sensitivity [loop], Eta_Omega [loop], Correction [loop], Reticon [loop]);
	    }
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_753", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif

}

/*----------------------------------------------------------------------------*\
**                                PolarizationC__ApplyPMD
**  Input parameters:
**    PMD p values (3)
**    Number (which PMD, 1,2 or 3)
**    PMD readouts
**    date (string)
**  Output parameters:
**    Corrected PMD data
**  Other interfaces:
**  Description:
**    Calibrates PMD data in order to get reflectances by dividing them by 2p
**    Also IntensityC__ApplyPMD is called later
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	18.4.94
**  Author:		Bernd Aberle, DLR/WT-DA-SE
**  Updated:    17.04.2000 Bernd Aberle, DLR/IMF, case Self->Coefficient[Number] <= 1e-16 considered
**			causing float overflow
\*----------------------------------------------------------------------------*/
FunctionStatus_e PolarizationC__ApplyPMD(const PolarizationC_s* Self, PMD_Number_e Number, float PMD[], const char *PixelDate)
{
    int loop;
    char ErrorString[LOG_FIELD_SIZE];
    FunctionStatus_e Status = SUCCESS;

#ifdef DEBUG
    float InputPMD [PMD_SIZE];
    FILE *out;
    char debug_filename[80];

    if (Debug__isActive(debug_ptr, DBG_POLARIZATION_APPLY_PMD, DBG_LEVEL_0))
    {	
	for (loop = 0; loop < PMD_SIZE; loop++)
	    InputPMD [loop] = PMD [loop];
    }
#endif

    /*-----------------------------------------------------------------------*\
    ** Correct each PMD pixel for Polarization
    \*-----------------------------------------------------------------------*/
    if ( Self->Coefficient[Number] <= 1e-16 || Self->Coefficient[Number] > 1.0 )
    {
	sprintf(ErrorString, 
		"Invalid or zero fractional polarization value for PMD %d, PixelDate %s", Number+1, PixelDate);

	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "3.5_221", WARN, LOG_PROBLEMS, ErrorString, "");

	Status = ERROR;
    }
    else
    {
	for (loop = 0; loop < PMD_SIZE; loop++)
	    PMD[loop] /= (2.0 * Self->Coefficient[Number]);
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", DbgFileName("PolarizationC__ApplyPMD", PixelDate) );

    if (Debug__isActive(debug_ptr, DBG_POLARIZATION_APPLY_PMD, DBG_LEVEL_0))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    if (Number == PMD_1)
	    {
		fprintf(out, "\n");
		fprintf(out, 
			"Pixel      PMD_Number Wavelength Coeff      InputPMD   PMD \n");
		fprintf(out, 
			"[]         []         []         []         []         []  \n\n");
	    }
	
	    for (loop = 0; loop < PMD_SIZE; loop++)
	    {
		fprintf(out, "%10d %10d %*.*g %*.*g %*.*g %*.*g\n", 
			loop,
			Number, 
			DEBUG_WIDTH, DEBUG_PRECISION, Self->Wavelength[Number],
			DEBUG_WIDTH, DEBUG_PRECISION, Self->Coefficient[Number],
			DEBUG_WIDTH, DEBUG_PRECISION, InputPMD[loop],
			DEBUG_WIDTH, DEBUG_PRECISION, PMD[loop]);
	    }
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_849", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif

    return Status;
}


/*----------------------------------------------------------------------------*\
**                          PolarizationC__CalculatePMD_Wavelength
**  Input parameters:
**    
**    Pixel Indizes of the Bands 1a to 4
**    Detector readouts 4 * 1024
**    DetectorGrid 4 * 1024
**    Preflight parameter Ksi 4 * 1024
**    
**  Output parameters:
**    Status of this routine
**    Return value : wavelength for the p value (fractional polarization)
**    
**  Other interfaces:
**    
**  Description:
**    
**    Determine the wavelength of the fractional polarization for PMD PMD_Number
**    (Used later for interpolation of effective polarization ratios over the
**    GOME spectrum)
**    
**    use following weighting function :
**    n(p) = 1/(n2 - n1 + 1) * Sum ( S_i * Ksi * i / S_average )
**    
**    Detector and Ksi must be fitted to that same wavelength grid in the
**    parameter DetectorGrid
**    
**  References:
**  Libraries:
**  Created:	3.2.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
float PolarizationC__CalculatePMD_Wavelength(const PixelIndex_s *PixelIndex, const float Detector[OPTICAL_CHANNELS][DETECTOR_PIXELS], 
					    const float *DetectorGrid[OPTICAL_CHANNELS], const float *Ksi[OPTICAL_CHANNELS], FunctionStatus_e *Status)
{
    /* build the mean value of the product of the Ksi values and the detector readouts corresponding to PMD 1 and
       sum up the products of the Ksi values, the detector readouts and indizes corresponding to PMD 1 : */

    float Wavelength = 0.0;
    int OpticalChannel;

    double value_1 = 0.0;
    double value_2 = 0.0;
    int BandIndx;
    int loop;
    float KsiValue;
    float DetectorValue;
    float weight_loop;
    int n_spectrum;
    int n_channel;

    *Status = SUCCESS;

    for (BandIndx=BAND_1A; BandIndx<=BAND_4; BandIndx++)
    {
	OpticalChannel = Band2Channel ((ScienceData_e)BandIndx);

	for (loop = PixelIndex[BandIndx].First; loop <= PixelIndex[BandIndx].Last; loop++)
	{
	    KsiValue = Ksi [OpticalChannel][loop];
	    DetectorValue = Detector [OpticalChannel][loop];

	    weight_loop = (float) loop + (DETECTOR_PIXELS * OpticalChannel);
	    value_1 += KsiValue * DetectorValue;
	    value_2 += KsiValue * DetectorValue * weight_loop;
	}
    }

    if (value_1 > SMALL)
    {
	n_spectrum = (int) (value_2 / value_1);				/* that's the index in the 4096 range */

	if (n_spectrum >= ((DETECTOR_PIXELS * OPTICAL_CHANNELS)) )
	    n_spectrum = (DETECTOR_PIXELS * OPTICAL_CHANNELS) - 1;

	if (n_spectrum < 0) n_spectrum = 0;

	OpticalChannel = (int) floor(n_spectrum/DETECTOR_PIXELS); 	/* this is the channel */
	n_channel = n_spectrum - (DETECTOR_PIXELS * OpticalChannel); 	/* that's the index in the 1024 range */
	Wavelength = DetectorGrid [OpticalChannel] [n_channel]; 	/* and this is the corresponding wavelength */
    }
    else *Status = ERROR;

    return Wavelength;
}


/*----------------------------------------------------------------------------*\
**                          PolarizationC__SetBands_for_PMD
**  Input parameters: PMD loop
**
**  Output parameters: BandsForPMD
**
**  Other interfaces:
**
**  Description:  
**    Returns band indizes in BandsForPMD, necessary for computing PMD polarization value
**    These bands are defined to be used as Detector values for the p (fractional polarization
**    values) calculation
**
**  References:
**  Libraries:
**  Created:	05 July 1995
**  Author:	Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void PolarizationC__SetBands_for_PMD ( PMD_Number_e loop_PMD, short BandsForPMD [BAND_DEFINED])
{
    short loop_band;

    /*---------------------------------------------------------------------------------------*\
    ** Select used bands
    \*---------------------------------------------------------------------------------------*/
    for (loop_band = 0; loop_band < BAND_DEFINED; loop_band++) BandsForPMD[loop_band] = -1;

    switch (loop_PMD)
    {
	/*
	  case PMD_1 : BandsForPMD[0] = BAND_1A; BandsForPMD[1] = BAND_1B; BandsForPMD[2] = BAND_2A; BandsForPMD[3] = BAND_2B; break;
	  case PMD_2 : BandsForPMD[0] = BAND_2B; BandsForPMD[1] = BAND_3; BandsForPMD[2] = BAND_4; break;		
	  case PMD_3 : BandsForPMD[0] = BAND_3; BandsForPMD[1] = BAND_4; break;

	  case PMD_1 : BandsForPMD[0] = BAND_2B; break;
	  case PMD_2 : BandsForPMD[0] = BAND_3; break;		
	  case PMD_3 : BandsForPMD[0] = BAND_4; break;
	  */

    case PMD_1 : BandsForPMD[0] = BAND_1B; BandsForPMD[1] = BAND_2A; BandsForPMD[2] = BAND_2B; break;
    case PMD_2 : BandsForPMD[0] = BAND_2B; BandsForPMD[1] = BAND_3; BandsForPMD[2] = BAND_4; break;		
    case PMD_3 : BandsForPMD[0] = BAND_3; BandsForPMD[1] = BAND_4; break;

    default : break;
    }
}


/*----------------------------------------------------------------------------*\
**                                Polarization::InterpolateGeometry
**  Input parameters:
**    IntgTime, SunAngleOrig, LoS_AngleOrig
**    PosAngle_s structure contains three Azimuth and three Line-of-Sight angles
**
**  Output parameters:
**    SunAngle, LoS_Angle
**
**  Other interfaces:
**
**  Description:
**
**    Interpolate sun and line of sight angles for the given integration time.
**    The original angles are three angles at the left border, the middle and
**    the right border of the 1.5 sec ground pixel.
**
**    The output angles are the angles at the left border, the middle and
**    the right border of the IntgTime sec ground pixel.
**
**    An interpolation of the angles is done if IntgTime is less than 1.5 sec,
**    otherwise the original values are only copied.
**
**  References:
**  Libraries:
**  Created:	26.1.94
**  Author:		...., DLR/WT-DA-...
\*----------------------------------------------------------------------------*/
void PolarizationC__InterpolGeometry (float IntgTime, const PosAngle_s SunAngleOrig[], const PosAngle_s LoS_AngleOrig[], 
				     PosAngle_s SunAngle[], PosAngle_s LoS_Angle[])
{
    PosAngle_s LoS_AngleCorr[ANGLES_DEFINED];
    short loop;

    /*-----------------------------------------------------------------------*\
    ** If IntgTime greater 1.5s, just copy original values
    \*-----------------------------------------------------------------------*/
    if (IntgTime >= GROUND_PIXEL_TIME || IntgTime <= 0.0)
    {
	for (loop = 0; loop < ANGLES_DEFINED; loop++)
	{
	    SunAngle[loop].Zenith = SunAngleOrig[loop].Zenith;
	    LoS_Angle[loop].Zenith = LoS_AngleOrig[loop].Zenith;
	    SunAngle[loop].Azimuth = SunAngleOrig[loop].Azimuth;
	    LoS_Angle[loop].Azimuth = LoS_AngleOrig[loop].Azimuth;
	}

	return;
    }

    /*-----------------------------------------------------------------------*\
    ** Because LoS_Azimuth jumps about 180 deg. between left and right scan, 
    ** we must correct for that amount. Therefore use LoS_AngleCorr[].Azimuth 
    ** later for interpolation
    \*-----------------------------------------------------------------------*/
    for (loop = 0; loop < ANGLES_DEFINED; loop++)
	LoS_AngleCorr[loop].Azimuth = LoS_AngleOrig[loop].Azimuth;

    if (IntgTime <= GROUND_PIXEL_TIME/2.0)
    {
	/* if (IntgTime <= GROUND_PIXEL_TIME/2 && IntgTime > 0) */

	if (LoS_AngleOrig[ANGLE_3].Azimuth - LoS_AngleOrig[ANGLE_2].Azimuth > 170.0) 
	    LoS_AngleCorr[ANGLE_2].Azimuth = LoS_AngleOrig[ANGLE_2].Azimuth + 180.0;

	else if (LoS_AngleOrig[ANGLE_2].Azimuth - LoS_AngleOrig[ANGLE_3].Azimuth > 170.0) 
	    LoS_AngleCorr[ANGLE_2].Azimuth = LoS_AngleOrig[ANGLE_2].Azimuth - 180.0;
    }
    else
    { 
	if (LoS_AngleOrig[ANGLE_2].Azimuth - LoS_AngleOrig[ANGLE_1].Azimuth > 170.0) 
	    LoS_AngleCorr[ANGLE_2].Azimuth = LoS_AngleOrig[ANGLE_2].Azimuth - 180.0;

	else if (LoS_AngleOrig[ANGLE_1].Azimuth - LoS_AngleOrig[ANGLE_2].Azimuth > 170.0) 
	    LoS_AngleCorr[ANGLE_2].Azimuth = LoS_AngleOrig[ANGLE_2].Azimuth + 180.0;
    }


    /*-----------------------------------------------------------------------*\
    ** If IntgTime smaller than 1.5s, interpolate three new angles 
    \*-----------------------------------------------------------------------*/

    /*-----------------------------------------------------------------------*\
    ** Angle three is always the same
    \*-----------------------------------------------------------------------*/
    SunAngle[ANGLE_3].Zenith = SunAngleOrig[ANGLE_3].Zenith;
    LoS_Angle[ANGLE_3].Zenith = LoS_AngleOrig[ANGLE_3].Zenith;
    SunAngle[ANGLE_3].Azimuth = SunAngleOrig[ANGLE_3].Azimuth;
    LoS_Angle[ANGLE_3].Azimuth = LoS_AngleOrig[ANGLE_3].Azimuth;

    /*-----------------------------------------------------------------------*\
    ** Angle two is always in the second half of the scanned area,
    ** so interpolate between points 2 and 3
    \*-----------------------------------------------------------------------*/
    SunAngle[ANGLE_2].Zenith = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime/2.0, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
						    SunAngleOrig[ANGLE_2].Zenith, SunAngleOrig[ANGLE_3].Zenith);

    SunAngle[ANGLE_2].Azimuth = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime/2.0, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
						     SunAngleOrig[ANGLE_2].Azimuth, SunAngleOrig[ANGLE_3].Azimuth);

    LoS_Angle[ANGLE_2].Zenith = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime/2.0, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
						     LoS_AngleOrig[ANGLE_2].Zenith, LoS_AngleOrig[ANGLE_3].Zenith);

    LoS_Angle[ANGLE_2].Azimuth = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime/2.0, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
						      LoS_AngleCorr[ANGLE_2].Azimuth, LoS_AngleCorr[ANGLE_3].Azimuth);

    /*-----------------------------------------------------------------------*\
    **  Angle one depends on IntgTime
    \*-----------------------------------------------------------------------*/
    if (IntgTime <= GROUND_PIXEL_TIME/2.0)
    {
	/* if (IntgTime <= GROUND_PIXEL_TIME/2 && IntgTime > 0) */

	SunAngle[ANGLE_1].Zenith = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
							SunAngleOrig[ANGLE_2].Zenith, SunAngleOrig[ANGLE_3].Zenith);

	LoS_Angle[ANGLE_1].Zenith = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
							 LoS_AngleOrig[ANGLE_2].Zenith, LoS_AngleOrig[ANGLE_3].Zenith);

	SunAngle[ANGLE_1].Azimuth = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
							 SunAngleOrig[ANGLE_2].Azimuth, SunAngleOrig[ANGLE_3].Azimuth);

	LoS_Angle[ANGLE_1].Azimuth = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, GROUND_PIXEL_TIME/2.0, GROUND_PIXEL_TIME, 
							  LoS_AngleCorr[ANGLE_2].Azimuth, LoS_AngleCorr[ANGLE_3].Azimuth);
    }
    else
    { 
	/* if (IntgTime < GROUND_PIXEL_TIME && IntgTime > GROUND_PIXEL_TIME/2.0) */

	SunAngle[ANGLE_1].Zenith = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, 0.0, GROUND_PIXEL_TIME/2.0, 
							SunAngleOrig[ANGLE_1].Zenith, SunAngleOrig[ANGLE_2].Zenith);

	LoS_Angle[ANGLE_1].Zenith = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, 0.0, GROUND_PIXEL_TIME/2.0, 
							 LoS_AngleOrig[ANGLE_1].Zenith, LoS_AngleOrig[ANGLE_2].Zenith);

	SunAngle[ANGLE_1].Azimuth = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, 0.0, GROUND_PIXEL_TIME/2.0, 
							 SunAngleOrig[ANGLE_1].Azimuth, SunAngleOrig[ANGLE_2].Azimuth);

	LoS_Angle[ANGLE_1].Azimuth = LinearInterpolation (GROUND_PIXEL_TIME - IntgTime, 0.0, GROUND_PIXEL_TIME/2.0, 
							  LoS_AngleCorr[ANGLE_1].Azimuth, LoS_AngleCorr[ANGLE_2].Azimuth);
    }



}


/*----------------------------------------------------------------------------*\
**                                PolarizationC__Calculate_p7
**  Input parameters:
**    float IntgTime, const PosAngle_s SunAngleOrig[], const PosAngle_s LoS_AngleOrig[]
**    const char *PixelDate
**
**  Output parameters:
**    float *p_seven_error, Error of the fractional polarization value
**    float *ParamChi, plane of polarization angle at center of the scanned region
**    FunctionStatus_e *Status
**    Return value : fractional polarization in the UV part. Calculated according
**      Raleigh single scattering model
**
**  Other interfaces:
**  Description:
**    The input angles are :
**      In SunAngleOrig : three Azimuth, three Zenith angles
**      In LoS_AngleOrig : three Azimuth, three Zenith angles
**      They are at the left border, the middle and
**      the right border of the 1.5 sec ground pixel
**
**    A call to PolarizationC__InterpolGeometry fitts those angles to the given
**    Integration time
**    A call to PolarizationC__AdjustGeometry converts those angles from satellite
**    height to the given atmospheric height
**    A call to PolarizationC__SeventhPoint calculates Q/I ratio and fractional p
**    values for each location at the left border, the middle and the right border 
**    of the ground pixel according IntgTime
**    The three Q/I ratios are weighted to build a mean value. From that value
**    a fractional polarization p for the whole groung pixel is calculated.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**
**  Created:	1.11.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
float PolarizationC__Calculate_p7(float IntgTime, const P7_Geometry_s* AnglesOrig, float *p_seven_error, float *ParamChi, FunctionStatus_e *Status, const char *PixelDate) 
{
    char ErrorString[LOG_FIELD_SIZE];

    short loop;
    float p_seven = INVALID_FRAC_POL;

    float Q_I_ratio [ANGLES_DEFINED];

    float X_s_s         = 0.0;
    float Q_I_ratio_mean;

    FunctionStatus_e p7_Status = SUCCESS;

    *Status = SUCCESS;

    /*----------------------------------------------------------------------------*\
    ** Interpolate angles to the location of the scanned region :
    ** obsolete since usage of ESA propagator
    \*----------------------------------------------------------------------------*/
    /*
      obsolete since ESA propagator
      PolarizationC__InterpolGeometry (IntgTime, SunAngleOrig, LoS_AngleOrig, SunAngle_IT, LoS_Angle_IT);
      */

    for (loop = 0; loop < ANGLES_DEFINED && p7_Status == SUCCESS; loop++)
    {
	Q_I_ratio[loop] = 0.0;

	/*----------------------------------------------------------------------------*\
	** Adjust viewing geometry from satellite to atmospheric height :
	** obsolete since usage of ESA propagator
	\*----------------------------------------------------------------------------*/
	/*
	  obsolete since ESA propagator
	  *Status = PolarizationC__AdjustGeometry(theta_sat, theta_0_sat, phi_phi_0_sat, EARTH_RADIUS, SAT_HEIGHT, 
	  SMALL, ATM_HEIGHT, &theta, &theta_0, &phi_phi_0, &do_arbitrary, &do_twilight);
	  */
	
	if (*Status == SUCCESS && AnglesOrig->Twilight[loop] == False)
	{
	    /*----------------------------------------------------------------------------*\
	    ** call Seventh point algorithm :
	    \*----------------------------------------------------------------------------*/
	    *Status = PolarizationC__SeventhPoint(AnglesOrig->Theta[loop], AnglesOrig->Theta_0[loop], AnglesOrig->Phi_Phi_0[loop],
						 RHO_N, &p_seven, p_seven_error, &X_s_s, &Q_I_ratio[loop], SMALL, PixelDate);

	    /* return the plane of polarization angle at center of scanned pixel */
	    if (loop == (int) (ANGLES_DEFINED/2) ) *ParamChi = X_s_s;

	    if (*Status != SUCCESS) p7_Status = ERROR;
	}
	else
	{
	    sprintf(ErrorString, "Seventh point - possibly twilight flag set - too dark - Date : %s", PixelDate);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_1233", WARN, LOG_CANNOT_CALCULATE, ErrorString, "");

	    p7_Status = ERROR;
	}
    }
	
    if (p7_Status == SUCCESS)
    {
	/* Interpolate Q_I_ratio's with weighting */
	Q_I_ratio_mean = ( 0.5*Q_I_ratio[ANGLE_1] + Q_I_ratio[ANGLE_2] + 0.5*Q_I_ratio[ANGLE_3] ) / 2.0;
	
	p_seven = 1.0 - ( (1.0 + Q_I_ratio_mean) / 2.0 );
    }
    else
    {
	*Status = ERROR;
    }

    /*-----------------------------------------------------------------------*\
    ** return value
    \*-----------------------------------------------------------------------*/
    return (p_seven);
}


/*----------------------------------------------------------------------------*\
**                                PolarizationC__SeventhPoint
**  Input parameters:
**    float theta_deg, 			LoS Zenith in degrees
**    float theta_0_deg, 		Solar Zenith in degrees
**    float phi_minus_phi_0_deg, 	Difference between LoS Azimuth and Solar Azimuth (degrees)
**    float rho_n,  , 			Physical constant
**    float smallval,                   comparator if zero or not for float values
**    const char *PixelDate,		time/date string of the ground pixel
**
**  Output parameters:
**    float * p_seven,  		fractional polarization
**    float * p_seven_error, 		error of the fractional polarization
**    float * X_s_s, 			plane of polarization angle (also called Chi)
**    float * Q_I_ratio			Ratio of single scattering Stokes parameters Q and I
**
**  Other interfaces:
**
**  Description:
**    Calculate an effective polarisation ratio beta using 
**    Raleigh single scattering model
**    Algorithm by Piet Stammes (KNMI Netherlands)
**
**  References:
**  Libraries:
**    libm
**  Created:	14.6.94
**  Author:		Algorithm by Piet Stammes, KNMI, Netherlands
** 			implemented in C code by Bernd Aberle, DLR/WT-DA-SE
**  
**  Revision:	09.06.1995
**		Double precision.  Error propagation.  (R. Spurr)
\*----------------------------------------------------------------------------*/
FunctionStatus_e PolarizationC__SeventhPoint ( float theta_deg, float theta_0_deg, float phi_minus_phi_0_deg, float rho_n,  float * p_seven,  float * p_seven_error, float  * X_s_s, float * Q_I_ratio, float smallval, const char *PixelDate)
{
    FunctionStatus_e Status = SUCCESS;
    int loop;
    char ErrorString[LOG_FIELD_SIZE];

    /* the input angles :
       printf("\ntheta : %g  theta_0 : %g  phi_minus_phi_0_deg : %g \n", theta, theta_0, phi_minus_phi_0_deg);
       */

    double phi_minus_phi_0;
    double theta;
    double theta_0;

    double beta_prime;

    double cth0;
    double sth0;
    double cth1;
    double sth1;
    double cphi;
    double sphi;

    double big_theta;
    double cos_big_theta;
    double sin_big_theta;
		
    double P_s_s;
    double big_delta;
    double pss1;

    double u_prime, v_prime, m_prime, cos_2chi;

    /*-----------------------------------------------------------------------*\
    ** For error calculation
    \*-----------------------------------------------------------------------*/
    float constant_depol_error = 0.05;
    float constant_angle_error = 0.1;

    double de[4];

    double dth[3];
    double dpss[4];

    double dp7[4];
    double du_prime[3], dv_prime[3], dm_prime[3];
    double sum = 0.0;


    /*-----------------------------------------------------------------------*\
    ** Initialize
    \*-----------------------------------------------------------------------*/
    *p_seven = 0.0;
    *p_seven_error = 0.0;


    /*-----------------------------------------------------------------------*\
    ** 0. Set up angles and cosines/sines
    \*-----------------------------------------------------------------------*/
    phi_minus_phi_0 = fabs(phi_minus_phi_0_deg) * DEG_TO_RAD;
    theta   = theta_deg * DEG_TO_RAD;
    theta_0 = theta_0_deg * DEG_TO_RAD;

    cth0 = cos ( theta_0 );
    sth0 = sin ( theta_0 );
    cth1 = cos ( theta );
    sth1 = sin ( theta );
    cphi = cos ( phi_minus_phi_0 );
    sphi = sqrt(1.0 - cphi*cphi);

    /*-----------------------------------------------------------------------*\
    ** 1. Calculate big_theta, angle of scatter
    \*-----------------------------------------------------------------------*/
    cos_big_theta = -cth0 * cth1 + sth0 * sth1 * cphi;
    sin_big_theta = sqrt(1.0 - cos_big_theta*cos_big_theta);

    if (cos_big_theta >= -1.0 && cos_big_theta <= 1.0) big_theta = acos(cos_big_theta);
    else
    {
	sprintf(ErrorString, "cos_big_theta out of range in calculating seventh point - Date : %s", PixelDate);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_1371", ERROR, LOG_CANNOT_CALCULATE, ErrorString, "");
	Status = ERROR;
    }

    /*-----------------------------------------------------------------------*\
    ** 2. Calculate P_s_s (the degree of polarisation due to single scattering)
    \*-----------------------------------------------------------------------*/
    big_delta = ((1 + rho_n) / (1 - rho_n)) - 1;
    pss1 = ( 1.0 + big_delta + pow(cos_big_theta, 2.0) );
    P_s_s = ( 1.0 - pow(cos_big_theta, 2.0) ) / pss1;

    /*-----------------------------------------------------------------------*\
    ** 3. Check the definition and sign of the input angles :
    \*-----------------------------------------------------------------------*/
    if ( 	smallval < fabs(theta) &&
		fabs(theta) < M_PI_2 &&
		smallval < theta_0 &&
		fabs(phi_minus_phi_0) <= M_PI )
    {
	/* if we are here, the input angles are valid, we can use them to calculate X_s_s, the polarisation plane angle */

	/*-----------------------------------------------------------------------*\
	** 4.  Calculate beta_prime and X_s_s (Plane of polarization angle)
	\*-----------------------------------------------------------------------*/
	beta_prime = 0.0;

	v_prime = sth1 * sin_big_theta;
	if ( fabs(v_prime) > smallval )
	{
	    u_prime = cth0 + cth1 * cos_big_theta;
	    m_prime = u_prime / v_prime;
	    if ( fabs(m_prime) > 1.0 )
	    {
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_1404", FATAL, LOG_CANNOT_CALCULATE, 
			 "sin_beta_prime out of range in calculating seventh point", "");
		if (Status != ERROR) Status = WARN;
	    }
	    else
	    {
		beta_prime = asin(m_prime);
		if (m_prime >= 0.0) *X_s_s = M_PI - beta_prime;
		else *X_s_s = -beta_prime;
		cos_2chi = 1.0 - 2.0 * m_prime * m_prime;
	    }
	}
	else
	{
	    sprintf(ErrorString, "sin_beta_prime for seventh point calculation (Nadir view) - Date : %s", PixelDate);
	    Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_1419", ERROR, LOG_CANNOT_CALCULATE, ErrorString, "");
	    Status = ERROR;
	}
    }
    else
    {
	/* printf("\ntheta : %g  theta_0 : %g  phi_minus_phi_0 : %g \n", theta, theta_0, phi_minus_phi_0); */
	/* if we are here, the input angles aren't valid */

	sprintf(ErrorString, "- Invalid p7 angles : phi_phi_0 : %f theta : %f theta_0 : %f - Date : %s", 
		phi_minus_phi_0, theta_deg, theta_0_deg, PixelDate);

	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_1431", ERROR, LOG_CANNOT_CALCULATE, ErrorString, "");
	Status = ERROR;
    }

    /*-----------------------------------------------------------------------*\
    ** 5. Calculate the fractional polarisation value Coefficient[6] (the seventh value)
    \*-----------------------------------------------------------------------*/
    if (Status != WARN)
    {
	*Q_I_ratio = P_s_s * cos_2chi;
	*p_seven = 1.0 - ( (1.0 + *Q_I_ratio) / 2.0 );
    }


    /* fprintf (stdout, "big_theta : %g P_s_s : %g p_seven : %g Chi : %g \n", big_theta, P_s_s, *p_seven, *X_s_s); */

    /*-----------------------------------------------------------------------*\
    ** 6. Calculate the Error on p_seven (Gaussian error propagation)
    \*-----------------------------------------------------------------------*/
    if (Status != WARN && Status != ERROR)
    {
	/* 6a. error setup */

	de[0] = DEG_TO_RAD * constant_angle_error;
	de[1] = DEG_TO_RAD * constant_angle_error;
	de[2] = DEG_TO_RAD * constant_angle_error;
	de[3] = 2.0 * constant_depol_error * rho_n / (1.0-rho_n) / (1.0-rho_n); 

	/* 6b.  Partial derivatives of P_s_s */

	dth[0] = - ( sth0*cth1+cth0*sth1*cphi );
	dth[1] = - ( sth1*cth0+cth1*sth0*cphi );
	dth[2] = + ( sth0*sth1*sphi );
	for ( loop = 0; loop <= 2; loop++ )
	{
	    dpss[loop] =  2.0*(2.0+big_delta)*cos_big_theta*dth[loop] / pss1 / pss1;
	}
	dpss[3] = - P_s_s / pss1;

	/* 6c.  Partial derivatives of p7 */

	du_prime[0] = -sth0;
	du_prime[1] = -sth1 * cos_big_theta;
	du_prime[2] = 0.0;
	dv_prime[0] = 0.0;
	dv_prime[1] = cth1 * sin_big_theta;
	dv_prime[2] = 0.0;
	for ( loop = 0; loop <= 2; loop++ )
	{
	    du_prime[loop] = du_prime[loop] - cth1 * dth[loop];
	    dv_prime[loop] = dv_prime[loop] + ( sth1 * cos_big_theta * dth[loop] / sin_big_theta );
	    dm_prime[loop] = ( du_prime[loop] - m_prime * dv_prime[loop] ) / v_prime;
	    dp7[loop]      = 0.5 * cos_2chi * dpss[loop] - 2.0 * P_s_s * m_prime * dm_prime[loop];
	}
	dp7[3] = 0.5 * dpss[3] * cos_2chi;

	/* seventh point error */

	for ( loop = 0; loop <= 3; loop++ )
	{
	    sum = sum + dp7[loop] * dp7[loop] * de[loop] * de[loop];
	}
	*p_seven_error = sqrt(sum);		

    }
    else
    {
	/* the above error messages are sufficient
	   sprintf(ErrorString, "No error propagation for Seventh point algorithm - Date : %s", PixelDate);
	   Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_1500", ERROR, LOG_CANNOT_CALCULATE, ErrorString, "");
	   */
    }

    return Status;
}



/*----------------------------------------------------------------------------*\
**                                PolarizationC__AdjustGeometry
**  Input parameters:
**    Line-of-sight zenith angle at Satellite (theta_sat)        [degs]
**            Solar zenith angle at Satellite (theta_0_sat)      [degs]
**        Relative azimuth angle at Satellite (phi_phi_0_sat)    [degs]
**        Earth Radius     at sub-satellite point                [km]
**        Satellite height at sub-satellite point                [km]
**        Atmospheric height for output angles    (atm_height)   [km]
**  Output parameters:
**    Line-of-sight zenith angle at atm_height  (theta)        [degs]
**            Solar zenith angle at atm_height (theta_0)       [degs]
**        Relative azimuth angle at atm_height (phi_phi_0 )    [degs]
**    Arbitrariness of Azimuth definition   (do_arbitrary)     [t/f]
**    twilight flag                         (do_twilight)      [t/f]
**  Other interfaces:
**    Input files INITIAL
**  Description:
**    Converts geometrical angles at satellite to given atmospheric height
**  References:
**    This function is described in the Module ...
**  Libraries:
**    libm
**  Created:	26.1.94
**  Author:		Rob Spurr, DLR/WT-DA-SE
** 			transferred to C++ code by Bernd Aberle, DLR/WT-DA-SE
**                      Double precision  09 June 1995
\*----------------------------------------------------------------------------*/
FunctionStatus_e PolarizationC__AdjustGeometry (float theta_sat, float theta_0_sat, float phi_phi_0_sat, float earth_radius, float sat_height, float smallval, float atm_height,float * theta, float *theta_0, float * phi_phi_0, Bool * do_arbitrary, Bool * do_twilight)
{
/*
  Revised version of old GOMEware routine called FIND_ANGLES
  This FORTRAN version prepared for Operational GDP processing

  Date : 8 June 1994
  Author : R. Spurr, DLR

  From the Geolocation information at the satellite, this routine
  calculates appropriate geometrical angles at any height in the atmosphere

  Applications -

  (1) Finds correct geometrical angles at top of atmosphere
  required as input to any radiative transfer algorithm
  (AMF, GOMETRAN, etc..). [ Level 1-2 processing ]

  (2) Finds correct geometrical angles required for computation of
  "seventh point" polarisation value at given atmospheric height
  [ Level 0-1 processing, polarisation correction algorithm ]

  Input arguments 
  ===============

  1. Geolocation information :

  theta_sat  = scan angle at the satellite (+/-)    [degs]
  theta_0_sat = solar zenith angle at the satellite  [degs]
  phi_phi_0_sat = relative azimuth at the satellite  [degs]
  earth_radius = radius of earth at subsatellite point [km]
  sat_height   = height of satellite above subsatellite point [km] 

  NOTE - Calculation done for relative azimuths in range [-180,180].  If the input azimuth
  is in range [0,360], this will be given a dummy assignment in range [-180,180].
  OUTPUT PHI_PHI0 IS IN THE RANGE [-180,180].

  2.  Other parameters

  atm_height   = height in atmosphere at which angles are required [km]
  smallval     = smallness parameter for special cases


  Output arguments
  ================

  theta_0    = Solar zenith at given atmosphere height  [degs]
  theta     = Line-of-sight zenith at given atmosphere height  [degs]
  phi_phi_0  = Relative azimuth angle at given atm. height [degs]
  do_arbitrary = arbitrariness flag (True if relative azimuth not defined)
  do_twilight  = twilight flag (True if atmosphere not illuminated)

  */

    FunctionStatus_e Status = SUCCESS;

/* Local variables. */

    double  BETA, DELTA, RELAZ, BETAR, DELTAR, RELAZR, GAMMA;
    double  SIDES, NUMER, DENOM, TERM;
    double  LOSZ, SOLZ, RAZIM;
    int 	JARBIT;
    Bool 	IARBO;


/* Initialise outputs. */

    *theta_0 = 0.0;
    *theta  = 0.0;
    *phi_phi_0 = 0.0;
    *do_arbitrary = False;
    *do_twilight  = False;

/* Set up dummy values, and convert azimuth to appropriate range */

    DELTA = theta_0_sat;
    GAMMA = fabs( theta_sat );
    RELAZ = phi_phi_0_sat;
    if ( RELAZ > 180.0 )  RELAZ = RELAZ - 360.0;
    if ( RELAZ < -180.0 ) RELAZ = RELAZ + 360.0;

/* Determine arbitrariness */
  
    if ( DELTA <= smallval ) DELTA = 0.0;
    if ( GAMMA <= smallval ) GAMMA = 0.0;

/* [ JARBIT = 0 for general case, no arbitrariness ] */

    JARBIT = 0;

/* [ If vertical optical path downwards, then JARBIT = 1 ] */

    if ( GAMMA == 0.0 ) JARBIT = 1;

/* [ If sun directly overhead, JARBIT = 2 ] */
/* [ If both conditions hold, set JARBIT = 3 ] */

    if ( DELTA == 0.0 ) JARBIT = JARBIT + 2;


/* C***Original code from GOMEware, not altered... */

/* Step 3. Adjust these angles DELTA and RELAZ to given atmospheric height */
/* *************************************************************************/

/* Initialise */

    LOSZ = 0.0;
    SOLZ = 0.0;
    RAZIM = 0.0;
    IARBO = False;

/* (3a) First find the angle subtended at earth's centre */
/*      and thence line of sight zenith at given atmospheric height */

    BETA = 0.0;

    if ( JARBIT != 1 )
    {
	SIDES = ( earth_radius + sat_height ) /
	    ( earth_radius + atm_height );
	BETA  = - GAMMA + RAD_TO_DEG *
	    asin ( SIDES * sin ( GAMMA * DEG_TO_RAD ));
    }
    LOSZ = GAMMA + BETA;

/* (3b) Next find new relative azimuth RAZIM, allowing for special cases */

    DELTAR = DELTA * DEG_TO_RAD;
    BETAR  = BETA * DEG_TO_RAD;

    if ( JARBIT != 0 )
    {
	if ( JARBIT != 1 && JARBIT != 3 && BETA > smallval)
	{
	    JARBIT = 0;                     /* moved out from under sun */
	    RAZIM = 180.0;
	}
    }
    else
    {

	RELAZR = RELAZ * DEG_TO_RAD;
	NUMER = sin(DELTAR) * sin(RELAZR);

	DENOM = cos(BETAR) * sin(DELTAR) * cos(RELAZR)
	    - sin(BETAR) * cos(DELTAR);

	if ( DELTA <= smallval )
	{
	    if ( BETA <= smallval )        /* Special case DELTA=0 */
	    {
		JARBIT = 2;
	    }
	    else
	    {
		RAZIM = 180.0;
	    }
	}
	else if ( fabs(RELAZ) <= smallval )
	{
	
	    if (fabs(BETA-DELTA) < smallval )    /* Special case RELAZ=0 */
	    {
		JARBIT = 2;
	    }
	    else
	    {
		if ( BETA < DELTA ) RAZIM = 0.0;
		if ( BETA > DELTA ) RAZIM = 180.0;
	    }
	}

	/* Special case, RELAZ = 180: */

	else if (fabs(RELAZ) >= (180.0-smallval) )
	    RAZIM = 180.0;
	
	else if ( fabs(DENOM) <= smallval )
	{
	    if ( RELAZ < 0.0 ) RAZIM = -90.0;
	    if ( RELAZ > 0.0 ) RAZIM = +90.0;
	}
	else
	{
	    RAZIM = RAD_TO_DEG * atan ( NUMER/DENOM );
	    if ( RELAZ>0.0 && RAZIM<0.0 ) RAZIM=RAZIM+180.0;
	    if ( RELAZ<0.0 && RAZIM>0.0 ) RAZIM=RAZIM-180.0;
	}
    }

/* (3c)  Now find new solar zenith angle for given atmospheric height */

    switch (JARBIT)
    {
    case 0 :
	{
	    SOLZ = RAD_TO_DEG * acos ( cos(DELTAR) * cos(BETAR) +
				       sin(DELTAR) * sin(BETAR) * cos(RELAZR) );
	    break;
	}

    case 1 :
	{
	    SOLZ = DELTA;
	    IARBO = True;
	    break;
	}

    case 2 :
	{
	    SOLZ = BETA;
	    break;
	}

    case 3 :
	{
	    SOLZ = 0.0;
	    IARBO = True;
	    break;
	}
    }


/* C*** End of original code */

/* Assign output variables */

    *theta_0   = SOLZ;
    *theta    = LOSZ;
    *phi_phi_0 = RAZIM;
    *do_arbitrary = IARBO;

/* Set twilight flag. */

    SIDES = 1.0/(1.0 + (atm_height/earth_radius));
    TERM = 90.0 + RAD_TO_DEG * acos (SIDES);
    if ( *theta_0 >= TERM ) *do_twilight = True;

/* Finish */

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                PolarizationC__GDF
**  Input parameters:
**    Gradient at first PMD point (grad_first)
**    Wavelengths and p's at p7 and first PMD point, x0, p0, x1, p1
**    Wavelengths of the given band (BAND_1A .. BAND_4)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**  Output parameters:
**    Computed polarisations for given band (BandCoefficients)
**  Other interfaces:
**    Call to numerical recipes routine rtbis
**  Description:
**    Finds a p value for each wavelength of the range between the 7th point
**    and first PMD p value using an analytic function (GDF)
**  References:
**    This function is described in the Module ...
**  Libraries:  numrecip
**  Version     1.1
**  Created:	28.7.94
**  Author:		Rob Spurr, DLR/WT-DA-SE
**  Updated:		Bernd Aberle, 16 October 1995, Put this algorithm to this separate function 
\*----------------------------------------------------------------------------*/
void 
PolarizationC__GDF(float grad_first, float x0, float p0, float x1, float p1,
		  const float *Lambda, float *BandCoefficients, 
		  short Begin, short End, const char *PixelDate)
{
#ifdef DEBUG
    FILE *out;
    char debug_filename[80];
#endif
    int loop;
    float beta, w0, pbar;
    float alpha, w, fw, kf, xmin, xchange, xstar, pstar, mstar;
    float apar, bpar, cpar;
    double b1, b2;
    double f1;
    int badcase = 0;
    FunctionStatus_e Status = SUCCESS;
    char ErrorString[LOG_FIELD_SIZE];

    /*-----------------------------------------------------------------------*\
    **  Calculate GDF parameters
    \*-----------------------------------------------------------------------*/
    alpha = 4.0 * ( p0 - p1 ) / grad_first;
		
    if ( alpha < 0.0 )
    {
	mstar = grad_first;
	xstar = x1;
	pstar = p1;
	badcase = 1;
    }
    if ( alpha > 0.0 )
    {
	xchange = 325.0;
	xmin = 0.5 * ( x1 + xchange );
	apar = grad_first * 0.5 / ( x1 - xmin );
	bpar = - 2.0 * apar * xmin;
	cpar = p1 - bpar * x1 - apar * x1 * x1;
	xstar = xmin - 0.5;
	pstar = apar * xstar * xstar + bpar * xstar + cpar;
	mstar = 2.0 * apar * xstar + bpar;
	badcase = 2;
    }
    if ( alpha == 0.0 )
    {
      sprintf(ErrorString, "p0 is equal to p1 - cannot interpolate using analytic gfd/power curves PixelDate : %s", PixelDate);
      Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_1855", WARN, LOG_CANNOT_CALCULATE, ErrorString,"");
      return;
    }

    xd1 =  (xstar - x0);
    k1 = 4.0 * ( p0 - pstar ) / mstar;
    b2 = 100.0;
    b1 = 0.00001;
    beta = rtbis(gdf_function, b1, b2, SMALL, &Status);

    if ( (Status == FATAL) || (Status == ERROR) )
    {
	sprintf(ErrorString, "Bisection routine failure - cannot interpolate using analytic gfd/power curves PixelDate : %s", PixelDate);
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "4.1_1855", WARN, LOG_CANNOT_CALCULATE, ErrorString,"");

	beta = 0.0;
	badcase = 3;
    }
    else
    {
	f1 = (double) (xd1 * beta);
	if ( f1 > BIGNUM ) 
	{
	    f1 = 0.0;
	}
	else
	{
	    f1 = exp(-f1);
	}
	kf = (float) (f1/((1.0+f1)*(1.0+f1)));
			
	w0 = (p0-pstar)/(0.25-kf);
			
	pbar = p0 - 0.25 * w0;
	
	
	/*-----------------------------------------------------------------------*\
	**  Apply Analytic curves
	\*-----------------------------------------------------------------------*/
	for ( loop = Begin; loop <= End; loop++ )
	{
	    w = Lambda[loop];
	
	
	    /*-----------------------------------------------------------------------*\
	    ** ..If band wavelength between 7th point and interpolation change-over, 
	    ** use Generalised Distribution Function 
	    \*-----------------------------------------------------------------------*/
	    if ( w > x0 && w <= x1)
	    {
		if ( badcase == 1 )
		{
		    fw = (float) ( exp( - ( w - x0 ) * beta ) );
		    BandCoefficients[loop] = pbar + w0 * fw / ( ( 1.0 + fw ) * ( 1.0 + fw ) );
		}
		if ( badcase == 2 )
		{
		    if ( w > x0 && w <= xstar )
		    {
			fw = (float) ( exp( - ( w - x0 ) * beta ) );
			BandCoefficients[loop] = pbar + w0 * fw / ( ( 1.0 + fw ) * ( 1.0 + fw ) );
		    }
		    if ( w > xstar )
		    {
			BandCoefficients[loop] = apar * w * w + bpar * w + cpar;
		    }
		}
	    }
	}
    }

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG
    sprintf (debug_filename, "%s", "PolarizationC__Compute.dbg" );
    if (Debug__isActive(debug_ptr, DBG_POLARIZATION_APPLY, DBG_LEVEL_1))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    fprintf(out,"\n");
	    fprintf(out,"beta= %f",beta);
	    fprintf(out,"\nw0= %f",w0);
	    fprintf(out,"\npbar= %f\n",pbar);
	
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "4.1_1932", WARN, LOG_CANNOT_OPEN, debug_filename, "permissions");
	}
    }
#endif
}


/*----------------------------------------------------------------------------*\
**                              PolarizationC__Read
**  Input parameters:
**	Stream		input stream to read polarization
**  Output parameters:
**	Param		read Polarization parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all polarization parameter entries from the given 
**	input stream into the polarization structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
PolarizationC__read(PolarizationC_s *Param, FILE* Stream,
		   Bool convert_sun_2_host)
{
    FunctionStatus_e Status = SUCCESS;
    long NoOfBytes = 0;  
    char LogBuffer[LOG_FIELD_SIZE];
 
    /*--------------------------------------------------------------------*\
    ** read Polarization coefficients
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->Coefficient, 1, sizeof( Param->Coefficient),
			Stream);

    /*--------------------------------------------------------------------*\
    **  read Wavelength of the corresponding polarization coefficients
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->Wavelength, 1, sizeof( Param->Wavelength),
			Stream);

    /*--------------------------------------------------------------------*\
    **  read Errors of the corresponding
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->Error, 1, sizeof( Param->Error),
			Stream);

    /*--------------------------------------------------------------------*\
    **  read Plane of polarization angle
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( &Param->Chi, 1, sizeof( Param->Chi),
			Stream);

    /*--------------------------------------------------------------------*\
    **  check 
    \*--------------------------------------------------------------------*/
    if (NoOfBytes != POLARIZATION_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "Polarization Parameter, #read bytes: %d", NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.3__2208", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	PolarizationC__Sun2Host( Param);
    }

    return Status;
}

/*----------------------------------------------------------------------------*\
**                           PolarizationC__Show
**  Input parameters:
**	Stream		output stream
**	Param		polarization structure
**
**  Output parameters:
**
**  Other interfaces:
**	polarization structure written to the output stream
**
**  Description:
**	Function to write the polarization structure to the output stream in
**	ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PolarizationC__show(const PolarizationC_s *Param, FILE* Stream)
{
    int	i;

    fprintf( Stream, "Polarisation Parameter\n");
    /*--------------------------------------------------------------------*\
    ** write Polarization coefficients
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Index\tWavelength\tCoefficient\tError\n");
    for ( i = 0; i < POLARIZATION_COEFFICIENTS; i++)
    {
	fprintf( Stream, "%5d\t%f\t%f\t%f\n",
		 i, Param->Wavelength[ i], Param->Coefficient[ i], Param->Error[ i]);
    }

    /*--------------------------------------------------------------------*\
    **  write Plane of polarization angle
    \*--------------------------------------------------------------------*/
    fprintf( Stream, "Polarisation Plane Angle (Chi) %f\n", Param->Chi);
}


/*----------------------------------------------------------------------------*\
**                           PolarizationC__Sun2Host
**  Input parameters:
**    Polarization		in Sun memory representation
**  Output parameters:
**    Polarization		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the Polarization struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
PolarizationC__Sun2Host(PolarizationC_s *Param)
{
    int	i;

    for ( i = 0; i < POLARIZATION_COEFFICIENTS; i++)
    {
	/* float */
	Param->Coefficient[ i] = SunFloat2Float( (unsigned char*)
						 &Param->Coefficient[ i]);
	Param->Wavelength[ i] = SunFloat2Float( (unsigned char*)
						&Param->Wavelength[ i]);
	Param->Error[ i] = SunFloat2Float( (unsigned char*)
					   &Param->Error[ i]);
    }
    /* float */
    Param->Chi = SunFloat2Float( (unsigned char*)&Param->Chi);
}


/*============================================================================*\
**  private operations
\*============================================================================*/

/*-----------------------------------------------------------------------*\
** function for use in Compute polarisation interpolation
\*-----------------------------------------------------------------------*/
double
gdf_function(double x)
{
    double faux = (x*xd1 > BIGEXP) ? 0. : exp ( - x * xd1 );
    return x * faux * k1 + ( 1.0 - faux ) * ( 1.0 + faux );
}



