/*----------------------------------------------------------------------------*\
**                                Reflectivity.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)Reflectivity.c	5.1 02/26/99, Copyright DLR 1998"

#include <math.h>
#include "reflvity.h"
#include "log.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)Reflectivity.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("Reflectivity")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                Reflectivity__Check
**  Input parameters:
**    IntensityGrid1 (1024) : grid for a given channel 1
**    IntensityGrid2 (1024) : grid for a given channel 2 which is partly overlapping channel 1
**    Reflectivity1 (1024) : reflectivity for channel 1 claculated by Reflectivity__Compute(...)
**    Reflectivity2 (1024) : reflectivity for channel 2 overlapping channel 1
**    Begin1 (first pixel in 1024 array for channel 1)
**    End1 (last pixel in 1024 array for channel 1)
**    Begin2 (first pixel in 1024 array for channel 2)
**    End2 (last pixel in 1024 array for channel 2)
**    Integration time (only for debugging output)
**    Wavelength : a Wavelength in overlap region between the two channels, where to build the
**      reflectivity ratio
**    PixelDate (ASCII string)
**
**  Output parameters:
**    *Result : 0 if reflectivity ratio is between 1.0 and JUMP_INDICATOR_GOOD
**              1 if reflectivity ratio is between JUMP_INDICATOR_GOOD and JUMP_INDICATOR_BAD
**              2 if reflectivity ratio is greater than JUMP_INDICATOR_BAD or no check at all has been done
**    FunctionStatus_e : SUCCESS if it was possible to build mean reflectivities at the given wavelength.
**      otherwise set to ERROR.
**
**  Other interfaces:
**
**  Description:
**    Requires two channels with overlap region. Builds then a mean value of reflectivity from 10 points
**    for each of the channels around the same wavelength. Builds then a ratio of these mean reflectivities.
**    Detects then if there is a jump in reflectivity by comparing with a limit defined as a constant in this
**    module.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	12.1.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e Reflectivity__Check(const float IntensityGrid1[], const float Reflectivity1[], short Begin1, short End1, 
				     const float IntensityGrid2[], const float Reflectivity2[], short Begin2, short End2, const float IntgTime,
				     int *Result, float Wavelength, const char *PixelDate)
{
    int loop;
    int counter;
    int loopwin;
    float MeanValue1;
    float MeanValue2;
    float Ratio;

#ifdef DEBUG
    FILE *out;
    char debug_filename[80];
#endif

    /*--------------------------------------------------------------------------------------------*\
    ** initialize result (= reflectivity jump flag)
    \*--------------------------------------------------------------------------------------------*/
    *Result = 0;

    /*--------------------------------------------------------------------------------------------*\
    ** build mean value of reflectivity for first channel
    \*--------------------------------------------------------------------------------------------*/
    loop = End1;
    counter = 0;
    MeanValue1 = 0.0;

    while (IntensityGrid1[loop] > Wavelength) loop--;

    for (loopwin = -5; loopwin <= 5; loopwin++)
    {
	if (loop + loopwin <= End1 && loop + loopwin >= Begin1)
	    MeanValue1 += Reflectivity1[loop + loopwin];

	counter++;
    }

    if (counter < 7)
	return ERROR;
    else
	MeanValue1 /= counter;



    /*--------------------------------------------------------------------------------------------*\
    ** build mean value of reflectivity for second channel
    \*--------------------------------------------------------------------------------------------*/
    loop = Begin2;
    counter = 0;
    MeanValue2 = 0.0;

    while (IntensityGrid2[loop] < Wavelength) loop++;

    for (loopwin = -5; loopwin <= 5; loopwin++)
    {
	if (loop + loopwin <= End2 && loop + loopwin >= Begin2)
	    MeanValue2 += Reflectivity2[loop + loopwin];

	counter++;
    }

    if (counter < 7)
	return ERROR;
    else
	MeanValue2 /= counter;

    /*--------------------------------------------------------------------------------------------*\
    ** check and set flag
    \*--------------------------------------------------------------------------------------------*/
    Ratio = 0;

    if (MeanValue2 > MeanValue1)
    {
	if (MeanValue1 > SMALL_REFLEC || MeanValue1 < -SMALL_REFLEC) Ratio = MeanValue2 / MeanValue1;
    }
    else
    {
	if (MeanValue2 > SMALL_REFLEC || MeanValue2 < -SMALL_REFLEC) Ratio = MeanValue1 / MeanValue2;
    }

    if (Ratio >= JUMP_INDICATOR_ONE && Ratio <= JUMP_INDICATOR_GOOD) *Result = 0;
    else if (Ratio > JUMP_INDICATOR_GOOD && Ratio <= JUMP_INDICATOR_BAD) *Result = 1;
    else *Result = 2;

    /*-----------------------------------------------------------------------*\
    ** test output
    \*-----------------------------------------------------------------------*/
#ifdef DEBUG	
    sprintf (debug_filename, "%s", "Reflectivity__Check.dbg");
    if (Debug__isActive(debug_ptr, DBG_INTENSITY, DBG_LEVEL_1))
    {
	if ((out = fopen(debug_filename, "a")) != NULL)
	{
	    fprintf(out, "%-12.5g %-12.5g %-12.5f %-10d  %-7.3f %s\n", 
		    MeanValue1, 
		    MeanValue2, 
		    Ratio, 
		    *Result,
		    IntgTime, 
		    PixelDate);		
	    fclose(out);
	}
	else
	{
	    Log__Msg(LOG_EXTRACT_DATA, SS_ID, "3.17_167", WARN, LOG_CANNOT_OPEN,
		     debug_filename, "permissions");
	}
    }
#endif


    /*-----------------------------------------------------------------------*\
    **
    \*-----------------------------------------------------------------------*/
    return SUCCESS;
}


/*----------------------------------------------------------------------------*\
**                                Reflectivity__Compute
**  Input parameters:
**    SolarZenith angle in radiants
**    SunCalibrated (1024) : 		complete calibrated irradiance (photons/...)
**    RadianceCalibrated (1024) :	complete calibrated radiance (photons/...)
**    Begin (first pixel in 1024 array)
**    End (last pixel in 1024 array)
**
**  Output parameters:
**    Reflectivity (1024) : reflectivity for this channel : (earth * pi) (/ sun * cos(SolarZenithAngle))
**
**  Other interfaces:
**
**  Description:
**    calculates the reflectivity for this channel ( (earth * pi) (/ sun * cos(SolarZenithAngle)) )
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**  Created:	12.1.96
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void Reflectivity__Compute(const float SolarZenith, const float SunCalibrated[], const float RadianceCalibrated[], float Reflectivity[], short Begin, short End)
{
    int loop;
    float tmp1;

    /*--------------------------------------------------------------------------------------------*\
    ** calculate the reflectivity for this channel : (earth * pi) (/ sun * cos(SolarZenithAngle))
    \*--------------------------------------------------------------------------------------------*/
    for (loop = Begin; loop <= End; loop++)
    {
	tmp1 =  SunCalibrated[loop] * cos(SolarZenith * M_PI / 180.0);

	if (tmp1 > SMALL_REFLEC || tmp1 < -SMALL_REFLEC)
	    Reflectivity[loop] = RadianceCalibrated[loop] * M_PI / tmp1;
	else
	    Reflectivity[loop] = 0.0;
    }
}


/*============================================================================*\
**  private operations
\*============================================================================*/


