/*----------------------------------------------------------------------------*\
**                                SDH.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SDH.c	5.2 08/19/02, Copyright DLR 1998"

#include <stdlib.h>
#include "sdh.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)SDH.c 5.2 08/19/02, Copyright DLR 1998")


#define	SS_ID	("SDH")


/*============================================================================*\
**  public operations
\*============================================================================*/


/*----------------------------------------------------------------------------*\
**                            SDH__generate
**  Input parameters:
**    DataSetType               Type of Data Set( e.g. LED__)
**    OrbitNumber               OrbitNumber
**    NrOfFiles                 number of Input Data Reference
**    SoftwareVersion           software version of the GDP
**    TimeCor                   time correlation
**    StateVector               State vector
**    ProductType               Product type of Input Data Reference
**    ProcessingInfo            Processing Information of product
**
**  Output parameters:
**    SDH                       Specific Data Set Header (SDH)
**    Generate_SDH              SUCCESS = SDH generation successful
**                              else    = not successful
**
**  Other interfaces:
**    Output file ....
**  Description:
**    Function to generate the Specific Data Set Header (SDH). The SDH
**    is generated with some fix values and some input values( see Inputs).
**    The Header is generated entry by entry. This ensures that there are no
**    unnessessary wholes(chars) within the generated SDH.
**    ....
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:    10.8.94
**  Author:             Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SDH__generate( SDH_s *SDH, const char* DataSetType, unsigned long OrbitNumber,
              short NrOfFiles,
              const char* SoftwareVersion, const TimeCor_s* TimeCor,
              const StateVectorC_s* StateVector, const char* ProductType,
              const char** ProcessingInfo)
{
    FunctionStatus_e    Stat;
    int                 i;

    Stat = SUCCESS;

    /*--------------------------------------------------------------------------*\
    ** copy Data Set Type
    \*--------------------------------------------------------------------------*/
    for ( i = 0; i < SDH_DATA_SET_TYPE__SIZE; i++)
        SDH->DataSetType[ i] = DataSetType[ i];

    /*--------------------------------------------------------------------------*\
    ** copy number of input data reference
    \*--------------------------------------------------------------------------*/
    SDH->Nref = NrOfFiles;

    /*--------------------------------------------------------------------------*\
    ** copy PIR
    \*--------------------------------------------------------------------------*/
    for ( i = 0; i < NrOfFiles; i++)
    {
        Stat = PIR__generate( SDH->PIR_Of_SDH[ i], OrbitNumber, ProductType,
                             ProcessingInfo[ i]);
       if (Stat != SUCCESS)
        {
            Stat = FATAL;
            Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.4_1544", Stat, LOG_NUM,
                      "error generating PIR", "");
        }
    }

    /*--------------------------------------------------------------------------*\
    ** copy GDP version
    \*--------------------------------------------------------------------------*/
    for ( i = 0; i < LEVEL_1_PRODUCT_VERSION_SIZE; i++)
        SDH->SoftwareVersion[ i] = SoftwareVersion[ i];

    /*--------------------------------------------------------------------------*\
    ** copy struct time correlation
    \*--------------------------------------------------------------------------*/
    SDH->TimeCor = *TimeCor;

    /*--------------------------------------------------------------------------*\
    ** copy struct state vector
    \*--------------------------------------------------------------------------*/
    SDH->StateVector = *StateVector;

    return Stat;
}


/*----------------------------------------------------------------------------*\
**                            SDH__Sun2Host
**  Input parameters:
**    SDH			in Sun memory representation
**  Output parameters:
**    SDH			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the SDH struct
**    from Sun memory representation into the memory representation of the
**    running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SDH__Sun2Host(SDH_s* Param)
{
  /* short */
  Param->Nraw = SunShort2Short( (unsigned char*)&Param->Nraw);
}

/*----------------------------------------------------------------------------*\
**                                SDH__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		SDH
**
**  Other interfaces:
**
**  Description:
**	Function to read all SDH entries from the given 
**	input stream into the SDH structure
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SDH__read(SDH_s *Param, FILE* Stream, Bool convert_sun_2_host)
{
    int i;
    FunctionStatus_e Status = SUCCESS;
    long NoOfReadBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** Read Data set Type
    \*--------------------------------------------------------------------*/
    NoOfReadBytes = fread( Param->DataSetType, 1,
			   SDH_DATA_SET_TYPE__SIZE, Stream);
    if (NoOfReadBytes != (SDH_DATA_SET_TYPE__SIZE)) {
	Status = FATAL;
	sprintf( LogBuffer, "SDH, Data Set Type, #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__181", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** read number of input data references, input data references itself
    \*------------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->Nref, 1, sizeof( Param->Nref), Stream);
    if (NoOfReadBytes != sizeof( Param->Nref))
    {
	Status = FATAL;
	sprintf(LogBuffer,
		"SDH, number of input data references (Nref), #read bytes:"
		" %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__196", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*----------------------------------------------------------------*\
    ** create Nref
    \*----------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	Param->Nref = SunShort2Short( (unsigned char*)&Param->Nref);
    }

    /*------------------------------------------------------------------------*\
    ** create PIRs
    \*------------------------------------------------------------------------*/
    if (Param->Nref == 0)
    {
	Param->PIR_Of_SDH = 0;
    }
    else {
#ifdef __cplusplus
	Param->PIR_Of_SDH = new char[ Param->Nref][ PIR_SIZE];
#else
	Param->PIR_Of_SDH = (char (*)[PIR_SIZE])
	    malloc(sizeof(char) * Param->Nref * PIR_SIZE);
#endif
	if (Param->PIR_Of_SDH == 0) 
	{
	    Status = FATAL;
	    sprintf( LogBuffer,
		     "SDH, memory for input data reference, #bytes to "
		     "allocate: %d",
		     Param->Nref * PIR_SIZE);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__230", Status,
		      LOG_CANNOT_ALLOCATE, LogBuffer, "");
	return Status;
	}

	/*----------------------------------------------------------------*\
	** Read input data references (PIRs)
	\*----------------------------------------------------------------*/
	for (i = 0; i < Param->Nref; i++) {
	    if (PIR__read(Param->PIR_Of_SDH[ i], Stream) != SUCCESS)
	    {
		Status = FATAL;
		sprintf( LogBuffer,
			 "SDH, Input Data Reference [ %4d], #read bytes: "
			 "%d", i, NoOfReadBytes);
		Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__245", Status,
			  LOG_CANNOT_READ, LogBuffer, "");
		return Status;
	    }
	}
    }

    /*--------------------------------------------------------------------*\
    ** Read versions
    \*--------------------------------------------------------------------*/
    NoOfReadBytes = fread( Param->SoftwareVersion, 1,
			   LEVEL_1_PRODUCT_VERSION_SIZE, Stream);
    if (NoOfReadBytes != LEVEL_1_PRODUCT_VERSION_SIZE) 
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SDH, SoftwareVersion, #read bytes: %d", NoOfReadBytes);
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.2__262", Status, 
		 LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** Read time correlation
    \*--------------------------------------------------------------------*/
    if (TimeCor__read(&Param->TimeCor, Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SDH, Time Correlation, #read bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__275", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** read StateVector
    \*--------------------------------------------------------------------*/
    if (StateVectorC__read(&Param->StateVector, Stream, convert_sun_2_host)
	!= SUCCESS)
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SDH, StateVector, #read bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__289", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** read Nraw
    \*------------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->Nraw, 1, sizeof( Param->Nraw), Stream);
    if (NoOfReadBytes != sizeof( Param->Nraw)) {
	Status = FATAL;
	sprintf( LogBuffer, "SDH, Nraw, #read bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__301", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*------------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	SDH__Sun2Host( Param);
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                SDH__Show
**  Input parameters:
**	Stream		output stream
**	Param		SDH structure
**
**  Output parameters:
**
**  Other interfaces:
**	SDH structure written to the output stream
**
**  Description:
**	Function to write the SDH structure to the
**	output stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SDH__show(const SDH_s *Param, FILE * Stream)
{
  int			i;

  /*----------------------------------------------------------------------*\
  ** show Data Set Type
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "Data Set Type       : ");
  for (i = 0; i < sizeof( Param->DataSetType); i++)
    fprintf( Stream, "%c", Param->DataSetType[ i]);
  fprintf( Stream, "\n");

  /*----------------------------------------------------------------------*\
  ** show input data references (PIRs)
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "Nref                : %3d\n", Param->Nref);
  for ( i = 0; i < Param->Nref; i++)
  {
    PIR__show(Param->PIR_Of_SDH[ i], Stream);
  }

  /*----------------------------------------------------------------------*\
  ** show version
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "SoftwareVersion     : ");
  for (i = 0; i < sizeof( Param->SoftwareVersion); i++)
    fprintf( Stream, "%c", Param->SoftwareVersion[ i]);
  fprintf( Stream, "\n");

  /*----------------------------------------------------------------------*\
  ** show time correlation
  \*----------------------------------------------------------------------*/
  TimeCor__show(&Param->TimeCor, Stream);

  /*----------------------------------------------------------------------*\
  ** show StateVector
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "State Vector:\n");
  StateVectorC__show(&Param->StateVector, Stream);

  /*----------------------------------------------------------------------*\
  ** show Nraw
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "Nraw: %3d\n", Param->Nraw);

}


/*----------------------------------------------------------------------------*\
**                                SDH__Write
**  Input parameters:
**	Stream			output stream
**	Param			SDH
**  Output parameters:
**
**  Other interfaces:
**
**  Description:
**	Function to write the SDH to the given output stream
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SDH__Write( FILE * Stream, const SDH_s *Param)
{
    int			i;
    FunctionStatus_e	Status			= SUCCESS;
    long			NoOfBytes		= 0;
    long			NoOfWrittenBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*----------------------------------------------------------------------*\
    ** Write Data set Type
    \*----------------------------------------------------------------------*/
    NoOfWrittenBytes = fwrite( Param->DataSetType, 1,
			       SDH_DATA_SET_TYPE__SIZE, Stream);
    if (NoOfWrittenBytes != (SDH_DATA_SET_TYPE__SIZE))
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SDH, Data Set Type, #written bytes: %d", NoOfWrittenBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__423", Status, LOG_CANNOT_WRITE,
		  LogBuffer, "");
	return Status;
    }
    NoOfBytes += NoOfWrittenBytes;

    /*----------------------------------------------------------------------*\
    ** Write number of input data references, input data references itself
    \*----------------------------------------------------------------------*/
    NoOfWrittenBytes = fwrite( &Param->Nref, 1, sizeof( Param->Nref), Stream);
    if (NoOfWrittenBytes != sizeof( Param->Nref))
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SDH, number of input data references (Nref), #written bytes: %d",
		 NoOfWrittenBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__439", Status,
		  LOG_CANNOT_WRITE, LogBuffer, "");
	return Status;
    }
    NoOfBytes += NoOfWrittenBytes;

    /*----------------------------------------------------------------------*\
    ** Write input data references (PIRs)
    \*----------------------------------------------------------------------*/
    for ( i = 0; i < Param->Nref; i++)
    {
	if (PIR__write(Param->PIR_Of_SDH[ i], Stream) != SUCCESS)
	{
	    Status = FATAL;
	    sprintf( LogBuffer, "SDH, Input Data Reference[ %4d]", i);
	    Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__454", Status,
		      LOG_CANNOT_WRITE, LogBuffer, "");
	    return Status;
	}
	NoOfBytes += PIR_SIZE;
    }

    /*----------------------------------------------------------------------*\
    ** Write versions
    \*----------------------------------------------------------------------*/
    NoOfWrittenBytes = fwrite( Param->SoftwareVersion, 1,
			       LEVEL_1_PRODUCT_VERSION_SIZE, Stream);
    if (NoOfWrittenBytes != LEVEL_1_PRODUCT_VERSION_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "SDH, SoftwareVersion, #written bytes: %d",
		 NoOfWrittenBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__471", Status, LOG_CANNOT_WRITE,
		  LogBuffer, "");
	return Status;
    }
    NoOfBytes += NoOfWrittenBytes;

    /*----------------------------------------------------------------------*\
    ** Write time correlation
    \*----------------------------------------------------------------------*/
    if (TimeCor__write(&Param->TimeCor, Stream) != SUCCESS)
    {
	Status = FATAL;
	sprintf( LogBuffer, "SDH, Time correlation");
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__484", Status, LOG_CANNOT_WRITE,
		  LogBuffer, "");
	return Status;
    }
    NoOfBytes += TIME_COR_SIZE;
  
    /*----------------------------------------------------------------------*\
    ** Write StateVector
    \*----------------------------------------------------------------------*/
    if (StateVectorC__write(&Param->StateVector, Stream) != SUCCESS)
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__496", Status,
		  LOG_CANNOT_WRITE, "SDH, StateVector", "");
	return Status;
    }
    NoOfBytes += STATE_VECTOR_SIZE;
  
    /*----------------------------------------------------------------------*\
    ** Write Nraw
    \*----------------------------------------------------------------------*/
    NoOfWrittenBytes = fwrite( &Param->Nraw, 1, sizeof( Param->Nraw), Stream);
    if (NoOfWrittenBytes != sizeof( Param->Nraw))
    {
	Status = FATAL;
	sprintf( LogBuffer, "SDH, Nraw, #written bytes: %d", NoOfWrittenBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__510", Status,
		  LOG_CANNOT_WRITE, LogBuffer, "");
	return Status;
    }
    NoOfBytes += NoOfWrittenBytes;

    /*----------------------------------------------------------------------*\
    ** check
    \*----------------------------------------------------------------------*/
    if (NoOfBytes != SDH_STATIC__SIZE + Param->Nref * PIR_SIZE)
    {
	Status = FATAL;
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__522", Status,
		  LOG_CANNOT_WRITE, "SDH", "");
	return Status;
    }

    return Status;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
