/*----------------------------------------------------------------------------*\
**                                SMCD.c
**
**  Description:
** 
**  Version:    5.2
**  Date:       08/19/02
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SMCD.c	5.2 08/19/02, Copyright DLR 1998"

#include "smcd.h"
#include "dataconv.h"
#include "log.h"


#define	SccsId	("@(#)SMCD.c 5.2 08/19/02, Copyright DLR 1998")


#define	SS_ID	("SMCD")


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                SMCD__ReadTime
**  Input parameters:
**	Stream			input file
**      FilePos			file position of the read
**      SMCD_Size		size of the SMCD
**  Output parameters:
**      SMCD_Time		the time of the SMCD
**	SMCD__ReadTime		SUCCESS == time read
**				FATAL	== time not read
**
**  Other interfaces:
**    Input file Level 1 Product defined via Stream
**
**  Description:
**	Function to read the next SMCD and return the Time. If the input
**	variable SMCD is 0, only a local read of the SMCD is done. If
**      the input variable SMCD is not 0, the read SMCD is returned
**      via the variable SMCD.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SMCD__ReadTime(FILE *Stream, long FilePos, long SMCD_Size, UTC_s *SMCD_Time,
	       PCD_s *SMCD, Bool convert_sun_2_host)
{
    FunctionStatus_e	Status	= SUCCESS;
    SMCD_s		SMCD_Local;
    SMCD_s		*SMCD_pt;
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** check SMCD is defined
    \*--------------------------------------------------------------------*/
    if (SMCD == 0)
    {
	SMCD_pt = &SMCD_Local;
    }
    else
    {
	SMCD_pt = (SMCD_s*)SMCD;
    }

    /*--------------------------------------------------------------------*\
    ** read the first SMCD
    \*--------------------------------------------------------------------*/
    fseek ( Stream, FilePos, SEEK_SET);
    if (SMCD__read(SMCD_pt, Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "the SMCD of the given FilePos %d from the Level 1 Product", FilePos);
	Log__Msg( LOG_EXTRACT_DATA, SS_ID, "5.2__83", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** define the search condition
    \*--------------------------------------------------------------------*/
    *SMCD_Time = SMCD_pt->Date;

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                SMCD__GetSpectrumParameter
**  Input parameters:
**    ProductType	product type
**    SPH1		SPH1 of Level 1 Product
**    SMCD		SMCD of the of Level 1 Product 
**
**  Output parameters:
**    PixelPar			pixel specific product parameter
**    GetSpectrumParameter	SUCCESS	= Parameter extracted
**				else	= error during parameter extraction
**
**  Other interfaces:
**
**  Description:
**    Function to get the pixel specific product parameter from the SMCD.
**    The Parameter are listed under Output Parameter.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	7.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SMCD__GetSpectrumParameter(ProductType_e ProductType, const SPH1_s *SPH1,
			   const SMCD_s *SMCD, PixelPar_s *PixelPar)
{
    FunctionStatus_e	Status		= SUCCESS;
    int			i;

    /*----------------------------------------------------------------------*\
    ** get the common pixel specific product parameter
    \*----------------------------------------------------------------------*/
    PixelPar->Date			= SMCD->Date;
    for (i = 0; i < SCIENCE_DATA_DEFINED; i++)
	PixelPar->ScienceIndex[ i]		= SMCD->ScienceIndex[ i];
    PixelPar->IndexSpectralCalPar		= SMCD->IndexSpectralCalPar;
    PixelPar->IndexLeakagePar		= SMCD->IndexLeakagePar;
    PixelPar->DarkCurrentCorFactor	= SMCD->DarkCurrentCorFactor;
    PixelPar->DarkCurrentNoiseCorFactor	= SMCD->DarkCurrentNoiseCorFactor;

    /*----------------------------------------------------------------------*\
    ** get the pixel specific auxiliary parameter ( subset counter, PMD, 
    ** AverageMode, RawDataCorFactor... )
    \*----------------------------------------------------------------------*/
    SPH1__getAuxiliaryParameter(SPH1, SMCD->AuxiliaryData,
				&PixelPar->AuxiliaryPar);

    /*----------------------------------------------------------------------*\
    ** get the earthshine specific pixel parameter
    \*----------------------------------------------------------------------*/
    if (ProductType == PR_SUNSHINE)
	{
	    PixelPar->Specific.Sun.SunAngle = SMCD->SunAngle;
	    PixelPar->Specific.Sun.BSDF_Angle = SMCD->MoonAngle;
	    if (SMCD->UseInReference < 0.0)
		PixelPar->Specific.Sun.UseInReference = False;
	    else
		PixelPar->Specific.Sun.UseInReference = True;
	}
    else
	if (ProductType == PR_MOONSHINE)
	    {
		PixelPar->Specific.Moon.SunAngle			= SMCD->SunAngle;
		PixelPar->Specific.Moon.MoonAngle			= SMCD->MoonAngle;
		PixelPar->Specific.Moon.IlluminatedFraction	= SMCD->UseInReference;
	    }
	else
	    Status = FATAL;

    return Status;
}


/*----------------------------------------------------------------------------*\
**                           SMCD__Sun2Host
**  Input parameters:
**    SMCD			in Sun memory representation
**  Output parameters:
**    SMCD			in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the SMCD struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	6.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SMCD__Sun2Host( SMCD_s *Param)
{
  int	i;

  /* float */
  Param->SunAngle.Zenith = SunFloat2Float( (unsigned char*)
						&Param->SunAngle.Zenith);
  /* float */
  Param->SunAngle.Azimuth = SunFloat2Float( (unsigned char*)
					&Param->SunAngle.Azimuth);
  /* float */
  Param->MoonAngle.Zenith = SunFloat2Float( (unsigned char*)
						&Param->MoonAngle.Zenith);
  /* float */
  Param->MoonAngle.Azimuth = SunFloat2Float( (unsigned char*)
					&Param->MoonAngle.Azimuth);
  /* float */
  Param->UseInReference = SunFloat2Float( (unsigned char*)
					&Param->UseInReference);
  /* float */
  Param->DarkCurrentCorFactor = SunFloat2Float( (unsigned char*)
						&Param->DarkCurrentCorFactor);
  /* float */
  Param->DarkCurrentNoiseCorFactor = SunFloat2Float( (unsigned char*)
					&Param->DarkCurrentNoiseCorFactor);
   /* short */
  Param->IndexSpectralCalPar = SunShort2Short( (unsigned char*)
						&Param->IndexSpectralCalPar);
   /* short */
  Param->IndexLeakagePar = SunShort2Short( (unsigned char*)
						&Param->IndexLeakagePar);
  for ( i = 0; i < AUXILIARY_DATA_SIZE; i++)
  {
    /* short */
    Param->AuxiliaryData[ i] = SunShort2Short( (unsigned char*)
						&Param->AuxiliaryData[ i]);
  }
  for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
  {
    /* short */
    Param->ScienceIndex[ i] = SunShort2Short( (unsigned char*)
						&Param->ScienceIndex[ i]);
  }
}

/*----------------------------------------------------------------------------*\
**                                SMCD__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		SMCD
**
**  Other interfaces:
**
**  Description:
**	Function to read all SMCD entries from the given 
**	input stream into the SMCD structure
**
**  References:
**  Libraries:
**  Created:	6.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SMCD__read(SMCD_s *Param, FILE* Stream, Bool convert_sun_2_host)
{
    int i;
    FunctionStatus_e Status = SUCCESS;
    long NoOfReadBytes = 0;
    char LogBuffer[LOG_FIELD_SIZE];
			 
    /*------------------------------------------------------------------------*\
     * read Date and Time of Ground Pixel End in UTC format
     \*-----------------------------------------------------------------------*/
    if (UTC__read(&Param->Date, Stream, convert_sun_2_host) != SUCCESS)
    {
	Status = FATAL;
	sprintf( LogBuffer, "SMCD UTC time, #read bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__272", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** read Zenith and Azimuth
    \*--------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->SunAngle.Zenith, 1,
			   sizeof( Param->SunAngle.Zenith), Stream);
    NoOfReadBytes += fread( &Param->SunAngle.Azimuth, 1,
			    sizeof( Param->SunAngle.Azimuth), Stream);
    NoOfReadBytes += fread( &Param->MoonAngle.Zenith, 1,
			    sizeof( Param->MoonAngle.Zenith), Stream);
    NoOfReadBytes += fread( &Param->MoonAngle.Azimuth, 1,
			    sizeof( Param->MoonAngle.Azimuth), Stream);
    NoOfReadBytes += fread( &Param->UseInReference, 1,
			    SMCD_USE_IN_REFERENCE_SIZE, Stream);
    if (NoOfReadBytes != SMCD_SUN_ANGLE_SIZE + SMCD_MOON_ANGLE_SIZE +
	SMCD_USE_IN_REFERENCE_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SMCD, Sun/Moon angles, #read bytes: %d",
		 NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__297", Status, 
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*--------------------------------------------------------------------*\
    ** read Dark Current & Noise Correction factor
    \*--------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->DarkCurrentCorFactor, 1,
			   sizeof( Param->DarkCurrentCorFactor), Stream);
    NoOfReadBytes += fread(&Param->DarkCurrentNoiseCorFactor, 1,
			   sizeof( Param->DarkCurrentNoiseCorFactor), 
			   Stream);
    if (NoOfReadBytes != SMCD_DARK_CURRENT_AND_NOISE_COR_SIZE) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SMCD, Dark Current & Noice Correction Factor, #read "
		 "bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__315", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** read index to Leakage calibration parameter
    \*------------------------------------------------------------------------*/
    NoOfReadBytes = fread( &Param->IndexSpectralCalPar, 1,
			   SMCD_INDEX_SPECTRAL_CAL_PAR, Stream);
    NoOfReadBytes += fread( &Param->IndexLeakagePar, 1,
			    SMCD_INDEX_LEAKAGE_COR_PAR, Stream);
    if ( NoOfReadBytes != (SMCD_INDEX_SPECTRAL_CAL_PAR +
			   SMCD_INDEX_LEAKAGE_COR_PAR)) {
	Status = FATAL;
	sprintf( LogBuffer,
		 "SMCD, Index to leakage calibration parameter, #read "
		 "bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__333", Status, 
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }

    /*------------------------------------------------------------------------*\
    ** read MPH_L1, SPH_L1, AuxiliaryData, ScienceIndex
    \*------------------------------------------------------------------------*/
    NoOfReadBytes = xCD__Read( Stream, Param->MPH_L1, Param->SPH_L1,
			       Param->AuxiliaryData, Param->ScienceIndex);
    if (NoOfReadBytes != (SMCD_MPH_EXTRACT_SIZE + SMCD_SPH_EXTRACT_SIZE +
			  SMCD_INSTRUMENT_HEADER_SIZE +
			  SCIENCE_DATA_DEFINED * SMCD_SCIENCE_INDICE_SIZE)){
	Status = FATAL;
	sprintf( LogBuffer,
		 "SMCD, xCD, #read bytes: %d", NoOfReadBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.2__349", Status,
		  LOG_CANNOT_READ, LogBuffer, "");
	return Status;
    }
	
    /*------------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*------------------------------------------------------------------------*/
    if (convert_sun_2_host)
    {
	SMCD__Sun2Host(Param);
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                                SMCD__Show
**  Input parameters:
**	Stream		output stream
**	Param		SMCD Structure
**
**  Output parameters:
**
**  Other interfaces:
**	SMCD Structure written to the output stream
**
**  Description:
**	Function to write the SMCD Structure to the output
**	stream in ASCII.
**
**  References:
**  Libraries:
**  Created:	26.7.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SMCD__show(const SMCD_s *Param, FILE * Stream, Bool FullInfo)
{
  int i;
  char ASCII_Time[ 30];

  /*----------------------------------------------------------------------*\
  * write Date and Time of Ground Pixel End in UTC format
  \*----------------------------------------------------------------------*/
  UTC_2_ASCII( Param->Date, ASCII_Time);
  fprintf( Stream, "UTC Days : %u, Msecs: %u, ASCII: %s\n",
	Param->Date.Days, Param->Date.Msecs, ASCII_Time);

  /*----------------------------------------------------------------------*\
  * write rest
  \*----------------------------------------------------------------------*/
  fprintf( Stream, "Sun Zenith and Azimuth\n");
  fprintf( Stream, "%f %f\n\n", Param->SunAngle.Zenith,
		 		Param->SunAngle.Azimuth);
  fprintf( Stream, "Moon Zenith and Azimuth\n");
  fprintf( Stream, "%f %f\n\n", Param->MoonAngle.Zenith,
		 		Param->MoonAngle.Azimuth);
  fprintf( Stream, "Used for MeanSunRef flag  %hd\n\n",
					 Param->UseInReference);
  fprintf( Stream, "Dark Current & Noise Correction Factor\n");
  fprintf( Stream, "%f %f\n\n", Param->DarkCurrentCorFactor,
		 		Param->DarkCurrentNoiseCorFactor);
  if (FullInfo)
  {
    fprintf( Stream, "MPH_L1_SIZE %d", MPH_L1_SIZE);
    for (i = 0; i < MPH_L1_SIZE; i++)
    {
      if (i % 10 == 0)
        fprintf( Stream, "\ni: %3d", i);
      fprintf( Stream, "%3d ", Param->MPH_L1[ i]);
    }
    fprintf( Stream, "\n\n");

    fprintf( Stream, "SPH_L1_SIZE %d", SPH_L1_SIZE);
    for (i = 0; i < SPH_L1_SIZE; i++)
    {
      if (i % 10 == 0)
        fprintf( Stream, "\ni: %3d", i);
      fprintf( Stream, "%3d ", Param->SPH_L1[ i]);
    }
    fprintf( Stream, "\n\n");

    fprintf( Stream, "AUXILIARY_DATA_SIZE %d", AUXILIARY_DATA_SIZE);
    for (i = 0; i < AUXILIARY_DATA_SIZE; i++)
    {
      if (i % 10 == 0)
        fprintf( Stream, "\ni: %3d", i);
      fprintf( Stream, "%5d ", Param->AuxiliaryData[ i]);
    }
    fprintf( Stream, "\n\n");
  }

  fprintf( Stream, "Spectral Parameter Index  %hd\n\n",
						Param->IndexSpectralCalPar);
  fprintf( Stream, "Leakage Parameter Index   %hd\n\n", Param->IndexLeakagePar);

  fprintf( Stream, "Science Data Index\n");
  for (i = 0; i < SCIENCE_DATA_DEFINED; i++)
  {
    fprintf( Stream, "%s\t%d\n", ScienceData2Str((ScienceData_e)i),
			 Param->ScienceIndex[ i]);
  }
  fprintf( Stream, "\n");
}


/*----------------------------------------------------------------------------*\
**                                xCD__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		PCD
**
**  Other interfaces:
**
**  Description:
**	Function to read the common entries of PCD, SCD, MCD from the given 
**	input stream
**
**  References:
**  Libraries:
**  Created:	6.2.96
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
long
xCD__Read(FILE * Stream, char MPH_L1[ MPH_L1_SIZE], char SPH_L1[ SPH_L1_SIZE],
	  GOME_Word_t AuxiliaryData[ AUXILIARY_DATA_SIZE],
	  short ScienceIndex[ SCIENCE_DATA_DEFINED])
{
  int			i;
  FunctionStatus_e	Status		= SUCCESS;
  long			NoOfBytes	= 0;
  long			NoOfReadBytes;
  char LogBuffer[LOG_FIELD_SIZE];
			 
  /*--------------------------------------------------------------------*\
  ** read extracted data from MPH, SPH, Auxiliary Data
  \*--------------------------------------------------------------------*/
  if ( Status == SUCCESS)
  {
    NoOfReadBytes = fread( MPH_L1, 1, MPH_L1_SIZE, Stream);
    if ( NoOfReadBytes != MPH_L1_SIZE)
    {
      Status = FATAL;
      sprintf( LogBuffer, "xCD, MPH_L1, #read bytes: %d", NoOfReadBytes);
      Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.2__2589", Status, LOG_CANNOT_READ,
	LogBuffer, "");
    }
    else
    {
      NoOfBytes += NoOfReadBytes;
    }
  }

  /*--------------------------------------------------------------------*\
  ** Read extracted data from SPH
  \*--------------------------------------------------------------------*/
  if ( Status == SUCCESS)
  {
    NoOfReadBytes = fread( SPH_L1, 1, SPH_L1_SIZE, Stream);
    if ( NoOfReadBytes != SPH_L1_SIZE)
    {
      Status = FATAL;
      sprintf( LogBuffer, "xCD, SPH_L1, #read bytes: %d", NoOfReadBytes);
      Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.2__2608", Status, LOG_CANNOT_READ,
	LogBuffer, "");
    }
    else
    {
      NoOfBytes += NoOfReadBytes;
    }
  }

  /*--------------------------------------------------------------------*\
  ** Read Auxiliary
  \*--------------------------------------------------------------------*/
  if (Status == SUCCESS)
  {
    NoOfReadBytes = fread( AuxiliaryData, 1,
			 AUXILIARY_DATA_SIZE * sizeof( GOME_Word_t), Stream);
    if ( NoOfReadBytes != AUXILIARY_DATA_SIZE * sizeof( GOME_Word_t))
    {
      Status = FATAL;
      sprintf( LogBuffer, "xCD, Auxiliary Data, #read bytes: %d",
	 NoOfReadBytes);
      Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.2__2629", Status, LOG_CANNOT_READ,
	LogBuffer, "");
    }
    else
    {
      NoOfBytes += NoOfReadBytes;
    }
  }

  /*--------------------------------------------------------------------*\
  ** read the Indexes of all the Spectral Bands
  \*--------------------------------------------------------------------*/
  if ( Status == SUCCESS)
  {
    NoOfReadBytes = 0;
    for ( i = 0; i < SCIENCE_DATA_DEFINED; i++)
    {
      NoOfReadBytes += fread( &ScienceIndex[ i], 1,
			 SHORT_SIZE, Stream);
    }
    if (NoOfReadBytes != SCIENCE_DATA_DEFINED * SHORT_SIZE)
    {
      Status = FATAL;
      sprintf( LogBuffer, "xCD, science data index , #read bytes: %d",
	 NoOfReadBytes);
      Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "4.2__2654", Status, LOG_CANNOT_READ,
	LogBuffer, "");
    }
    else
    {
      NoOfBytes += NoOfReadBytes;
    }
  }

  return NoOfBytes;
}


/*============================================================================*\
**  private operations
\*============================================================================*/

 
