/*----------------------------------------------------------------------------*\
**                                SpectralC.c
**
**  Description:
** 
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**      This file contains the following functions and methods:
\*----------------------------------------------------------------------------*/
#pragma ident "@(#)SpectralC.c	5.1 02/26/99, Copyright DLR 1998"

#include "specralc.h"
#include "dataconv.h"
#include "log.h"
#ifdef DEBUG
#include "debug.h"
#endif



#define	SccsId	("@(#)SpectralC.c 5.1 02/26/99, Copyright DLR 1998")


#define	SS_ID	("Spectral")


#ifdef DEBUG
/*----------------------------------------------------------------------------*\
**  for debugging
\*----------------------------------------------------------------------------*/
extern Debug_s* debug_ptr;
#endif


/*============================================================================*\
**  public operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                           SpectralC__Sun2Host
**  Input parameters:
**    Spectral		in Sun memory representation
**  Output parameters:
**    Spectral		in memory representation of the host
**
**  Other interfaces:
**
**  Description:
**    This function converts all entries of the Spectral struct from Sun
**    memory representation into the memory representation of the running host.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	15.3.95
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SpectralC__Sun2Host( SpectralC_s *Param)
{
    int	i, j;
    for ( j = 0; j < OPTICAL_CHANNELS; j++)
    {
	for ( i = 0; i < SPECTRAL_FITT_ORDER; i++)
	{
	    Param->Coefficient[ j][ i] = SunDouble2Double( (unsigned char*)
							   &Param->Coefficient[ j][ i]);
	}
	Param->Error[ j] = SunDouble2Double( (unsigned char*)
					     &Param->Error[ j]);
    }
}

/*----------------------------------------------------------------------------*\
**                              SpectralC__Read
**  Input parameters:
**	Stream		input stream
**  Output parameters:
**	Param		read Spectral Calibration parameter
**
**  Other interfaces:
**
**  Description:
**	Function to read all Spectral Calibration parameter from the given 
**	input stream into the Spectral Calibration parameter structure
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
FunctionStatus_e
SpectralC__read(SpectralC_s *Param, FILE* Stream, Bool CONVERT_SUN_2_HOST)
{
    FunctionStatus_e	Status		= SUCCESS;
    long			NoOfBytes	= 0;
    char LogBuffer[LOG_FIELD_SIZE];

    /*--------------------------------------------------------------------*\
    ** read Spectral Parameter
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->Coefficient, 1, sizeof( Param->Coefficient),
			Stream);
    /*--------------------------------------------------------------------*\
    ** read Spectral Parameter
    \*--------------------------------------------------------------------*/
    NoOfBytes += fread( Param->Error, 1, sizeof( Param->Error),
			Stream);
    if (NoOfBytes != SPECTRAL_SIZE)
    {
	Status = FATAL;
	sprintf( LogBuffer, "Spectral Calibration Parameter, #read bytes: %d",
		 NoOfBytes);
	Log__Msg( LOG_LEVEL_0_TO_1, SS_ID, "5.1__116", Status, LOG_CANNOT_READ,
		  LogBuffer, "");
    return Status;
    }

    /*----------------------------------------------------------------------*\
    ** convert from Sun memory representation to host format
    \*----------------------------------------------------------------------*/
    if (CONVERT_SUN_2_HOST)
    {
	SpectralC__Sun2Host( Param);
    }

    return Status;
}


/*----------------------------------------------------------------------------*\
**                           SpectralC__Show
**  Input parameters:
**	Stream		output stream
**	Param		Spectral parameter structure
**
**  Output parameters:
**
**  Other interfaces:
**	Spectral parameter structure written to the output stream
**
**  Description:
**	Function to write the Spectral parameter structure to the output
**	stream in ASCII.
**
**  References:
**    This function is described in the Module ...
**  Libraries:
**    ....
**  Created:	17.11.94
**  Author:		Meinhard Wolfmueller, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void
SpectralC__show(const SpectralC_s *Param, FILE* Stream)
{
    int	i, j;

/*----------------------------------------------------------------------------*\
    **  show spectral parameter
    \*--------------------------------------------------------------------*/
    for ( i = 0; i < SPECTRAL_FITT_ORDER; i++)
    {
	for ( j = 0; j < OPTICAL_CHANNELS; j++)
	    fprintf( Stream, "\t%e", Param->Coefficient[ j][ i]);
	fprintf( Stream, "\n");
    }

    fprintf( Stream, "Spectral Error\n");
    for ( j = 0; j < OPTICAL_CHANNELS; j++)
	fprintf( Stream, "\t%lf", Param->Error[ j]);
    fprintf( Stream, "\n");
}


/*----------------------------------------------------------------------------*\
**                                SpectralC__GetWavelength
**  Input parameters:
**    Self: SpectralC_s structure
**    Self: SpectralC_s structure
**    Index: pixel index in which the polynom will be evaluated
**  Output parameters:
**  Other interfaces:
**  Description:
**    Evaluates the polynom given by the coefficients in the pixel Index
**    and return this as a wavelength value
**  References:
**  Libraries:
**  Created:	19.6.94
**  Author:		Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
float
SpectralC__GetWavelength(const SpectralC_s *Self, OpticalChannel_e Channel, short Index)
{
    return EvalPolynom_d((double) Index, Self->Coefficient[Channel], SPECTRAL_FITT_ORDER);
}



/*----------------------------------------------------------------------------*\
**                                SpectralC__Apply
**  Input parameters:
**    Self: SpectralC_s structure
**    Self: SpectralC_s structure
**    Begin, End index
**    CallIndex: a calling index used for the debug file name
**  Output parameters:
**    LambdaGrid
**  Other interfaces:
**  Description:
**    Evaluates the polynom given by the coefficients in the pixel indexes
**    from Begin to End and return LambdaGrid with the wavelength values
**  References:
**  Libraries:
**  Created:	19.10.94
**  Author:		Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void 
SpectralC__apply(const SpectralC_s *Self, ScienceData_e Band, float *LambdaGrid,
		short Begin, short End, short CallIndex)
{
    short i;
    OpticalChannel_e Channel = Band2Channel(Band);

#ifdef DEBUG
    /*------------------------------------------------------------------------*\
    **  Declaration and opening the DebugStream
    \*------------------------------------------------------------------------*/
    char debug_filename[MAX_FILENAME_LENGTH];
    FILE* debug_file_ptr = NULL;

    sprintf(debug_filename, "%s.%.3d.dbg", "SpectralC__Apply", CallIndex);
    if (Debug__isActive(debug_ptr, DBG_SPECTRAL, DBG_LEVEL_2) 
	&& (debug_file_ptr = fopen(debug_filename, "a")) == NULL)
    {
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_81", INFO, LOG_CANNOT_OPEN,
		 "debug file 'SpectralC__Apply.XXX.dbg'", "permissions");
    }
    /*------------------------------------------------------------------------*\
    **  Generation of level 2 debugging information
    **  Coefficients
    \*------------------------------------------------------------------------*/
    if (debug_file_ptr != NULL)
    {
	fprintf(debug_file_ptr, "\n%s\nCoefficients\n", ScienceData2Str(Band));
	for (i=0; i<SPECTRAL_FITT_ORDER; i++)
	{
	    fprintf(debug_file_ptr, "\t%e\n", Self->Coefficient[Channel][i]);
	}
	fprintf(debug_file_ptr, "\nPixel\tWavelength\n[-] \t[nm]\n\n");
    }
#endif

    for (i=Begin; i<=End; i++)
    {
	LambdaGrid[i] = EvalPolynom_d((double) i, Self->Coefficient[Channel], SPECTRAL_FITT_ORDER);

#ifdef DEBUG
	/*--------------------------------------------------------------------*\
	**  Generation of level 2 debugging information
	**  Pixel	Wavelength
	\*--------------------------------------------------------------------*/
	if (debug_file_ptr != NULL)
	{
	    fprintf(debug_file_ptr, "%.4d\t%f\n", i, LambdaGrid[i]);
	}
#endif
    }

#ifdef DEBUG
    if ((debug_file_ptr != NULL) && (fclose(debug_file_ptr) != 0))
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "4.1_116", INFO, LOG_CANNOT_CLOSE,
		 "debug file 'SpectralC__Apply.XXX.dbg'", "");
#endif
}


/*============================================================================*\
**  private operations
\*============================================================================*/

/*----------------------------------------------------------------------------*\
**                                EvalPolynom_f
**  Input parameters:
**    X: The point to evaluate the polynom
**    Coefficient: Describe the polynom
**    Grad: grad of the polynom
**  Output parameters:
**  Other interfaces:
**  Description:
**    This function evaluates a polynom of grad Grad described by Coefficient 
**    in the value x
**  References:
**  Libraries:
**  Created:    22.3.94
**  Author: 	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
float
EvalPolynom_f(float X, const float *Coefficient, short Grad)
{
    float Result = 0.0, Mult = 1.0;
    short i;
 
    for (i=0; i<Grad; i++)
    {
        Result += Coefficient[i]*Mult;
        Mult *= X;
    }
    return Result;
}


/*----------------------------------------------------------------------------*\
**                                EvalPolynom_d
**  Input parameters:
**    X: The point to evaluate the polynom
**    Coefficient: Describe the polynom
**    Grad: grad of the polynom
**  Output parameters:
**  Other interfaces:
**  Description:
**    This function evaluates a polynom of grad Grad described by Coefficient 
**    in the value x
**  References:
**  Libraries:
**  Created:    22.3.94
**  Author: 	Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
double
EvalPolynom_d(double X, const double *Coefficient, short Grad)
{
    double Result = 0.0, Mult = 1.0;
    short i;
 
    for (i=0; i<Grad; i++)
    {
        Result += Coefficient[i]*Mult;
        Mult *= X;
    }
    return Result;
}
 
