/*----------------------------------------------------------------------------*\
**                              spline.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function spline
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)spline.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"


/*----------------------------------------------------------------------------*\
**                                spline
**  Input parameters:
**    Vector x[1..n], y[1..n]
**    double yp1, ypn
**    int n
**  Output parameters:
**    Vector y2[1..n]
**  Other interfaces: none
**  Description:
**    Given arrays x[1..n] and y[1..n] containing a tabulated function,
**    y=f(x), with x1<x2<..<xn, and given values yp1 and ypn for the
**    first derivative of the interpolating function at points 1 and n,
**    respectively, this routine returns and array y2[1..n] that
**    contains the second derivatives of the interpolating function at
**    the tabulated points xi. If yp1 and/or ypn are larger than
**    NATURAL_SPLINE (0.99e30), the routine is signalled to set the
**    corresponding boundary condition for a natural spline, with zero
**    second derivative on that boundary
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    28.1.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void spline( double x[], double y[], short n, double yp1, double ypn, double y2[])
{
	short i,k;
	double p,qn,sig,un,*u,*vector();
	void free_vector();

	u=vector__new(1,n-1);
	if (yp1 > 0.99e30)
		y2[1]=u[1]=0.0;
	else {
		y2[1] = -0.5;
		u[1]=(3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
	}
	for (i=2;i<=n-1;i++) {
		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
		p=sig*y2[i-1]+2.0;
		y2[i]=(sig-1.0)/p;
		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}
	if (ypn > 0.99e30)
		qn=un=0.0;
	else {
		qn=0.5;
		un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
	}
	y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
	for (k=n-1;k>=1;k--)
		y2[k]=y2[k]*y2[k+1]+u[k];
	vector__delete(u,1,n-1);
}
