/*----------------------------------------------------------------------------*\
**                              svdfit.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function svdfit
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)svdfit.c	5.1 02/26/99, Copyright DLR 1994"

#include "numrecip.h"
#include "log.h"


#define	SS_ID	("svdfit")

#define	TOL	(1.0e-20)

/*----------------------------------------------------------------------------*\
**                                svdfit
**  Input parameters:
**    vector x[1..ndata], y[1..ndata], sign[1..ndata]
**    short ndata
**    vector a[1..ma]
**    short ma
**    (* funcs)(double, double *, short) user supplied routine
**  Output parameters:
**    matrix u[1..ndata][1..ma]
**    matrix v[1..ma][1..ma]
**    vector w[1..ma]
**    double *chisq
**    Status: tells if there are some problems during the computation
**  Other interfaces: none
**  Description:
**    Backsubstitution routine. Solves A*X = B for a vector X
**    Given a set of points x,y with individual standard deviations
**    given by sig, user x^2 minimization to determine the coefficients
**    a of the fitting function      --  
**                               y = \     a[i]*afunc[i](x)
**                                   /_[i]
**    The fitting equations are solved using SVD of the matrix ndata
**    by ma. matrixes u, v and w provide workspace on input.
**    The program returns values for the ma fit parameters a and X^2
**    chisq. funcs(x, afunc, ma) returns the ma basis functions
**    evaluated a X = X in the array afunc[1..ma].
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    16.2.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
void svdfit( double x[], double y[], double sig[], short ndata, double a[], short ma, double **u, double **v, double w[], double *chisq, void (* funcs)(double, double[], short), FunctionStatus_e * Status)
{
	short j,i;
	double wmax,tmp,thresh,sum,*b,*afunc;

	*Status = SUCCESS;
	b=vector__new(1,ndata);
	afunc=vector__new(1,ma);
	for (i=1;i<=ndata;i++) {
		(*funcs)(x[i],afunc,ma);
		tmp=1.0/sig[i];
		for (j=1;j<=ma;j++) u[i][j]=afunc[j]*tmp;
		b[i]=y[i]*tmp;
	}
	svdcmp(u,ndata,ma,w,v,Status);
	if ((*Status == ERROR) || (*Status == FATAL)) {
		Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_69", *Status, LOG_NUMRECIP, "Can't compute a singular value descomposition", "");
		return;
	}
	wmax=0.0;
	for (j=1;j<=ma;j++)
		if (w[j] > wmax) wmax=w[j];
	thresh=TOL*wmax;
	for (j=1;j<=ma;j++)
		if (w[j] < thresh) w[j]=0.0;
	svbksb(u,w,v,ndata,ma,b,a);
	*chisq=0.0;
	for (i=1;i<=ndata;i++) {
		(*funcs)(x[i],afunc,ma);
		for (sum=0.0,j=1;j<=ma;j++) sum += a[j]*afunc[j];
		*chisq += (tmp=(y[i]-sum)/sig[i],tmp*tmp);
	}
	vector__delete(afunc,1,ma);
	vector__delete(b,1,ndata);
}
