/*----------------------------------------------------------------------------*\
**                              triangle.c
**
**  Numerical Recipes in C
**
**  Version:	5.1
**  Date:		02/26/99
**  Contents:
**    This file contains the function triangle
\*----------------------------------------------------------------------------*/
#include <stdlib.h>
#include "numrecip.h"

#pragma ident	"@(#)triangle.c	5.1 02/26/99, Copyright DLR 1994"


#define	MAX_KERNEL_SIZE	(100)

/*----------------------------------------------------------------------------*\
**                                triangle
**  Input parameters: float DataIn [DETECTOR_PIXELS], short KernelHalfSize,
**    short Begin, short End
**  Output parameters: float DataOut [DETECTOR_PIXELS]
**  Other interfaces:
**  Description:
**    Smoothes DataIn by a triangular convolution. The width of the triangle is
**    defined by KernelHalfSize.
**    Used by Straylight and Fixed
**  References:
**  Libraries:
**  Created:	26.1.95
**  Author:		Wolfgang Balzer, DLR/WT-DA-BS
**  Integrated in GDP:	Bernd Aberle, DLR/WT-DA-SE
**  Optimized by:	Diego Loyola, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void triangle(float DataIn [], float DataOut [], short KernelHalfSize, short Begin, short End)
{
	short	loopi, loopj;
	float	sum, fak, weight;
	const float HalfSize = (float)KernelHalfSize;
	float kernel[MAX_KERNEL_SIZE];

	/* build a triangle-formed kernel like 0..max..0 */
	for (loopj = -KernelHalfSize; loopj <= KernelHalfSize && KernelHalfSize + loopj < MAX_KERNEL_SIZE; loopj++)
		kernel[KernelHalfSize + loopj] = (HalfSize - abs(loopj)) / HalfSize;

	/* convolute data and kernel */
	convol(DataIn, DataOut, Begin, End, kernel, KernelHalfSize);
}

/*----------------------------------------------------------------------------*\
**                                convol
**  Input parameters: float DataIn [DETECTOR_PIXELS], short KernelHalfSize,
**    short Begin, short End
**  Output parameters: float DataOut [DETECTOR_PIXELS]
**  Other interfaces:
**  Description:
**    Smoothes DataIn by a triangular convolution. The width of the triangle is
**    defined by KernelHalfSize
**  References:
**  Libraries:
**  Created:	26.1.95
**  Author:		Bernd Aberle, DLR/WT-DA-SE
\*----------------------------------------------------------------------------*/
void convol(float DataIn[], float DataOut[], short Begin, short End, float Kernel[], short KernelHalfSize)
{
	short	loopi, loopj;
	float	sum, fak;
	const float HalfSize = (float)KernelHalfSize;

	for (loopi = Begin; loopi <= End; loopi++)
	{
		fak = sum = 0.0;
		for (loopj = -KernelHalfSize; loopj <= KernelHalfSize && KernelHalfSize + loopj < MAX_KERNEL_SIZE; loopj++)
		{
			if ((loopi+loopj >= Begin) && (loopi+loopj <= End)) 
			{
				/* build the sum of the kernel */
				fak += Kernel[KernelHalfSize + loopj];

				/* build the sum of the kernel mutiplied by data */
				sum += Kernel[KernelHalfSize + loopj] * DataIn[loopi + loopj];
			}
		}

		/* substitute middle pixel by division of both sums */
		DataOut[loopi] = (fak==0.) ? 0. : sum/fak;
	}
}

