/*----------------------------------------------------------------------------*\
**                              zbrent.c
**
**  Numerical Recipes in C
**
**  Version:    5.1
**  Date:       02/26/99
**  Contents:
**    This file contains the function zbrent
\*----------------------------------------------------------------------------*/
#pragma ident	"@(#)zbrent.c	5.1 02/26/99, Copyright DLR 1994"

#include <math.h>
#include "log.h"
#include "numrecip.h"


#define	SS_ID	("zbrent")


#define	ITMAX	(100)
/* Maximum allowed number of iterations */ 
#define	EPS	(3.0e-8)
/* Machine doubleing point precision */ 
/*----------------------------------------------------------------------------*\
**                                zbrent
**  Input parameters:
**    double (*func)(double)
**    double x1, x2, tol
**  Output parameters:
**    Status: tells if there are some problems during the computation
**  Other interfaces: none
**  Description:
**  Using Brent's method, find the root of a function func know to lie
**  between x1 and x2. The root, returned as zbrent, will be refined until
**  its accuracy is tol.
**  References:
**    Numerical Recipes in C
**  Libraries:
**    Math  (math functions and constants)
**  Created:    9.3.93
**  Changed by: Diego Loyola, DLR/WT-DA-BS
\*----------------------------------------------------------------------------*/
double zbrent(double (*func)(double), double x1, double x2, double tol, FunctionStatus_e * Status)
{
	short iter;
	double a=x1,b=x2,c,d,e,min1,min2;
	double fa=(*func)(a),fb=(*func)(b),fc,p,q,r,s,tol1,xm;

	*Status = SUCCESS;
	if (fb*fa > 0.0) 
	{
		/* Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_50", ERROR, LOG_NUMRECIP, "Root must be bracketed", ""); */
		*Status = ERROR;
		return -1;
	}
	fc=fb;
	for (iter=1;iter<=ITMAX;iter++) {
		if (fb*fc > 0.0) {
			c=a;
			fc=fa;
			e=d=b-a;
		}
		if (fabs(fc) < fabs(fb)) {
			a=b;
			b=c;
			c=a;
			fa=fb;
			fb=fc;
			fc=fa;
		}
		tol1=2.0*EPS*fabs(b)+0.5*tol;
		xm=0.5*(c-b);
		if (fabs(xm) <= tol1 || fb == 0.0) return b;
		if (fabs(e) >= tol1 && fabs(fa) > fabs(fb)) {
			s=fb/fa;
			if (a == c) {
				p=2.0*xm*s;
				q=1.0-s;
			} else {
				q=fa/fc;
				r=fb/fc;
				p=s*(2.0*xm*q*(q-r)-(b-a)*(r-1.0));
				q=(q-1.0)*(r-1.0)*(s-1.0);
			}
			if (p > 0.0)  q = -q;
			p=fabs(p);
			min1=3.0*xm*q-fabs(tol1*q);
			min2=fabs(e*q);
			if (2.0*p < (min1 < min2 ? min1 : min2)) {
				e=d;
				d=p/q;
			} else {
				d=xm;
				e=d;
			}
		} else {
			d=xm;
			e=d;
		}
		a=b;
		fa=fb;
		if (fabs(d) > tol1)
			b += d;
		else
			b += (xm > 0.0 ? fabs(tol1) : -fabs(tol1));
		fb=(*func)(b);
	}
	Log__Msg(LOG_LEVEL_0_TO_1, SS_ID, "5.1_106", ERROR, LOG_NUMRECIP, "Maximum number of iterations exceeded", "");
	*Status = ERROR;
	return b;
}

